@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-log.h"
 #include "core-syslog.h"
 
 #define PR_TIMEOUT	(0.5)	/* pr_lock timeout in seconds */
@@ -32,58 +33,85 @@ static FILE	*log_file = NULL;
 
 static inline FILE *pr_file(void)
 {
-	return (g_opt_flags & OPT_FLAGS_STDOUT) ? stdout : stderr;
+	return (g_opt_flags & OPT_FLAGS_STDERR) ? stderr : stdout;
 }
 
+/*
+ *  pr_log_file_flush()
+ *	ensure log data is flushed as much as possible
+ */
+static void pr_log_file_flush(void)
+{
+
+	if (log_file) {
+		const int fd = fileno(log_file);
+
+		(void)fflush(log_file);
+		if (fd >= 0)
+			shim_fsync(fd);
+	}
+}
 
 #if defined(HAVE_ATOMIC_COMPARE_EXCHANGE) &&	\
     defined(HAVE_ATOMIC_STORE)
 
+/*
+ *  pr_lock_usable()
+ *	inline helper, return true if locking is usable
+ */
+static inline bool pr_lock_usable(void)
+{
+	/* lockless logging - not lockable */
+	if (g_opt_flags & OPT_FLAGS_LOG_LOCKLESS)
+		return false;
+	/* something wrong with shared data? */
+	if (!g_shared)
+		return false;
+	/* locking is usable */
+	return true;
+}
+
 /*
  *  pr_lock_init()
  */
 void pr_lock_init(void)
 {
-	if (g_opt_flags & OPT_FLAGS_LOG_LOCKLESS)
-		return;
-	if (!g_shared)
+	if (!pr_lock_usable())
 		return;
 
-	g_shared->pr_pid = -1;
-	g_shared->pr_lock_count = 0;
-	g_shared->pr_atomic_lock = 0;
-	g_shared->pr_whence = -1.0;
+	g_shared->pr.pid = -1;
+	g_shared->pr.lock_count = 0;
+	g_shared->pr.atomic_lock = 0;
+	g_shared->pr.whence = -1.0;
 }
 
 /*
- *  pr_spin_lock()
- *	simple spin lock
+ *  pr_spin_lock_pid()
+ *	simple spin lock, if pid is -1, getpid
  */
-static void pr_spin_lock(void)
+static void pr_spin_lock_pid(pid_t pid)
 {
 	double timeout_time;
 	pid_t val, orig;
-	pid_t pid;
 
-	if (g_opt_flags & OPT_FLAGS_LOG_LOCKLESS)
-		return;
-	if (!g_shared)
+	if (!pr_lock_usable())
 		return;
 
 	timeout_time = stress_time_now() + PR_TIMEOUT;
-	pid = getpid();
+	if (pid == (pid_t)-1)
+		pid = getpid();
 
 	for (;;) {
 		while (stress_time_now() < timeout_time) {
 			orig = 0;
 			val = pid;
-			if (__atomic_compare_exchange(&g_shared->pr_atomic_lock, &orig,
+			if (__atomic_compare_exchange(&g_shared->pr.atomic_lock, &orig,
 			    &val, 0, __ATOMIC_SEQ_CST, __ATOMIC_SEQ_CST))
 				return;
 			/* Owner dead? force unlock and retry */
-			if ((kill(orig, 0) < 0) && (errno == ESRCH)) {
+			if ((shim_kill(orig, 0) < 0) && (errno == ESRCH)) {
 				val = 0;
-				__atomic_exchange(&g_shared->pr_atomic_lock, &val, &orig, __ATOMIC_SEQ_CST);
+				__atomic_exchange(&g_shared->pr.atomic_lock, &val, &orig, __ATOMIC_SEQ_CST);
 				continue;
 			}
 			shim_sched_yield();
@@ -94,7 +122,7 @@ static void pr_spin_lock(void)
 		 *  and re-try. Urgh.
 		 */
 		val = 0;
-		__atomic_exchange(&g_shared->pr_atomic_lock, &val, &orig, __ATOMIC_SEQ_CST);
+		__atomic_exchange(&g_shared->pr.atomic_lock, &val, &orig, __ATOMIC_SEQ_CST);
 		/* reset the timeout */
 		timeout_time = stress_time_now() + PR_TIMEOUT;
 	}
@@ -108,13 +136,11 @@ static void pr_spin_unlock(void)
 {
 	int zero;
 
-	if (g_opt_flags & OPT_FLAGS_LOG_LOCKLESS)
-		return;
-	if (!g_shared)
+	if (!pr_lock_usable())
 		return;
 
 	zero = 0;
-	__atomic_store(&g_shared->pr_atomic_lock, &zero, __ATOMIC_SEQ_CST);
+	__atomic_store(&g_shared->pr.atomic_lock, &zero, __ATOMIC_SEQ_CST);
 }
 
 /*
@@ -124,18 +150,16 @@ static void pr_spin_unlock(void)
  */
 static void pr_lock_acquire(const pid_t pid)
 {
-	if (g_opt_flags & OPT_FLAGS_LOG_LOCKLESS)
-		return;
-	if (!g_shared)
+	if (!pr_lock_usable())
 		return;
 
 	for (;;) {
 		int32_t count;
 		double whence, now;
 
-		pr_spin_lock();
-		count = g_shared->pr_lock_count;
-		whence = g_shared->pr_whence;
+		pr_spin_lock_pid(pid);
+		count = g_shared->pr.lock_count;
+		whence = g_shared->pr.whence;
 		now = stress_time_now();
 
 		/*
@@ -143,9 +167,9 @@ static void pr_lock_acquire(const pid_t pid)
 		 * up, so pass ownership over
 		 */
 		if (((now - whence) > PR_TIMEOUT) || count == 0) {
-			g_shared->pr_pid = pid;	/* Indicate we now own lock */
-			g_shared->pr_lock_count++;
-			g_shared->pr_whence = now;
+			g_shared->pr.pid = pid;	/* Indicate we now own lock */
+			g_shared->pr.lock_count++;
+			g_shared->pr.whence = now;
 			pr_spin_unlock();
 			return;
 		}
@@ -155,36 +179,34 @@ static void pr_lock_acquire(const pid_t pid)
 }
 
 /*
- *  pr_lock()
+ *  pr_lock_pid()
  *	attempt to get a lock, it's just too bad
  *	if it fails, this is just to stop messages
  *	getting intermixed.
  *
- *  pr_lock can be called multiple times by a process, only
+ *  pr_lock_pid can be called multiple times by a process, only
  *  the first call acquires the lock
  */
-void pr_lock(void)
+static void pr_lock_pid(pid_t pid)
 {
-	pid_t pid;
 	double now;
 
-	if (g_opt_flags & OPT_FLAGS_LOG_LOCKLESS)
-		return;
-	if (!g_shared)
+	if (!pr_lock_usable())
 		return;
 
-	pid = getpid();
-	pr_spin_lock();
+	if (pid == (pid_t)-1)
+		pid = getpid();
+	pr_spin_lock_pid(pid);
 	/* Already own lock? */
-	if (g_shared->pr_pid == pid) {
-		g_shared->pr_lock_count++;
+	if (g_shared->pr.pid == pid) {
+		g_shared->pr.lock_count++;
 		pr_spin_unlock();
 		return;
 	}
 
-	if (g_shared->pr_pid == -1) {
+	if (g_shared->pr.pid == -1) {
 		/* No owner, acquire lock */
-		g_shared->pr_pid = pid;
+		g_shared->pr.pid = pid;
 		pr_spin_unlock();
 		pr_lock_acquire(pid);
 		return;
@@ -196,12 +218,12 @@ void pr_lock(void)
 	 *  take ownership.
 	 */
 	now = stress_time_now();
-	if (((now - g_shared->pr_whence) > PR_TIMEOUT) ||
-	    (kill(g_shared->pr_pid, 0) == ESRCH)) {
+	if (((now - g_shared->pr.whence) > PR_TIMEOUT) ||
+	    (shim_kill(g_shared->pr.pid, 0) == ESRCH)) {
 		/* force acquire */
-		g_shared->pr_pid = pid;
-		g_shared->pr_lock_count = 0;
-		g_shared->pr_whence = now;
+		g_shared->pr.pid = pid;
+		g_shared->pr.lock_count = 0;
+		g_shared->pr.whence = now;
 		pr_spin_unlock();
 		return;
 	}
@@ -210,61 +232,81 @@ void pr_lock(void)
 	pr_lock_acquire(pid);
 }
 
+void pr_lock(void)
+{
+	pr_lock_pid((pid_t)-1);
+}
+
 /*
- *  pr_unlock()
+ *  pr_unlock_pid()
  *	attempt to unlock
  */
-void pr_unlock(void)
+static void pr_unlock_pid(pid_t pid)
 {
-	pid_t pid;
-
-	if (g_opt_flags & OPT_FLAGS_LOG_LOCKLESS)
-		return;
-	if (!g_shared)
+	if (!pr_lock_usable())
 		return;
 
-	pid = getpid();
-	pr_spin_lock();
+	if (pid == (pid_t)-1)
+		pid = getpid();
+	pr_spin_lock_pid(pid);
 	/* Do we own the lock? */
-	if (g_shared->pr_pid == pid) {
-		g_shared->pr_lock_count--;
-		if (g_shared->pr_lock_count == 0) {
-			g_shared->pr_pid = -1;
-			g_shared->pr_whence = -1.0;
+	if (g_shared->pr.pid == pid) {
+		g_shared->pr.lock_count--;
+		if (g_shared->pr.lock_count == 0) {
+			g_shared->pr.pid = -1;
+			g_shared->pr.whence = -1.0;
 		}
 	}
 	pr_spin_unlock();
 }
 
+void pr_unlock(void)
+{
+	pr_unlock_pid((pid_t)-1);
+}
+
 /*
  *  pr_lock_exited()
  *	process has terminated, ensure locks are cleared on it
  */
 void pr_lock_exited(const pid_t pid)
 {
-	if (g_opt_flags & OPT_FLAGS_LOG_LOCKLESS)
-		return;
-	if (!g_shared)
+	if (!pr_lock_usable())
 		return;
 
-	pr_spin_lock();
-	if (g_shared->pr_pid == pid) {
-		g_shared->pr_pid = -1;
-		g_shared->pr_lock_count = 0;
-		g_shared->pr_whence = 0.0;
+	pr_spin_lock_pid(pid);
+	if (g_shared->pr.pid == pid) {
+		g_shared->pr.pid = -1;
+		g_shared->pr.lock_count = 0;
+		g_shared->pr.whence = 0.0;
 	}
 	pr_spin_unlock();
 }
 #else
 
+static inline bool pr_lock_usable(void)
+{
+	return false;
+}
+
 void pr_lock_init(void)
 {
 }
 
+static void pr_lock_pid(pid_t pid)
+{
+	(void)pid;
+}
+
 void pr_lock(void)
 {
 }
 
+static void pr_unlock_pid(pid_t pid)
+{
+	(void)pid;
+}
+
 void pr_unlock(void)
 {
 }
@@ -355,8 +397,9 @@ static int pr_msg(
 {
 	int ret = 0;
 	char ts[32];
+	pid_t pid = getpid();
 
-	pr_lock();
+	pr_lock_pid(pid);
 
 	if (g_opt_flags & OPT_FLAGS_TIMESTAMP) {
 		struct timeval tv;
@@ -381,48 +424,48 @@ static int pr_msg(
 		*ts = '\0';
 	}
 
-	if ((flag & (PR_FAIL | PR_WARN)) || (g_opt_flags & flag)) {
+	if ((flag & (OPT_FLAGS_PR_FAIL | OPT_FLAGS_PR_WARN)) || (g_opt_flags & flag)) {
 		char buf[4096];
 		const char *type = "";
 
-		if (flag & PR_ERROR)
+		if (flag & OPT_FLAGS_PR_ERROR)
 			type = "error:";
-		if (flag & PR_DEBUG)
+		if (flag & OPT_FLAGS_PR_DEBUG)
 			type = "debug:";
-		if (flag & PR_INFO)
+		if (flag & OPT_FLAGS_PR_INFO)
 			type = "info: ";
-		if (flag & PR_FAIL)
+		if (flag & OPT_FLAGS_PR_FAIL)
 			type = "fail: ";
-		if (flag & PR_WARN)
+		if (flag & OPT_FLAGS_PR_WARN)
 			type = "warn: ";
-		if (flag & PR_METRICS)
+		if (flag & OPT_FLAGS_PR_METRICS)
 			type = "metrc:";
 
 		if (g_opt_flags & OPT_FLAGS_LOG_BRIEF) {
 			size_t n = (size_t)vsnprintf(buf, sizeof(buf), fmt, ap);
 			if (log_file) {
 				VOID_RET(size_t, fwrite(buf, 1, n, log_file));
-				(void)fflush(log_file);
+				pr_log_file_flush();
 			}
 			VOID_RET(size_t, fwrite(buf, 1, n, fp));
 		} else {
-			size_t n = (size_t)snprintf(buf, sizeof(buf), "%s%s [%d] ",
-				ts, type, (int)getpid());
+			size_t n = (size_t)snprintf(buf, sizeof(buf), "%s%s [%" PRIdMAX "] ",
+				ts, type, (intmax_t)pid);
 			ret = vsnprintf(buf + n, sizeof(buf) - n, fmt, ap);
 			(void)fprintf(fp, "%s: %s", g_app_name, buf);
 			if (log_file) {
 				(void)fprintf(log_file, "%s: %s", g_app_name, buf);
-				(void)fflush(log_file);
+				pr_log_file_flush();
 			}
 		}
 		(void)fflush(fp);
 
-		if (flag & PR_FAIL) {
+		if (flag & OPT_FLAGS_PR_FAIL) {
 			abort_fails++;
 			if (abort_fails >= ABORT_FAILURES) {
 				if (!abort_msg_emitted) {
 					abort_msg_emitted = true;
-					keep_stressing_set_flag(false);
+					stress_continue_set_flag(false);
 					(void)fprintf(fp, "info: %d failures "
 						"reached, aborting stress "
 						"process\n", ABORT_FAILURES);
@@ -434,12 +477,12 @@ static int pr_msg(
 #if defined(HAVE_SYSLOG_H)
 		/* Log messages if syslog requested, don't log DEBUG */
 		if ((g_opt_flags & OPT_FLAGS_SYSLOG) &&
-		    (!(flag & PR_DEBUG))) {
+		    (!(flag & OPT_FLAGS_PR_DEBUG))) {
 			syslog(LOG_INFO, "%s", buf);
 		}
 #endif
 	}
-	pr_unlock();
+	pr_unlock_pid(pid);
 	return ret;
 }
 
@@ -452,7 +495,7 @@ void pr_dbg(const char *fmt, ...)
 	va_list ap;
 
 	va_start(ap, fmt);
-	(void)pr_msg(pr_file(), PR_DEBUG, fmt, ap);
+	(void)pr_msg(pr_file(), OPT_FLAGS_PR_DEBUG, fmt, ap);
 	va_end(ap);
 }
 
@@ -466,7 +509,7 @@ void pr_dbg_skip(const char *fmt, ...)
 
 	va_start(ap, fmt);
 	if (!(g_opt_flags & OPT_FLAGS_SKIP_SILENT))
-		(void)pr_msg(pr_file(), PR_DEBUG, fmt, ap);
+		(void)pr_msg(pr_file(), OPT_FLAGS_PR_DEBUG, fmt, ap);
 	va_end(ap);
 }
 
@@ -479,7 +522,7 @@ void pr_inf(const char *fmt, ...)
 	va_list ap;
 
 	va_start(ap, fmt);
-	(void)pr_msg(pr_file(), PR_INFO, fmt, ap);
+	(void)pr_msg(pr_file(), OPT_FLAGS_PR_INFO, fmt, ap);
 	va_end(ap);
 }
 
@@ -493,7 +536,7 @@ void pr_inf_skip(const char *fmt, ...)
 
 	va_start(ap, fmt);
 	if (!(g_opt_flags & OPT_FLAGS_SKIP_SILENT))
-		(void)pr_msg(pr_file(), PR_INFO, fmt, ap);
+		(void)pr_msg(pr_file(), OPT_FLAGS_PR_INFO, fmt, ap);
 	va_end(ap);
 }
 
@@ -506,7 +549,7 @@ void pr_err(const char *fmt, ...)
 	va_list ap;
 
 	va_start(ap, fmt);
-	(void)pr_msg(pr_file(), PR_ERROR, fmt, ap);
+	(void)pr_msg(pr_file(), OPT_FLAGS_PR_ERROR, fmt, ap);
 	va_end(ap);
 }
 
@@ -520,7 +563,7 @@ void pr_err_skip(const char *fmt, ...)
 
 	va_start(ap, fmt);
 	if (!(g_opt_flags & OPT_FLAGS_SKIP_SILENT))
-		(void)pr_msg(pr_file(), PR_ERROR, fmt, ap);
+		(void)pr_msg(pr_file(), OPT_FLAGS_PR_ERROR, fmt, ap);
 	va_end(ap);
 }
 
@@ -533,7 +576,7 @@ void pr_fail(const char *fmt, ...)
 	va_list ap;
 
 	va_start(ap, fmt);
-	(void)pr_msg(pr_file(), PR_FAIL, fmt, ap);
+	(void)pr_msg(pr_file(), OPT_FLAGS_PR_FAIL, fmt, ap);
 	va_end(ap);
 }
 
@@ -546,7 +589,7 @@ void pr_tidy(const char *fmt, ...)
 	va_list ap;
 
 	va_start(ap, fmt);
-	(void)pr_msg(pr_file(), g_caught_signal ? PR_INFO : PR_DEBUG, fmt, ap);
+	(void)pr_msg(pr_file(), g_shared && g_shared->caught_sigint ? OPT_FLAGS_PR_INFO : OPT_FLAGS_PR_DEBUG, fmt, ap);
 	va_end(ap);
 }
 
@@ -559,7 +602,7 @@ void pr_warn(const char *fmt, ...)
 	va_list ap;
 
 	va_start(ap, fmt);
-	(void)pr_msg(pr_file(), PR_WARN, fmt, ap);
+	(void)pr_msg(pr_file(), OPT_FLAGS_PR_WARN, fmt, ap);
 	va_end(ap);
 }
 
@@ -573,7 +616,7 @@ void pr_warn_skip(const char *fmt, ...)
 
 	va_start(ap, fmt);
 	if (!(g_opt_flags & OPT_FLAGS_SKIP_SILENT))
-		(void)pr_msg(pr_file(), PR_WARN, fmt, ap);
+		(void)pr_msg(pr_file(), OPT_FLAGS_PR_WARN, fmt, ap);
 	va_end(ap);
 }
 
@@ -586,6 +629,6 @@ void pr_metrics(const char *fmt, ...)
 	va_list ap;
 
 	va_start(ap, fmt);
-	(void)pr_msg(pr_file(), PR_METRICS, fmt, ap);
+	(void)pr_msg(pr_file(), OPT_FLAGS_PR_METRICS, fmt, ap);
 	va_end(ap);
 }
