@@ -22,14 +22,20 @@
 #define STRESS_PRAGMA_(x) _Pragma (#x)
 #define STRESS_PRAGMA(x) STRESS_PRAGMA_(x)
 
-#if defined(__clang__) &&	\
-    NEED_CLANG(4, 0, 0) &&	\
+#if defined(HAVE_PRAGMA_NO_HARD_DFP) &&	\
+    defined(HAVE_COMPILER_GCC) &&	\
+    defined(HAVE_PRAGMA)
+#define STRESS_PRAGMA_NO_HARD_DFP	 _Pragma("GCC target (\"no-hard-dfp\")")
+#endif
+
+#if defined(HAVE_COMPILER_CLANG) &&	\
+    NEED_CLANG(4, 0, 0) &&		\
     defined(HAVE_PRAGMA)
 #define STRESS_PRAGMA_PUSH		_Pragma("GCC diagnostic push")
 #define STRESS_PRAGMA_POP		_Pragma("GCC diagnostic pop")
 #define STRESS_PRAGMA_WARN_OFF		_Pragma("GCC diagnostic ignored \"-Weverything\"")
-#elif defined(__GNUC__) &&	\
-      defined(HAVE_PRAGMA) &&	\
+#elif defined(HAVE_COMPILER_GCC) &&	\
+      defined(HAVE_PRAGMA) &&		\
       NEED_GNUC(4, 6, 0)
 #define STRESS_PRAGMA_PUSH		_Pragma("GCC diagnostic push")
 #define STRESS_PRAGMA_POP		_Pragma("GCC diagnostic pop")
@@ -44,28 +50,26 @@
 #define STRESS_PRAGMA_WARN_OFF
 #endif
 
-#if defined(__clang__) &&	\
-    NEED_CLANG(8, 0, 0) &&	\
+#if defined(HAVE_COMPILER_CLANG) &&	\
+    NEED_CLANG(8, 0, 0) &&		\
     defined(HAVE_PRAGMA)
 #define STRESS_PRAGMA_WARN_CPP_OFF	_Pragma("GCC diagnostic ignored \"-Wcpp\"")
-#elif defined(__GNUC__) &&	\
-      defined(HAVE_PRAGMA) &&	\
+#elif defined(HAVE_COMPILER_GCC) &&	\
+      defined(HAVE_PRAGMA) &&		\
       NEED_GNUC(10, 0, 0)
 #define STRESS_PRAGMA_WARN_CPP_OFF	_Pragma("GCC diagnostic ignored \"-Wcpp\"")
 #else
 #define STRESS_PRAGMA_WARN_CPP_OFF
 #endif
 
-#if defined(__ICC) &&			\
-    defined(__INTEL_COMPILER) &&	\
-    defined(__INTEL_COMPILER_UPDATE)
+#if defined(HAVE_COMPILER_ICC)
 #define PRAGMA_UNROLL_N(n)	STRESS_PRAGMA(unroll)
 #define PRAGMA_UNROLL		STRESS_PRAGMA(unroll)
-#elif defined(__clang__) &&       \
+#elif defined(HAVE_COMPILER_CLANG) &&	\
     NEED_CLANG(9, 0, 0)
 #define PRAGMA_UNROLL_N(n)	STRESS_PRAGMA(unroll n)
 #define PRAGMA_UNROLL		STRESS_PRAGMA(unroll)
-#elif defined(__GNUC__) &&      \
+#elif defined(HAVE_COMPILER_GCC) &&      \
     NEED_GNUC(10, 0, 0)
 #define PRAGMA_UNROLL_N(n)	STRESS_PRAGMA(GCC unroll n)
 #define PRAGMA_UNROLL		STRESS_PRAGMA(GCC unroll 8)
