@@ -18,6 +18,8 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
+#include "core-sched.h"
 
 #ifndef SCHED_FLAG_RESET_ON_FORK
 #define SCHED_FLAG_RESET_ON_FORK        (0x01)
@@ -83,46 +85,6 @@ static const char prefix[] = "sched";
 #if defined(SCHED_DEADLINE) &&	\
     defined(__linux__)
 #define HAVE_STRESS_SET_DEADLINE_SCHED	(1)
-
-int stress_set_deadline_sched(
-	const pid_t pid,
-	const uint64_t period,
-	const uint64_t runtime,
-	const uint64_t deadline,
-	const bool quiet)
-{
-	int rc;
-	struct shim_sched_attr attr;
-
-	(void)memset(&attr, 0, sizeof(attr));
-
-	attr.size = sizeof(attr);
-	attr.sched_policy = SCHED_DEADLINE;
-	/* make sched_deadline task be able to fork*/
-	attr.sched_flags = SCHED_FLAG_RESET_ON_FORK;
-	if (g_opt_flags & OPT_FLAGS_DEADLINE_GRUB)
-		attr.sched_flags |= SCHED_FLAG_RECLAIM;
-
-	attr.sched_nice = 0;
-	attr.sched_priority = 0;
-	if (!quiet)
-		pr_dbg("%s: setting scheduler class '%s' (period=%" PRIu64
-			", runtime=%" PRIu64 ", deadline=%" PRIu64 ")\n",
-			prefix, "deadline", period, runtime, deadline);
-	attr.sched_runtime = runtime;
-	attr.sched_deadline = deadline;
-	attr.sched_period = period;
-
-	rc = shim_sched_setattr(pid, &attr, 0);
-	if (rc < 0) {
-		rc = -errno;
-		if (!quiet)
-			pr_inf("%s: cannot set scheduler: errno=%d (%s)\n",
-				prefix, errno, strerror(errno));
-		return rc;
-	}
-	return 0;
-}
 #endif
 
 #define HAVE_STRESS_SET_SCHED	(1)
@@ -154,7 +116,7 @@ int stress_set_sched(
 	uint64_t sched_deadline = 0;
 #endif
 
-	(void)memset(&param, 0, sizeof(param));
+	(void)shim_memset(&param, 0, sizeof(param));
 
 	switch (sched) {
 	case UNDEFINED:	/* No preference, don't set */
@@ -197,7 +159,7 @@ int stress_set_sched(
 	case SCHED_DEADLINE:
 		min = sched_get_priority_min(sched);
 		max = sched_get_priority_max(sched);
-		(void)memset(&attr, 0, sizeof(attr));
+		(void)shim_memset(&attr, 0, sizeof(attr));
 		attr.size = sizeof(attr);
 		attr.sched_policy = SCHED_DEADLINE;
 		attr.sched_flags = SCHED_FLAG_RESET_ON_FORK;
