@@ -25,6 +25,7 @@
 #include "core-builtin.h"
 #include "core-cpu-cache.h"
 #include "core-pragma.h"
+#include "core-shim.h"
 
 #if defined(__NR_pkey_get)
 #define HAVE_PKEY_GET
@@ -91,9 +92,9 @@ UNEXPECTED
 #endif
 
 #if defined(HAVE_RUSAGE_WHO_T)
-#define shim_rusage_who_t	__rusage_who_t
+typedef __rusage_who_t	shim_rusage_who_t;
 #else
-#define shim_rusage_who_t	int
+typedef int		shim_rusage_who_t;
 #endif
 
 #if defined(HAVE_LINUX_MODULE_H)
@@ -251,6 +252,45 @@ static int shim_emulate_fallocate(int fd, off_t offset, off_t len)
 	return 0;
 }
 
+/*
+ *  shim_posix_fallocate()
+ *	emulation of posix_fallocate using chunks of allocations
+ *	and add EINTR support (which is not POSIX, but this is
+ *	an emulation wrapper so too bad).
+ */
+int shim_posix_fallocate(int fd, off_t offset, off_t len)
+{
+#if defined(HAVE_POSIX_FALLOCATE)
+	const off_t chunk_len = (off_t)(1 * MB);
+
+	do {
+		const off_t sz = (len > chunk_len) ? chunk_len : len;
+		int ret;
+
+		errno = 0;
+		ret = posix_fallocate(fd, offset, sz);
+		if (ret < 0)
+			return ret;
+
+		/*
+		 *  stressor has been signaled to finish, fake
+		 *  an EINTR return
+		 */
+		if (!stress_continue_flag()) {
+			errno = EINTR;
+			return -1;
+		}
+		offset += sz;
+		len -= sz;
+	} while (len > 0);
+
+	return 0;
+#else
+	return shim_emulate_fallocate(fd, offset, len);
+#endif
+}
+
+
 /*
  * shim_fallocate()
  *	shim wrapper for fallocate system call
@@ -329,7 +369,7 @@ int shim_fallocate(int fd, int mode, off_t offset, off_t len)
 	/*
 	 *  posix_fallocate returns 0 for success, > 0 as errno
 	 */
-	ret = posix_fallocate(fd, offset, len);
+	ret = shim_posix_fallocate(fd, offset, len);
 	errno = 0;
 	if (ret != 0) {
 		/* failed, so retry with slower emulated fallocate */
@@ -2585,3 +2625,35 @@ int shim_set_mempolicy_home_node(
 #endif
 }
 
+/*
+ *  shim_fchmodat
+ *	shim for Linux fchmodat
+ */
+int shim_fchmodat(int dfd, const char *filename, mode_t mode, unsigned int flags)
+{
+#if defined(HAVE_FCHMODAT)
+	return fchmodat(dfd, filename, mode, flags);
+#elif defined(__NR_fchmodat)
+	/* No flags field in system call */
+	(void)flags;
+	return (int)syscall(__NR_fchmodat, dfd, filename, mode);
+#else
+	return shim_enosys(0, dfd, filename, mode, flags);
+#endif
+}
+
+/*
+ *  shim_fchmodat2
+ *	shim for Linux 6.6 fchmodat2
+ */
+int shim_fchmodat2(int dfd, const char *filename, mode_t mode, unsigned int flags)
+{
+#if defined(HAVE_FCHMODAT2)
+	return fchmodat2(dfd, filename, mode, flags);
+#elif defined(__NR_fchmodat2)
+	return (int)syscall(__NR_fchmodat2, dfd, filename, mode, flags);
+#else
+	return shim_enosys(0, dfd, filename, mode, flags);
+#endif
+}
+
