@@ -70,6 +70,7 @@
 	MACRO(dnotify)		\
 	MACRO(dup)		\
 	MACRO(dynlib)		\
+	MACRO(eigen)		\
 	MACRO(efivar)		\
 	MACRO(enosys)		\
 	MACRO(env)		\
@@ -88,6 +89,7 @@
 	MACRO(filename)		\
 	MACRO(flock)		\
 	MACRO(flushcache)	\
+	MACRO(fma)		\
 	MACRO(fork)		\
 	MACRO(forkheavy)	\
 	MACRO(fp)		\
@@ -166,7 +168,9 @@
 	MACRO(mmapfork)		\
 	MACRO(mmaphuge)		\
 	MACRO(mmapmany)		\
+	MACRO(module)		\
 	MACRO(mprotect)		\
+	MACRO(mpfr)		\
 	MACRO(mq)		\
 	MACRO(mremap)		\
 	MACRO(msg)		\
@@ -230,6 +234,7 @@
 	MACRO(rotate)		\
 	MACRO(rseq)		\
 	MACRO(rtc)		\
+	MACRO(schedmix)		\
 	MACRO(schedpolicy)	\
 	MACRO(sctp)		\
 	MACRO(seal)		\
@@ -245,6 +250,7 @@
 	MACRO(shm)		\
 	MACRO(shm_sysv)		\
 	MACRO(sigabrt)		\
+	MACRO(sigbus)		\
 	MACRO(sigchld)		\
 	MACRO(sigfd)		\
 	MACRO(sigfpe)		\
@@ -317,6 +323,7 @@
 	MACRO(vm_rw)		\
 	MACRO(vm_segv)		\
 	MACRO(vm_splice)	\
+	MACRO(vnni)		\
 	MACRO(wait)		\
 	MACRO(waitcpu)		\
 	MACRO(watchdog)		\
