@@ -46,7 +46,7 @@ static void MLOCKED_TEXT stress_pagein_handler(int signum)
 		siglongjmp(jmp_env, 1);
 }
 
-static void stress_thrash_state(char *state)
+static void stress_thrash_state(const char *state)
 {
 	stress_set_proc_state_str("thrash", state);
 }
@@ -78,6 +78,7 @@ int stress_pagein_self(const char *name)
 	if (!fpmap)
 		return -errno;
 
+	stress_thrash_state("pagein");
 	/*
 	 * Look for field 0060b000-0060c000 r--p 0000b000 08:01 1901726
 	 */
@@ -209,7 +210,8 @@ static inline void stress_compact_memory(void)
 	if (!thrash_run)
 		return;
 
-	VOID_RET(ssize_t, system_write("/proc/sys/vm/compact_memory", "1", 1));
+	stress_thrash_state("compact");
+	VOID_RET(ssize_t, stress_system_write("/proc/sys/vm/compact_memory", "1", 1));
 #endif
 }
 
@@ -228,7 +230,23 @@ static inline void stress_zone_reclaim(void)
 	mode[0] = '0' + (stress_mwc8() & 7);
 	mode[1] = '\0';
 
-	VOID_RET(ssize_t, system_write("/proc/sys/vm/zone_reclaim_mode", mode, 1));
+	stress_thrash_state("reclaim");
+	VOID_RET(ssize_t, stress_system_write("/proc/sys/vm/zone_reclaim_mode", mode, 1));
+#endif
+}
+
+/*
+ *  stress_kmemleak_scan()
+ *	trigger kernel memory leak scan
+ */
+static inline void stress_kmemleak_scan(void)
+{
+#if defined(__linux__)
+	if (!thrash_run)
+		return;
+
+	stress_thrash_state("scan");
+	VOID_RET(ssize_t, stress_system_write("/sys/kernel/debug/kmemleak", "scan", 4));
 #endif
 }
 
@@ -245,12 +263,13 @@ static inline void stress_slab_shrink(void)
 	/*
 	 *  older shrink interface, may fail
 	 */
-	VOID_RET(ssize_t, system_write("/sys/kernel/slab/cache/shrink", "1", 1));
+	VOID_RET(ssize_t, stress_system_write("/sys/kernel/slab/cache/shrink", "1", 1));
 
 	dir = opendir(slabpath);
 	if (!dir)
 		return;
 
+	stress_thrash_state("shrink");
 	/*
 	 *  shrink all slabs
 	 */
@@ -259,7 +278,7 @@ static inline void stress_slab_shrink(void)
 			char path[PATH_MAX];
 
 			(void)snprintf(path, sizeof(path), "%s/%s", slabpath, d->d_name);
-			VOID_RET(ssize_t, system_write(path, "1", 1));
+			VOID_RET(ssize_t, stress_system_write(path, "1", 1));
 		}
 	}
 	(void)closedir(dir);
@@ -278,7 +297,8 @@ static inline void stress_drop_caches(void)
 	str[0] = '1' + (char)method;
 	str[1] = '\0';
 
-	VOID_RET(ssize_t, system_write("/proc/sys/vm/drop_caches", str, 1));
+	stress_thrash_state("dropcache");
+	VOID_RET(ssize_t, stress_system_write("/proc/sys/vm/drop_caches", str, 1));
 	if (method++ >= 2)
 		method = 0;
 #endif
@@ -294,7 +314,8 @@ static inline void stress_merge_memory(void)
 	if (!thrash_run)
 		return;
 
-	VOID_RET(ssize_t, system_write("/proc/sys/mm/ksm/run", KSM_RUN_MERGE, 1));
+	stress_thrash_state("merge");
+	VOID_RET(ssize_t, stress_system_write("/proc/sys/mm/ksm/run", KSM_RUN_MERGE, 1));
 #endif
 }
 
@@ -366,25 +387,20 @@ int stress_thrash_start(void)
 
 		while (thrash_run) {
 			if ((stress_mwc8() & 0x3) == 0) {
-				stress_thrash_state("shrink");
 				stress_slab_shrink();
-
-				stress_thrash_state("pagein");
 				stress_pagein_all_procs();
 			}
 			if ((stress_mwc8() & 0x7) == 0) {
-				stress_thrash_state("dropcache");
 				stress_drop_caches();
 			}
-			stress_thrash_state("compact");
 			stress_compact_memory();
 
-			stress_thrash_state("merge");
 			stress_merge_memory();
 
-			stress_thrash_state("reclaim");
 			stress_zone_reclaim();
 
+			stress_kmemleak_scan();
+
 			stress_thrash_state("sleep");
 			(void)sleep(1);
 		}
@@ -408,11 +424,11 @@ void stress_thrash_stop(void)
 	if (!thrash_pid)
 		return;
 
-	(void)kill(thrash_pid, SIGALRM);
+	(void)shim_kill(thrash_pid, SIGALRM);
 	(void)shim_waitpid(thrash_pid, &status, 0);
-	if (kill(thrash_pid, 0) == 0) {
+	if (shim_kill(thrash_pid, 0) == 0) {
 		(void)shim_usleep(250000);
-		(void)kill(thrash_pid, SIGKILL);
+		(void)shim_kill(thrash_pid, SIGKILL);
 		(void)shim_waitpid(thrash_pid, &status, 0);
 	}
 
