@@ -21,6 +21,7 @@
 #include "core-builtin.h"
 #include "core-pragma.h"
 #include "core-thermal-zone.h"
+#include "core-vmstat.h"
 
 #if defined(HAVE_MACH_MACH_H)
 #include <mach/mach.h>
@@ -1162,12 +1163,6 @@ void stress_vmstat_start(void)
 				}
 #endif
 
-#if defined(__linux__)
-				for (ptr = therms, tz_info = g_shared->tz_info; tz_info; tz_info = tz_info->next) {
-					(void)snprintf(ptr, 8, " %6.2f", stress_get_tz_info(tz_info));
-					ptr += 7;
-				}
-#endif
 				stress_get_cpu_ghz(&avg_ghz, &min_ghz, &max_ghz);
 				if (avg_ghz > 0.0)
 					(void)snprintf(cpuspeed, sizeof(cpuspeed), "%5.2f %5.2f %5.2f",
@@ -1178,7 +1173,14 @@ void stress_vmstat_start(void)
 
 				pr_lock();
 				if (thermalstat_count == 0)
-					pr_inf("therm: AvGHz MnGhz MxGHz  LdA1  LdA5 LdA15 %s\n", therms);
+					pr_inf("therm: AvGHz MnGHz MxGHz  LdA1  LdA5 LdA15 %s\n", therms);
+
+#if defined(__linux__)
+				for (ptr = therms, tz_info = g_shared->tz_info; tz_info; tz_info = tz_info->next) {
+					(void)snprintf(ptr, 8, " %6.2f", stress_get_tz_info(tz_info));
+					ptr += 7;
+				}
+#endif
 				if (stress_get_load_avg(&min1, &min5, &min15) < 0)  {
 					pr_inf("therm: %18s %5.5s %5.5s %5.5s %s\n",
 						cpuspeed, "n/a", "n/a", "n/a", therms);
@@ -1226,11 +1228,12 @@ void stress_vmstat_start(void)
 		if (status_sleep == status_delay) {
 			const double runtime = round(stress_time_now() - g_shared->time_started);
 
-			pr_inf("status: %" PRIu32 " run, %" PRIu32 " exit, %" PRIu32 " reap, %" PRIu32 " fail, %s\n",
-				g_shared->stressors_started,
-				g_shared->stressors_exited,
-				g_shared->stressors_reaped,
-				g_shared->stressors_failed,
+			pr_inf("status: %" PRIu32 " run, %" PRIu32 " exit, %" PRIu32 " reap, %" PRIu32 " fail, %" PRIu32 " sigalarm, %s\n",
+				g_shared->instance_count.started,
+				g_shared->instance_count.exited,
+				g_shared->instance_count.reaped,
+				g_shared->instance_count.failed,
+				g_shared->instance_count.alarmed,
 				stress_duration_to_str(runtime, false));
 		}
 	}
