@@ -205,7 +205,7 @@ static pid_t stress_access_spawn(
 			if (j >= SIZEOF_ARRAY(modes))
 				j = 0;
 			shim_sched_yield();
-		} while(keep_stressing(args));
+		} while(stress_continue(args));
 		_exit(0);
 	}
 	return pid;
@@ -217,8 +217,8 @@ static void stress_access_reap(pid_t *pid)
 
 	if (*pid == -1)
 		return;
-	(void)kill(*pid, SIGKILL);
-	(void)waitpid(*pid, &status, 0);
+	(void)shim_kill(*pid, SIGKILL);
+	(void)shim_waitpid(*pid, &status, 0);
 	*pid = -1;
 }
 
@@ -266,7 +266,7 @@ static int stress_access(const stress_args_t *args)
 			args->name, filename2, errno, strerror(errno));
 		goto tidy;
 	}
-	fs_type = stress_fs_type(filename1);
+	fs_type = stress_get_fs_type(filename1);
 
 	/* metrics in a shared page for child stats to be available to parent */
 	metrics = (stress_metrics_t *)mmap(NULL, metrics_size, PROT_READ | PROT_WRITE,
@@ -413,8 +413,8 @@ static int stress_access(const stress_args_t *args)
 #endif
 			}
 		}
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	metrics[2].duration = metrics[0].duration + metrics[1].duration;
 	metrics[2].count = metrics[0].count + metrics[1].count;
