@@ -44,13 +44,24 @@ static void stress_alarm_sigusr1_handler(int sig)
 	_exit(0);
 }
 
+static void stress_alarm_stress_bogo_inc(const stress_args_t *args)
+{
+	sigset_t set;
+
+	sigemptyset(&set);
+	sigaddset(&set, SIGUSR1);
+	if (sigprocmask(SIG_BLOCK, &set, NULL) == 0) {
+		stress_bogo_inc(args);
+		(void)sigprocmask(SIG_UNBLOCK, &set, NULL);
+	}
+}
+
 /*
  *  stress_alarm
  *	stress alarm()
  */
 static int stress_alarm(const stress_args_t *args)
 {
-	const double t_end = stress_time_now() + (double)g_opt_timeout;
 	pid_t pid;
 
 	if (stress_sighandler(args->name, SIGALRM, stress_sighandler_nop, NULL) < 0)
@@ -60,9 +71,9 @@ static int stress_alarm(const stress_args_t *args)
 again:
 	pid = fork();
 	if (pid < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			return EXIT_SUCCESS;
 		pr_fail("%s: fork failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
@@ -89,8 +100,8 @@ again:
 			secs_left = sleep((unsigned int)INT_MAX);
 			if (secs_left == 0)
 				err_mask |= STRESS_SLEEP_INTMAX;
-			inc_counter(args);
-			if (!keep_stressing(args))
+			stress_alarm_stress_bogo_inc(args);
+			if (!stress_continue(args))
 				break;
 
 			/* zeros second interrupted sleep */
@@ -102,8 +113,8 @@ again:
 			secs_left = sleep(0);
 			if (secs_left != 0)
 				err_mask |= STRESS_SLEEP_ZERO;
-			inc_counter(args);
-			if (!keep_stressing(args))
+			stress_alarm_stress_bogo_inc(args);
+			if (!stress_continue(args))
 				break;
 
 			/* random duration interrupted sleep */
@@ -115,25 +126,25 @@ again:
 			secs_left = sleep(secs_left);
 			if (secs_left > secs_sleep)
 				err_mask |= STRESS_SLEEP_RANDOM;
-			inc_counter(args);
-		} while (keep_stressing(args));
+			stress_alarm_stress_bogo_inc(args);
+		} while (stress_continue(args));
 		_exit(err_mask);
 	} else {
 		int status;
 		const bool verify = !!(g_opt_flags & OPT_FLAGS_VERIFY);
 
-		while (keep_stressing(args) &&
-		       (stress_time_now() < t_end)) {
+		while (stress_continue(args) &&
+		       (stress_time_now() < args->time_end)) {
 			const uint64_t delay_ns = 1000 + stress_mwc32modn(10000);
 
-			(void)kill(pid, SIGALRM);
+			(void)shim_kill(pid, SIGALRM);
 			shim_nanosleep_uint64(delay_ns);
 			shim_sched_yield();
-			(void)kill(pid, SIGALRM);
+			(void)shim_kill(pid, SIGALRM);
 			shim_sched_yield();
 		}
 
-		(void)kill(pid, SIGUSR1);
+		(void)shim_kill(pid, SIGUSR1);
 		(void)waitpid(pid, &status, 0);
 
 		if (verify && WIFEXITED(status)) {
