@@ -19,6 +19,7 @@
  */
 #include "stress-ng.h"
 #include "core-builtin.h"
+#include "core-out-of-memory.h"
 #include "core-put.h"
 #include "core-pragma.h"
 
@@ -115,7 +116,7 @@ static int stress_bad_altstack_child(const stress_args_t *args)
 #else
 	UNEXPECTED
 #endif
-	uint32_t rnd;
+	NOCLOBBER uint32_t rnd;
 	int i, ret;
 	stack_t ss, old_ss;
 	size_t sz;
@@ -357,7 +358,7 @@ again:
 			return EXIT_SUCCESS;
 		pid = fork();
 		if (pid < 0) {
-			if (stress_redo_fork(errno))
+			if (stress_redo_fork(args, errno))
 				goto again;
 			if (!stress_continue(args))
 				return EXIT_SUCCESS;
