@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
 #include "core-pthread.h"
 
 #if defined(HAVE_SYS_MOUNT_H)
@@ -43,7 +44,7 @@ static const stress_help_t help[] = {
 static void MLOCKED_TEXT stress_bind_mount_child_handler(int signum)
 {
 	if (signum == SIGALRM) {
-		keep_stressing_set_flag(false);
+		stress_continue_set_flag(false);
 		return;
 	}
 	_exit(0);
@@ -143,9 +144,9 @@ bind_umount:
 			}
 #endif
 		}
-		inc_counter(args);
-	} while (keep_stressing_flag() &&
-		 (!args->max_ops || (get_counter(args) < args->max_ops)));
+		stress_bogo_inc(args);
+	} while (stress_continue_flag() &&
+		 (!args->max_ops || (stress_bogo_get(args) < args->max_ops)));
 
 	rate = (mount_count > 0.0) ? (double)mount_duration / mount_count : 0.0;
 	stress_metrics_set(args, 0, "microsecs per mount", rate * STRESS_DBL_MICROSECOND);
@@ -183,21 +184,29 @@ static int stress_bind_mount(const stress_args_t *args)
 		static char stack[CLONE_STACK_SIZE];
 		char *stack_top = (char *)stress_get_stack_top((void *)stack, CLONE_STACK_SIZE);
 
-		(void)memset(stack, 0, sizeof stack);
+		(void)shim_memset(stack, 0, sizeof stack);
 
 		pid = (pid_t)clone(stress_bind_mount_child,
 			stress_align_stack(stack_top),
 			CLONE_NEWUSER | CLONE_NEWNS | CLONE_VM | SIGCHLD,
 			(void *)&pargs, 0);
 		if (pid < 0) {
-			int rc = stress_exit_status(errno);
-
+			switch (errno) {
+			case ENOMEM:
+			case ENOSPC:
+			case EPERM:
+				return EXIT_NO_RESOURCE;
+			case ENOSYS:
+				return EXIT_NOT_IMPLEMENTED;
+			default:
+				break;
+			}
 			pr_fail("%s: clone failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
-			return rc;
+			return EXIT_FAILURE;
 		}
 		VOID_RET(int, shim_waitpid(pid, &status, 0));
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
@@ -216,6 +225,7 @@ stressor_info_t stress_bind_mount_info = {
 stressor_info_t stress_bind_mount_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_FILESYSTEM | CLASS_OS | CLASS_PATHOLOGICAL,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without Linux bind-mount options MS_BIND"
 };
