@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
 
 #if defined(HAVE_SYS_CAPABILITY_H)
 #include <sys/capability.h>
@@ -42,8 +43,8 @@ static int stress_capgetset_pid(
 	struct __user_cap_header_struct uch;
 	struct __user_cap_data_struct ucd[_LINUX_CAPABILITY_U32S_3];
 
-	(void)memset(&uch, 0, sizeof uch);
-	(void)memset(ucd, 0, sizeof ucd);
+	(void)shim_memset(&uch, 0, sizeof uch);
+	(void)shim_memset(ucd, 0, sizeof ucd);
 
 #if defined(_LINUX_CAPABILITY_VERSION_3)
 	uch.version = _LINUX_CAPABILITY_VERSION_3;
@@ -133,7 +134,7 @@ static int stress_capgetset_pid(
 	uch.pid = pid;
 	ret = capget(&uch, ucd);
 
-	inc_counter(args);
+	stress_bogo_inc(args);
 
 	return ret;
 }
@@ -150,13 +151,13 @@ static int stress_cap(const stress_args_t *args)
 		DIR *dir;
 
 		stress_capgetset_pid(args, 1, false, true);
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			break;
 		stress_capgetset_pid(args, args->pid, true, true);
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			break;
 		stress_capgetset_pid(args, getppid(), false, false);
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			break;
 
 		dir = opendir("/proc");
@@ -171,12 +172,12 @@ static int stress_cap(const stress_args_t *args)
 				if (sscanf(d->d_name, "%" SCNdMAX, &p) != 1)
 					continue;
 				stress_capgetset_pid(args, (pid_t)p, false, false);
-				if (!keep_stressing(args))
+				if (!stress_continue(args))
 					break;
 			}
 			(void)closedir(dir);
 		}
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	return EXIT_SUCCESS;
@@ -193,6 +194,7 @@ stressor_info_t stress_cap_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_OS,
 	.help = help,
+	.verify = VERIFY_ALWAYS,
 	.unimplemented_reason = "built without sys/capability.h"
 };
 #endif
