@@ -210,7 +210,7 @@ static int stress_chown(const stress_args_t *args)
 			(void)shim_usleep(100000);
 #endif
 			/* Timed out, then give up */
-			if (!keep_stressing_flag()) {
+			if (!stress_continue_flag()) {
 				rc = EXIT_SUCCESS;
 				goto tidy;
 			}
@@ -219,7 +219,7 @@ static int stress_chown(const stress_args_t *args)
 					"retries to open and gave up "
 					"(instance %" PRIu32 ")%s\n",
 					args->name, filename, retries, args->instance,
-					stress_fs_type(filename));
+					stress_get_fs_type(filename));
 				rc = EXIT_NO_RESOURCE;
 				goto tidy;
 			}
@@ -235,23 +235,23 @@ static int stress_chown(const stress_args_t *args)
 		if ((ret < 0) && (ret != -EPERM))
 			pr_fail("%s: fchown failed, errno=%d (%s)%s\n",
 				args->name, errno, strerror(errno),
-				stress_fs_type(filename));
+				stress_get_fs_type(filename));
 
 		ret = do_chown(chown, filename, cap_chown, uid, gid);
 		if ((ret < 0) && (ret != -EPERM)) {
 			pr_fail("%s: chown %s failed, errno=%d (%s)%s\n",
 				args->name, filename, errno, strerror(errno),
-				stress_fs_type(filename));
+				stress_get_fs_type(filename));
 		}
 		ret = do_chown(lchown, filename, cap_chown, uid, gid);
 		if ((ret < 0) && (ret != -EPERM)) {
 			pr_fail("%s: lchown %s failed, errno=%d (%s)%s\n",
 				args->name, filename, errno, strerror(errno),
-				stress_fs_type(filename));
+				stress_get_fs_type(filename));
 		}
 		(void)shim_fsync(fd);
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	rc = EXIT_SUCCESS;
 tidy:
