@@ -146,7 +146,7 @@ static void *stress_close_func(void *arg)
 	(void)sigprocmask(SIG_BLOCK, &set, NULL);
 #endif
 
-	while (keep_stressing(args)) {
+	while (stress_continue(args)) {
 		const uint64_t delay =
 			max_delay_us ? stress_mwc64modn(max_delay_us) : 0;
 		int fds[FDS_TO_DUP], i, ret;
@@ -211,6 +211,7 @@ static int stress_close(const stress_args_t *args)
 	const uid_t uid = getuid();
 	const gid_t gid = getgid();
 	const bool not_root = !stress_check_capability(SHIM_CAP_IS_ROOT);
+	bool close_failure = false;
 	double max_duration = 0.0, duration = 0.0, count = 0.0, rate;
 #if defined(HAVE_FACCESSAT)
 	int file_fd = -1;
@@ -272,7 +273,7 @@ static int stress_close(const stress_args_t *args)
 		fd = -1;
 		t1 = stress_time_now();
 
-		switch (stress_mwc8modn(15)) {
+		switch (stress_mwc8() & 0x0f) {
 		case 0:
 			domain = stress_mwc8modn((uint8_t)SIZEOF_ARRAY(domains));
 			type = stress_mwc8modn((uint8_t)SIZEOF_ARRAY(types));
@@ -312,7 +313,7 @@ static int stress_close(const stress_args_t *args)
 			break;
 #endif
 		case 7:
-			if (pipe(pipefds) == 0) {
+			if (LIKELY(pipe(pipefds) == 0)) {
 				fd = pipefds[0];
 				(void)close(pipefds[1]);
 			}
@@ -365,13 +366,17 @@ static int stress_close(const stress_args_t *args)
 			fd = dup(0);
 #endif
 			break;
+		case 15:
+			fd = open("/dev/stdin", O_RDONLY);
+			break;
 		default:
+			fd = open(".", O_RDONLY | O_DIRECTORY);
 			break;
 		}
-		if (fd == -1)
+		if (UNLIKELY(fd == -1))
 			fd = open("/dev/null", O_RDWR);
 
-		if (fd != -1) {
+		if (LIKELY(fd != -1)) {
 			double t;
 
 			dupfd = dup(fd);
@@ -401,7 +406,7 @@ static int stress_close(const stress_args_t *args)
 			 * is a file descriptor referring to a file other than a directory
 			 */
 			ret = faccessat(file_fd, "./", F_OK, 0);
-			if (ret >= 0) {
+			if (UNLIKELY(ret >= 0)) {
 				pr_fail("%s: faccessat opened file descriptor succeeded unexpectedly, "
 					"errno=%d (%s)\n", args->name, errno, strerror(errno));
 				(void)close(ret);
@@ -415,11 +420,26 @@ static int stress_close(const stress_args_t *args)
 			VOID_RET(int, close(dup(STDOUT_FILENO)));
 
 			t = stress_time_now();
-			if (close(fd) == 0) {
+			if (LIKELY(close(fd) == 0)) {
 				duration += stress_time_now() - t;
 				count += 1.0;
-			}
 
+				/*
+				 *  A close on a successfully closed fd should fail
+				 */
+				if (close(fd) == 0) {
+					pr_fail("%s: unexpectedly able to close the same file %d twice\n",
+						args->name, fd);
+					close_failure = true;
+				} else {
+					if ((errno != EBADF) &&
+					    (errno != EINTR)) {
+						pr_fail("%s: expected error on close failure, error=%d (%s)\n",
+							args->name,  errno, strerror(errno));
+						close_failure = true;
+					}
+				}
+			}
 			if (dupfd != -1)
 				(void)close(dupfd);
 		}
@@ -434,10 +454,10 @@ static int stress_close(const stress_args_t *args)
 		max_duration *= 0.995;
 		if (max_duration < 1.0)
 			max_duration = 1.0;
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
-	rc = EXIT_SUCCESS;
+	rc = close_failure ? EXIT_FAILURE : EXIT_SUCCESS;
 tidy:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
@@ -467,13 +487,15 @@ tidy:
 
 stressor_info_t stress_close_info = {
 	.stressor = stress_close,
-	.class = CLASS_SCHEDULER | CLASS_OS,
+	.class = CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
 stressor_info_t stress_close_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_SCHEDULER | CLASS_OS,
+	.class = CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without pthread support"
 };
