@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
 
 #define MIN_COPY_FILE_BYTES	(128 * MB)
 #define MAX_COPY_FILE_BYTES	(MAX_FILE_LIMIT)
@@ -64,7 +65,7 @@ static int stress_copy_file_fill(
 	char buf[COPY_FILE_MAX_BUF_SIZE];
 	ssize_t sz = size;
 
-	(void)memset(buf, stress_mwc8(), sizeof(buf));
+	(void)shim_memset(buf, stress_mwc8(), sizeof(buf));
 
 	if (lseek(fd, off, SEEK_SET) < 0)
 		return -1;
@@ -76,7 +77,7 @@ static int stress_copy_file_fill(
 		if (n < 0)
 			return -1;
 		sz -= n;
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			return 0;
 	}
 	return 0;
@@ -136,7 +137,7 @@ static int stress_copy_file_range_verify(
 #endif
 
 		n = STRESS_MINIMUM(bytes_in, bytes_out);
-		if (memcmp(buf_in, buf_out, (size_t)n) != 0)
+		if (shim_memcmp(buf_in, buf_out, (size_t)n) != 0)
 			return -1;
 		bytes_left -= n;
 		*off_in += n;
@@ -158,6 +159,7 @@ static int stress_copy_file(const stress_args_t *args)
 	char filename[PATH_MAX - 5], tmp[PATH_MAX];
 	uint64_t copy_file_bytes = DEFAULT_COPY_FILE_BYTES;
 	double duration = 0.0, bytes = 0.0, rate;
+	const bool verify = !!(g_opt_flags & OPT_FLAGS_VERIFY);
 
 	if (!stress_get_setting("copy-file-bytes", &copy_file_bytes)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
@@ -190,7 +192,7 @@ static int stress_copy_file(const stress_args_t *args)
 		rc = stress_exit_status(errno);
 		pr_fail("%s: ftruncated failed, errno=%d (%s)%s\n",
 			args->name, errno, strerror(errno),
-			stress_fs_type(tmp));
+			stress_get_fs_type(tmp));
 		goto tidy_in;
 	}
 
@@ -215,10 +217,10 @@ static int stress_copy_file(const stress_args_t *args)
 		off_out_orig = (shim_loff_t)stress_mwc64modn(copy_file_bytes - DEFAULT_COPY_FILE_SIZE);
 		off_out = off_out_orig;
 
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			break;
 		stress_copy_file_fill(args, fd_in, (off_t)off_in, DEFAULT_COPY_FILE_SIZE);
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			break;
 		t = stress_time_now();
 		copy_ret = shim_copy_file_range(fd_in, &off_in, fd_out,
@@ -231,10 +233,17 @@ static int stress_copy_file(const stress_args_t *args)
 			    (errno == EINTR) ||
 			    (errno == ENOSPC))
 				continue;
+			if (errno == ENOSYS) {
+				pr_inf_skip("%s: copy_file_range failed, system "
+					"call not implemented, skipping stressor\n",
+					args->name);
+				rc = EXIT_NO_RESOURCE;
+				goto tidy_out;
+			}
 			if (errno == EINVAL) {
 				pr_inf_skip("%s: copy_file_range failed, the "
 					"kernel splice may be not implemented "
-					"for the file system, skipping stressor.\n",
+					"for the file system, skipping stressor\n",
 					args->name);
 				rc = EXIT_NO_RESOURCE;
 				goto tidy_out;
@@ -243,7 +252,7 @@ static int stress_copy_file(const stress_args_t *args)
 				args->name, errno, strerror(errno));
 			goto tidy_out;
 		}
-		if (g_opt_flags & OPT_FLAGS_VERIFY) {
+		if (verify) {
 			int verify_ret;
 			off_in = off_in_orig;
 			off_out = off_out_orig;
@@ -255,7 +264,7 @@ static int stress_copy_file(const stress_args_t *args)
 					args->name, (intmax_t)off_in_orig, (intmax_t)off_out_orig);
 			}
 		}
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			break;
 
 		/*
@@ -273,11 +282,11 @@ static int stress_copy_file(const stress_args_t *args)
 		 */
 		VOID_RET(ssize_t, shim_copy_file_range(fd_in, &off_in, fd_out,
 						&off_out, DEFAULT_COPY_FILE_SIZE, ~0U));
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			break;
 		(void)shim_fsync(fd_out);
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 	rc = EXIT_SUCCESS;
 
 	rate = (duration > 0.0) ? bytes / duration : 0.0;
@@ -308,6 +317,7 @@ stressor_info_t stress_copy_file_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_OPTIONAL,
 	.help = help,
 	.unimplemented_reason = "built without clone() system call"
 };
