@@ -19,6 +19,7 @@
  */
 #include "stress-ng.h"
 #include "core-arch.h"
+#include "core-bitops.h"
 #include "core-builtin.h"
 #include "core-cpu.h"
 #include "core-pragma.h"
@@ -59,11 +60,11 @@
 
 /* do nothing */
 #if defined(HAVE_ASM_NOP)
-#define FORCE_DO_NOTHING() __asm__ __volatile__("nop;")
+#define FORCE_DO_NOTHING() 	stress_asm_nop()
 #elif defined(HAVE_ASM_NOTHING)
-#define FORCE_DO_NOTHING() __asm__ __volatile__("")
+#define FORCE_DO_NOTHING() 	stress_asm_nothing()
 #else
-#define FORCE_DO_NOTHING() while (0)
+#define FORCE_DO_NOTHING() 	while (0)
 #endif
 
 #if defined(HAVE_INT128_T)
@@ -248,59 +249,13 @@ static void OPTIMIZE3 TARGET_CLONES stress_cpu_gcd(const char *name)
 static void OPTIMIZE3 TARGET_CLONES stress_cpu_bitops(const char *name)
 {
 	uint32_t i, i_sum = 0;
-	const uint32_t sum = 0x8aac0aab;
+	const uint32_t sum = 0x8aac4aab;
 
 	for (i = 0; i < 16384; i++) {
-		{
-			register uint32_t r;
-#if defined(HAVE_BUILTIN_BITREVERSE)
-			r = __builtin_bitreverse32(i);
-#else
-			register uint32_t v, s = (sizeof(v) * 8) - 1;
-
-			/* Reverse bits */
-			r = v = i;
-			for (v >>= 1; v; v >>= 1, s--) {
-				r <<= 1;
-				r |= v & 1;
-			}
-			r <<= s;
-#endif
-			i_sum += r;
-		}
-		{
-			/* parity check */
-			register uint32_t v = i;
-
-			v ^= v >> 16;
-			v ^= v >> 8;
-			v ^= v >> 4;
-			v &= 0xf;
-			i_sum += (0x6996 >> v) & 1;
-		}
-		{
-#if defined(HAVE_BUILTIN_POPCOUNT)
-			i_sum += __builtin_popcount((unsigned int)i);
-#else
-			/* Brian Kernighan count bits */
-			register uint32_t j, v = i;
-
-			for (j = 0; v; j++)
-				v &= v - 1;
-			i_sum += j;
-#endif
-		}
-		{
-			/* round up to nearest highest power of 2 */
-			register uint32_t v = i - 1;
-
-			v |= v >> 1;
-			v |= v >> 2;
-			v |= v >> 4;
-			v |= v >> 8;
-			v |= v >> 16;
-			i_sum += v;
-		}
+		i_sum += stress_bitreverse32(i);
+		i_sum += stress_parity32(i);
+		i_sum += stress_popcount32(i);
+		i_sum += stress_nextpwr2(i);
 	}
 	if ((g_opt_flags & OPT_FLAGS_VERIFY) && (i_sum != sum))
 		pr_fail("%s: bitops error detected, failed "
