@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
 
 #if defined(HAVE_CRYPT_H)
 #include <crypt.h>
@@ -59,7 +60,7 @@ static int stress_crypt_id(
 	const size_t prefix_len,
 	const char *method,
 	const char *passwd,
-	char *salt,
+	const char *salt,
 	stress_metrics_t *metrics)
 {
 	char *encrypted;
@@ -68,10 +69,10 @@ static int stress_crypt_id(
 #if defined (HAVE_CRYPT_R)
 	static struct crypt_data data;
 
-	(void)memset(&data, 0, sizeof(data));
+	(void)shim_memset(&data, 0, sizeof(data));
 #endif
 	(void)shim_strlcpy(newsalt, salt, sizeof(newsalt));
-	(void)memcpy(newsalt, prefix, prefix_len);
+	(void)shim_memcpy(newsalt, prefix, prefix_len);
 	errno = 0;
 
 #if defined (HAVE_CRYPT_R)
@@ -154,7 +155,7 @@ static int stress_crypt(const stress_args_t *args)
 			passwd[i] = seedchars[stress_mwc32modn((uint32_t)sizeof(seedchars))];
 		passwd[i] = '\0';
 
-		for (i = 0; keep_stressing(args) && (i < SIZEOF_ARRAY(crypt_methods)); i++) {
+		for (i = 0; stress_continue(args) && (i < SIZEOF_ARRAY(crypt_methods)); i++) {
 			int ret;
 
 			ret = stress_crypt_id(args,
@@ -165,11 +166,11 @@ static int stress_crypt(const stress_args_t *args)
 			if (ret < 0)
 				failed++;
 			else
-				inc_counter(args);
+				stress_bogo_inc(args);
 		}
 		if (failed)
 			break;
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
@@ -197,6 +198,7 @@ stressor_info_t stress_crypt_info = {
 stressor_info_t stress_crypt_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_CPU,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without gcrypt library"
 };
