@@ -17,10 +17,11 @@
  *
  */
 #include "stress-ng.h"
+#include "core-affinity.h"
 #include "core-cpu-cache.h"
 
 static const stress_help_t help[] = {
-	{ NULL,	"dekker N",		"start N workers that exercise ther Dekker algorithm" },
+	{ NULL,	"dekker N",		"start N workers that exercise the Dekker algorithm" },
 	{ NULL,	"dekker-ops N",		"stop after N dekker mutex bogo operations" },
 	{ NULL,	NULL,			NULL }
 };
@@ -107,7 +108,7 @@ static void stress_dekker_p1(const stress_args_t *args)
 	check0 = dekker->m.check;
 	dekker->m.check--;
 	check1 = dekker->m.check;
-	inc_counter(args);
+	stress_bogo_inc(args);
 
 	dekker->m.turn = 0;
 	dekker->m.wants_to_enter[1] = false;
@@ -130,6 +131,7 @@ static int stress_dekker(const stress_args_t *args)
 	const size_t sz = STRESS_MAXIMUM(args->page_size, sizeof(*dekker));
 	pid_t pid;
 	double rate, duration, count;
+	int parent_cpu;
 
 	dekker = (dekker_t *)mmap(NULL, sz, PROT_READ | PROT_WRITE,
 			MAP_ANONYMOUS | MAP_SHARED, -1, 0);
@@ -140,23 +142,26 @@ static int stress_dekker(const stress_args_t *args)
 	}
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
+	parent_cpu = stress_get_cpu();
 	pid = fork();
 	if (pid < 0) {
 		pr_inf_skip("%s: cannot create child process, skipping stressor\n", args->name);
 		return EXIT_NO_RESOURCE;
 	} else if (pid == 0) {
 		/* Child */
-		while (keep_stressing(args))
+		(void)stress_change_cpu(args, parent_cpu);
+
+		while (stress_continue(args))
 			stress_dekker_p0(args);
 		_exit(0);
 	} else {
 		int status;
 
 		/* Parent */
-		while (keep_stressing(args))
+		while (stress_continue(args))
 			stress_dekker_p1(args);
-		(void)kill(pid, SIGKILL);
-		(void)waitpid(pid, &status, 0);
+		(void)shim_kill(pid, SIGKILL);
+		(void)shim_waitpid(pid, &status, 0);
 	}
 
 	duration = dekker->p0.duration + dekker->p1.duration;
