@@ -20,6 +20,7 @@
 #include "stress-ng.h"
 #include "core-arch.h"
 #include "core-cpu.h"
+#include "core-out-of-memory.h"
 #include "core-pragma.h"
 
 static const stress_help_t help[] = {
@@ -39,9 +40,9 @@ static const stress_help_t help[] = {
 #define MAX_SYSCALL	(2048)		/* Guess */
 #endif
 
-#if defined(__linux__) &&       \
-    !defined(__TINYC__) &&	\
-    !defined(__PCC__) &&	\
+#if defined(__linux__) &&       	\
+    !defined(HAVE_COMPILER_TCC) &&	\
+    !defined(HAVE_COMPILER_PCC) &&	\
     defined(STRESS_ARCH_X86_64)
 #define STRESS_EXERCISE_X86_SYSCALL
 #endif
@@ -200,7 +201,7 @@ static void exercise_syscall(
 	NOCLOBBER bool enosys = false;
 	const pid_t pid = getpid();
 
-	if (!keep_stressing(args))
+	if (!stress_continue(args))
 		_exit(EXIT_SUCCESS);
 
 	itimer_set(args);
@@ -596,6 +597,9 @@ static const long skip_syscalls[] = {
 #if defined(SYS_fchmodat)
 	SYS_fchmodat,
 #endif
+#if defined(SYS_fchmodat2)
+	SYS_fchmodat2,
+#endif
 #if defined(SYS_fchown)
 	SYS_fchown,
 #endif
@@ -929,6 +933,9 @@ static const long skip_syscalls[] = {
 #if defined(SYS_madvise)
 	SYS_madvise,
 #endif
+#if defined(SYS_map_shadow_stack)
+	SYS_map_shadow_stack,
+#endif
 #if defined(SYS_mbind)
 	SYS_mbind,
 #endif
@@ -1975,6 +1982,9 @@ static const long skip_syscalls[] = {
 #if defined(__NR_fchmodat)
 	__NR_fchmodat,
 #endif
+#if defined(__NR_fchmodat2)
+	__NR_fchmodat2,
+#endif
 #if defined(__NR_fchown)
 	__NR_fchown,
 #endif
@@ -2377,6 +2387,9 @@ static const long skip_syscalls[] = {
 #if defined(__NR_madvise1)
 	__NR_madvise1,
 #endif
+#if defined(__NR_map_shadow_stack)
+	__NR_map_shadow_stack,
+#endif
 #if defined(__NR_mbind)
 	__NR_mbind,
 #endif
@@ -3515,7 +3528,7 @@ static inline int stress_do_syscall(
 	if (syscall_find(number)) {
 		return rc;
 	}
-	if (!keep_stressing(args))
+	if (!stress_continue(args))
 		return 0;
 	pid = fork();
 	if (pid < 0) {
@@ -3583,8 +3596,8 @@ static inline int stress_do_syscall(
 		 */
 		ret = waitpid(pid, &status, 0);
 		if (ret < 0) {
-			(void)kill(pid, SIGKILL);
-			(void)waitpid(pid, &status, 0);
+			(void)shim_kill(pid, SIGKILL);
+			(void)shim_waitpid(pid, &status, 0);
 		}
 		rc = WEXITSTATUS(status);
 
@@ -3593,7 +3606,7 @@ static inline int stress_do_syscall(
 			if ((!random) || (number < 65536))
 				syscall_add(number);
 		}
-		inc_counter(args);
+		stress_bogo_inc(args);
 	}
 	return rc;
 }
@@ -3611,13 +3624,13 @@ static int stress_enosys(const stress_args_t *args)
 #endif
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 again:
-	if (!keep_stressing(args))
+	if (!stress_continue(args))
 		return EXIT_SUCCESS;
 	pid = fork();
 	if (pid < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			goto deinit;
 		pr_err("%s: fork failed: errno=%d: (%s)\n",
 			args->name, errno, strerror(errno));
@@ -3630,9 +3643,14 @@ again:
 			if (errno != EINTR)
 				pr_dbg("%s: waitpid(): errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
-			(void)kill(pid, SIGALRM);
+			(void)shim_kill(pid, SIGALRM);
+
+			/* Still alive, kill it */
+			if (shim_kill(pid, 0) == 0) {
+				stress_force_killed_bogo(args);
+				(void)shim_kill(pid, SIGKILL);
+			}
 			(void)shim_waitpid(pid, &status, 0);
-			(void)kill(pid, SIGKILL);
 		} else if (WIFSIGNALED(status)) {
 			pr_dbg("%s: child died: %s (instance %d)\n",
 				args->name, stress_strsignal(WTERMSIG(status)),
@@ -3661,14 +3679,14 @@ again:
 		ssize_t j;
 
 		/* Child, wrapped to catch OOMs */
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			_exit(0);
 
 		stress_parent_died_alarm();
 		(void)sched_settings_apply(true);
 
 		/* Make sure this is killable by OOM killer */
-		stress_set_oom_adjustment(args->name, true);
+		stress_set_oom_adjustment(args, true);
 
 		//limit_procs(2);
 
@@ -3680,39 +3698,39 @@ again:
 
 			/* Low sequential syscalls */
 			for (number = 0; number < MAX_SYSCALL + 1024; number++) {
-				if (!keep_stressing(args))
+				if (!stress_continue(args))
 					goto finish;
 				stress_do_syscall(args, number, false);
 			}
 
 			/* Random syscalls */
 			for (j = 0; j < 1024; j++) {
-				if (!keep_stressing(args))
+				if (!stress_continue(args))
 					goto finish;
 				stress_do_syscall(args, stress_mwc8() & mask, true);
-				if (!keep_stressing(args))
+				if (!stress_continue(args))
 					goto finish;
 				stress_do_syscall(args, stress_mwc16() & mask, true);
-				if (!keep_stressing(args))
+				if (!stress_continue(args))
 					goto finish;
 				stress_do_syscall(args, stress_mwc32() & mask, true);
-				if (!keep_stressing(args))
+				if (!stress_continue(args))
 					goto finish;
 				stress_do_syscall(args, (long)(stress_mwc64() & mask), true);
 			}
 
 			/* Various bit masks */
 			for (number = 1; number; number <<= 1) {
-				if (!keep_stressing(args))
+				if (!stress_continue(args))
 					goto finish;
 				stress_do_syscall(args, number, false);
-				if (!keep_stressing(args))
+				if (!stress_continue(args))
 					goto finish;
 				stress_do_syscall(args, number | 1, false);
-				if (!keep_stressing(args))
+				if (!stress_continue(args))
 					goto finish;
 				stress_do_syscall(args, number | (number << 1U), false);
-				if (!keep_stressing(args))
+				if (!stress_continue(args))
 					goto finish;
 				stress_do_syscall(args, ~number, false);
 			}
@@ -3722,12 +3740,12 @@ again:
 				long n;
 
 				for (n = 0; n < 0x100; n++) {
-					if (!keep_stressing(args))
+					if (!stress_continue(args))
 						goto finish;
 					stress_do_syscall(args, n + number, false);
 				}
 			}
-		} while (keep_stressing(args));
+		} while (stress_continue(args));
 finish:
 		syscall_free();
 		_exit(EXIT_SUCCESS);
