@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-out-of-memory.h"
 
 static const stress_help_t help[] = {
 	{ NULL,	"env N",	"start N workers setting environment vars" },
@@ -75,7 +76,7 @@ static int stress_env_child(const stress_args_t *args, void *context)
 			MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
 		if (value == MAP_FAILED) {
 			pr_inf_skip("%s: could not allocate %zd bytes for environment variable value, "
-				"errno = %d (%s), skipping stressor\n",
+				"errno=%d (%s), skipping stressor\n",
 				args->name, arg_max,
 				errno, strerror(errno));
 			return EXIT_NO_RESOURCE;
@@ -108,7 +109,7 @@ static int stress_env_child(const stress_args_t *args, void *context)
 
 		/* Low memory avoidance, re-start */
 		if (stress_low_memory(arg_max * 2)) {
-			(void)kill(getpid(), SIGKILL);
+			(void)shim_kill(getpid(), SIGKILL);
 			_exit(EXIT_SUCCESS);
 		}
 
@@ -142,8 +143,8 @@ static int stress_env_child(const stress_args_t *args, void *context)
 					pr_fail("%s: unsentenv on variable %s failed, errno=%d (%s)\n",
 						args->name, name, errno, strerror(errno));
 				}
-				inc_counter(args);
-				if (!keep_stressing(args))
+				stress_bogo_inc(args);
+				if (!stress_continue(args))
 					goto reap;
 			}
 			i = 0;
@@ -151,9 +152,9 @@ static int stress_env_child(const stress_args_t *args, void *context)
 			stress_mwc_get_seed(&seed_w, &seed_z);
 		} else {
 			i++;
-			inc_counter(args);
+			stress_bogo_inc(args);
 		}
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 reap:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
