@@ -18,6 +18,8 @@
  *
  */
 #include "stress-ng.h"
+#include "core-affinity.h"
+#include "core-builtin.h"
 
 #if defined(HAVE_SYS_EVENTFD_H)
 #include <sys/eventfd.h>
@@ -52,7 +54,7 @@ static int stress_eventfd(const stress_args_t *args)
 {
 	pid_t pid;
 	int fd1, fd2, test_fd, rc;
-	int flags = 0;
+	int flags = 0, parent_cpu;
 	bool eventfd_nonblock = false;
 
 	(void)stress_get_setting("eventfd-nonblock", &eventfd_nonblock);
@@ -91,15 +93,16 @@ static int stress_eventfd(const stress_args_t *args)
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 again:
+	parent_cpu = stress_get_cpu();
 	pid = fork();
 	if (pid < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
 
 		(void)close(fd1);
 		(void)close(fd2);
 
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			return EXIT_SUCCESS;
 		pr_fail("%s: fork failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
@@ -107,10 +110,11 @@ again:
 	} else if (pid == 0) {
 		int n = 0;
 
+		(void)stress_change_cpu(args, parent_cpu);
 		stress_parent_died_alarm();
 		(void)sched_settings_apply(true);
 
-		while (keep_stressing_flag()) {
+		while (stress_continue_flag()) {
 			uint64_t val = 0;
 			ssize_t ret;
 			char re[7];
@@ -119,10 +123,10 @@ again:
 			VOID_RET(ssize_t, read(fd1, re, sizeof(re)));
 
 			for (;;) {
-				if (!keep_stressing_flag())
+				if (!stress_continue_flag())
 					goto exit_child;
 				ret = read(fd1, &val, sizeof(val));
-				if (ret < 0) {
+				if (UNLIKELY(ret < 0)) {
 					if ((errno == EAGAIN) ||
 					    (errno == EINTR))
 						continue;
@@ -130,7 +134,7 @@ again:
 						args->name, errno, strerror(errno));
 					goto exit_child;
 				}
-				if (ret < (ssize_t)sizeof(val)) {
+				if (UNLIKELY(ret < (ssize_t)sizeof(val))) {
 					pr_fail("%s child short read, got %zd, expecting %zd bytes\n",
 						args->name, ret, (ssize_t)sizeof(val));
 					goto exit_child;
@@ -141,11 +145,11 @@ again:
 			/*
 			 *  Periodically exercise with invalid writes
 			 */
-			if (n++ >= 64) {
+			if (UNLIKELY(n++ >= 64)) {
 				n = 0;
 
 				/* Exercise write using small buffer */
-				(void)memset(re, 0, sizeof(re));
+				(void)shim_memset(re, 0, sizeof(re));
 				VOID_RET(ssize_t, write(fd1, re, sizeof(re)));
 
 				/* Exercise write on buffer out of range */
@@ -155,10 +159,10 @@ again:
 
 			val = 1;
 			for (;;) {
-				if (!keep_stressing_flag())
+				if (!stress_continue_flag())
 					goto exit_child;
 				ret = write(fd2, &val, sizeof(val));
-				if (ret < 0) {
+				if (UNLIKELY(ret < 0)) {
 					if ((errno == EAGAIN) ||
 					    (errno == EINTR))
 						continue;
@@ -166,7 +170,7 @@ again:
 						args->name, errno, strerror(errno));
 					goto exit_child;
 				}
-				if (ret < (ssize_t)sizeof(val)) {
+				if (UNLIKELY(ret < (ssize_t)sizeof(val))) {
 					pr_fail("%s child short write, got %zd, expecting %zd bytes\n",
 						args->name, ret, (ssize_t)sizeof(val));
 					goto exit_child;
@@ -193,11 +197,11 @@ exit_child:
 			(void)stress_read_fdinfo(self, stress_mwc1() ? fd1 : fd2);
 
 			for (;;) {
-				if (!keep_stressing_flag())
+				if (!stress_continue_flag())
 					goto exit_parent;
 
 				ret = write(fd1, &val, sizeof(val));
-				if (ret < 0) {
+				if (UNLIKELY(ret < 0)) {
 					if ((errno == EAGAIN) ||
 					    (errno == EINTR))
 						continue;
@@ -205,7 +209,7 @@ exit_child:
 						args->name, errno, strerror(errno));
 					goto exit_parent;
 				}
-				if (ret < (ssize_t)sizeof(val)) {
+				if (UNLIKELY(ret < (ssize_t)sizeof(val))) {
 					pr_fail("%s parent short write, got %zd, expecting %zd bytes\n",
 						args->name, ret, (ssize_t)sizeof(val));
 					goto exit_parent;
@@ -214,11 +218,11 @@ exit_child:
 			}
 
 			for (;;) {
-				if (!keep_stressing_flag())
+				if (!stress_continue_flag())
 					goto exit_parent;
 
 				ret = read(fd2, &val, sizeof(val));
-				if (ret < 0) {
+				if (UNLIKELY(ret < 0)) {
 					if ((errno == EAGAIN) ||
 					    (errno == EINTR))
 						continue;
@@ -226,19 +230,19 @@ exit_child:
 						args->name, errno, strerror(errno));
 					goto exit_parent;
 				}
-				if (ret < (ssize_t)sizeof(val)) {
+				if (UNLIKELY(ret < (ssize_t)sizeof(val))) {
 					pr_fail("%s parent short read, got %zd, expecting %zd bytes\n",
 						args->name, ret, (ssize_t)sizeof(val));
 					goto exit_parent;
 				}
 				break;
 			}
-			inc_counter(args);
-		} while (keep_stressing(args));
+			stress_bogo_inc(args);
+		} while (stress_continue(args));
 exit_parent:
 		stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
-		(void)kill(pid, SIGKILL);
+		(void)shim_kill(pid, SIGKILL);
 		(void)shim_waitpid(pid, &status, 0);
 		(void)close(fd1);
 		(void)close(fd2);
@@ -258,6 +262,7 @@ stressor_info_t stress_eventfd_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_FILESYSTEM | CLASS_SCHEDULER | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without sys/eventfd.h or eventfd() support"
 };
