@@ -18,7 +18,9 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
 #include "core-capabilities.h"
+#include "core-mounts.h"
 
 #if defined(HAVE_SYS_FANOTIFY_H)
 #include <sys/fanotify.h>
@@ -435,7 +437,7 @@ static void stress_fanotify_read_events(
 	metadata = (struct fanotify_event_metadata *)buffer;
 
 	while (FAN_EVENT_OK(metadata, len)) {
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 		if ((metadata->fd != FAN_NOFD) && (metadata->fd >= 0)) {
 #if defined(FAN_OPEN)
@@ -462,7 +464,7 @@ static void stress_fanotify_read_events(
 			if (metadata->mask & FAN_RENAME)
 				account->rename++;
 #endif
-			inc_counter(args);
+			stress_bogo_inc(args);
 			(void)close(metadata->fd);
 		}
 		metadata = FAN_EVENT_NEXT(metadata, len);
@@ -476,12 +478,14 @@ static void stress_fanotify_read_events(
 static int stress_fanotify(const stress_args_t *args)
 {
 	char pathname[PATH_MAX - 16], filename[PATH_MAX], filename2[PATH_MAX];
-	const pid_t ppid = getpid();
 	pid_t pid;
 	int ret, rc = EXIT_SUCCESS;
 	stress_fanotify_account_t account;
 
-	(void)memset(&account, 0, sizeof(account));
+	if (stress_sigchld_set_handler(args) < 0)
+		return EXIT_NO_RESOURCE;
+
+	(void)shim_memset(&account, 0, sizeof(account));
 
 	stress_temp_dir_args(args, pathname, sizeof(pathname));
 	(void)stress_mk_filename(filename, sizeof(filename), pathname, "fanotify_file");
@@ -495,7 +499,7 @@ static int stress_fanotify(const stress_args_t *args)
 	pid = fork();
 
 	/* do all mount points */
-	(void)memset(mnts, 0, sizeof(mnts));
+	(void)shim_memset(mnts, 0, sizeof(mnts));
 
 	n_mnts = stress_mount_get(mnts, MAX_MNTS);
 	if (n_mnts < 1) {
@@ -523,7 +527,6 @@ static int stress_fanotify(const stress_args_t *args)
 			if (fd < 0) {
 				pr_fail("%s: creat %s failed, errno=%d (%s)\n",
 					args->name, filename, errno, strerror(errno));
-				(void)kill(ppid, SIGALRM);
 				_exit(EXIT_FAILURE);
 			}
 			(void)close(fd);
@@ -533,7 +536,6 @@ static int stress_fanotify(const stress_args_t *args)
 			if (fd < 0) {
 				pr_fail("%s: open %s O_WRONLY failed, errno=%d (%s)\n",
 					args->name, filename, errno, strerror(errno));
-				(void)kill(ppid, SIGALRM);
 				_exit(EXIT_FAILURE);
 			}
 			VOID_RET(ssize_t, write(fd, "test", 4));
@@ -544,7 +546,6 @@ static int stress_fanotify(const stress_args_t *args)
 			if (fd < 0) {
 				pr_fail("%s: open %s O_RDONLY failed, errno=%d (%s)\n",
 					args->name, filename, errno, strerror(errno));
-				(void)kill(ppid, SIGALRM);
 				_exit(EXIT_FAILURE);
 			}
 			VOID_RET(ssize_t, read(fd, buffer, sizeof(buffer)));
@@ -555,7 +556,7 @@ static int stress_fanotify(const stress_args_t *args)
 			else
 				(void)shim_unlink(filename2);
 
-		} while (keep_stressing(args));
+		} while (stress_continue(args));
 
 		_exit(EXIT_SUCCESS);
 	} else {
@@ -568,6 +569,7 @@ static int stress_fanotify(const stress_args_t *args)
     defined(FAN_REPORT_DFID_NAME)
 		int fan_fd2;
 #endif
+		double t, duration;
 
 		fanotify_event_init_invalid();
 
@@ -620,7 +622,6 @@ static int stress_fanotify(const stress_args_t *args)
 		max_fd = fan_fd1;
 #endif
 #endif
-
 		ret = test_fanotify_mark(mnts);
 		if (ret < 0) {
 			free(buffer);
@@ -641,6 +642,7 @@ static int stress_fanotify(const stress_args_t *args)
 			goto tidy;
 		}
 
+		t = stress_time_now();
 		do {
 			fd_set rfds;
 			size_t i;
@@ -692,7 +694,9 @@ static int stress_fanotify(const stress_args_t *args)
 			for (i = 0; i < SIZEOF_ARRAY(init_flags); i++) {
 				stress_fanotify_init_exercise(init_flags[i]);
 			}
-		} while (keep_stressing(args));
+		} while (stress_continue(args));
+
+		duration = stress_time_now() - t;
 
 		free(buffer);
 		fanotify_event_clear(fan_fd1);
@@ -704,20 +708,14 @@ static int stress_fanotify(const stress_args_t *args)
 			(void)close(fan_fd2);
 		}
 #endif
-		pr_inf("%s: "
-			"%" PRIu64 " open, "
-			"%" PRIu64 " close write, "
-			"%" PRIu64 " close nowrite, "
-			"%" PRIu64 " access, "
-			"%" PRIu64 " modify, "
-			"%" PRIu64 " rename\n",
-			args->name,
-			account.open,
-			account.close_write,
-			account.close_nowrite,
-			account.access,
-			account.modify,
-			account.rename);
+		if (duration > 0.0) {
+			stress_metrics_set(args, 0, "opens/sec", (double)account.open / duration);
+			stress_metrics_set(args, 1, "close writes/sec", (double)account.close_write / duration);
+			stress_metrics_set(args, 2, "close no-writes/sec", (double)account.close_nowrite / duration);
+			stress_metrics_set(args, 3, "accesses/sec", (double)account.access / duration);
+			stress_metrics_set(args, 4, "modifies/sec", (double)account.modify / duration);
+			/* stress_metrics_set(args, 5, "renames/sec", (double)account.rename / duration); */
+		}
 	}
 tidy:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
@@ -725,7 +723,7 @@ tidy:
 	if (pid > 0) {
 		int status;
 
-		(void)kill(pid, SIGKILL);
+		(void)shim_kill(pid, SIGKILL);
 		(void)shim_waitpid(pid, &status, 0);
 	}
 	(void)shim_unlink(filename);
@@ -748,6 +746,7 @@ stressor_info_t stress_fanotify_info = {
 stressor_info_t stress_fanotify_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_FILESYSTEM | CLASS_SCHEDULER | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without sys/fanotify.h"
 };
