@@ -179,12 +179,13 @@ static int do_fcntl(
 
 			/* Exercise all permutations of SETFL flags */
 			if ((setfl_flag_count > 0) && (setfl_flag_perms)) {
-				static int index;
+				static size_t index;
 
 				VOID_RET(int, fcntl(fd, F_SETFL, setfl_flag_perms[index]));
 
 				index++;
-				index %= setfl_flag_count;
+				if (index >= setfl_flag_count)
+					index = 0;
 			}
 
 			new_flags = old_flags | O_APPEND;
@@ -781,7 +782,7 @@ static int stress_fcntl(const stress_args_t *args)
 		} else {
 			break;
 		}
-	} while (keep_stressing_flag() && ++retries < 100);
+	} while (stress_continue_flag() && ++retries < 100);
 
 	if ((fd < 0) || (retries >= 100)) {
 		pr_err("%s: creat: file %s took %d "
@@ -792,8 +793,8 @@ static int stress_fcntl(const stress_args_t *args)
 
 	do {
 		do_fcntl(args, fd, bad_fd, path_fd);
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	rc = EXIT_SUCCESS;
 tidy:
