@@ -18,6 +18,8 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
+#include "core-killpid.h"
 
 #if defined(HAVE_LINUX_FIEMAP_H)
 #include <linux/fiemap.h>
@@ -89,7 +91,7 @@ static int stress_fiemap_writer(
 		offset = stress_mwc64modn(len) & ~0x1fffUL;
 		if (lseek(fd, (off_t)offset, SEEK_SET) < 0)
 			break;
-		if (!inc_counter_lock(args, counter_lock, false))
+		if (!stress_bogo_inc_lock(args, counter_lock, false))
 			break;
 		if (write(fd, buf, sizeof(buf)) < 0) {
 			if (errno == ENOSPC)
@@ -100,7 +102,7 @@ static int stress_fiemap_writer(
 				goto tidy;
 			}
 		}
-		if (!inc_counter_lock(args, counter_lock, false))
+		if (!stress_bogo_inc_lock(args, counter_lock, false))
 			break;
 #if defined(FALLOC_FL_PUNCH_HOLE) && \
     defined(FALLOC_FL_KEEP_SIZE)
@@ -117,12 +119,12 @@ static int stress_fiemap_writer(
 				punch_hole = false;
 		}
 		(void)shim_usleep(1000);
-		if (!inc_counter_lock(args, counter_lock, false))
+		if (!stress_bogo_inc_lock(args, counter_lock, false))
 			break;
 #else
 		UNEXPECTED
 #endif
-	} while (inc_counter_lock(args, counter_lock, false));
+	} while (stress_bogo_inc_lock(args, counter_lock, false));
 	rc = EXIT_SUCCESS;
 tidy:
 	(void)close(fd);
@@ -168,7 +170,7 @@ static void stress_fiemap_ioctl(
 			break;
 		}
 		shim_sched_yield();
-		if (!keep_stressing(args)) {
+		if (!stress_continue(args)) {
 			free(fiemap);
 			break;
 		}
@@ -188,7 +190,7 @@ static void stress_fiemap_ioctl(
 		}
 		fiemap = tmp;
 
-		(void)memset(fiemap->fm_extents, 0, extents_size);
+		(void)shim_memset(fiemap->fm_extents, 0, extents_size);
 		fiemap->fm_extent_count = fiemap->fm_mapped_extents;
 		fiemap->fm_mapped_extents = 0;
 
@@ -200,7 +202,7 @@ static void stress_fiemap_ioctl(
 		}
 		free(fiemap);
 		shim_sched_yield();
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			break;
 #if !defined(O_SYNC)
 		if (c++ > COUNT_MAX) {
@@ -208,7 +210,7 @@ static void stress_fiemap_ioctl(
 			fdatasync(fd);
 		}
 #endif
-	} while (inc_counter_lock(args, counter_lock, true));
+	} while (stress_bogo_inc_lock(args, counter_lock, true));
 }
 
 /*
@@ -239,9 +241,9 @@ static inline pid_t stress_fiemap_spawn(const stress_args_t *args, const int fd)
 static int stress_fiemap(const stress_args_t *args)
 {
 	pid_t pids[MAX_FIEMAP_PROCS];
-	int ret, fd, rc = EXIT_FAILURE, status;
+	int ret, fd, rc = EXIT_FAILURE;
 	char filename[PATH_MAX];
-	size_t i, n;
+	size_t n;
 	uint64_t fiemap_bytes = DEFAULT_FIEMAP_SIZE;
 	struct fiemap fiemap;
 	const char *fs_type;
@@ -281,10 +283,10 @@ static int stress_fiemap(const stress_args_t *args)
 			args->name, filename, errno, strerror(errno));
 		goto dir_clean;
 	}
-	fs_type = stress_fs_type(filename);
+	fs_type = stress_get_fs_type(filename);
 	(void)shim_unlink(filename);
 
-	(void)memset(&fiemap, 0, sizeof(fiemap));
+	(void)shim_memset(&fiemap, 0, sizeof(fiemap));
 	fiemap.fm_length = ~0UL;
 	if (ioctl(fd, FS_IOC_FIEMAP, &fiemap) < 0) {
 		errno = EOPNOTSUPP;
@@ -300,7 +302,7 @@ static int stress_fiemap(const stress_args_t *args)
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	for (n = 0; n < MAX_FIEMAP_PROCS; n++) {
-		if (!keep_stressing(args)) {
+		if (!stress_continue(args)) {
 			rc = EXIT_SUCCESS;
 			goto reap;
 		}
@@ -312,13 +314,9 @@ static int stress_fiemap(const stress_args_t *args)
 	rc = stress_fiemap_writer(args, fd, fiemap_bytes);
 reap:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
+
 	/* And reap stressors */
-	for (i = 0; i < n; i++) {
-		(void)kill(pids[i], SIGKILL);
-	}
-	for (i = 0; i < n; i++) {
-		(void)shim_waitpid(pids[i], &status, 0);
-	}
+	stress_kill_and_wait_many(args, pids, n, SIGALRM, true);
 close_clean:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	(void)close(fd);
@@ -345,6 +343,7 @@ stressor_info_t stress_fiemap_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without linux/fiemap.h, linux/fs.h or ioctl() FS_IOC_FIEMAP support"
 };
