@@ -18,6 +18,8 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
+#include "core-killpid.h"
 
 #if defined(HAVE_SYS_SELECT_H)
 #include <sys/select.h>
@@ -123,7 +125,7 @@ static void stress_fifo_reader(
 			name, errno, strerror(errno));
 		return;
 	}
-	while (keep_stressing_flag()) {
+	while (stress_continue_flag()) {
 		ssize_t sz;
 #if defined(HAVE_SELECT)
 		int ret;
@@ -149,7 +151,7 @@ redo:
 			 * as this can happen on a highly
 			 * overloaded stressed system
 			 */
-			if (keep_stressing(args))
+			if (stress_continue(args))
 				goto redo;
 			break;
 		}
@@ -244,14 +246,14 @@ static int stress_fifo(const stress_args_t *args)
 		goto tidy;
 	}
 
-	(void)memset(pids, 0, sizeof(pids));
+	(void)shim_memset(pids, 0, sizeof(pids));
 	for (i = 0; i < fifo_readers; i++) {
 		pids[i] = fifo_spawn(args, stress_fifo_reader, args->name, fifoname, fifo_data_size);
 		if (pids[i] < 0) {
 			rc = EXIT_NO_RESOURCE;
 			goto reap;
 		}
-		if (!keep_stressing_flag()) {
+		if (!stress_continue_flag()) {
 			rc = EXIT_SUCCESS;
 			goto reap;
 		}
@@ -270,7 +272,7 @@ static int stress_fifo(const stress_args_t *args)
 		goto reap;
 	}
 
-	(void)memset(buf, 0xaa, sizeof(buf));
+	(void)shim_memset(buf, 0xaa, sizeof(buf));
 	buf[0] = 0ULL;
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
@@ -295,8 +297,8 @@ static int stress_fifo(const stress_args_t *args)
 		}
 		buf[0]++;
 		buf[0] &= ~wrap_mask;
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 	fifo_duration = stress_time_now() - t;
 
 	rate = (fifo_duration > 0.0) ? (fifo_count / fifo_duration) : 0.0;
@@ -307,14 +309,7 @@ static int stress_fifo(const stress_args_t *args)
 	(void)close(fd);
 reap:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
-	for (i = 0; i < fifo_readers; i++) {
-		if (pids[i] > 0) {
-			int status;
-
-			(void)stress_killpid(pids[i]);
-			(void)shim_waitpid(pids[i], &status, 0);
-		}
-	}
+	stress_kill_and_wait_many(args, pids, fifo_readers, SIGALRM, false);
 tidy:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	(void)shim_unlink(fifoname);
@@ -335,6 +330,7 @@ stressor_info_t stress_fifo_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_PIPE_IO | CLASS_OS | CLASS_SCHEDULER,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without sys/select.h"
 };
