@@ -18,6 +18,8 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
+#include "core-out-of-memory.h"
 
 #if defined(HAVE_SYS_STATVFS_H)
 #include <sys/statvfs.h>
@@ -103,7 +105,7 @@ static int stress_filename_probe_length(
 				break;
 			pr_err("%s: creat() failed when probing "
 				"for filename length, "
-				"errno = %d (%s)\n",
+				"errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			*sz_max = 0;
 			return -1;
@@ -164,7 +166,7 @@ static int stress_filename_probe(
 			    (errno != EILSEQ)) {
 				pr_err("%s: creat() failed when probing "
 					"for allowed filename characters, "
-					"errno = %d (%s)\n",
+					"errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 				pr_inf("%s: perhaps retry and use "
 					"--filename-opts posix\n", args->name);
@@ -348,11 +350,11 @@ static int stress_filename(const stress_args_t *args)
 	}
 
 #if defined(HAVE_SYS_STATVFS_H)
-	(void)memset(&buf, 0, sizeof(buf));
+	(void)shim_memset(&buf, 0, sizeof(buf));
 	if (statvfs(pathname, &buf) < 0) {
 		pr_fail("%s: statvfs %s failed, errno=%d (%s)%s\n",
 			args->name, pathname, errno, strerror(errno),
-			stress_fs_type(pathname));
+			stress_get_fs_type(pathname));
 		goto tidy_dir;
 	}
 #else
@@ -385,7 +387,7 @@ static int stress_filename(const stress_args_t *args)
 
 	if (stress_filename_probe_length(args, filename, ptr, &sz_max) < 0) {
 		pr_fail("%s: failed to determine maximum filename length%s\n",
-			args->name, stress_fs_type(pathname));
+			args->name, stress_get_fs_type(pathname));
 		goto tidy_dir;
 	}
 
@@ -420,16 +422,16 @@ static int stress_filename(const stress_args_t *args)
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 again:
-	if (!keep_stressing_flag()) {
+	if (!stress_continue_flag()) {
 		/* Time to die */
 		rc = EXIT_SUCCESS;
 		goto tidy_dir;
 	}
 	pid = fork();
 	if (pid < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
-		if (!keep_stressing(args)) {
+		if (!stress_continue(args)) {
 			rc = EXIT_SUCCESS;
 			goto tidy_dir;
 		}
@@ -444,8 +446,9 @@ again:
 			if (errno != EINTR)
 				pr_dbg("%s: waitpid(): errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
-			(void)kill(pid, SIGTERM);
-			(void)kill(pid, SIGKILL);
+			stress_force_killed_bogo(args);
+			(void)shim_kill(pid, SIGTERM);
+			(void)shim_kill(pid, SIGKILL);
 			(void)shim_waitpid(pid, &status, 0);
 		} else if (WIFSIGNALED(status)) {
 			pr_dbg("%s: child died: %s (instance %d)\n",
@@ -478,7 +481,7 @@ again:
 		(void)sched_settings_apply(true);
 
 		/* Make sure this is killable by OOM killer */
-		stress_set_oom_adjustment(args->name, true);
+		stress_set_oom_adjustment(args, true);
 
 		i = 0;
 		sz = 1;
@@ -493,68 +496,68 @@ again:
 			/* Should succeed */
 			stress_filename_generate(ptr, 1, ch);
 			stress_filename_test(args, filename, 1, true, mypid);
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				break;
 			stress_filename_generate_random(ptr, 1, chars_allowed);
 			stress_filename_test(args, filename, 1, true, mypid);
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				break;
 
 			/* Should succeed */
 			stress_filename_generate(ptr, sz_max, ch);
 			stress_filename_test(args, filename, sz_max, true, mypid);
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				break;
 			stress_filename_generate_random(ptr, sz_max, chars_allowed);
 			stress_filename_test(args, filename, sz_max, true, mypid);
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				break;
 
 			/* Should succeed */
 			stress_filename_generate(ptr, sz_max - 1, ch);
 			stress_filename_test(args, filename, sz_max - 1, true, mypid);
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				break;
 			stress_filename_generate_random(ptr, sz_max - 1, chars_allowed);
 			stress_filename_test(args, filename, sz_max - 1, true, mypid);
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				break;
 
 			/* Should fail */
 			stress_filename_generate(ptr, sz_max + 1, ch);
 			stress_filename_test(args, filename, sz_max + 1, false, mypid);
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				break;
 			stress_filename_generate_random(ptr, sz_max + 1, chars_allowed);
 			stress_filename_test(args, filename, sz_max + 1, false, mypid);
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				break;
 
 			/* Should succeed */
 			stress_filename_generate(ptr, sz, ch);
 			stress_filename_test(args, filename, sz, true, mypid);
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				break;
 			stress_filename_generate_random(ptr, sz, chars_allowed);
 			stress_filename_test(args, filename, sz, true, mypid);
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				break;
 
 			/* Should succeed */
 			stress_filename_generate(ptr, rnd_sz, ch);
 			stress_filename_test(args, filename, rnd_sz, true, mypid);
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				break;
 			stress_filename_generate_random(ptr, rnd_sz, chars_allowed);
 			stress_filename_test(args, filename, rnd_sz, true, mypid);
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				break;
 
 			sz++;
 			if (sz > sz_max)
 				sz = 1;
-			inc_counter(args);
-		} while (keep_stressing(args));
+			stress_bogo_inc(args);
+		} while (stress_continue(args));
 		_exit(EXIT_SUCCESS);
 	}
 	rc = EXIT_SUCCESS;
