@@ -17,21 +17,62 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  */
+
 #include "stress-ng.h"
+#include "core-arch.h"
+#include "core-builtin.h"
+#include "core-pragma.h"
+
+#if defined(STRESS_ARCH_S390)
+#undef ALWAYS_INLINE
+#define ALWAYS_INLINE
+#endif
+
 #include "core-put.h"
 
+#if defined(STRESS_ARCH_SH4)
+#undef HAVE_COMPLEX_H
+#undef HAVE_COMPLEX
+#endif
+
 #if defined(HAVE_COMPLEX_H)
 #include <complex.h>
 #endif
 
-typedef void (*stress_funccall_func)(const stress_args_t *argse);
+#if defined(STRESS_ARCH_S390)
+/*
+ *  Use decimal floating point for s390
+ *  as some cpus don't support hard decimal
+ *  floating point
+ */
+#if defined(STRESS_PRAGMA_NO_HARD_DFP)
+STRESS_PRAGMA_NO_HARD_DFP
+#else
+/*
+ *  Otheriwse for s390 assume no decimal
+ *  floating point is supported as the
+ *  least risky default
+ */
+#undef HAVE_FLOAT_DECIMAL32
+#undef HAVE_FLOAT_DECIMAL64
+#undef HAVE_FLOAT_DECIMAL128
+#endif
+#endif
+
+#if defined(STRESS_ARCH_SH4)
+#undef HAVE_FLOAT_DECIMAL32
+#undef HAVE_FLOAT_DECIMAL64
+#undef HAVE_FLOAT_DECIMAL128
+#endif
+
+typedef bool (*stress_funccall_func)(const stress_args_t *args);
 
 typedef struct {
 	const char              *name;  /* human readable form of stressor */
 	const stress_funccall_func   func;   /* the funccall method function */
 } stress_funccall_method_info_t;
 
-static const stress_funccall_method_info_t funccall_methods[];
+static const stress_funccall_method_info_t stress_funccall_methods[];
 
 static const stress_help_t help[] = {
 	{ NULL,	"funccall N",		"start N workers exercising 1 to 9 arg functions" },
@@ -69,7 +110,7 @@ static inline void stress_complex_float_t_put(stress_complex_float_t a)
 #endif
 }
 
-static inline void stress_complex_double_t_put(stress_complex_double_t a)
+static inline ALWAYS_INLINE void stress_complex_double_t_put(stress_complex_double_t a)
 {
 #if defined(HAVE_CREAL) &&	\
     defined(HAVE_CIMAG)
@@ -79,7 +120,7 @@ static inline void stress_complex_double_t_put(stress_complex_double_t a)
 #endif
 }
 
-static inline void stress_complex_long_double_t_put(stress_complex_long_double_t a)
+static inline ALWAYS_INLINE void stress_complex_long_double_t_put(stress_complex_long_double_t a)
 {
 #if defined(HAVE_CREALL) &&	\
     defined(HAVE_CIMAGL)
@@ -91,7 +132,7 @@ static inline void stress_complex_long_double_t_put(stress_complex_long_double_t
 
 #endif
 
-static inline float stress_mwcfloat(void)
+static inline ALWAYS_INLINE float stress_mwcfloat(void)
 {
 	const uint32_t r = stress_mwc32();
 
@@ -105,64 +146,71 @@ static inline double stress_mwcdouble(void)
 	return (double)r / (double)(0xffffffffffffffffULL);
 }
 
-#define stress_funccall_type(type, rndfunc)				\
-static void NOINLINE stress_funccall_ ## type(const stress_args_t *args);	\
+/*
+ *  comparison functions, simple integer types use direct comparison,
+ *  floating pointing use precision as equal values should never been
+ *  compared for floating point
+ */
+#define cmp_type(a, b, type)	(a != b)
+#define cmp_fp(a, b, type)	(fabs((double)(a - b)) > (type)0.0001)
+#define cmp_cmplx(a, b, type)	(cabs((double complex)(a - b)) > (double)0.0001)
+#define cmp_ignore(a, b, type)	(0)
+
+#define stress_funccall_type(type, rndfunc, cmpfunc)			\
+static bool NOINLINE 							\
+stress_funccall_ ## type(const stress_args_t *args);			\
 									\
-static void NOINLINE stress_funccall_ ## type(const stress_args_t *args)	\
+static bool NOINLINE							\
+stress_funccall_ ## type(const stress_args_t *args)			\
 {									\
 	register int ii;						\
-	type a, b, c, d, e, f, g, h, i;					\
+	type res_old;							\
 									\
-	a = rndfunc();							\
-	b = rndfunc();							\
-	c = rndfunc();							\
-	d = rndfunc();							\
-	e = rndfunc();							\
-	f = rndfunc();							\
-	g = rndfunc();							\
-	h = rndfunc();							\
-	i = rndfunc();							\
+	type a = rndfunc();						\
+	type b = rndfunc();						\
+	type c = rndfunc();						\
+	type d = rndfunc();						\
+	type e = rndfunc();						\
+	type f = rndfunc();						\
+	type g = rndfunc();						\
+	type h = rndfunc();						\
+	type i = rndfunc();						\
 									\
-	do {								\
-		for (ii = 0; ii < 1000; ii++) {				\
-			type res = 					\
-			(stress_funccall_ ## type ## _1(a) + 		\
-			 stress_funccall_ ## type ## _2(a, b) +		\
-			 stress_funccall_ ## type ## _3(a, b,		\
-				c) +					\
-			 stress_funccall_ ## type ## _4(a, b, 		\
-				c, d) + 				\
-			 stress_funccall_ ## type ## _5(a, b,		\
-				c, d, e) +				\
-			 stress_funccall_ ## type ## _6(a, b,		\
-				c, d, e, f) + 				\
-			 stress_funccall_ ## type ## _7(a, b,		\
-				c, d, e, f, g) + 			\
-			 stress_funccall_ ## type ## _8(a, b,		\
-				c, d, e, f, g, h) + 			\
-			 stress_funccall_ ## type ## _9(a, b,		\
-				c, d, e, f, g, h, i));			\
+	(void)shim_memset(&res_old, 0, sizeof(res_old));		\
 									\
-			res += 						\
-			(stress_funcdeep_ ## type ## _2(a, b) +		\
-			 stress_funcdeep_ ## type ## _3(a, b,		\
-				c) + 					\
-			 stress_funcdeep_ ## type ## _4(a, b, 		\
-				c, d) +					\
-			 stress_funcdeep_ ## type ## _5(a, b,		\
-				c, d, e) +				\
-			 stress_funcdeep_ ## type ## _6(a, b,		\
-				c, d, e, f) +				\
-			 stress_funcdeep_ ## type ## _7(a, b,		\
-				c, d, e, f, g) +			\
-			 stress_funcdeep_ ## type ## _8(a, b,		\
-				c, d, e, f, g, h) +			\
-			 stress_funcdeep_ ## type ## _9(a, b,		\
-				c, d, e, f, g, h, i));			\
-			type ## _put(res);				\
+	for (ii = 0; ii < 1000; ii++) {					\
+		type res_new = 						\
+		 stress_funccall_ ## type ## _1(a) + 			\
+		 stress_funccall_ ## type ## _2(a, b) +			\
+		 stress_funccall_ ## type ## _3(a, b, c) +		\
+		 stress_funccall_ ## type ## _4(a, b, c, d) + 		\
+		 stress_funccall_ ## type ## _5(a, b, c, d, e) +	\
+		 stress_funccall_ ## type ## _6(a, b, c, d, e, f) + 	\
+		 stress_funccall_ ## type ## _7(a, b, c, d, e, f, g) + 	\
+		 stress_funccall_ ## type ## _8(a, b, c, d, e, f, g, h) + \
+		 stress_funccall_ ## type ## _9(a, b, c, d, e, f, g, h, i); \
+									\
+		res_new += 						\
+		 stress_funcdeep_ ## type ## _1(a) +			\
+		 stress_funcdeep_ ## type ## _2(a, b) +			\
+		 stress_funcdeep_ ## type ## _3(a, b, c) + 		\
+		 stress_funcdeep_ ## type ## _4(a, b, c, d) +		\
+		 stress_funcdeep_ ## type ## _5(a, b, c, d, e) +	\
+		 stress_funcdeep_ ## type ## _6(a, b, c, d, e, f) +	\
+		 stress_funcdeep_ ## type ## _7(a, b, c, d, e, f, g) +	\
+		 stress_funcdeep_ ## type ## _8(a, b, c, d, e, f, g, h) +\
+		 stress_funcdeep_ ## type ## _9(a, b, c, d, e, f, g, h, i); \
+		type ## _put(res_new);					\
+		if (ii == 0) {						\
+			res_old = res_new;				\
+		} else {						\
+			if (cmpfunc(res_old, res_new, type)) {		\
+				return false;				\
 			}						\
-		inc_counter(args);					\
-	} while (keep_stressing(args));					\
+		}							\
+	}								\
+	stress_bogo_inc(args);						\
+	return true;							\
 }
 
 #define stress_funccall_1(type)				\
@@ -372,6 +420,16 @@ static type NOINLINE stress_funccall_ ## type ## _9(	\
 	return a + b + c + d + e + f + g + h + i;	\
 }							\
 
+#define stress_funcdeep_1(type)				\
+static type NOINLINE stress_funcdeep_ ## type ## _1(	\
+	const type a);					\
+							\
+static type NOINLINE stress_funcdeep_ ## type ## _1(	\
+	const type a)					\
+{							\
+	return a;					\
+}							\
+
 #define stress_funcdeep_2(type)				\
 static type NOINLINE stress_funcdeep_ ## type ## _2(	\
 	const type a,					\
@@ -563,6 +621,7 @@ stress_funccall_6(bool)
 stress_funccall_7(bool)
 stress_funccall_8(bool)
 stress_funccall_9(bool)
+stress_funcdeep_1(bool)
 stress_funcdeep_2(bool)
 stress_funcdeep_3(bool)
 stress_funcdeep_4(bool)
@@ -571,7 +630,7 @@ stress_funcdeep_6(bool)
 stress_funcdeep_7(bool)
 stress_funcdeep_8(bool)
 stress_funcdeep_9(bool)
-stress_funccall_type(bool, stress_mwc1)
+stress_funccall_type(bool, stress_mwc1, cmp_type)
 
 stress_funccall_1(uint8_t)
 stress_funccall_2(uint8_t)
@@ -582,6 +641,7 @@ stress_funccall_6(uint8_t)
 stress_funccall_7(uint8_t)
 stress_funccall_8(uint8_t)
 stress_funccall_9(uint8_t)
+stress_funcdeep_1(uint8_t)
 stress_funcdeep_2(uint8_t)
 stress_funcdeep_3(uint8_t)
 stress_funcdeep_4(uint8_t)
@@ -590,7 +650,7 @@ stress_funcdeep_6(uint8_t)
 stress_funcdeep_7(uint8_t)
 stress_funcdeep_8(uint8_t)
 stress_funcdeep_9(uint8_t)
-stress_funccall_type(uint8_t, stress_mwc8)
+stress_funccall_type(uint8_t, stress_mwc8, cmp_type)
 
 stress_funccall_1(uint16_t)
 stress_funccall_2(uint16_t)
@@ -601,6 +661,7 @@ stress_funccall_6(uint16_t)
 stress_funccall_7(uint16_t)
 stress_funccall_8(uint16_t)
 stress_funccall_9(uint16_t)
+stress_funcdeep_1(uint16_t)
 stress_funcdeep_2(uint16_t)
 stress_funcdeep_3(uint16_t)
 stress_funcdeep_4(uint16_t)
@@ -609,7 +670,7 @@ stress_funcdeep_6(uint16_t)
 stress_funcdeep_7(uint16_t)
 stress_funcdeep_8(uint16_t)
 stress_funcdeep_9(uint16_t)
-stress_funccall_type(uint16_t, stress_mwc16)
+stress_funccall_type(uint16_t, stress_mwc16, cmp_type)
 
 stress_funccall_1(uint32_t)
 stress_funccall_2(uint32_t)
@@ -620,6 +681,7 @@ stress_funccall_6(uint32_t)
 stress_funccall_7(uint32_t)
 stress_funccall_8(uint32_t)
 stress_funccall_9(uint32_t)
+stress_funcdeep_1(uint32_t)
 stress_funcdeep_2(uint32_t)
 stress_funcdeep_3(uint32_t)
 stress_funcdeep_4(uint32_t)
@@ -628,7 +690,7 @@ stress_funcdeep_6(uint32_t)
 stress_funcdeep_7(uint32_t)
 stress_funcdeep_8(uint32_t)
 stress_funcdeep_9(uint32_t)
-stress_funccall_type(uint32_t, stress_mwc32)
+stress_funccall_type(uint32_t, stress_mwc32, cmp_type)
 
 stress_funccall_1(uint64_t)
 stress_funccall_2(uint64_t)
@@ -639,6 +701,7 @@ stress_funccall_6(uint64_t)
 stress_funccall_7(uint64_t)
 stress_funccall_8(uint64_t)
 stress_funccall_9(uint64_t)
+stress_funcdeep_1(uint64_t)
 stress_funcdeep_2(uint64_t)
 stress_funcdeep_3(uint64_t)
 stress_funcdeep_4(uint64_t)
@@ -647,7 +710,7 @@ stress_funcdeep_6(uint64_t)
 stress_funcdeep_7(uint64_t)
 stress_funcdeep_8(uint64_t)
 stress_funcdeep_9(uint64_t)
-stress_funccall_type(uint64_t, stress_mwc64)
+stress_funccall_type(uint64_t, stress_mwc64, cmp_type)
 
 #if defined(HAVE_INT128_T)
 stress_funccall_1(__uint128_t)
@@ -659,6 +722,7 @@ stress_funccall_6(__uint128_t)
 stress_funccall_7(__uint128_t)
 stress_funccall_8(__uint128_t)
 stress_funccall_9(__uint128_t)
+stress_funcdeep_1(__uint128_t)
 stress_funcdeep_2(__uint128_t)
 stress_funcdeep_3(__uint128_t)
 stress_funcdeep_4(__uint128_t)
@@ -667,7 +731,7 @@ stress_funcdeep_6(__uint128_t)
 stress_funcdeep_7(__uint128_t)
 stress_funcdeep_8(__uint128_t)
 stress_funcdeep_9(__uint128_t)
-stress_funccall_type(__uint128_t, stress_mwc64)
+stress_funccall_type(__uint128_t, stress_mwc64, cmp_type)
 #endif
 
 stress_funccall_1(float)
@@ -679,6 +743,7 @@ stress_funccall_6(float)
 stress_funccall_7(float)
 stress_funccall_8(float)
 stress_funccall_9(float)
+stress_funcdeep_1(float)
 stress_funcdeep_2(float)
 stress_funcdeep_3(float)
 stress_funcdeep_4(float)
@@ -687,7 +752,7 @@ stress_funcdeep_6(float)
 stress_funcdeep_7(float)
 stress_funcdeep_8(float)
 stress_funcdeep_9(float)
-stress_funccall_type(float, stress_mwcfloat)
+stress_funccall_type(float, stress_mwcfloat, cmp_fp)
 
 stress_funccall_1(double)
 stress_funccall_2(double)
@@ -698,6 +763,7 @@ stress_funccall_6(double)
 stress_funccall_7(double)
 stress_funccall_8(double)
 stress_funccall_9(double)
+stress_funcdeep_1(double)
 stress_funcdeep_2(double)
 stress_funcdeep_3(double)
 stress_funcdeep_4(double)
@@ -706,7 +772,7 @@ stress_funcdeep_6(double)
 stress_funcdeep_7(double)
 stress_funcdeep_8(double)
 stress_funcdeep_9(double)
-stress_funccall_type(double, stress_mwcdouble)
+stress_funccall_type(double, stress_mwcdouble, cmp_fp)
 
 stress_funccall_1(stress_long_double_t)
 stress_funccall_2(stress_long_double_t)
@@ -717,6 +783,7 @@ stress_funccall_6(stress_long_double_t)
 stress_funccall_7(stress_long_double_t)
 stress_funccall_8(stress_long_double_t)
 stress_funccall_9(stress_long_double_t)
+stress_funcdeep_1(stress_long_double_t)
 stress_funcdeep_2(stress_long_double_t)
 stress_funcdeep_3(stress_long_double_t)
 stress_funcdeep_4(stress_long_double_t)
@@ -725,7 +792,7 @@ stress_funcdeep_6(stress_long_double_t)
 stress_funcdeep_7(stress_long_double_t)
 stress_funcdeep_8(stress_long_double_t)
 stress_funcdeep_9(stress_long_double_t)
-stress_funccall_type(stress_long_double_t, stress_mwc64)
+stress_funccall_type(stress_long_double_t, stress_mwc64, cmp_fp)
 
 #if defined(HAVE_COMPLEX_H) &&			\
     defined(HAVE_COMPLEX) &&			\
@@ -741,6 +808,7 @@ stress_funccall_6(stress_complex_float_t)
 stress_funccall_7(stress_complex_float_t)
 stress_funccall_8(stress_complex_float_t)
 stress_funccall_9(stress_complex_float_t)
+stress_funcdeep_1(stress_complex_float_t)
 stress_funcdeep_2(stress_complex_float_t)
 stress_funcdeep_3(stress_complex_float_t)
 stress_funcdeep_4(stress_complex_float_t)
@@ -749,7 +817,7 @@ stress_funcdeep_6(stress_complex_float_t)
 stress_funcdeep_7(stress_complex_float_t)
 stress_funcdeep_8(stress_complex_float_t)
 stress_funcdeep_9(stress_complex_float_t)
-stress_funccall_type(stress_complex_float_t, stress_mwcdouble)
+stress_funccall_type(stress_complex_float_t, stress_mwcdouble, cmp_cmplx)
 
 stress_funccall_1(stress_complex_double_t)
 stress_funccall_2(stress_complex_double_t)
@@ -760,6 +828,7 @@ stress_funccall_6(stress_complex_double_t)
 stress_funccall_7(stress_complex_double_t)
 stress_funccall_8(stress_complex_double_t)
 stress_funccall_9(stress_complex_double_t)
+stress_funcdeep_1(stress_complex_double_t)
 stress_funcdeep_2(stress_complex_double_t)
 stress_funcdeep_3(stress_complex_double_t)
 stress_funcdeep_4(stress_complex_double_t)
@@ -768,7 +837,7 @@ stress_funcdeep_6(stress_complex_double_t)
 stress_funcdeep_7(stress_complex_double_t)
 stress_funcdeep_8(stress_complex_double_t)
 stress_funcdeep_9(stress_complex_double_t)
-stress_funccall_type(stress_complex_double_t, stress_mwcdouble)
+stress_funccall_type(stress_complex_double_t, stress_mwcdouble, cmp_cmplx)
 
 stress_funccall_1(stress_complex_long_double_t)
 stress_funccall_2(stress_complex_long_double_t)
@@ -779,6 +848,7 @@ stress_funccall_6(stress_complex_long_double_t)
 stress_funccall_7(stress_complex_long_double_t)
 stress_funccall_8(stress_complex_long_double_t)
 stress_funccall_9(stress_complex_long_double_t)
+stress_funcdeep_1(stress_complex_long_double_t)
 stress_funcdeep_2(stress_complex_long_double_t)
 stress_funcdeep_3(stress_complex_long_double_t)
 stress_funcdeep_4(stress_complex_long_double_t)
@@ -787,20 +857,12 @@ stress_funcdeep_6(stress_complex_long_double_t)
 stress_funcdeep_7(stress_complex_long_double_t)
 stress_funcdeep_8(stress_complex_long_double_t)
 stress_funcdeep_9(stress_complex_long_double_t)
-stress_funccall_type(stress_complex_long_double_t, stress_mwcdouble)
-#endif
-
-/*
- *  The PCC compiler complains at ALWAYS_INLINE, so disable it
- */
-#if defined(__PCC__)
-#undef ALWAYS_INLINE
-#define ALWAYS_INLINE
+stress_funccall_type(stress_complex_long_double_t, stress_mwcdouble, cmp_cmplx)
 #endif
 
 #if defined(HAVE_FLOAT_DECIMAL32) &&	\
-    !defined(__clang__)
-static inline void ALWAYS_INLINE _Decimal32_put(const _Decimal32 a)
+    !defined(HAVE_COMPILER_CLANG)
+static inline void _Decimal32_put(const _Decimal32 a)
 {
 	g_put_val.double_val = (double)a;
 }
@@ -814,6 +876,7 @@ stress_funccall_6(_Decimal32)
 stress_funccall_7(_Decimal32)
 stress_funccall_8(_Decimal32)
 stress_funccall_9(_Decimal32)
+stress_funcdeep_1(_Decimal32)
 stress_funcdeep_2(_Decimal32)
 stress_funcdeep_3(_Decimal32)
 stress_funcdeep_4(_Decimal32)
@@ -822,12 +885,12 @@ stress_funcdeep_6(_Decimal32)
 stress_funcdeep_7(_Decimal32)
 stress_funcdeep_8(_Decimal32)
 stress_funcdeep_9(_Decimal32)
-stress_funccall_type(_Decimal32, (_Decimal32)stress_mwc64)
+stress_funccall_type(_Decimal32, (_Decimal32)stress_mwc64, cmp_ignore)
 #endif
 
 #if defined(HAVE_FLOAT_DECIMAL64) &&	\
-    !defined(__clang__)
-static inline void ALWAYS_INLINE _Decimal64_put(const _Decimal64 a)
+    !defined(HAVE_COMPILER_CLANG)
+static inline void _Decimal64_put(const _Decimal64 a)
 {
 	g_put_val.double_val = (double)a;
 }
@@ -841,6 +904,7 @@ stress_funccall_6(_Decimal64)
 stress_funccall_7(_Decimal64)
 stress_funccall_8(_Decimal64)
 stress_funccall_9(_Decimal64)
+stress_funcdeep_1(_Decimal64)
 stress_funcdeep_2(_Decimal64)
 stress_funcdeep_3(_Decimal64)
 stress_funcdeep_4(_Decimal64)
@@ -849,12 +913,12 @@ stress_funcdeep_6(_Decimal64)
 stress_funcdeep_7(_Decimal64)
 stress_funcdeep_8(_Decimal64)
 stress_funcdeep_9(_Decimal64)
-stress_funccall_type(_Decimal64, (_Decimal64)stress_mwc64)
+stress_funccall_type(_Decimal64, (_Decimal64)stress_mwc64, cmp_ignore)
 #endif
 
 #if defined(HAVE_FLOAT_DECIMAL128) &&	\
-    !defined(__clang__)
-static inline void ALWAYS_INLINE _Decimal128_put(const _Decimal128 a)
+    !defined(HAVE_COMPILER_CLANG)
+static inline void _Decimal128_put(const _Decimal128 a)
 {
 	g_put_val.double_val = (double)a;
 }
@@ -868,6 +932,7 @@ stress_funccall_6(_Decimal128)
 stress_funccall_7(_Decimal128)
 stress_funccall_8(_Decimal128)
 stress_funccall_9(_Decimal128)
+stress_funcdeep_1(_Decimal128)
 stress_funcdeep_2(_Decimal128)
 stress_funcdeep_3(_Decimal128)
 stress_funcdeep_4(_Decimal128)
@@ -876,12 +941,12 @@ stress_funcdeep_6(_Decimal128)
 stress_funcdeep_7(_Decimal128)
 stress_funcdeep_8(_Decimal128)
 stress_funcdeep_9(_Decimal128)
-stress_funccall_type(_Decimal128, (_Decimal128)stress_mwc64)
+stress_funccall_type(_Decimal128, (_Decimal128)stress_mwc64, cmp_ignore)
 #endif
 
 #if defined(HAVE_FLOAT16) &&	\
-    !defined(__clang__)
-static inline void ALWAYS_INLINE __fp16_put(const __fp16 a)
+    !defined(HAVE_COMPILER_CLANG)
+static inline void __fp16_put(const __fp16 a)
 {
 	g_put_val.float_val = (float)a;
 }
@@ -895,6 +960,7 @@ stress_funccall_6(__fp16)
 stress_funccall_7(__fp16)
 stress_funccall_8(__fp16)
 stress_funccall_9(__fp16)
+stress_funcdeep_1(__fp16)
 stress_funcdeep_2(__fp16)
 stress_funcdeep_3(__fp16)
 stress_funcdeep_4(__fp16)
@@ -903,12 +969,12 @@ stress_funcdeep_6(__fp16)
 stress_funcdeep_7(__fp16)
 stress_funcdeep_8(__fp16)
 stress_funcdeep_9(__fp16)
-stress_funccall_type(__fp16, (__fp16)stress_mwc32)
+stress_funccall_type(__fp16, (__fp16)stress_mwc32, cmp_fp)
 #endif
 
 #if defined(HAVE_FLOAT32) &&	\
-    !defined(__clang__)
-static inline void ALWAYS_INLINE _Float32_put(const _Float32 a)
+    !defined(HAVE_COMPILER_CLANG)
+static inline void _Float32_put(const _Float32 a)
 {
 	g_put_val.float_val = (float)a;
 }
@@ -922,6 +988,7 @@ stress_funccall_6(_Float32)
 stress_funccall_7(_Float32)
 stress_funccall_8(_Float32)
 stress_funccall_9(_Float32)
+stress_funcdeep_1(_Float32)
 stress_funcdeep_2(_Float32)
 stress_funcdeep_3(_Float32)
 stress_funcdeep_4(_Float32)
@@ -930,12 +997,12 @@ stress_funcdeep_6(_Float32)
 stress_funcdeep_7(_Float32)
 stress_funcdeep_8(_Float32)
 stress_funcdeep_9(_Float32)
-stress_funccall_type(_Float32, (_Float32)stress_mwc32)
+stress_funccall_type(_Float32, (_Float32)stress_mwc32, cmp_ignore)
 #endif
 
 #if defined(HAVE_FLOAT64) &&	\
-    !defined(__clang__)
-static inline void ALWAYS_INLINE _Float64_put(const _Float64 a)
+    !defined(HAVE_COMPILER_CLANG)
+static inline void _Float64_put(const _Float64 a)
 {
 	g_put_val.double_val = (double)a;
 }
@@ -949,6 +1016,7 @@ stress_funccall_6(_Float64)
 stress_funccall_7(_Float64)
 stress_funccall_8(_Float64)
 stress_funccall_9(_Float64)
+stress_funcdeep_1(_Float64)
 stress_funcdeep_2(_Float64)
 stress_funcdeep_3(_Float64)
 stress_funcdeep_4(_Float64)
@@ -957,13 +1025,13 @@ stress_funcdeep_6(_Float64)
 stress_funcdeep_7(_Float64)
 stress_funcdeep_8(_Float64)
 stress_funcdeep_9(_Float64)
-stress_funccall_type(_Float64, (_Float64)stress_mwc64)
+stress_funccall_type(_Float64, (_Float64)stress_mwc64, cmp_ignore)
 #endif
 
 
 #if defined(HAVE_FLOAT80) &&	\
-    !defined(__clang__)
-static inline void ALWAYS_INLINE __float80_put(const __float80 a)
+    !defined(HAVE_COMPILER_CLANG)
+static inline void __float80_put(const __float80 a)
 {
 	g_put_val.double_val = (double)a;
 }
@@ -977,6 +1045,7 @@ stress_funccall_6(__float80)
 stress_funccall_7(__float80)
 stress_funccall_8(__float80)
 stress_funccall_9(__float80)
+stress_funcdeep_1(__float80)
 stress_funcdeep_2(__float80)
 stress_funcdeep_3(__float80)
 stress_funcdeep_4(__float80)
@@ -985,12 +1054,12 @@ stress_funcdeep_6(__float80)
 stress_funcdeep_7(__float80)
 stress_funcdeep_8(__float80)
 stress_funcdeep_9(__float80)
-stress_funccall_type(__float80, (__float80)stress_mwc64)
+stress_funccall_type(__float80, (__float80)stress_mwc64, cmp_fp)
 #endif
 
 #if defined(HAVE_FLOAT128) &&	\
-    !defined(__clang__)
-static inline void ALWAYS_INLINE __float128_put(const __float128 a)
+    !defined(HAVE_COMPILER_CLANG)
+static inline void __float128_put(const __float128 a)
 {
 	g_put_val.double_val = (double)a;
 }
@@ -1004,6 +1073,7 @@ stress_funccall_6(__float128)
 stress_funccall_7(__float128)
 stress_funccall_8(__float128)
 stress_funccall_9(__float128)
+stress_funcdeep_1(__float128)
 stress_funcdeep_2(__float128)
 stress_funcdeep_3(__float128)
 stress_funcdeep_4(__float128)
@@ -1012,13 +1082,16 @@ stress_funcdeep_6(__float128)
 stress_funcdeep_7(__float128)
 stress_funcdeep_8(__float128)
 stress_funcdeep_9(__float128)
-stress_funccall_type(__float128, (__float128)stress_mwc64)
+stress_funccall_type(__float128, (__float128)stress_mwc64, cmp_fp)
 #endif
 
+static bool stress_funccall_all(const stress_args_t *args);
+
 /*
  * Table of func call stress methods
  */
-static const stress_funccall_method_info_t funccall_methods[] = {
+static const stress_funccall_method_info_t stress_funccall_methods[] = {
+	{ "all",	stress_funccall_all },
 	{ "bool",	stress_funccall_bool },
 	{ "uint8",	stress_funccall_uint8_t },
 	{ "uint16",	stress_funccall_uint16_t },
@@ -1029,23 +1102,23 @@ static const stress_funccall_method_info_t funccall_methods[] = {
 #endif
 	{ "float",	stress_funccall_float },
 #if defined(HAVE_FLOAT16) &&	\
-    !defined(__clang__)
+    !defined(HAVE_COMPILER_CLANG)
 	{ "float16",	stress_funccall___fp16 },
 #endif
 #if defined(HAVE_FLOAT32) &&	\
-    !defined(__clang__)
+    !defined(HAVE_COMPILER_CLANG)
 	{ "float32",	stress_funccall__Float32 },
 #endif
 #if defined(HAVE_FLOAT64) &&	\
-    !defined(__clang__)
+    !defined(HAVE_COMPILER_CLANG)
 	{ "float64",	stress_funccall__Float64 },
 #endif
 #if defined(HAVE_FLOAT80) &&	\
-    !defined(__clang__)
+    !defined(HAVE_COMPILER_CLANG)
 	{ "float80",	stress_funccall___float80 },
 #endif
 #if defined(HAVE_FLOAT128) &&	\
-    !defined(__clang__)
+    !defined(HAVE_COMPILER_CLANG)
 	{ "float128",	stress_funccall___float128 },
 #endif
 	{ "double",	stress_funccall_double },
@@ -1059,38 +1132,67 @@ static const stress_funccall_method_info_t funccall_methods[] = {
 	{ "clongdouble",stress_funccall_stress_complex_long_double_t },
 #endif
 #if defined(HAVE_FLOAT_DECIMAL32) &&	\
-    !defined(__clang__)
+    !defined(HAVE_COMPILER_CLANG)
 	{ "decimal32",	stress_funccall__Decimal32 },
 #endif
 #if defined(HAVE_FLOAT_DECIMAL64) &&	\
-     !defined(__clang__)
+     !defined(HAVE_COMPILER_CLANG)
 	{ "decimal64",	stress_funccall__Decimal64 },
 #endif
 #if defined(HAVE_FLOAT_DECIMAL128) &&	\
-    !defined(__clang__)
+    !defined(HAVE_COMPILER_CLANG)
 	{ "decimal128",	stress_funccall__Decimal128 },
 #endif
-	{ NULL,		NULL },
 };
 
+static stress_metrics_t stress_funccall_metrics[SIZEOF_ARRAY(stress_funccall_methods)];
+
+static bool stress_funccall_exercise(const stress_args_t *args, const size_t method)
+{
+	bool success;
+	double t;
+
+	t = stress_time_now();
+	success = stress_funccall_methods[method].func(args);
+	stress_funccall_metrics[method].duration += stress_time_now() - t;
+	stress_funccall_metrics[method].count += 1.0;
+
+	if (!success && (method != 0)) {
+		pr_fail("%s: verification failed with a nested %s function call return value\n",
+			args->name, stress_funccall_methods[method].name);
+	}
+	return success;
+}
+
+static bool stress_funccall_all(const stress_args_t *args)
+{
+	size_t i;
+	bool success = true;
+
+	for (i = 1; success && (i < SIZEOF_ARRAY(stress_funccall_methods)); i++) {
+		success &= stress_funccall_exercise(args, i);
+	}
+	return success;
+}
+
 /*
  *  stress_set_funccall_method()
  *	set the default funccal stress method
  */
 static int stress_set_funccall_method(const char *name)
 {
-	stress_funccall_method_info_t const *info;
+	size_t i;
 
-	for (info = funccall_methods; info->func; info++) {
-		if (!strcmp(info->name, name)) {
-			stress_set_setting("funccall-method", TYPE_ID_UINTPTR_T, &info);
+	for (i = 0; i < SIZEOF_ARRAY(stress_funccall_methods); i++) {
+		if (!strcmp(stress_funccall_methods[i].name, name)) {
+			stress_set_setting("funccall-method", TYPE_ID_SIZE_T, &i);
 			return 0;
 		}
 	}
 
 	(void)fprintf(stderr, "funccall-method must be one of:");
-	for (info = funccall_methods; info->func; info++) {
-		(void)fprintf(stderr, " %s", info->name);
+	for (i = 0; i < SIZEOF_ARRAY(stress_funccall_methods); i++) {
+		(void)fprintf(stderr, " %s", stress_funccall_methods[i].name);
 	}
 	(void)fprintf(stderr, "\n");
 
@@ -1103,22 +1205,41 @@ static int stress_set_funccall_method(const char *name)
  */
 static int stress_funccall(const stress_args_t *args)
 {
-	const stress_funccall_method_info_t *funccall_method = &funccall_methods[3];
+	size_t funccall_method = 0;
+	bool success;
+
+	size_t i, j;
+
+	for (i = 0; i < SIZEOF_ARRAY(stress_funccall_metrics); i++) {
+		stress_funccall_metrics[i].duration = 0.0;
+		stress_funccall_metrics[i].count = 0.0;
+	}
 
 	(void)stress_get_setting("funccall-method", &funccall_method);
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
-	funccall_method->func(args);
+	do {
+		success = stress_funccall_exercise(args, funccall_method);
+	} while (success && stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
-	return EXIT_SUCCESS;
-}
 
+	for (i = 1, j = 0; i < SIZEOF_ARRAY(stress_funccall_metrics); i++) {
+		double rate = (stress_funccall_metrics[i].duration > 0) ?
+			stress_funccall_metrics[i].count / stress_funccall_metrics[i].duration : 0.0;
 
-static void stress_funccall_set_default(void)
-{
-	stress_set_funccall_method("uint64");
+		if (rate > 0.0) {
+			char msg[64];
+
+			(void)snprintf(msg, sizeof(msg), "%s function invocations per sec",
+					stress_funccall_methods[i].name);
+			stress_metrics_set(args, j, msg, rate);
+			j++;
+		}
+	}
+
+	return success ? EXIT_SUCCESS : EXIT_FAILURE;
 }
 
 static const stress_opt_set_func_t opt_set_funcs[] = {
@@ -1128,8 +1249,8 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
 
 stressor_info_t stress_funccall_info = {
 	.stressor = stress_funccall,
-	.set_default = stress_funccall_set_default,
 	.class = CLASS_CPU,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
