@@ -18,7 +18,9 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
 #include "core-capabilities.h"
+#include "core-mounts.h"
 
 #if defined(HAVE_LINUX_SYSCTL_H)
 #include <linux/sysctl.h>
@@ -201,12 +203,12 @@ static int stress_get(const stress_args_t *args)
 
 		(void)mypid;
 
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 
 		VOID_RET(pid_t, getppid());
 
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 
 #if defined(HAVE_SWAPCONTEXT) &&        \
@@ -215,22 +217,9 @@ static int stress_get(const stress_args_t *args)
 			ucontext_t context;
 
 			VOID_RET(int, getcontext(&context));
-			if (!keep_stressing_flag())
+			if (!stress_continue_flag())
 				break;
 		}
-
-#if defined(HAVE_SWAPCONTEXT) &&        \
-    defined(HAVE_UCONTEXT_H)
-		{
-			ucontext_t context;
-
-			VOID_RET(int, getcontext(&context));
-			if (!keep_stressing_flag())
-				break;
-		}
-#else
-		UNEXPECTED
-#endif
 #else
 		UNEXPECTED
 #endif
@@ -240,7 +229,7 @@ static int stress_get(const stress_args_t *args)
 			char name[128];
 
 			VOID_RET(int, shim_getdomainname(name, sizeof(name)));
-			if (!keep_stressing_flag())
+			if (!stress_continue_flag())
 				break;
 		}
 #else
@@ -250,7 +239,7 @@ static int stress_get(const stress_args_t *args)
 #if defined(HAVE_GETHOSTID)
 		{
 			VOID_RET(long, gethostid());
-			if (!keep_stressing_flag())
+			if (!stress_continue_flag())
 				break;
 		}
 #else
@@ -262,7 +251,7 @@ static int stress_get(const stress_args_t *args)
 			char name[128];
 
 			VOID_RET(int, gethostname(name, sizeof(name)));
-			if (!keep_stressing_flag())
+			if (!stress_continue_flag())
 				break;
 		}
 #else
@@ -274,7 +263,7 @@ static int stress_get(const stress_args_t *args)
 			if (!ptr) {
 				pr_fail("%s: getcwd %s failed, errno=%d (%s)%s\n",
 					args->name, path, errno, strerror(errno),
-					stress_fs_type(path));
+					stress_get_fs_type(path));
 			} else {
 				/* getcwd returned a string: is it the same as path? */
 				if (strncmp(ptr, path, sizeof(path))) {
@@ -284,7 +273,7 @@ static int stress_get(const stress_args_t *args)
 				}
 			}
 		}
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 
 		VOID_RET(gid_t, getgid());
@@ -292,7 +281,7 @@ static int stress_get(const stress_args_t *args)
 		VOID_RET(uid_t, getuid());
 		VOID_RET(uid_t, geteuid());
 
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 
 		/*
@@ -319,10 +308,10 @@ static int stress_get(const stress_args_t *args)
 		 *  system call if possible. Memset gids to zero to keep
 		 *  valgrind happy.
 		 */
-		(void)memset(gids, 0, sizeof(gids));
+		(void)shim_memset(gids, 0, sizeof(gids));
 		VOID_RET(long, syscall(__NR_getgroups, -1, gids));
 #endif
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 
 #if defined(HAVE_GETPGRP)
@@ -343,7 +332,7 @@ static int stress_get(const stress_args_t *args)
 			pid = stress_get_unused_pid_racy(false);
 			VOID_RET(pid_t, getpgid(pid));
 		}
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 #else
 		UNEXPECTED
@@ -368,7 +357,7 @@ static int stress_get(const stress_args_t *args)
 			if (verify && errno && (errno != EINVAL) && (ret < 0))
 				pr_fail("%s: getpriority failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
-			if (!keep_stressing_flag())
+			if (!stress_continue_flag())
 				break;
 		}
 		/* Exercise getpriority calls using non-zero who argument */
@@ -387,7 +376,7 @@ static int stress_get(const stress_args_t *args)
 			if (verify && (ret < 0))
 				pr_fail("%s: getresgid failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
-			if (!keep_stressing_flag())
+			if (!stress_continue_flag())
 				break;
 		}
 #else
@@ -402,7 +391,7 @@ static int stress_get(const stress_args_t *args)
 			if (verify && (ret < 0))
 				pr_fail("%s: getresuid failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
-			if (!keep_stressing_flag())
+			if (!stress_continue_flag())
 				break;
 		}
 #else
@@ -416,7 +405,7 @@ static int stress_get(const stress_args_t *args)
 			if (verify && (ret < 0))
 				pr_fail("%s: getrlimit(%zu, ..) failed, errno=%d (%s)\n",
 					args->name, i, errno, strerror(errno));
-			if (!keep_stressing_flag())
+			if (!stress_continue_flag())
 				break;
 		}
 
@@ -468,7 +457,7 @@ static int stress_get(const stress_args_t *args)
 				/* Exercise invalid pids */
 				(void)prlimit(pid, rlimits[i], NULL, &rlims[0]);
 			}
-			if (!keep_stressing_flag())
+			if (!stress_continue_flag())
 				break;
 		}
 #else
@@ -488,7 +477,7 @@ static int stress_get(const stress_args_t *args)
 			size_t kern_version_len = 0;
 			struct __sysctl_args sysctl_args;
 
-			(void)memset(&sysctl_args, 0, sizeof(sysctl_args));
+			(void)shim_memset(&sysctl_args, 0, sizeof(sysctl_args));
 			sysctl_args.name = name;
 			sysctl_args.nlen = SIZEOF_ARRAY(name);
 			sysctl_args.oldval = kern_version;
@@ -507,7 +496,7 @@ static int stress_get(const stress_args_t *args)
 			if (rusages[i].verify && verify && (ret < 0) && (errno != ENOSYS))
 				pr_fail("%s: getrusage(%s, ..) failed, errno=%d (%s)\n",
 					args->name, rusages[i].name, errno, strerror(errno));
-			if (!keep_stressing_flag())
+			if (!stress_continue_flag())
 				break;
 		}
 
@@ -522,7 +511,7 @@ static int stress_get(const stress_args_t *args)
 			pid = stress_get_unused_pid_racy(false);
 			VOID_RET(int, getsid(pid));
 		}
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 		break;
 #else
 		UNEXPECTED
@@ -534,7 +523,7 @@ static int stress_get(const stress_args_t *args)
 		(void)shim_getcpu(NULL, &node, NULL);
 		(void)shim_getcpu(&cpu, NULL, NULL);
 		(void)shim_getcpu(NULL, NULL, NULL);
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 
 		t = time(NULL);
@@ -548,7 +537,7 @@ static int stress_get(const stress_args_t *args)
 		 *  with it to increase kernel test coverage
 		 */
 		t1 = time(&t2);
-		if (memcmp(&t1, &t2, sizeof(t1))) {
+		if (shim_memcmp(&t1, &t2, sizeof(t1))) {
 			pr_fail("%s: time failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 
@@ -564,7 +553,7 @@ static int stress_get(const stress_args_t *args)
 
 		t1 = shim_time(&t2);
 		if ((t == (time_t)-1) && (errno != ENOSYS)) {
-			if (memcmp(&t1, &t2, sizeof(t1))) {
+			if (shim_memcmp(&t1, &t2, sizeof(t1))) {
 				pr_fail("%s: time failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 			}
@@ -627,7 +616,7 @@ static int stress_get(const stress_args_t *args)
 
 			if (!uname_segv) {
 				ret = sigsetjmp(jmp_env, 1);
-				if (!keep_stressing(args))
+				if (!stress_continue(args))
 					break;
 				if (ret != 0) {
 					uname_segv = true;
@@ -690,7 +679,7 @@ static int stress_get(const stress_args_t *args)
 
 #if defined(HAVE_SYS_TIMEX_H) &&	\
     defined(HAVE_ADJTIME)
-		(void)memset(&delta, 0, sizeof(delta));
+		(void)shim_memset(&delta, 0, sizeof(delta));
 		ret = adjtime(&delta, &tv);
 		if (cap_sys_time && verify && (ret < 0) && (errno != EPERM))
 			pr_fail("%s: adjtime failed, errno=%d (%s)\n",
@@ -757,8 +746,8 @@ static int stress_get(const stress_args_t *args)
 		UNEXPECTED
 #endif
 
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
