@@ -17,6 +17,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
 #include "core-hash.h"
 #if defined(HAVE_XXHASH_H)
 #include <xxhash.h>
@@ -55,8 +56,6 @@ static const stress_help_t help[] = {
 	{ NULL,	 NULL,			NULL }
 };
 
-static stress_hash_method_info_t hash_methods[];
-
 /*
  *  stress_hash_generic()
  *	stress test generic string hash function
@@ -83,7 +82,7 @@ static void stress_hash_generic(
 		stress_mwc_reseed();
 	}
 
-	(void)memset(bucket->buckets, 0, bucket->size);
+	(void)shim_memset(bucket->buckets, 0, bucket->size);
 
 	stress_uint8rnd4((uint8_t *)bucket->buffer, bucket->n_keys);
 	/* Make it ASCII range ' '..'_' */
@@ -569,25 +568,10 @@ static void stress_hash_method_sobel(
 	stress_hash_generic(name, hmi, bucket, stress_hash_sobel_wrapper, 0x2a7cdb61, 0x2a7cdb61);
 }
 
-/*
- *  stress_hash_all()
- *	iterate over all hash stressor methods
- */
 static HOT OPTIMIZE3 void stress_hash_all(
 	const char *name,
 	const struct stress_hash_method_info *hmi,
-	const stress_bucket_t *bucket)
-{
-	static int i = 1;	/* Skip over stress_hash_all */
-	const struct stress_hash_method_info *h = &hash_methods[i];
-
-	(void)hmi;
-
-	h->func(name, h, bucket);
-	i++;
-	if (!hash_methods[i].func)
-		i = 1;
-}
+	const stress_bucket_t *bucket);
 
 /*
  * Table of has stress methods
@@ -623,9 +607,28 @@ static stress_hash_method_info_t hash_methods[] = {
     defined(HAVE_LIB_XXHASH)
 	{ "xxh64",		stress_hash_method_xxh64,	NULL },
 #endif
-	{ NULL,			NULL,				NULL }
 };
 
+/*
+ *  stress_hash_all()
+ *	iterate over all hash stressor methods
+ */
+static HOT OPTIMIZE3 void stress_hash_all(
+	const char *name,
+	const struct stress_hash_method_info *hmi,
+	const stress_bucket_t *bucket)
+{
+	static size_t i = 1;	/* Skip over stress_hash_all */
+	const struct stress_hash_method_info *h = &hash_methods[i];
+
+	(void)hmi;
+
+	h->func(name, h, bucket);
+	i++;
+	if (i >= SIZEOF_ARRAY(hash_methods))
+		i = 1;
+}
+
 static stress_hash_stats_t hash_stats[SIZEOF_ARRAY(hash_methods)];
 
 /*
@@ -636,7 +639,7 @@ static int stress_set_hash_method(const char *name)
 {
 	size_t i;
 
-	for (i = 0; hash_methods[i].name; i++) {
+	for (i = 0; i < SIZEOF_ARRAY(hash_methods); i++) {
 		if (!strcmp(hash_methods[i].name, name)) {
 			stress_set_setting("hash-method", TYPE_ID_SIZE_T, &i);
 			return 0;
@@ -644,7 +647,7 @@ static int stress_set_hash_method(const char *name)
 	}
 
 	(void)fprintf(stderr, "hash-method must be one of:");
-	for (i = 0; hash_methods[i].name; i++) {
+	for (i = 0; i < SIZEOF_ARRAY(hash_methods); i++) {
 		(void)fprintf(stderr, " %s", hash_methods[i].name);
 	}
 	(void)fprintf(stderr, "\n");
@@ -675,7 +678,7 @@ static int HOT OPTIMIZE3 stress_hash(const stress_args_t *args)
 	}
 	buffer = calloc(bucket.n_keys + 64, sizeof(*buffer));
 	if (!buffer) {
-		pr_inf_skip("%s: failed to allocate %" PRIu32 " byte bufffer, skipping stressor\n",
+		pr_inf_skip("%s: failed to allocate %" PRIu32 " byte buffer, skipping stressor\n",
 			args->name, bucket.n_keys);
 		free(bucket.buckets);
 		return EXIT_NO_RESOURCE;
@@ -685,7 +688,7 @@ static int HOT OPTIMIZE3 stress_hash(const stress_args_t *args)
 	(void)stress_get_setting("hash-method", &hash_method);
 	hm = &hash_methods[hash_method];
 
-	for (i = 0; hash_methods[i].name; i++) {
+	for (i = 0; i < SIZEOF_ARRAY(hash_methods); i++) {
 		hash_stats[i].duration = 0.0;
 		hash_stats[i].total = false;
 		hash_stats[i].chi_squared = 0.0;
@@ -699,17 +702,17 @@ static int HOT OPTIMIZE3 stress_hash(const stress_args_t *args)
 
 	do {
 		(void)hm->func(args->name, hm, &bucket);
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	if (args->instance == 0) {
 		pr_lock();
 		pr_inf("%s: %12.12s %15s %10s\n",
 			args->name, "hash", "hashes/sec", "chi squared");
-		for (i = 1; hash_methods[i].name; i++) {
+		for (i = 1; i < SIZEOF_ARRAY(hash_methods); i++) {
 			stress_hash_stats_t *stats = hash_methods[i].stats;
 
-			if (stats->duration > 0.0 && stats->total > 0) {
+			if ((stats->duration > 0.0) && (stats->total > 0)) {
 				const double rate = (double)((stats->duration > 0.0) ?
 					(double)stats->total / stats->duration : (double)0.0);
 
