@@ -143,7 +143,7 @@ static int stress_heapsort(const stress_args_t *args)
 				}
 			}
 		}
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 
 		/* Reverse sort */
@@ -167,7 +167,7 @@ static int stress_heapsort(const stress_args_t *args)
 				}
 			}
 		}
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 		/* And re-order  */
 		stress_sort_data_int32_mangle(data, n);
@@ -194,11 +194,11 @@ static int stress_heapsort(const stress_args_t *args)
 				}
 			}
 		}
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	do_jmp = false;
 	(void)stress_sigrestore(args->name, SIGALRM, &old_action);
@@ -225,7 +225,8 @@ stressor_info_t stress_heapsort_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_CPU_CACHE | CLASS_CPU | CLASS_MEMORY,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_OPTIONAL,
 	.help = help,
-	.unimplemented_reason = "built without BSD library"
+	.unimplemented_reason = "built without the BSD library"
 };
 #endif
