@@ -18,6 +18,9 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
+#include "core-killpid.h"
+#include "core-out-of-memory.h"
 
 static const stress_help_t help[] = {
 	{ NULL,	"hrtimers N",	  "start N workers that exercise high resolution timers" },
@@ -43,7 +46,7 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
 static uint64_t timer_counter;
 static uint64_t max_ops;
 static timer_t timerid;
-static double start;
+static double time_end;
 static long ns_delay;
 static int overrun;
 void *lock;
@@ -63,12 +66,12 @@ static void stress_hrtimers_set(struct itimerspec *timer)
 }
 
 /*
- *  stress_hrtimers_keep_stressing(args)
+ *  stress_hrtimers_stress_continue(args)
  *      returns true if we can keep on running a stressor
  */
-static bool HOT OPTIMIZE3 stress_hrtimers_keep_stressing(void)
+static bool HOT OPTIMIZE3 stress_hrtimers_stress_continue(void)
 {
-	return (LIKELY(keep_stressing_flag()) &&
+	return (LIKELY(stress_continue_flag()) &&
 		LIKELY(!max_ops || ((timer_counter) < max_ops)));
 }
 
@@ -84,7 +87,7 @@ static void MLOCKED_TEXT OPTIMIZE3 stress_hrtimers_handler(int sig)
 	(void)sig;
 
 	timer_counter++;
-	if (UNLIKELY(!stress_hrtimers_keep_stressing()))
+	if (UNLIKELY(!stress_hrtimers_stress_continue()))
 		goto cancel;
 
 	if (UNLIKELY((timer_counter & 4095) == 0)) {
@@ -104,18 +107,18 @@ static void MLOCKED_TEXT OPTIMIZE3 stress_hrtimers_handler(int sig)
 		if ((timer_counter & 65535) == 0) {
 			if ((sigpending(&mask) == 0) && (sigismember(&mask, SIGINT)))
 				goto cancel;
-			if ((stress_time_now() - start) > (double)g_opt_timeout)
+			if (stress_time_now() > time_end)
 				goto cancel;
 		}
 	}
 	return;
 
 cancel:
-	keep_stressing_set_flag(false);
+	stress_continue_set_flag(false);
 	/* Cancel timer if we detect no more runs */
-	(void)memset(&timer, 0, sizeof(timer));
+	(void)shim_memset(&timer, 0, sizeof(timer));
 	(void)timer_settime(timerid, 0, &timer, NULL);
-	(void)kill(getpid(), SIGALRM);
+	(void)shim_kill(getpid(), SIGALRM);
 }
 
 
@@ -130,26 +133,31 @@ static int stress_hrtimer_process(const stress_args_t *args)
 	struct itimerspec timer;
 	sigset_t mask;
 
+	time_end = args->time_end;
+
 	(void)sigemptyset(&mask);
 	(void)sigaddset(&mask, SIGINT);
 	(void)sigprocmask(SIG_SETMASK, &mask, NULL);
 
 	VOID_RET(int, stress_set_sched(getpid(), SCHED_RR, UNDEFINED, true));
 
-	start = stress_time_now();
-
-	(void)memset(&action, 0, sizeof action);
+	(void)shim_memset(&action, 0, sizeof action);
 	action.sa_handler = stress_hrtimers_handler;
 	(void)sigemptyset(&action.sa_mask);
 
 	if (sigaction(SIGRTMIN, &action, NULL) < 0)
 		return EXIT_FAILURE;
 
-	(void)memset(&sev, 0, sizeof(sev));
+	(void)shim_memset(&sev, 0, sizeof(sev));
 	sev.sigev_notify = SIGEV_SIGNAL;
 	sev.sigev_signo = SIGRTMIN;
 	sev.sigev_value.sival_ptr = &timerid;
 	if (timer_create(CLOCK_REALTIME, &sev, &timerid) < 0) {
+		if ((errno == EAGAIN) || (errno == ENOMEM)) {
+			pr_inf_skip("%s: timer_create, errno=%d (%s), skipping stessor\n",
+				args->name, errno, strerror(errno));
+			return EXIT_NO_RESOURCE;
+		}
 		pr_fail("%s: timer_create failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		return EXIT_FAILURE;
@@ -164,9 +172,9 @@ static int stress_hrtimer_process(const stress_args_t *args)
 
 	do {
 		shim_usleep(100000);
-	} while (stress_hrtimers_keep_stressing());
+	} while (stress_hrtimers_stress_continue());
 
-	add_counter_lock(args, lock, timer_counter);
+	stress_bogo_add_lock(args, lock, timer_counter);
 
 	if (timer_delete(timerid) < 0) {
 		pr_fail("%s: timer_delete failed, errno=%d (%s)\n",
@@ -179,12 +187,14 @@ static int stress_hrtimer_process(const stress_args_t *args)
 static int stress_hrtimers(const stress_args_t *args)
 {
 	pid_t pids[PROCS_MAX];
-	const pid_t parent = getpid();
 	size_t i;
         bool hrtimers_adjust = false;
 	double start_time = -1.0, end_time;
 	sigset_t mask;
 
+	if (stress_sigchld_set_handler(args) < 0)
+		return EXIT_NO_RESOURCE;
+
 	lock = stress_lock_create();
 	if (!lock) {
 		pr_inf("%s: cannot create lock, skipping stressor\n", args->name);
@@ -196,21 +206,20 @@ static int stress_hrtimers(const stress_args_t *args)
 	ns_delay = hrtimers_adjust ? 10000 : -1;
 	max_ops = args->max_ops / PROCS_MAX;
 
-	(void)memset(pids, 0, sizeof(pids));
+	(void)shim_memset(pids, 0, sizeof(pids));
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	for (i = 0; i < PROCS_MAX; i++) {
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			goto reap;
 
 		pids[i] = fork();
 		if (pids[i] == 0) {
 			/* Child */
 			stress_parent_died_alarm();
-			stress_set_oom_adjustment(args->name, true);
+			stress_set_oom_adjustment(args, true);
 			(void)sched_settings_apply(true);
 			stress_hrtimer_process(args);
-			(void)kill(parent, SIGALRM);
 			_exit(EXIT_SUCCESS);
 		}
 	}
@@ -222,30 +231,19 @@ static int stress_hrtimers(const stress_args_t *args)
 	start_time = stress_time_now();
 	do {
 		shim_usleep(100000);
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 reap:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
-	for (i = 0; i < PROCS_MAX; i++) {
-		if (pids[i] > 0)
-			(void)kill(pids[i], SIGALRM);
-	}
+	stress_kill_and_wait_many(args, pids, PROCS_MAX, SIGALRM, true);
 	end_time = stress_time_now();
 
-	for (i = 0; i < PROCS_MAX; i++) {
-		if (pids[i] > 0) {
-			int status;
-
-			VOID_RET(int, shim_waitpid(pids[i], &status, 0));
-		}
-	}
-
 	if (start_time >= 0.0) {
 		const double dt = end_time - start_time;
 
 		if (dt > 0.0) {
-			const double rate = (double)get_counter(args) / dt;
+			const double rate = (double)stress_bogo_get(args) / dt;
 
 			pr_dbg("%s: hrtimer signals at %.3f MHz\n", args->name, rate / 1000000.0);
 			stress_metrics_set(args, 0, "hrtimer signals per sec", rate);
@@ -268,6 +266,7 @@ stressor_info_t stress_hrtimers_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_SCHEDULER,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without librt or hrtimer support"
 };
