@@ -18,11 +18,19 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
+#include "core-out-of-memory.h"
 #include "io-uring.h"
 
 #if defined(HAVE_LINUX_IO_URING_H)
 #include <linux/io_uring.h>
 #endif
+#if defined(HAVE_SYS_XATTR_H)
+#include <sys/xattr.h>
+#undef HAVE_ATTR_XATTR_H
+#elif defined(HAVE_ATTR_XATTR_H)
+#include <attr/xattr.h>
+#endif
 
 #if !defined(O_DSYNC)
 #define O_DSYNC		(0)
@@ -31,7 +39,7 @@
 static const stress_help_t help[] = {
 	{ NULL,	"io-uring N",		"start N workers that issue io-uring I/O requests" },
 	{ NULL,	"io-uring-ops N",	"stop after N bogo io-uring I/O requests" },
-	{ NULL,	NULL,		NULL }
+	{ NULL,	NULL,			NULL }
 };
 
 #if defined(HAVE_LINUX_IO_URING_H) &&	\
@@ -53,16 +61,17 @@ static const stress_help_t help[] = {
      defined(HAVE_IORING_OP_CLOSE) ||	\
      defined(HAVE_IORING_OP_MADVISE) ||	\
      defined(HAVE_IORING_OP_STATX) || 	\
+     defined(HAVE_IORING_OP_SETXATTR) || \
      defined(HAVE_IORING_OP_GETXATTR) || \
      defined(HAVE_IORING_OP_SYNC_FILE_RANGE))
 
-
-
 /*
  *  io uring file info
  */
 typedef struct {
 	int fd;			/* file descriptor */
+	int fd_at;		/* file path descriptor */
+	int fd_dup;		/* file descriptor to dup */
 	char *filename;		/* filename */
 	struct iovec *iovecs;	/* iovecs array 1 per block to submit */
 	size_t iovecs_sz;	/* size of iovecs allocation */
@@ -107,9 +116,16 @@ typedef struct {
 	size_t sq_size;
 	size_t cq_size;
 	size_t sqes_size;
+	size_t sqes_entries;
 } stress_io_uring_submit_t;
 
-typedef void (*stress_io_uring_setup)(stress_io_uring_file_t *io_uring_file, struct io_uring_sqe *sqe);
+typedef struct {
+	size_t  index;
+	uint8_t opcode;
+	bool	supported;
+} stress_io_uring_user_data_t;
+
+typedef void (*stress_io_uring_setup)(const stress_io_uring_file_t *io_uring_file, struct io_uring_sqe *sqe, const void *extra);
 
 /*
  *  opcode to human readable name lookup
@@ -126,7 +142,7 @@ static const char *stress_io_uring_opcode_name(const uint8_t opcode);
  *  shim_io_uring_setup
  *	wrapper for io_uring_setup()
  */
-static int shim_io_uring_setup(unsigned entries, struct io_uring_params *p)
+static inline int shim_io_uring_setup(unsigned entries, struct io_uring_params *p)
 {
 	return (int)syscall(__NR_io_uring_setup, entries, p);
 }
@@ -135,7 +151,7 @@ static int shim_io_uring_setup(unsigned entries, struct io_uring_params *p)
  *  shim_io_uring_enter
  *	wrapper for io_uring_enter()
  */
-static int shim_io_uring_enter(
+static inline int shim_io_uring_enter(
 	int fd,
 	unsigned int to_submit,
 	unsigned int min_complete,
@@ -184,8 +200,14 @@ static int stress_setup_io_uring(
 	stress_uring_io_cq_ring_t *cring = &submit->cq_ring;
 	struct io_uring_params p;
 
-	(void)memset(&p, 0, sizeof(p));
-	submit->io_uring_fd = shim_io_uring_setup(256, &p);
+	(void)shim_memset(&p, 0, sizeof(p));
+	/*
+	 *  16 is plenty, with too many we end up with lots of cache
+	 *  misses, with too few we end up with ring filling. This
+	 *  seems to be a good fit with the set of requests being
+	 *  issue by this stressor
+	 */
+	submit->io_uring_fd = shim_io_uring_setup(16, &p);
 	if (submit->io_uring_fd < 0) {
 		if (errno == ENOSYS) {
 			pr_inf_skip("%s: io-uring not supported by the kernel, skipping stressor\n",
@@ -241,6 +263,7 @@ static int stress_setup_io_uring(
 	sring->flags = VOID_ADDR_OFFSET(submit->sq_mmap, p.sq_off.flags);
 	sring->array = VOID_ADDR_OFFSET(submit->sq_mmap, p.sq_off.array);
 
+	submit->sqes_entries = p.sq_entries;
 	submit->sqes_size = p.sq_entries * sizeof(struct io_uring_sqe);
 	submit->sqes_mmap = mmap(NULL, submit->sqes_size,
 			PROT_READ | PROT_WRITE, MAP_SHARED | MAP_POPULATE,
@@ -297,9 +320,7 @@ static void stress_close_io_uring(stress_io_uring_submit_t *submit)
  */
 static inline int stress_io_uring_complete(
 	const stress_args_t *args,
-	stress_io_uring_submit_t *submit,
-	const uint8_t opcode,
-	bool *supported)
+	stress_io_uring_submit_t *submit)
 {
 	stress_uring_io_cq_ring_t *cring = &submit->cq_ring;
 	struct io_uring_cqe *cqe;
@@ -307,38 +328,66 @@ static inline int stress_io_uring_complete(
 	int ret = EXIT_SUCCESS;
 
 	for (;;) {
-		shim_mb();
+		stress_asm_mb();
 
 		/* Empty? */
 		if (head == *cring->tail)
 			break;
 
 		cqe = &cring->cqes[head & *submit->cq_ring.ring_mask];
-		if ((cqe->res < 0)
-#ifdef HAVE_IORING_OP_FALLOCATE
-			&& (opcode != IORING_OP_FALLOCATE)
-#endif
-		) {
-			const int err = abs(cqe->res);
+		stress_io_uring_user_data_t *const user_data =
+			(stress_io_uring_user_data_t *)(uintptr_t)cqe->user_data;
+		if (cqe->res < 0) {
+			int err;
 
+			err = abs(cqe->res);
 			/* Silently ignore some completion errors */
 			if ((err == EOPNOTSUPP) || (err == ENOTDIR)) {
-				*supported = false;
+				user_data->supported = false;
 			} else  {
-				pr_fail("%s: completion opcode=%d (%s), error=%d (%s)\n",
-					args->name, opcode,
-					stress_io_uring_opcode_name(opcode),
+				switch (err) {
+				/* Silently ignore some errors */
+				case ENOSPC:
+				case EFBIG:
+					goto next_head;
+				case ENOENT:
+					if (user_data->opcode == IORING_OP_ASYNC_CANCEL)
+						goto next_head;
+					break;
+				case EINVAL:
+					if (user_data->opcode == IORING_OP_FALLOCATE)
+						goto next_head;
+					break;
+#if defined(HAVE_IORING_OP_GETXATTR) &&	\
+    defined(XATTR_CREATE)
+				case ENODATA:
+					if (user_data->opcode == IORING_OP_GETXATTR)
+						goto next_head;
+					break;
+#endif
+#if defined(HAVE_IORING_OP_SETXATTR) &&	\
+    defined(XATTR_CREATE)
+				case EEXIST:
+					if (user_data->opcode == IORING_OP_SETXATTR)
+						goto next_head;
+					break;
+#endif
+				default:
+					break;
+				}
+				pr_fail("%s: completion opcode 0x%2.2x (%s), error=%d (%s)\n",
+					args->name, user_data->opcode,
+					stress_io_uring_opcode_name(user_data->opcode),
 					err, strerror(err));
 				ret = EXIT_FAILURE;
 			}
 		}
+next_head:
 		head++;
 	}
 
 	*cring->head = head;
-	shim_mb();
-	if (ret == EXIT_SUCCESS)
-		inc_counter(args);
+	stress_asm_mb();
 
 	return ret;
 }
@@ -349,69 +398,141 @@ static inline int stress_io_uring_complete(
  */
 static int stress_io_uring_submit(
 	const stress_args_t *args,
-	stress_io_uring_setup setup_func,
-	stress_io_uring_file_t *io_uring_file,
+	const stress_io_uring_setup setup_func,
+	const stress_io_uring_file_t *io_uring_file,
 	stress_io_uring_submit_t *submit,
-	bool *supported)
+	stress_io_uring_user_data_t *user_data,
+	const void *extra_data)
 {
 	stress_uring_io_sq_ring_t *sring = &submit->sq_ring;
 	unsigned index = 0, tail = 0, next_tail = 0;
 	struct io_uring_sqe *sqe;
 	int ret;
-	uint8_t opcode;
 
 	next_tail = tail = *sring->tail;
 	next_tail++;
-	shim_mb();
+	stress_asm_mb();
 	index = tail & *submit->sq_ring.ring_mask;
 	sqe = &submit->sqes_mmap[index];
-	(void)memset(sqe, 0, sizeof(*sqe));
+	(void)shim_memset(sqe, 0, sizeof(*sqe));
 
-	setup_func(io_uring_file, sqe);
-	opcode = sqe->opcode;
+	setup_func(io_uring_file, sqe, extra_data);
+	/* Save opcode for later completion error reporting */
+	sqe->user_data = (uint64_t)(uintptr_t)user_data;
 
 	sring->array[index] = index;
 	tail = next_tail;
 	if (*sring->tail != tail) {
-		shim_mb();
+		stress_asm_mb();
 		*sring->tail = tail;
-		shim_mb();
-	}
-
-	ret = shim_io_uring_enter(submit->io_uring_fd, 1,
-		1, IORING_ENTER_GETEVENTS);
-	if (ret < 0) {
-		/* Silently ignore ENOSPC failures */
-		if (errno == ENOSPC)	
-			return EXIT_SUCCESS;
-		pr_fail("%s: io_uring_enter failed, opcode=%d (%s), errno=%d (%s)\n",
-			args->name, opcode,
-			stress_io_uring_opcode_name(opcode),
-			errno, strerror(errno));
-		if (errno == EOPNOTSUPP)
-			*supported = false;
+		stress_asm_mb();
+	} else {
 		return EXIT_FAILURE;
 	}
 
-	return stress_io_uring_complete(args, submit, opcode, supported);
+retry:
+	ret = shim_io_uring_enter(submit->io_uring_fd, 1,
+		1, IORING_ENTER_GETEVENTS);
+	if (ret < 0) {
+		if (errno == EBUSY){
+			stress_io_uring_complete(args, submit);
+			goto retry;
+		}
+		/* Silently ignore ENOSPC or cancel opcode failures */
+		if ((errno == ENOSPC) || (sqe->opcode == IORING_OP_ASYNC_CANCEL))
+			return EXIT_SUCCESS;
+		pr_fail("%s: io_uring_enter failed, opcode=%d (%s), errno=%d (%s)\n",
+			args->name, sqe->opcode,
+			stress_io_uring_opcode_name(sqe->opcode),
+			errno, strerror(errno));
+		if (errno == EOPNOTSUPP)
+			user_data->supported = false;
+		return EXIT_FAILURE;
+	}
+	stress_bogo_inc(args);
+	return EXIT_SUCCESS;
 }
 
+#if defined(HAVE_IORING_OP_ASYNC_CANCEL)
+/*
+ *  stress_io_uring_cancel_setup()
+ *	setup cancel submit over io_uring
+ */
+static void stress_io_uring_async_cancel_setup(
+	const stress_io_uring_file_t *io_uring_file,
+	struct io_uring_sqe *sqe,
+	const void *extra_info)
+{
+	(void)io_uring_file;
+
+	const struct io_uring_sqe *sqe_to_cancel =
+		(const struct io_uring_sqe *)extra_info;
+
+	sqe->fd = sqe_to_cancel->fd;
+	sqe->flags = 2;
+	sqe->opcode = IORING_OP_ASYNC_CANCEL;
+	sqe->addr = sqe_to_cancel->addr;
+	sqe->off = 0;
+	sqe->len = 0;
+	sqe->splice_fd_in = 0;
+}
+
+/*
+ *  stress_io_uring_cancel_rdwr()
+ *	try to cancel pending read/writes
+ */
+static void stress_io_uring_cancel_rdwr(
+	const stress_args_t *args,
+	const stress_io_uring_file_t *io_uring_file,
+	stress_io_uring_submit_t *submit)
+{
+	size_t i;
+	stress_io_uring_user_data_t user_data;
+
+	user_data.supported = true;
+	user_data.index = -1;
+	user_data.opcode = IORING_OP_ASYNC_CANCEL;
+
+	for (i = 0; i < submit->sqes_entries; i++) {
+		struct io_uring_sqe *sqe_to_cancel = &submit->sqes_mmap[i];
+
+		if (!sqe_to_cancel->addr)
+			continue;
+
+		switch (sqe_to_cancel->opcode) {
+		case IORING_OP_READ:
+		case IORING_OP_READV:
+		case IORING_OP_WRITE:
+		case IORING_OP_WRITEV:
+			(void)stress_io_uring_submit(args, stress_io_uring_async_cancel_setup,
+				io_uring_file, submit, &user_data, (void *)sqe_to_cancel);
+			break;
+		default:
+			break;
+		}
+	}
+	(void)stress_io_uring_complete(args, submit);
+}
+#endif
+
 #if defined(HAVE_IORING_OP_READV)
 /*
  *  stress_io_uring_readv_setup()
  *	setup readv submit over io_uring
  */
 static void stress_io_uring_readv_setup(
-	stress_io_uring_file_t *io_uring_file,
-	struct io_uring_sqe *sqe)
+	const stress_io_uring_file_t *io_uring_file,
+	struct io_uring_sqe *sqe,
+	const void *extra_info)
 {
+	(void)extra_info;
+
 	sqe->fd = io_uring_file->fd;
 	sqe->flags = 0;
 	sqe->opcode = IORING_OP_READV;
 	sqe->addr = (uintptr_t)io_uring_file->iovecs;
 	sqe->len = io_uring_file->blocks;
 	sqe->off = (uint64_t)stress_mwc8() * io_uring_file->blocks;
-	sqe->user_data = (uintptr_t)io_uring_file;
 }
 #endif
 
@@ -421,16 +542,18 @@ static void stress_io_uring_readv_setup(
  *	setup writev submit over io_uring
  */
 static void stress_io_uring_writev_setup(
-	stress_io_uring_file_t *io_uring_file,
-	struct io_uring_sqe *sqe)
+	const stress_io_uring_file_t *io_uring_file,
+	struct io_uring_sqe *sqe,
+	const void *extra_info)
 {
+	(void)extra_info;
+
 	sqe->fd = io_uring_file->fd;
 	sqe->flags = 0;
 	sqe->opcode = IORING_OP_WRITEV;
 	sqe->addr = (uintptr_t)io_uring_file->iovecs;
 	sqe->len = io_uring_file->blocks;
 	sqe->off = (uint64_t)stress_mwc8() * io_uring_file->blocks;
-	sqe->user_data = (uintptr_t)io_uring_file;
 }
 #endif
 
@@ -440,16 +563,18 @@ static void stress_io_uring_writev_setup(
  *	setup read submit over io_uring
  */
 static void stress_io_uring_read_setup(
-	stress_io_uring_file_t *io_uring_file,
-	struct io_uring_sqe *sqe)
+	const stress_io_uring_file_t *io_uring_file,
+	struct io_uring_sqe *sqe,
+	const void *extra_info)
 {
+	(void)extra_info;
+
 	sqe->fd = io_uring_file->fd;
 	sqe->flags = 0;
 	sqe->opcode = IORING_OP_READ;
 	sqe->addr = (uintptr_t)io_uring_file->iovecs[0].iov_base;
 	sqe->len = io_uring_file->iovecs[0].iov_len;
 	sqe->off = (uint64_t)stress_mwc8() * io_uring_file->blocks;
-	sqe->user_data = (uintptr_t)io_uring_file;
 }
 #endif
 
@@ -459,16 +584,18 @@ static void stress_io_uring_read_setup(
  *	setup write submit over io_uring
  */
 static void stress_io_uring_write_setup(
-	stress_io_uring_file_t *io_uring_file,
-	struct io_uring_sqe *sqe)
+	const stress_io_uring_file_t *io_uring_file,
+	struct io_uring_sqe *sqe,
+	const void *extra_info)
 {
+	(void)extra_info;
+
 	sqe->fd = io_uring_file->fd;
 	sqe->flags = 0;
 	sqe->opcode = IORING_OP_WRITE;
 	sqe->addr = (uintptr_t)io_uring_file->iovecs[0].iov_base;
 	sqe->len = io_uring_file->iovecs[0].iov_len;
 	sqe->off = (uint64_t)stress_mwc8() * io_uring_file->blocks;
-	sqe->user_data = (uintptr_t)io_uring_file;
 }
 #endif
 
@@ -478,14 +605,16 @@ static void stress_io_uring_write_setup(
  *	setup fsync submit over io_uring
  */
 static void stress_io_uring_fsync_setup(
-	stress_io_uring_file_t *io_uring_file,
-	struct io_uring_sqe *sqe)
+	const stress_io_uring_file_t *io_uring_file,
+	struct io_uring_sqe *sqe,
+	const void *extra_info)
 {
+	(void)extra_info;
+
 	sqe->fd = io_uring_file->fd;
 	sqe->opcode = IORING_OP_FSYNC;
 	sqe->len = 0;
 	sqe->off = 0;
-	sqe->user_data = (uintptr_t)io_uring_file;
 	sqe->ioprio = 0;
 	sqe->buf_index = 0;
 	sqe->rw_flags = 0;
@@ -498,10 +627,12 @@ static void stress_io_uring_fsync_setup(
  *	setup nop submit over io_uring
  */
 static void stress_io_uring_nop_setup(
-	stress_io_uring_file_t *io_uring_file,
-	struct io_uring_sqe *sqe)
+	const stress_io_uring_file_t *io_uring_file,
+	struct io_uring_sqe *sqe,
+	const void *extra_info)
 {
 	(void)io_uring_file;
+	(void)extra_info;
 
 	sqe->opcode = IORING_OP_NOP;
 }
@@ -513,9 +644,12 @@ static void stress_io_uring_nop_setup(
  *	setup fallocate submit over io_uring
  */
 static void stress_io_uring_fallocate_setup(
-	stress_io_uring_file_t *io_uring_file,
-	struct io_uring_sqe *sqe)
+	const stress_io_uring_file_t *io_uring_file,
+	struct io_uring_sqe *sqe,
+	const void *extra_info)
 {
+	(void)extra_info;
+
 	sqe->fd = io_uring_file->fd;
 	sqe->opcode = IORING_OP_FALLOCATE;
 	sqe->off = 0;			/* offset */
@@ -533,9 +667,12 @@ static void stress_io_uring_fallocate_setup(
  *	setup fadvise submit over io_uring
  */
 static void stress_io_uring_fadvise_setup(
-	stress_io_uring_file_t *io_uring_file,
-	struct io_uring_sqe *sqe)
+	const stress_io_uring_file_t *io_uring_file,
+	struct io_uring_sqe *sqe,
+	const void *extra_info)
 {
+	(void)extra_info;
+
 	sqe->fd = io_uring_file->fd;
 	sqe->opcode = IORING_OP_FADVISE;
 	sqe->off = 0;			/* offset */
@@ -557,13 +694,15 @@ static void stress_io_uring_fadvise_setup(
  *	setup close submit over io_uring
  */
 static void stress_io_uring_close_setup(
-	stress_io_uring_file_t *io_uring_file,
-	struct io_uring_sqe *sqe)
+	const stress_io_uring_file_t *io_uring_file,
+	struct io_uring_sqe *sqe,
+	const void *extra_info)
 {
 	(void)io_uring_file;
+	(void)extra_info;
 
 	/* don't worry about bad fd if dup fails */
-	sqe->fd = dup(fileno(stdin));
+	sqe->fd = dup(io_uring_file->fd_dup);
 	sqe->opcode = IORING_OP_CLOSE;
 	sqe->ioprio = 0;
 	sqe->off = 0;
@@ -580,9 +719,12 @@ static void stress_io_uring_close_setup(
  *	setup madvise submit over io_uring
  */
 static void stress_io_uring_madvise_setup(
-	stress_io_uring_file_t *io_uring_file,
-	struct io_uring_sqe *sqe)
+	const stress_io_uring_file_t *io_uring_file,
+	struct io_uring_sqe *sqe,
+	const void *extra_info)
 {
+	(void)extra_info;
+
 	sqe->fd = io_uring_file->fd;
 	sqe->opcode = IORING_OP_MADVISE;
 	sqe->addr = (uintptr_t)io_uring_file->iovecs[0].iov_base;
@@ -604,19 +746,29 @@ static void stress_io_uring_madvise_setup(
  *	setup statx submit over io_uring
  */
 static void stress_io_uring_statx_setup(
-	stress_io_uring_file_t *io_uring_file,
-	struct io_uring_sqe *sqe)
+	const stress_io_uring_file_t *io_uring_file,
+	struct io_uring_sqe *sqe,
+	const void *extra_info)
 {
-	static shim_statx_t statxbuf;
+	(void)io_uring_file;
+	(void)sqe;
+	(void)extra_info;
 
-	sqe->opcode = IORING_OP_STATX;
-	sqe->fd = io_uring_file->fd;
-	sqe->addr = (uintptr_t)io_uring_file->filename;
-	sqe->addr2 = (uintptr_t)&statxbuf;
-	sqe->statx_flags = AT_EMPTY_PATH;
-	sqe->ioprio = 0;
-	sqe->buf_index = 0;
-	sqe->flags = 0;
+#if defined(STATX_SIZE)
+	if (io_uring_file->fd_at >= 0) {
+		static shim_statx_t statxbuf;
+
+		sqe->opcode = IORING_OP_STATX;
+		sqe->fd = io_uring_file->fd_at;
+		sqe->addr = (uintptr_t)"";
+		sqe->addr2 = (uintptr_t)&statxbuf;
+		sqe->statx_flags = AT_EMPTY_PATH;
+		sqe->ioprio = 0;
+		sqe->buf_index = 0;
+		sqe->flags = 0;
+		sqe->len = STATX_SIZE;
+	}
+#endif
 }
 #endif
 
@@ -626,10 +778,12 @@ static void stress_io_uring_statx_setup(
  *	setup sync_file_range submit over io_uring
  */
 static void stress_io_uring_sync_file_range_setup(
-	stress_io_uring_file_t *io_uring_file,
-	struct io_uring_sqe *sqe)
+	const stress_io_uring_file_t *io_uring_file,
+	struct io_uring_sqe *sqe,
+	const void *extra_info)
 {
-	sqe->opcode = IORING_OP_SYNC_FILE_RANGE;
+	(void)extra_info;
+
 	sqe->fd = io_uring_file->fd;
 	sqe->off = stress_mwc16() & ~511UL;
 	sqe->len = stress_mwc32() & ~511UL;
@@ -640,39 +794,84 @@ static void stress_io_uring_sync_file_range_setup(
 }
 #endif
 
-#if defined(HAVE_IORING_OP_GETXATTR)
+#if defined(HAVE_IORING_OP_SETXATTR) &&	\
+    defined(XATTR_CREATE)
+/*
+ *  stress_io_uring_setxattr_setup()
+ *	setup setxattr submit over io_uring
+ */
+static void stress_io_uring_setxattr_setup(
+	const stress_io_uring_file_t *io_uring_file,
+	struct io_uring_sqe *sqe,
+	const void *extra_info)
+{
+	(void)io_uring_file;
+	(void)extra_info;
+
+	static char attr_value[] = "ioring-xattr-data";
+
+	sqe->opcode = IORING_OP_SETXATTR;
+	sqe->fd = 0;
+	sqe->off = (uintptr_t)attr_value;
+	sqe->len = sizeof(attr_value);
+	sqe->flags = 0;
+	sqe->addr = (uintptr_t)"user.var_test";
+	sqe->ioprio = 0;
+	sqe->rw_flags = 0;
+	sqe->buf_index = 0;
+	sqe->addr3 = (uintptr_t)io_uring_file->filename;
+        sqe->xattr_flags = XATTR_CREATE;
+}
+#endif
+
+
+#if defined(HAVE_IORING_OP_GETXATTR) &&	\
+    defined(XATTR_CREATE)
 /*
  *  stress_io_uring_getxattr_setup()
  *	setup getxattr submit over io_uring
  */
 static void stress_io_uring_getxattr_setup(
-	stress_io_uring_file_t *io_uring_file,
-	struct io_uring_sqe *sqe)
+	const stress_io_uring_file_t *io_uring_file,
+	struct io_uring_sqe *sqe,
+	const void *extra_info)
 {
-	char value[1024];
+	(void)io_uring_file;
+	(void)extra_info;
 
-	sqe->opcode = IORING_OP_STATX;
-	sqe->fd = io_uring_file->fd;
-	sqe->addr = (uintptr_t)io_uring_file->filename;
-	sqe->addr2 = (uintptr_t)value;
-	sqe->len = sizeof(value);
-	sqe->xattr_flags = 0;
-	sqe->ioprio = 0;
-	sqe->buf_index = 0;
+	static char attr_value[128];
+
+	sqe->opcode = IORING_OP_GETXATTR;
+	sqe->fd = 0;
+	sqe->off = (uintptr_t)attr_value;
+	sqe->len = sizeof(attr_value);
 	sqe->flags = 0;
+	sqe->addr = (uintptr_t)"user.var_test";
+	sqe->ioprio = 0;
+	sqe->rw_flags = 0;
+	sqe->buf_index = 0;
+	sqe->addr3 = (uintptr_t)io_uring_file->filename;
+        sqe->xattr_flags = 0;
 }
 #endif
 
 
+/*
+ *  We have some duplications here because we want to perform more than
+ *  one of these io-urion ops per round before we do a completion so we
+ *  can add more activity onto the ring for a bit more activity/stress.
+ */
 static const stress_io_uring_setup_info_t stress_io_uring_setups[] = {
 #if defined(HAVE_IORING_OP_READV)
 	{ IORING_OP_READV,	"IORING_OP_READV", 	stress_io_uring_readv_setup },
+	{ IORING_OP_READV,	"IORING_OP_READV", 	stress_io_uring_readv_setup },
 #endif
 #if defined(HAVE_IORING_OP_WRITEV)
 	{ IORING_OP_WRITEV,	"IORING_OP_WRITEV",	stress_io_uring_writev_setup },
 #endif
 #if defined(HAVE_IORING_OP_READ)
 	{ IORING_OP_READ,	"IORING_OP_READ",	stress_io_uring_read_setup },
+	{ IORING_OP_READ,	"IORING_OP_READ",	stress_io_uring_read_setup },
 #endif
 #if defined(HAVE_IORING_OP_WRITE)
 	{ IORING_OP_WRITE,	"IORING_OP_WRITE",	stress_io_uring_write_setup },
@@ -682,6 +881,9 @@ static const stress_io_uring_setup_info_t stress_io_uring_setups[] = {
 #endif
 #if defined(HAVE_IORING_OP_NOP)
 	{ IORING_OP_NOP,	"IORING_OP_NOP",	stress_io_uring_nop_setup },
+	{ IORING_OP_NOP,	"IORING_OP_NOP",	stress_io_uring_nop_setup },
+	{ IORING_OP_NOP,	"IORING_OP_NOP",	stress_io_uring_nop_setup },
+	{ IORING_OP_NOP,	"IORING_OP_NOP",	stress_io_uring_nop_setup },
 #endif
 #if defined(HAVE_IORING_OP_FALLOCATE)
 	{ IORING_OP_FALLOCATE,	"IORING_OP_FALLOCATE",	stress_io_uring_fallocate_setup },
@@ -697,12 +899,18 @@ static const stress_io_uring_setup_info_t stress_io_uring_setups[] = {
 #endif
 #if defined(HAVE_IORING_OP_STATX)
 	{ IORING_OP_STATX,	"IORING_OP_STATX",	stress_io_uring_statx_setup },
+	{ IORING_OP_STATX,	"IORING_OP_STATX",	stress_io_uring_statx_setup },
 #endif
 #if defined(HAVE_IORING_OP_SYNC_FILE_RANGE)
 	{ IORING_OP_SYNC_FILE_RANGE, "IORING_OP_SYNC_FILE_RANGE", stress_io_uring_sync_file_range_setup },
 #endif
-#if defined(HAVE_IORING_OP_GETXATTR)
-	{ IORING_OP_GETXATTR, "IORING_OP_GETXATTR",	stress_io_uring_getxattr_setup },
+#if defined(HAVE_IORING_OP_SETXATTR) &&	\
+    defined(XATTR_CREATE)
+	{ IORING_OP_SETXATTR,	"IORING_OP_SETXATTR",	stress_io_uring_setxattr_setup },
+#endif
+#if defined(HAVE_IORING_OP_GETXATTR) && \
+    defined(XATTR_CREATE)
+	{ IORING_OP_GETXATTR,	"IORING_OP_GETXATTR",	stress_io_uring_getxattr_setup },
 #endif
 };
 
@@ -721,12 +929,11 @@ static const char *stress_io_uring_opcode_name(const uint8_t opcode)
 	return "unknown";
 }
 
-
 /*
  *  stress_io_uring
  *	stress asynchronous I/O
  */
-static int stress_io_uring(const stress_args_t *args)
+static int stress_io_uring_child(const stress_args_t *args, void *context)
 {
 	int ret, rc;
 	char filename[PATH_MAX];
@@ -737,11 +944,14 @@ static int stress_io_uring(const stress_args_t *args)
 	off_t file_size = (off_t)blocks * block_size;
 	stress_io_uring_submit_t submit;
 	const pid_t self = getpid();
-	bool supported[SIZEOF_ARRAY(stress_io_uring_setups)];
+	stress_io_uring_user_data_t user_data[SIZEOF_ARRAY(stress_io_uring_setups)];
 
-	(void)memset(&submit, 0, sizeof(submit));
-	(void)memset(&io_uring_file, 0, sizeof(io_uring_file));
+	(void)context;
 
+	(void)shim_memset(&submit, 0, sizeof(submit));
+	(void)shim_memset(&io_uring_file, 0, sizeof(io_uring_file));
+
+	io_uring_file.fd_dup = fileno(stdin);
 	io_uring_file.file_size = file_size;
 	io_uring_file.blocks = blocks;
 	io_uring_file.block_size = block_size;
@@ -773,7 +983,7 @@ static int stress_io_uring(const stress_args_t *args)
 			stress_io_uring_unmap_iovecs(&io_uring_file);
 			return EXIT_NO_RESOURCE;
 		}
-		(void)memset(io_uring_file.iovecs[i].iov_base, stress_mwc8(), block_size);
+		(void)shim_memset(io_uring_file.iovecs[i].iov_base, stress_mwc8(), block_size);
 		file_size -= iov_length;
 	}
 
@@ -798,7 +1008,12 @@ static int stress_io_uring(const stress_args_t *args)
 			args->name, filename, errno, strerror(errno));
 		goto clean;
 	}
-	(void)shim_unlink(filename);
+#if defined(O_PATH)
+	io_uring_file.fd_at = open(filename, O_PATH);
+#else
+	io_uring_file.fd_at = -1;
+#endif
+	stress_file_rw_hint_short(io_uring_file.fd);
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
@@ -806,38 +1021,57 @@ static int stress_io_uring(const stress_args_t *args)
 	 *  Assume all opcodes are supported
 	 */
 	for (j = 0; j < SIZEOF_ARRAY(stress_io_uring_setups); j++) {
-		supported[j] = true;
+		user_data[j].supported = true;
+		user_data[j].index = j;
+		user_data[j].opcode = stress_io_uring_setups[j].opcode;
 	}
 
 	rc = EXIT_SUCCESS;
 	i = 0;
 	do {
-		for (j = 0; j < SIZEOF_ARRAY(stress_io_uring_setups); j++) {
-			if (supported[j]) {
+		for (j = 0; (j < SIZEOF_ARRAY(stress_io_uring_setups)) && stress_continue_flag(); j++) {
+			if (user_data[j].supported) {
 				rc = stress_io_uring_submit(args,
 					stress_io_uring_setups[j].setup_func,
-					&io_uring_file, &submit, &supported[j]);
-				if ((rc != EXIT_SUCCESS) || !keep_stressing(args))
+					&io_uring_file, &submit, &user_data[j], NULL);
+				if ((rc != EXIT_SUCCESS) || !stress_continue(args))
 					break;
 			}
 		}
+		stress_io_uring_complete(args, &submit);
 
-		if (i++ > 1024) {
+		if (i++ >= 4096) {
 			i = 0;
 			(void)stress_read_fdinfo(self, submit.io_uring_fd);
 		}
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
+	pr_dbg("%s: submits completed, closing uring and unlinking file\n", args->name);
+#if defined(HAVE_IORING_OP_ASYNC_CANCEL)
+	stress_io_uring_cancel_rdwr(args, &io_uring_file, &submit);
+#endif
 	(void)close(io_uring_file.fd);
 clean:
+	if (io_uring_file.fd_at >= 0)
+		(void)close(io_uring_file.fd_at);
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	stress_close_io_uring(&submit);
 	stress_io_uring_unmap_iovecs(&io_uring_file);
+	(void)shim_unlink(filename);
 	(void)stress_temp_dir_rm_args(args);
 	return rc;
 }
 
+/*
+ *  stress_io_uring
+ *      stress asynchronous I/O
+ */
+static int stress_io_uring(const stress_args_t *args)
+{
+	return stress_oomable_child(args, NULL, stress_io_uring_child, STRESS_OOMABLE_NORMAL);
+}
+
 stressor_info_t stress_io_uring_info = {
 	.stressor = stress_io_uring,
 	.class = CLASS_IO | CLASS_OS,
@@ -848,6 +1082,7 @@ stressor_info_t stress_io_uring_info = {
 stressor_info_t stress_io_uring_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_IO | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without linux/io_uring.h or syscall() support"
 };
