@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
 #include "core-io-priority.h"
 
 #if defined(HAVE_SYS_UIO_H)
@@ -82,33 +83,33 @@ static int stress_ioprio(const stress_args_t *args)
 		if (shim_ioprio_get(IOPRIO_WHO_PROCESS, args->pid) < 0) {
 			if (errno != EINVAL) {
 				pr_fail("%s: ioprio_get(OPRIO_WHO_PROCESS, %" PRIdMAX"), "
-					"errno = %d (%s)\n",
+					"errno=%d (%s)\n",
 					args->name, (intmax_t)args->pid, errno, strerror(errno));
 				goto cleanup_file;
 			}
 		}
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			break;
 		if (shim_ioprio_get(IOPRIO_WHO_PROCESS, 0) < 0) {
 			if (errno != EINVAL) {
 				pr_fail("%s: ioprio_get(OPRIO_WHO_PROCESS, 0), "
-					"errno = %d (%s)\n",
+					"errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 				goto cleanup_file;
 			}
 		}
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			break;
 #if defined(HAVE_GETPGRP)
 		if (shim_ioprio_get(IOPRIO_WHO_PGRP, grp) < 0) {
 			if (errno != EINVAL) {
 				pr_fail("%s: ioprio_get(OPRIO_WHO_PGRP, %" PRIdMAX "), "
-					"errno = %d (%s)\n",
+					"errno=%d (%s)\n",
 					args->name, (intmax_t)grp, errno, strerror(errno));
 				goto cleanup_file;
 			}
 		}
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			break;
 #else
 		UNEXPECTED
@@ -116,12 +117,12 @@ static int stress_ioprio(const stress_args_t *args)
 		if (shim_ioprio_get(IOPRIO_WHO_PGRP, 0) < 0) {
 			if (errno != EINVAL) {
 				pr_fail("%s: ioprio_get(OPRIO_WHO_PGRP, 0), "
-					"errno = %d (%s)\n",
+					"errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 				goto cleanup_file;
 			}
 		}
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			break;
 		/*
 		 *  Exercise invalid ioprio_get arguments
@@ -134,16 +135,16 @@ static int stress_ioprio(const stress_args_t *args)
 		if (shim_ioprio_get(IOPRIO_WHO_USER, (int)uid) < 0) {
 			if (errno != EINVAL) {
 				pr_fail("%s: ioprio_get(OPRIO_WHO_USR, %d), "
-					"errno = %d (%s)\n",
+					"errno=%d (%s)\n",
 					args->name, uid, errno, strerror(errno));
 				goto cleanup_file;
 			}
 		}
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			break;
 
 		for (i = 0; i < MAX_IOV; i++) {
-			(void)memset(buffer[i], stress_mwc8(), BUF_SIZE);
+			(void)shim_memset(buffer[i], stress_mwc8(), BUF_SIZE);
 			iov[i].iov_base = buffer[i];
 			iov[i].iov_len = BUF_SIZE;
 		}
@@ -155,10 +156,10 @@ static int stress_ioprio(const stress_args_t *args)
 				goto cleanup_file;
 			}
 		}
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			break;
 		(void)shim_fsync(fd);
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			break;
 
 		/*
@@ -175,13 +176,13 @@ static int stress_ioprio(const stress_args_t *args)
 				pr_fail("%s: ioprio_set("
 					"IOPRIO_WHO_PROCESS, %" PRIdMAX ", "
 					"(IOPRIO_CLASS_IDLE, 0)), "
-					"errno = %d (%s)\n",
+					"errno=%d (%s)\n",
 					args->name, (intmax_t)args->pid,
 					errno, strerror(errno));
 				goto cleanup_file;
 			}
 		}
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			break;
 
 		if (pwritev(fd, iov, MAX_IOV, (off_t)512 * stress_mwc16()) < 0) {
@@ -191,10 +192,10 @@ static int stress_ioprio(const stress_args_t *args)
 				goto cleanup_file;
 			}
 		}
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			break;
 		(void)shim_fsync(fd);
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			break;
 
 		for (i = 0; i < 8; i++) {
@@ -204,7 +205,7 @@ static int stress_ioprio(const stress_args_t *args)
 					pr_fail("%s: ioprio_set("
 						"IOPRIO_WHO_PROCESS, %" PRIdMAX ", "
 						"(IOPRIO_CLASS_BE, %d)), "
-						"errno = %d (%s)\n",
+						"errno=%d (%s)\n",
 						args->name, (intmax_t)args->pid,
 						i, errno, strerror(errno));
 					goto cleanup_file;
@@ -219,7 +220,7 @@ static int stress_ioprio(const stress_args_t *args)
 			}
 			(void)shim_fsync(fd);
 		}
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			break;
 		for (i = 0; i < 8; i++) {
 			if (shim_ioprio_set(IOPRIO_WHO_PROCESS, args->pid,
@@ -228,7 +229,7 @@ static int stress_ioprio(const stress_args_t *args)
 					pr_fail("%s: ioprio_set("
 						"IOPRIO_WHO_PROCESS, %" PRIdMAX ", "
 						"(IOPRIO_CLASS_RT, %d)), "
-						"errno = %d (%s)\n",
+						"errno=%d (%s)\n",
 						args->name, (intmax_t)args->pid,
 						i, errno, strerror(errno));
 					goto cleanup_file;
@@ -243,8 +244,8 @@ static int stress_ioprio(const stress_args_t *args)
 			}
 			(void)shim_fsync(fd);
 		}
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	rc = EXIT_SUCCESS;
 
@@ -268,6 +269,7 @@ stressor_info_t stress_ioprio_info = {
 stressor_info_t stress_ioprio_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without sys/uio.h, ioprio_get(), ioprio_set() or pwritev() support"
 };
