@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
 
 #define MIN_ITIMER_FREQ		(1)
 #define MAX_ITIMER_FREQ		(100000000)
@@ -25,6 +26,7 @@
 
 static const stress_help_t help[] = {
 	{ NULL,	"itimer N",	"start N workers exercising interval timers" },
+	{ NULL,	"itimer-freq F","set the itimer frequency, limited by jiffy clock rate" },
 	{ NULL,	"itimer-ops N",	"stop after N interval timer bogo operations" },
 	{ NULL,	"itimer-rand",	"enable random interval timer frequency" },
 	{ NULL, NULL,		NULL }
@@ -61,7 +63,7 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
 static volatile uint64_t itimer_counter = 0;
 static uint64_t max_ops;
 static double rate_us;
-static double start;
+static double time_end;
 
 static const shim_itimer_which_t stress_itimers[] = {
 #if defined(ITIMER_REAL)
@@ -105,12 +107,12 @@ static void stress_itimer_set(struct itimerval *timer)
 }
 
 /*
- *  stress_itimer_keep_stressing(args)
+ *  stress_itimer_stress_continue(args)
  *      returns true if we can keep on running a stressor
  */
-static bool HOT OPTIMIZE3 stress_itimer_keep_stressing(void)
+static bool HOT OPTIMIZE3 stress_itimer_stress_continue(void)
 {
-	return (LIKELY(keep_stressing_flag()) &&
+	return (LIKELY(stress_continue_flag()) &&
 		LIKELY(!max_ops || (itimer_counter < max_ops)));
 }
 
@@ -125,7 +127,7 @@ static void stress_itimer_handler(int sig)
 
 	(void)sig;
 
-	if (!stress_itimer_keep_stressing())
+	if (!stress_itimer_stress_continue())
 		goto cancel;
 	itimer_counter++;
 
@@ -134,17 +136,17 @@ static void stress_itimer_handler(int sig)
 			goto cancel;
 	/* High freq timer, check periodically for timeout */
 	if ((itimer_counter & 65535) == 0)
-		if ((stress_time_now() - start) > (double)g_opt_timeout)
+		if (stress_time_now() > time_end)
 			goto cancel;
-	if (keep_stressing_flag()) {
+	if (stress_continue_flag()) {
 		stress_itimer_set(&timer);
 		return;
 	}
 
 cancel:
-	keep_stressing_set_flag(false);
+	stress_continue_set_flag(false);
 	/* Cancel timer if we detect no more runs */
-	(void)memset(&timer, 0, sizeof(timer));
+	(void)shim_memset(&timer, 0, sizeof(timer));
 	(void)setitimer(ITIMER_PROF, &timer, NULL);
 }
 
@@ -158,12 +160,13 @@ static int stress_itimer(const stress_args_t *args)
 	sigset_t mask;
 	uint64_t itimer_freq = DEFAULT_ITIMER_FREQ;
 
+	time_end = args->time_end;
+
 	(void)sigemptyset(&mask);
 	(void)sigaddset(&mask, SIGINT);
 	(void)sigprocmask(SIG_SETMASK, &mask, NULL);
 
 	max_ops = args->max_ops;
-	start = stress_time_now();
 
 	if (!stress_get_setting("itimer-freq", &itimer_freq)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
@@ -201,8 +204,8 @@ static int stress_itimer(const stress_args_t *args)
 			(void)getitimer(stress_itimers[i], &t);
 		}
 
-		set_counter(args, itimer_counter);
-	} while (keep_stressing(args));
+		stress_bogo_set(args, itimer_counter);
+	} while (stress_continue(args));
 
 	if (itimer_counter == 0) {
 		pr_fail("%s: did not handle any itimer SIGPROF signals\n",
@@ -211,7 +214,7 @@ static int stress_itimer(const stress_args_t *args)
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
-	(void)memset(&timer, 0, sizeof(timer));
+	(void)shim_memset(&timer, 0, sizeof(timer));
 	(void)setitimer(ITIMER_PROF, &timer, NULL);
 	return EXIT_SUCCESS;
 }
