@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
 #include "core-capabilities.h"
 #include "core-net.h"
 
@@ -70,41 +71,47 @@ struct shim_kcmp_epoll_slot {
 do {									\
 	int rc = SHIM_KCMP(pid1, pid2, type, idx1, idx2);		\
 									\
-	if (rc < 0) {	 						\
+	if (UNLIKELY(rc < 0)) {	 					\
 		if (errno == EPERM) {					\
 			pr_inf("%s: %s", capfail, args->name);		\
 			goto reap;					\
 		}							\
-		if ((errno != EINVAL) && (errno != ENOSYS))		\
-			pr_fail("%s kcmp " # type " failed, "		\
+		if ((errno != EINVAL) &&				\
+		    (errno != ENOSYS) &&				\
+		    (errno != EBADF)) {					\
+			pr_fail("%s: kcmp " # type " failed, "		\
 				"errno=%d (%s)\n", args->name,		\
 				errno, strerror(errno));		\
+			ret = EXIT_FAILURE;				\
+		}							\
 	}								\
-	if (!keep_stressing_flag())					\
-		goto reap;						\
 } while (0)
 
 #define KCMP_VERIFY(pid1, pid2, type, idx1, idx2, res)			\
 do {									\
 	int rc = SHIM_KCMP(pid1, pid2, type, idx1, idx2);		\
 									\
-	if (rc != res) {						\
+	if (UNLIKELY(rc != res)) {					\
 		if (rc < 0) {						\
 			if (errno == EPERM) {				\
 				pr_inf("%s: %s", capfail, args->name);	\
 				goto reap;				\
 			}						\
-			if ((errno != EINVAL) && (errno != ENOSYS))	\
-				pr_fail("%s kcmp " # type " failed, "	\
+			if ((errno != EINVAL) &&			\
+			    (errno != ENOSYS) &&			\
+			    (errno != EBADF)) {				\
+				pr_fail("%s: kcmp " # type " failed, "	\
 					"errno=%d (%s)\n", args->name,	\
 					errno, strerror(errno));	\
+				ret = EXIT_FAILURE;			\
+			}						\
 		} else {						\
 			pr_fail( "%s: kcmp " # type			\
 			" returned %d, expected: %d\n",			\
 			args->name, rc, ret);				\
 		}							\
 	}								\
-	if (!keep_stressing_flag())					\
+	if (!stress_continue_flag())					\
 		goto reap;						\
 } while (0)
 
@@ -179,7 +186,7 @@ static int stress_kcmp(const stress_args_t *args)
 		goto again;
 	}
 
-	(void)memset(&ev, 0, sizeof(ev));
+	(void)shim_memset(&ev, 0, sizeof(ev));
 	ev.data.fd = efd;
 	ev.events = EPOLLIN | EPOLLET;
 	if (epoll_ctl(efd, EPOLL_CTL_ADD, sfd, &ev) < 0) {
@@ -194,10 +201,10 @@ static int stress_kcmp(const stress_args_t *args)
 again:
 	pid1 = fork();
 	if (pid1 < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
 		(void)close(fd1);
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			goto finish;
 		pr_fail("%s: fork failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
@@ -212,7 +219,7 @@ again:
 		(void)sched_settings_apply(true);
 
 		/* Child */
-		while (keep_stressing_flag())
+		while (stress_continue_flag())
 			(void)pause();
 
 		/* will never get here */
@@ -228,6 +235,7 @@ again:
 	} else {
 		/* Parent */
 		int fd2, status, pid2;
+		const bool verify = !!(g_opt_flags & OPT_FLAGS_VERIFY);
 
 		pid2 = getpid();
 		if ((fd2 = open("/dev/null", O_WRONLY)) < 0) {
@@ -269,7 +277,7 @@ again:
 
 #if defined(HAVE_SYS_EPOLL_H) &&	\
     NEED_GLIBC(2,3,2)
-			if (efd != -1) {
+			if ((sfd != -1) && (efd != -1)) {
 				struct shim_kcmp_epoll_slot slot;
 
 				slot.efd = (uint32_t)efd;
@@ -282,7 +290,7 @@ again:
 #endif
 
 			/* Same simple checks */
-			if (g_opt_flags & OPT_FLAGS_VERIFY) {
+			if (verify) {
 				KCMP_VERIFY(pid1, pid1, SHIM_KCMP_FILE, fd1, fd1, 0);
 				KCMP_VERIFY(pid1, pid1, SHIM_KCMP_FILES, 0, 0, 0);
 				KCMP_VERIFY(pid1, pid1, SHIM_KCMP_FS, 0, 0, 0);
@@ -293,7 +301,7 @@ again:
 				KCMP_VERIFY(pid1, pid2, SHIM_KCMP_SYSVSEM, 0, 0, 0);
 #if defined(HAVE_SYS_EPOLL_H) &&	\
     NEED_GLIBC(2,3,2)
-				if (efd != -1) {
+				if ((sfd != -1) && (efd != -1)) {
 					struct shim_kcmp_epoll_slot slot;
 
 					slot.efd = (uint32_t)efd;
@@ -314,12 +322,12 @@ again:
 			if (!is_root)
 				(void)SHIM_KCMP(1, pid2, SHIM_KCMP_SIGHAND, 0, 0);
 
-			inc_counter(args);
-		} while (keep_stressing(args));
+			stress_bogo_inc(args);
+		} while (stress_continue(args));
 reap:
 		if (fd2 >= 0)
 			(void)close(fd2);
-		(void)kill(pid1, SIGKILL);
+		(void)shim_kill(pid1, SIGKILL);
 		(void)shim_waitpid(pid1, &status, 0);
 		(void)close(fd1);
 	}
@@ -346,6 +354,7 @@ stressor_info_t stress_kcmp_info = {
 stressor_info_t stress_kcmp_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_OS,
+	.verify = VERIFY_OPTIONAL,
 	.help = help,
 	.unimplemented_reason = "built without kcmp() system call support"
 };
