@@ -167,8 +167,8 @@ static int stress_klog(const stress_args_t *args)
 		/* unusual length */
 		(void)shim_klogctl(SYSLOG_ACTION_READ_ALL, buffer, 0);
 
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
@@ -187,6 +187,7 @@ stressor_info_t stress_klog_info = {
 stressor_info_t stress_klog_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without syslog() system call or klogctl()"
 };
