@@ -18,6 +18,7 @@
  */
 #include "stress-ng.h"
 #include "core-arch.h"
+#include "core-builtin.h"
 
 #if defined(HAVE_LINUX_KVM_H)
 #include <linux/kvm.h>
@@ -105,6 +106,11 @@ static int stress_kvm(const stress_args_t *args)
 		if (vm_fd < 0) {
 			if (errno == EINTR)
 				goto tidy_kvm_fd;
+			if (errno == EBUSY) {
+				pr_inf_skip("%s: KVM device busy, skipping stressor\n", args->name);
+				(void)close(kvm_fd);
+				return EXIT_NO_RESOURCE;
+			}
 			pr_fail("%s: ioctl KVM_CREATE_VM failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			goto tidy_kvm_fd;
@@ -115,7 +121,7 @@ static int stress_kvm(const stress_args_t *args)
 		if (vm_mem == MAP_FAILED)
 			goto tidy_vm_fd;
 
-		(void)memset(&kvm_mem, 0, sizeof(kvm_mem));
+		(void)shim_memset(&kvm_mem, 0, sizeof(kvm_mem));
 		kvm_mem.slot = 0;
 		kvm_mem.guest_phys_addr = 0;
 		kvm_mem.memory_size = vm_mem_size;
@@ -139,7 +145,7 @@ static int stress_kvm(const stress_args_t *args)
 			goto tidy_vm_mmap;
 		}
 
-		(void)memcpy(vm_mem, &kvm_x86_kernel, sizeof(kvm_x86_kernel));
+		(void)shim_memcpy(vm_mem, &kvm_x86_kernel, sizeof(kvm_x86_kernel));
 		if (ioctl(vcpu_fd, KVM_GET_SREGS, &sregs) < 0) {
 			if (errno == EINTR)
 				goto tidy_vcpu_fd;
@@ -167,7 +173,7 @@ static int stress_kvm(const stress_args_t *args)
 			goto tidy_vcpu_fd;
 		}
 
-		(void)memset(&regs, 0, sizeof(regs));
+		(void)shim_memset(&regs, 0, sizeof(regs));
 		regs.rflags = 2;
 		regs.rip = 0;
 		if (ioctl(vcpu_fd, KVM_SET_REGS, &regs) < 0) {
@@ -195,7 +201,7 @@ static int stress_kvm(const stress_args_t *args)
 			goto tidy_vcpu_fd;
 		}
 
-		for (i = 0; i < 1000 && keep_stressing(args); i++) {
+		for (i = 0; i < 1000 && stress_continue(args); i++) {
 			uint8_t *port;
 
 			ret = ioctl(vcpu_fd, KVM_RUN, 0);
@@ -269,8 +275,8 @@ tidy_vm_fd:
 tidy_kvm_fd:
 		(void)close(kvm_fd);
 		if (run_ok)
-			inc_counter(args);
-	} while (keep_stressing(args));
+			stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
@@ -287,6 +293,7 @@ stressor_info_t stress_kvm_info = {
 stressor_info_t stress_kvm_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_DEV | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built on non-x86-64 without linux/kvm.h"
 };
