@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
 
 #define MIN_LEASE_BREAKERS	(1)
 #define MAX_LEASE_BREAKERS	(64)
@@ -89,7 +90,7 @@ static pid_t stress_lease_spawn(
 again:
 	pid = fork();
 	if (pid < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
 		return -1;
 	}
@@ -115,7 +116,7 @@ again:
 			}
 			(void)stress_get_lease(fd);
 			(void)close(fd);
-		} while (keep_stressing(args));
+		} while (stress_continue(args));
 
 		stress_set_proc_state(args->name, STRESS_STATE_WAIT);
 		_exit(EXIT_SUCCESS);
@@ -149,24 +150,24 @@ static int stress_try_lease(
 	 *  attempt a lease lock
 	 */
 	while (fcntl(fd, F_SETLEASE, lock) < 0) {
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			goto tidy;
 	}
 	(void)stress_get_lease(fd);
 
-	inc_counter(args);
+	stress_bogo_inc(args);
 	(void)shim_sched_yield();
 
 	/*
 	 *  attempt a lease unlock
 	 */
 	while (fcntl(fd, F_SETLEASE, F_UNLCK) < 0) {
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 		if (errno != EAGAIN) {
 			pr_fail("%s: fcntl failed: errno=%d: (%s)%s\n",
 				args->name, errno, strerror(errno),
-				stress_fs_type(filename));
+				stress_get_fs_type(filename));
 			break;
 		}
 	}
@@ -195,7 +196,7 @@ static int stress_lease(const stress_args_t *args)
 			lease_breakers = MIN_LEASE_BREAKERS;
 	}
 
-	(void)memset(l_pids, 0, sizeof(l_pids));
+	(void)shim_memset(l_pids, 0, sizeof(l_pids));
 
 	if (stress_sighandler(args->name, SIGIO, stress_lease_handler, NULL) < 0)
 		return EXIT_FAILURE;
@@ -237,7 +238,7 @@ static int stress_lease(const stress_args_t *args)
 		ret = stress_try_lease(args, filename, O_RDONLY, F_RDLCK);
 		if (ret != EXIT_SUCCESS)
 			break;
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 	t2 = stress_time_now();
 
 reap:
@@ -245,7 +246,7 @@ reap:
 
 	for (i = 0; i < lease_breakers; i++) {
 		if (l_pids[i]) {
-			(void)kill(l_pids[i], SIGKILL);
+			(void)shim_kill(l_pids[i], SIGKILL);
 			(void)shim_waitpid(l_pids[i], &status, 0);
 		}
 	}
@@ -275,6 +276,7 @@ stressor_info_t stress_lease_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without fcntl() F_SETLEASE, F_WRLCK or F_UNLCK commands"
 };
