@@ -18,6 +18,8 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
+#include "core-mounts.h"
 
 #if defined(HAVE_LIBGEN_H)
 #include <libgen.h>
@@ -95,7 +97,7 @@ static int stress_link_generic(
 	bool symlink_func = (linkfunc == symlink);
 	char *mnts[MOUNTS_MAX];
 
-	(void)memset(tmp_newpath, 0, sizeof(tmp_newpath));
+	(void)shim_memset(tmp_newpath, 0, sizeof(tmp_newpath));
 	(void)snprintf(tmp_newpath, sizeof(tmp_newpath),
 		"/tmp/stress-ng-%s-%d-%" PRIu64 "-link",
 		args->name, (int)getpid(), stress_mwc64());
@@ -141,7 +143,7 @@ static int stress_link_generic(
 		char testpath[PATH_MAX];
 		ssize_t rret;
 
-		for (i = 0; keep_stressing(args) && (i < n); i++) {
+		for (i = 0; stress_continue(args) && (i < n); i++) {
 			char newpath[PATH_MAX];
 			struct stat stbuf;
 
@@ -166,7 +168,7 @@ static int stress_link_generic(
 				rc = stress_exit_status(errno);
 				pr_fail("%s: %s failed, errno=%d (%s)%s\n",
 					args->name, funcname, errno, strerror(errno),
-					stress_fs_type(oldpath));
+					stress_get_fs_type(oldpath));
 				n = i;
 				break;
 			}
@@ -180,9 +182,9 @@ static int stress_link_generic(
 					char tmpdir[PATH_MAX], *dir;
 					int dir_fd;
 
-					shim_strlcpy(tmpfilename, newpath, sizeof(tmpfilename));
+					(void)shim_strlcpy(tmpfilename, newpath, sizeof(tmpfilename));
 					filename = basename(tmpfilename);
-					shim_strlcpy(tmpdir, newpath, sizeof(tmpdir));
+					(void)shim_strlcpy(tmpdir, newpath, sizeof(tmpdir));
 					dir = dirname(tmpdir);
 
 					/*
@@ -194,7 +196,7 @@ static int stress_link_generic(
 						if ((rret < 0) && (errno != ENOSYS)) {
 							pr_fail("%s: readlinkat failed, errno=%d (%s)%s\n",
 							args->name, errno, strerror(errno),
-							stress_fs_type(filename));
+							stress_get_fs_type(filename));
 						}
 						(void)close(dir_fd);
 					}
@@ -206,7 +208,7 @@ static int stress_link_generic(
 					rc = stress_exit_status(errno);
 					pr_fail("%s: readlink failed, errno=%d (%s)%s\n",
 						args->name, errno, strerror(errno),
-						stress_fs_type(newpath));
+						stress_get_fs_type(newpath));
 				} else {
 					buf[rret] = '\0';
 					if ((size_t)rret != oldpathlen)
@@ -222,7 +224,7 @@ static int stress_link_generic(
 				if (mounts_max > 0) {
 					/* Try hard link on differet random mount point */
 					const size_t idx = random_mount(mounts_max);
-					
+
 					ret = linkfunc(mnts[idx], tmp_newpath);
 					if (ret == 0)
 						(void)shim_unlink(tmp_newpath);
@@ -232,7 +234,7 @@ static int stress_link_generic(
 				rc = stress_exit_status(errno);
 				pr_fail("%s: lstat failed, errno=%d (%s)%s\n",
 					args->name, errno, strerror(errno),
-					stress_fs_type(newpath));
+					stress_get_fs_type(newpath));
 			}
 #if defined(O_DIRECTORY)
 			if (temp_dir_fd > 0)
@@ -262,6 +264,9 @@ static int stress_link_generic(
 #endif
 
 err_unlink:
+		/* time to finish, indicate so while the slow unlink occurs */
+		if (!stress_continue(args))
+			stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 #if defined(O_DIRECTORY)
 		if (temp_dir_fd > 0)
 			fsync(temp_dir_fd);
@@ -271,9 +276,8 @@ err_unlink:
 		if (temp_dir_fd > 0)
 			fsync(temp_dir_fd);
 #endif
-
-		inc_counter(args);
-	} while ((rc == EXIT_SUCCESS) && keep_stressing(args));
+		stress_bogo_inc(args);
+	} while ((rc == EXIT_SUCCESS) && stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
@@ -327,6 +331,7 @@ stressor_info_t stress_link_info = {
 stressor_info_t stress_link_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.opt_set_funcs = opt_set_funcs,
 	.help = hardlink_help,
 	.unimplemented_reason = "unsupported on Haiku"
