@@ -31,8 +31,8 @@
 struct list_entry;
 
 typedef void (*stress_list_func)(const stress_args_t *args,
-				 const size_t n,
-				 struct list_entry *data,
+				 struct list_entry *entries,
+				 const struct list_entry *entries_end,
 				 stress_metrics_t *metrics);
 
 typedef struct {
@@ -170,41 +170,38 @@ static void MLOCKED_TEXT stress_list_handler(int signum)
 
 static void OPTIMIZE3 stress_list_slistt(
 	const stress_args_t *args,
-	const size_t n,
-	struct list_entry *data,
+	struct list_entry *entries,
+	const struct list_entry *entries_end,
 	stress_metrics_t *metrics)
 {
-	size_t i;
 	register struct list_entry *entry, *head, *tail;
 	bool found = false;
 	double t;
 
-	entry = data;
-	head = entry;
-	tail = entry;
-	entry++;
-	for (i = 1; i < n; i++, entry++) {
+	head = entries;
+	tail = entries;
+	for (entry = entries + 1; entry < entries_end; entry++) {
 		tail->u.next = entry;
 		tail = entry;
 	}
 
 	t = stress_time_now();
-	for (entry = head, i = 0; i < n; i++, entry++) {
+	for (entry = head; entry < entries_end; entry++) {
 		register struct list_entry *find;
 
 		for (find = head; find; find = find->u.next) {
-			if (find == entry) {
+			if (UNLIKELY(find == entry)) {
 				found = true;
 				break;
 			}
 		}
 
-		if (!found)
+		if (UNLIKELY(!found))
 			pr_fail("%s: slistt entry #%zd not found\n",
-				args->name, i);
+				args->name, entry - entries);
 	}
 	metrics->duration += stress_time_now() - t;
-	metrics->count += (double)i;
+	metrics->count += (double)(entry - entries);
 
 	while (head) {
 		register struct list_entry *next = head->u.next;
@@ -215,41 +212,41 @@ static void OPTIMIZE3 stress_list_slistt(
 }
 
 #if defined(HAVE_SYS_QUEUE_LIST)
-static void stress_list_list(
+static void OPTIMIZE3 stress_list_list(
 	const stress_args_t *args,
-	const size_t n,
-	struct list_entry *data,
+	struct list_entry *entries,
+	const struct list_entry *entries_end,
 	stress_metrics_t *metrics)
 {
-	size_t i;
-	struct list_entry *entry;
+	register struct list_entry *entry;
 	struct listhead head;
 	bool found = false;
 	double t;
 
+	(void)shim_memset(&head, 0, sizeof(head));
 	LIST_INIT(&head);
 
-	for (entry = data, i = 0; i < n; i++, entry++) {
+	for (entry = entries; entry < entries_end; entry++) {
 		LIST_INSERT_HEAD(&head, entry, u.list_entries);
 	}
 
 	t = stress_time_now();
-	for (entry = data, i = 0; i < n; i++, entry++) {
-		struct list_entry *find;
+	for (entry = entries; entry < entries_end; entry++) {
+		register struct list_entry *find;
 
 		LIST_FOREACH(find, &head, u.list_entries) {
-			if (find == entry) {
+			if (UNLIKELY(find == entry)) {
 				found = true;
 				break;
 			}
 		}
 
-		if (!found)
+		if (UNLIKELY(!found))
 			pr_fail("%s: list entry #%zd not found\n",
-				args->name, i);
+				args->name, entry - entries);
 	}
 	metrics->duration += stress_time_now() - t;
-	metrics->count += (double)i;
+	metrics->count += (double)(entry - entries);
 
 	while (!LIST_EMPTY(&head)) {
 		entry = (struct list_entry *)LIST_FIRST(&head);
@@ -260,41 +257,41 @@ static void stress_list_list(
 #endif
 
 #if defined(HAVE_SYS_QUEUE_SLIST)
-static void stress_list_slist(
+static void OPTIMIZE3 stress_list_slist(
 	const stress_args_t *args,
-	const size_t n,
-	struct list_entry *data,
+	struct list_entry *entries,
+	const struct list_entry *entries_end,
 	stress_metrics_t *metrics)
 {
-	size_t i;
-	struct list_entry *entry;
+	register struct list_entry *entry;
 	struct slisthead head;
 	bool found = false;
 	double t;
 
+	(void)shim_memset(&head, 0, sizeof(head));
 	SLIST_INIT(&head);
 
-	for (entry = data, i = 0; i < n; i++, entry++) {
+	for (entry = entries; entry < entries_end; entry++) {
 		SLIST_INSERT_HEAD(&head, entry, u.slist_entries);
 	}
 
 	t = stress_time_now();
-	for (entry = data, i = 0; i < n; i++, entry++) {
-		struct list_entry *find;
+	for (entry = entries; entry < entries_end; entry++) {
+		register struct list_entry *find;
 
 		SLIST_FOREACH(find, &head, u.slist_entries) {
-			if (find == entry) {
+			if (UNLIKELY(find == entry)) {
 				found = true;
 				break;
 			}
 		}
 
-		if (!found)
+		if (UNLIKELY(!found))
 			pr_fail("%s: slist entry #%zd not found\n",
-				args->name, i);
+				args->name, entry - entries);
 	}
 	metrics->duration += stress_time_now() - t;
-	metrics->count += (double)i;
+	metrics->count += (double)(entry - entries);
 
 	while (!SLIST_EMPTY(&head)) {
 		SLIST_REMOVE_HEAD(&head, u.slist_entries);
@@ -304,41 +301,41 @@ static void stress_list_slist(
 #endif
 
 #if defined(HAVE_SYS_QUEUE_CIRCLEQ)
-static void stress_list_circleq(
+static void OPTIMIZE3 stress_list_circleq(
 	const stress_args_t *args,
-	const size_t n,
-	struct list_entry *data,
+	struct list_entry *entries,
+	const struct list_entry *entries_end,
 	stress_metrics_t *metrics)
 {
-	size_t i;
-	struct list_entry *entry;
+	register struct list_entry *entry;
 	struct circleqhead head;
 	bool found = false;
 	double t;
 
+	(void)shim_memset(&head, 0, sizeof(head));
 	CIRCLEQ_INIT(&head);
 
-	for (entry = data, i = 0; i < n; i++, entry++) {
+	for (entry = entries; entry < entries_end; entry++) {
 		CIRCLEQ_INSERT_TAIL(&head, entry, u.circleq_entries);
 	}
 
 	t = stress_time_now();
-	for (entry = data, i = 0; i < n; i++, entry++) {
-		struct list_entry *find;
+	for (entry = entries; entry < entries_end; entry++) {
+		register struct list_entry *find;
 
 		CIRCLEQ_FOREACH(find, &head, u.circleq_entries) {
-			if (find == entry) {
+			if (UNLIKELY(find == entry)) {
 				found = true;
 				break;
 			}
 		}
 
-		if (!found)
+		if (UNLIKELY(!found))
 			pr_fail("%s: circleq entry #%zd not found\n",
-				args->name, i);
+				args->name, entry - entries);
 	}
 	metrics->duration += stress_time_now() - t;
-	metrics->count += (double)i;
+	metrics->count += (double)(entry - entries);
 
 	while ((entry = (struct list_entry *)CIRCLEQ_FIRST(&head)) != (struct list_entry *)&head) {
 		CIRCLEQ_REMOVE(&head, entry, u.circleq_entries);
@@ -348,41 +345,41 @@ static void stress_list_circleq(
 #endif
 
 #if defined(HAVE_SYS_QUEUE_STAILQ)
-static void stress_list_stailq(
+static void OPTIMIZE3 stress_list_stailq(
 	const stress_args_t *args,
-	const size_t n,
-	struct list_entry *data,
+	struct list_entry *entries,
+	const struct list_entry *entries_end,
 	stress_metrics_t *metrics)
 {
-	size_t i;
-	struct list_entry *entry;
+	register struct list_entry *entry;
 	struct stailhead head;
 	bool found = false;
 	double t;
 
+	(void)shim_memset(&head, 0, sizeof(head));
 	STAILQ_INIT(&head);
 
-	for (entry = data, i = 0; i < n; i++, entry++) {
+	for (entry = entries; entry < entries_end; entry++) {
 		STAILQ_INSERT_TAIL(&head, entry, u.stailq_entries);
 	}
 
 	t = stress_time_now();
-	for (entry = data, i = 0; i < n; i++, entry++) {
-		struct list_entry *find;
+	for (entry = entries; entry < entries_end; entry++) {
+		register struct list_entry *find;
 
 		STAILQ_FOREACH(find, &head, u.stailq_entries) {
-			if (find == entry) {
+			if (UNLIKELY(find == entry)) {
 				found = true;
 				break;
 			}
 		}
 
-		if (!found)
+		if (UNLIKELY(!found))
 			pr_fail("%s: stailq entry #%zd not found\n",
-				args->name, i);
+				args->name, entry - entries);
 	}
 	metrics->duration += stress_time_now() - t;
-	metrics->count += (double)i;
+	metrics->count += (double)(entry - entries);
 
 	while ((entry = (struct list_entry *)STAILQ_FIRST(&head)) != NULL) {
 		STAILQ_REMOVE(&head, entry, list_entry, u.stailq_entries);
@@ -392,30 +389,30 @@ static void stress_list_stailq(
 #endif
 
 #if defined(HAVE_SYS_QUEUE_TAILQ)
-static void stress_list_tailq(
+static void OPTIMIZE3 stress_list_tailq(
 	const stress_args_t *args,
-	const size_t n,
-	struct list_entry *data,
+	struct list_entry *entries,
+	const struct list_entry *entries_end,
 	stress_metrics_t *metrics)
 {
-	size_t i;
-	struct list_entry *entry;
+	register struct list_entry *entry;
 	struct tailhead head;
 	bool found = false;
 	double t;
 
+	(void)shim_memset(&head, 0, sizeof(head));
 	TAILQ_INIT(&head);
 
-	for (entry = data, i = 0; i < n; i++, entry++) {
+	for (entry = entries; entry < entries_end; entry++) {
 		TAILQ_INSERT_TAIL(&head, entry, u.tailq_entries);
 	}
 
 	t = stress_time_now();
-	for (entry = data, i = 0; i < n; i++, entry++) {
-		struct list_entry *find;
+	for (entry = entries; entry < entries_end; entry++) {
+		register struct list_entry *find;
 
 		TAILQ_FOREACH(find, &head, u.tailq_entries) {
-			if (find == entry) {
+			if (UNLIKELY(find == entry)) {
 				found = true;
 				break;
 			}
@@ -423,10 +420,10 @@ static void stress_list_tailq(
 
 		if (!found)
 			pr_fail("%s: tailq entry #%zd not found\n",
-				args->name, i);
+				args->name, entry - entries);
 	}
 	metrics->duration += stress_time_now() - t;
-	metrics->count += (double)i;
+	metrics->count += (double)(entry - entries);
 
 	while ((entry = (struct list_entry *)TAILQ_FIRST(&head)) != NULL) {
 		TAILQ_REMOVE(&head, entry, u.tailq_entries);
@@ -437,8 +434,8 @@ static void stress_list_tailq(
 
 static void stress_list_all(
 	const stress_args_t *args,
-	const size_t n,
-	struct list_entry *data,
+	struct list_entry *entries,
+	const struct list_entry *entries_end,
 	stress_metrics_t *metrics);
 
 
@@ -467,13 +464,13 @@ static const stress_list_method_info_t list_methods[] = {
 
 static void stress_list_all(
 	const stress_args_t *args,
-	const size_t n,
-	struct list_entry *data,
+	struct list_entry *entries,
+	const struct list_entry *entries_end,
 	stress_metrics_t *metrics)
 {
 	static size_t index = 1;
 
-	list_methods[index].func(args, n, data, &metrics[index]);
+	list_methods[index].func(args, entries, entries_end, &metrics[index]);
 	index++;
 	if (index >= SIZEOF_ARRAY(list_methods))
 		index = 1;
@@ -516,7 +513,7 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
 static int stress_list(const stress_args_t *args)
 {
 	uint64_t v, list_size = DEFAULT_LIST_SIZE;
-	struct list_entry *entries, *entry;
+	struct list_entry *entries, *entry, *entries_end;
 	size_t n, i, j, bit, list_method = 0;
 	struct sigaction old_action;
 	int ret;
@@ -546,6 +543,7 @@ static int stress_list(const stress_args_t *args)
 			"out of memory, skipping stressor\n", args->name, n);
 		return EXIT_NO_RESOURCE;
 	}
+	entries_end = entries + n;
 
 	ret = sigsetjmp(jmp_env, 1);
 	if (ret) {
@@ -561,7 +559,7 @@ static int stress_list(const stress_args_t *args)
 	}
 
 	v = 0;
-	for (entry = entries, i = 0, bit = 0; i < n; i++, entry++) {
+	for (entry = entries, bit = 0; entry < entries_end; entry++) {
 		if (!bit) {
 			v = stress_mwc64();
 			bit = 1;
@@ -578,17 +576,17 @@ static int stress_list(const stress_args_t *args)
 	do {
 		uint64_t rnd;
 
-		func(args, n, entries, metrics);
+		func(args, entries, entries_end, metrics);
 
 		rnd = stress_mwc64();
-		for (entry = entries, i = 0; i < n; i++, entry++) {
+		for (entry = entries; entry < entries_end; entry++) {
 			register uint64_t value = entry->value ^ rnd;
 
 			entry->value = shim_ror64(value);
 		}
 
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	do_jmp = false;
 	(void)stress_sigrestore(args->name, SIGALRM, &old_action);
