@@ -37,9 +37,14 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
 	{ 0,			NULL },
 };
 
-#if (((defined(__GNUC__) || defined(__clang__)) && 	\
+#if (((defined(HAVE_COMPILER_GCC) ||			\
+       defined(HAVE_COMPILER_CLANG) ||			\
+       defined(HAVE_COMPILER_ICC) ||			\
+       defined(HAVE_COMPILER_ICX) ||			\
+       defined(HAVE_COMPILER_TCC) ||			\
+       defined(HAVE_COMPILER_PCC)) &&			\
        defined(STRESS_ARCH_X86)) ||			\
-     (defined(__GNUC__) && 				\
+     (defined(HAVE_COMPILER_GCC) && 			\
       (defined(HAVE_ATOMIC_ADD_FETCH) ||		\
        defined(HAVE_ATOMIC_FETCH_ADD)) &&		\
       defined(__ATOMIC_SEQ_CST) &&			\
@@ -68,9 +73,9 @@ do {							\
 #else
 #define MEM_LOCK(ptr, inc)				\
 do {							\
-	asm volatile("lock addl %1,%0" :		\
-		     "+m" (*ptr) :			\
-		     "ir" (inc));			\
+	__asm__ __volatile__("lock addl %1,%0" :	\
+			     "+m" (*ptr) :		\
+			     "ir" (inc));		\
 } while (0)
 #endif
 
@@ -145,6 +150,8 @@ static int stress_lockbus(const stress_args_t *args)
 {
 	uint32_t *buffer;
 	int flags = MAP_ANONYMOUS | MAP_SHARED;
+	double t, rate;
+	NOCLOBBER double duration, count;
 #if defined(STRESS_ARCH_X86)
 	uint32_t *splitlock_ptr1, *splitlock_ptr2;
 	bool lockbus_nosplit = false;
@@ -175,11 +182,13 @@ static int stress_lockbus(const stress_args_t *args)
 	if (args->instance == 0)
 		pr_dbg("%s: splitlocks %s\n", args->name,
 			do_splitlock ? "enabled" : "disabled");
-	if (sigsetjmp(jmp_env, 1) && !keep_stressing(args))
+	if (sigsetjmp(jmp_env, 1) && !stress_continue(args))
 		goto done;
 #endif
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
+	duration = 0;
+	count = 0;
 	do {
 		uint32_t *ptr0 = buffer + (stress_mwc32modn(BUFFER_SIZE - CHUNK_SIZE) >> 2);
 #if defined(STRESS_ARCH_X86)
@@ -191,6 +200,7 @@ static int stress_lockbus(const stress_args_t *args)
 #endif
 		const uint32_t inc = 1;
 
+		t = stress_time_now();
 		MEM_LOCK_AND_INCx8(ptr0, inc);
 		MEM_LOCKx8(ptr1);
 		MEM_LOCKx8(ptr2);
@@ -222,15 +232,23 @@ static int stress_lockbus(const stress_args_t *args)
 			SYNC_BOOL_COMPARE_AND_SWAP(ptr2, zero, val);
 		}
 #endif
-
-		inc_counter(args);
-	} while (keep_stressing(args));
+		duration += (stress_time_now() - t);
+#if defined(HAVE_SYNC_BOOL_COMPARE_AND_SWAP)
+		count += (8.0 * 12.0) + 6.0;
+#else
+		count += (8.0 * 12.0);
+#endif
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 #if defined(STRESS_ARCH_X86)
 done:
 #endif
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
+	rate = (count > 0.0) ? duration / count : 0.0;
+	stress_metrics_set(args, 0, "nanosecs per memory lock operation", rate * STRESS_DBL_NANOSECOND);
+
 	(void)munmap((void *)buffer, BUFFER_SIZE);
 
 	return EXIT_SUCCESS;
