@@ -18,6 +18,8 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
+#include "core-pragma.h"
 #include "core-put.h"
 #include "core-target-clones.h"
 
@@ -35,14 +37,14 @@ static const stress_help_t help[] = {
 };
 
 #if defined(HAVE_VLA_ARG) &&	\
-    !defined(__PCC__)
+    !defined(HAVE_COMPILER_PCC)
 
 typedef float 	stress_matrix_3d_type_t;
 
 /*
  *  the matrix stress test has different classes of matrix stressor
  */
-typedef void (*stress_matrix_3d_func)(
+typedef void (*stress_matrix_3d_func_t)(
 	const size_t n,
 	stress_matrix_3d_type_t a[RESTRICT n][n][n],
 	stress_matrix_3d_type_t b[RESTRICT n][n][n],
@@ -50,9 +52,12 @@ typedef void (*stress_matrix_3d_func)(
 
 typedef struct {
 	const char			*name;		/* human readable form of stressor */
-	const stress_matrix_3d_func	func[2];	/* method functions, x by y by z, z by y by x */
+	const stress_matrix_3d_func_t	func[2];	/* method functions, x by y by z, z by y by x */
 } stress_matrix_3d_method_info_t;
 
+static const char *current_method = NULL;		/* current matrix method */
+static size_t method_all_index;				/* all method index */
+
 static const stress_matrix_3d_method_info_t matrix_3d_methods[];
 
 static int stress_set_matrix_3d_size(const char *opt)
@@ -92,12 +97,11 @@ static void OPTIMIZE3 TARGET_CLONES stress_matrix_3d_xyz_add(
 		for (j = 0; j < n; j++) {
 			register size_t k;
 
+PRAGMA_UNROLL_N(8)
 			for (k = 0; k < n; k++) {
 				r[i][j][k] = a[i][j][k] + b[i][j][k];
 			}
 		}
-		if (UNLIKELY(!keep_stressing_flag()))
-			return;
 	}
 }
 
@@ -123,8 +127,6 @@ static void OPTIMIZE3 TARGET_CLONES stress_matrix_3d_zyx_add(
 				r[i][j][k] = a[i][j][k] + b[i][j][k];
 			}
 		}
-		if (UNLIKELY(!keep_stressing_flag()))
-			return;
 	}
 }
 
@@ -146,12 +148,11 @@ static void OPTIMIZE3 TARGET_CLONES stress_matrix_3d_xyz_sub(
 		for (j = 0; j < n; j++) {
 			register size_t k;
 
+PRAGMA_UNROLL_N(8)
 			for (k = 0; k < n; k++) {
 				r[i][j][k] = a[i][j][k] - b[i][j][k];
 			}
 		}
-		if (UNLIKELY(!keep_stressing_flag()))
-			return;
 	}
 }
 
@@ -177,8 +178,6 @@ static void OPTIMIZE3 TARGET_CLONES stress_matrix_3d_zyx_sub(
 				r[i][j][k] = a[i][j][k] + b[i][j][k];
 			}
 		}
-		if (UNLIKELY(!keep_stressing_flag()))
-			return;
 	}
 }
 
@@ -206,8 +205,6 @@ static void OPTIMIZE3 TARGET_CLONES stress_matrix_3d_xyz_trans(
 				r[i][j][k] = a[k][j][i];
 			}
 		}
-		if (UNLIKELY(!keep_stressing_flag()))
-			return;
 	}
 }
 
@@ -235,8 +232,6 @@ static void OPTIMIZE3 TARGET_CLONES stress_matrix_3d_zyx_trans(
 				r[i][j][k] = a[k][j][i];
 			}
 		}
-		if (UNLIKELY(!keep_stressing_flag()))
-			return;
 	}
 }
 
@@ -261,12 +256,11 @@ static void OPTIMIZE3 TARGET_CLONES stress_matrix_3d_xyz_mult(
 		for (j = 0; j < n; j++) {
 			register size_t k;
 
+PRAGMA_UNROLL_N(8)
 			for (k = 0; k < n; k++) {
 				r[i][j][k] = v * a[i][j][k];
 			}
 		}
-		if (UNLIKELY(!keep_stressing_flag()))
-			return;
 	}
 }
 
@@ -295,8 +289,6 @@ static void OPTIMIZE3 TARGET_CLONES stress_matrix_3d_zyx_mult(
 				r[i][j][k] = v * a[i][j][k];
 			}
 		}
-		if (UNLIKELY(!keep_stressing_flag()))
-			return;
 	}
 }
 
@@ -321,12 +313,11 @@ static void OPTIMIZE3 TARGET_CLONES stress_matrix_3d_xyz_div(
 		for (j = 0; j < n; j++) {
 			register size_t k;
 
+PRAGMA_UNROLL_N(8)
 			for (k = 0; k < n; k++) {
 				r[i][j][k] = a[i][j][k] / v;
 			}
 		}
-		if (UNLIKELY(!keep_stressing_flag()))
-			return;
 	}
 }
 
@@ -355,8 +346,6 @@ static void OPTIMIZE3 TARGET_CLONES stress_matrix_3d_zyx_div(
 				r[i][j][k] = a[i][j][k] / v;
 			}
 		}
-		if (UNLIKELY(!keep_stressing_flag()))
-			return;
 	}
 }
 
@@ -379,12 +368,11 @@ static void OPTIMIZE3 TARGET_CLONES stress_matrix_3d_xyz_hadamard(
 		for (j = 0; j < n; j++) {
 			register size_t k;
 
+PRAGMA_UNROLL_N(8)
 			for (k = 0; k < n; k++) {
 				r[i][j][k] = a[i][j][k] * b[i][j][k];
 			}
 		}
-		if (UNLIKELY(!keep_stressing_flag()))
-			return;
 	}
 }
 
@@ -411,8 +399,6 @@ static void OPTIMIZE3 TARGET_CLONES stress_matrix_3d_zyx_hadamard(
 				r[i][j][k] = a[i][j][k] * b[i][j][k];
 			}
 		}
-		if (UNLIKELY(!keep_stressing_flag()))
-			return;
 	}
 }
 
@@ -438,12 +424,11 @@ static void OPTIMIZE3 TARGET_CLONES stress_matrix_3d_xyz_frobenius(
 		for (j = 0; j < n; j++) {
 			register size_t k;
 
+PRAGMA_UNROLL_N(8)
 			for (k = 0; k < n; k++) {
 				sum += a[i][j][k] * b[i][j][k];
 			}
 		}
-		if (UNLIKELY(!keep_stressing_flag()))
-			return;
 	}
 	stress_float_put((float)sum);
 }
@@ -474,8 +459,6 @@ static void OPTIMIZE3 TARGET_CLONES stress_matrix_3d_zyx_frobenius(
 				sum += a[i][j][k] * b[i][j][k];
 			}
 		}
-		if (UNLIKELY(!keep_stressing_flag()))
-			return;
 	}
 	stress_float_put((float)sum);
 }
@@ -500,12 +483,11 @@ static void OPTIMIZE3 TARGET_CLONES stress_matrix_3d_xyz_copy(
 		for (j = 0; j < n; j++) {
 			register size_t k;
 
+			/* unrolling does not help, the following turns into a memcpy */
 			for (k = 0; k < n; k++) {
 				r[i][j][k] = a[i][j][k];
 			}
 		}
-		if (UNLIKELY(!keep_stressing_flag()))
-			return;
 	}
 }
 
@@ -529,12 +511,11 @@ static void OPTIMIZE3 TARGET_CLONES stress_matrix_3d_zyx_copy(
 		for (j = 0; j < n; j++) {
 			register size_t i;
 
+			/* unrolling does not help, the following turns into a memcpy */
 			for (i = 0; i < n; i++) {
 				r[i][j][k] = a[i][j][k];
 			}
 		}
-		if (UNLIKELY(!keep_stressing_flag()))
-			return;
 	}
 }
 
@@ -556,12 +537,11 @@ static void OPTIMIZE3 TARGET_CLONES stress_matrix_3d_xyz_mean(
 		for (j = 0; j < n; j++) {
 			register size_t k;
 
+PRAGMA_UNROLL_N(8)
 			for (k = 0; k < n; k++) {
 				r[i][j][k] = (a[i][j][k] + b[i][j][k]) / (stress_matrix_3d_type_t)2.0;
 			}
 		}
-		if (UNLIKELY(!keep_stressing_flag()))
-			return;
 	}
 }
 
@@ -587,8 +567,6 @@ static void OPTIMIZE3 TARGET_CLONES stress_matrix_3d_zyx_mean(
 				r[i][j][k] = (a[i][j][k] + b[i][j][k]) / (stress_matrix_3d_type_t)2.0;
 			}
 		}
-		if (UNLIKELY(!keep_stressing_flag()))
-			return;
 	}
 }
 
@@ -610,15 +588,15 @@ static void OPTIMIZE3 TARGET_CLONES stress_matrix_3d_xyz_zero(
 	for (i = 0; i < n; i++) {
 		register size_t j;
 
+PRAGMA_UNROLL_N(8)
 		for (j = 0; j < n; j++) {
 			register size_t k;
 
+			/* unrolling does not help, the following turns into a memzero */
 			for (k = 0; k < n; k++) {
 				r[i][j][k] = 0.0;
 			}
 		}
-		if (UNLIKELY(!keep_stressing_flag()))
-			return;
 	}
 }
 
@@ -647,8 +625,6 @@ static void OPTIMIZE3 TARGET_CLONES stress_matrix_3d_zyx_zero(
 				r[i][j][k] = 0.0;
 			}
 		}
-		if (UNLIKELY(!keep_stressing_flag()))
-			return;
 	}
 }
 
@@ -673,12 +649,11 @@ static void OPTIMIZE3 TARGET_CLONES stress_matrix_3d_xyz_negate(
 		for (j = 0; j < n; j++) {
 			register size_t k;
 
+PRAGMA_UNROLL_N(8)
 			for (k = 0; k < n; k++) {
 				r[i][j][k] = -a[i][j][k];
 			}
 		}
-		if (UNLIKELY(!keep_stressing_flag()))
-			return;
 	}
 }
 
@@ -707,8 +682,6 @@ static void OPTIMIZE3 TARGET_CLONES stress_matrix_3d_zyx_negate(
 				r[i][j][k] = -a[i][j][k];
 			}
 		}
-		if (UNLIKELY(!keep_stressing_flag()))
-			return;
 	}
 }
 
@@ -733,12 +706,11 @@ static void OPTIMIZE3 TARGET_CLONES stress_matrix_3d_xyz_identity(
 		for (j = 0; j < n; j++) {
 			register size_t k;
 
+			/* loop unrolling does not improve the loop */
 			for (k = 0; k < n; k++) {
 				r[i][j][k] = ((i == j) && (j == k)) ? 1.0 : 0.0;
 			}
 		}
-		if (UNLIKELY(!keep_stressing_flag()))
-			return;
 	}
 }
 
@@ -767,8 +739,6 @@ static void OPTIMIZE3 TARGET_CLONES stress_matrix_3d_zyx_identity(
 				r[i][j][k] = ((i == j) && (j == k)) ? 1.0 : 0.0;
 			}
 		}
-		if (UNLIKELY(!keep_stressing_flag()))
-			return;
 	}
 }
 
@@ -785,7 +755,7 @@ static void stress_matrix_3d_zyx_all(
 	stress_matrix_3d_type_t r[RESTRICT n][n][n]);
 
 /*
- * Table of cpu stress methods, ordered x by y by z and z by y by x
+ * Table of matrix_3 stress methods, ordered x by y by z and z by y by x
  */
 static const stress_matrix_3d_method_info_t matrix_3d_methods[] = {
 	{ "all",		{ stress_matrix_3d_xyz_all,		stress_matrix_3d_zyx_all } },/* Special "all" test */
@@ -808,7 +778,7 @@ static stress_metrics_t matrix_3d_metrics[SIZEOF_ARRAY(matrix_3d_methods)];
 
 /*
  *  stress_matrix_3d_all()
- *	iterate over all cpu stressors
+ *	iterate over all matrix_3 stressors
  */
 static void OPTIMIZE3 stress_matrix_3d_xyz_all(
 	const size_t n,
@@ -816,21 +786,19 @@ static void OPTIMIZE3 stress_matrix_3d_xyz_all(
 	stress_matrix_3d_type_t b[RESTRICT n][n][n],
 	stress_matrix_3d_type_t r[RESTRICT n][n][n])
 {
-	static size_t i = 1;	/* Skip over stress_matrix_3d_all */
 	double t;
 
+	current_method = matrix_3d_methods[method_all_index].name;
+
 	t = stress_time_now();
-	matrix_3d_methods[i].func[0](n, a, b, r);
-	matrix_3d_metrics[i].duration += stress_time_now() - t;
-	matrix_3d_metrics[i].count += 1.0;
-	i++;
-	if (i >= SIZEOF_ARRAY(matrix_3d_methods))
-		i = 1;
+	matrix_3d_methods[method_all_index].func[0](n, a, b, r);
+	matrix_3d_metrics[method_all_index].duration += stress_time_now() - t;
+	matrix_3d_metrics[method_all_index].count += 1.0;
 }
 
 /*
  *  stress_matrix_3d_all()
- *	iterate over all cpu stressors
+ *	iterate over all matrix_3d stressors
  */
 static void OPTIMIZE3 stress_matrix_3d_zyx_all(
 	const size_t n,
@@ -838,18 +806,15 @@ static void OPTIMIZE3 stress_matrix_3d_zyx_all(
 	stress_matrix_3d_type_t b[RESTRICT n][n][n],
 	stress_matrix_3d_type_t r[RESTRICT n][n][n])
 {
-	static size_t i = 1;	/* Skip over stress_matrix_3d_all */
 	double t;
 
-	t = stress_time_now();
-	matrix_3d_methods[i].func[1](n, a, b, r);
-	matrix_3d_metrics[i].duration += stress_time_now() - t;
-	matrix_3d_metrics[i].count += 1.0;
-	i++;
-	if (i >= SIZEOF_ARRAY(matrix_3d_methods))
-		i = 1;
-}
+	current_method = matrix_3d_methods[method_all_index].name;
 
+	t = stress_time_now();
+	matrix_3d_methods[method_all_index].func[1](n, a, b, r);
+	matrix_3d_metrics[method_all_index].duration += stress_time_now() - t;
+	matrix_3d_metrics[method_all_index].count += 1.0;
+}
 
 /*
  *  stress_set_matrix_3d_method()
@@ -900,11 +865,13 @@ static inline int stress_matrix_3d_exercise(
 {
 	int ret = EXIT_NO_RESOURCE;
 	typedef stress_matrix_3d_type_t (*matrix_3d_ptr_t)[n][n];
-	size_t matrix_3d_size = round_up(args->page_size, (sizeof(stress_matrix_3d_type_t) * n * n * n));
+	size_t matrix_3d_size = sizeof(stress_matrix_3d_type_t) * n * n * n;
+	size_t matrix_3d_mmap_size = round_up(args->page_size, matrix_3d_size);
 	const size_t num_matrix_3d_methods = SIZEOF_ARRAY(matrix_3d_methods);
-	const stress_matrix_3d_func func = matrix_3d_methods[matrix_3d_method].func[matrix_3d_zyx];
+	const stress_matrix_3d_func_t func = matrix_3d_methods[matrix_3d_method].func[matrix_3d_zyx];
+	const bool verify = !!(g_opt_flags & OPT_FLAGS_VERIFY);
 
-	matrix_3d_ptr_t a, b = NULL, r = NULL;
+	matrix_3d_ptr_t a, b = NULL, r = NULL, s = NULL;
 	register size_t i, j;
 	const stress_matrix_3d_type_t v = 65535 / (stress_matrix_3d_type_t)((uint64_t)~0);
 	int flags = MAP_PRIVATE | MAP_ANONYMOUS;
@@ -912,29 +879,40 @@ static inline int stress_matrix_3d_exercise(
 	flags |= MAP_POPULATE;
 #endif
 
+	method_all_index = 1;
+	current_method = matrix_3d_methods[matrix_3d_method].name;
+
 	for (i = 0; i < num_matrix_3d_methods; i++) {
 		matrix_3d_metrics[i].duration = 0.0;
 		matrix_3d_metrics[i].count = 0.0;
 	}
 
-	a = (matrix_3d_ptr_t)mmap(NULL, matrix_3d_size,
+	a = (matrix_3d_ptr_t)mmap(NULL, matrix_3d_mmap_size,
 		PROT_READ | PROT_WRITE, flags, -1, 0);
 	if (a == MAP_FAILED) {
 		pr_fail("%s: matrix allocation failed, out of memory\n", args->name);
 		goto tidy_ret;
 	}
-	b = (matrix_3d_ptr_t)mmap(NULL, matrix_3d_size,
+	b = (matrix_3d_ptr_t)mmap(NULL, matrix_3d_mmap_size,
 		PROT_READ | PROT_WRITE, flags, -1, 0);
 	if (b == MAP_FAILED) {
 		pr_fail("%s: matrix allocation failed, out of memory\n", args->name);
 		goto tidy_a;
 	}
-	r = (matrix_3d_ptr_t)mmap(NULL, matrix_3d_size,
+	r = (matrix_3d_ptr_t)mmap(NULL, matrix_3d_mmap_size,
 		PROT_READ | PROT_WRITE, flags, -1, 0);
 	if (r == MAP_FAILED) {
 		pr_fail("%s: matrix allocation failed, out of memory\n", args->name);
 		goto tidy_b;
 	}
+	if (verify) {
+		s = (matrix_3d_ptr_t)mmap(NULL, matrix_3d_mmap_size,
+			PROT_READ | PROT_WRITE, flags, -1, 0);
+		if (s == MAP_FAILED) {
+			pr_fail("%s: matrix allocation failed, out of memory\n", args->name);
+			goto tidy_r;
+		}
+	}
 
 	/*
 	 *  Initialise matrices
@@ -961,8 +939,26 @@ static inline int stress_matrix_3d_exercise(
 		(void)func(n, a, b, r);
 		matrix_3d_metrics[matrix_3d_method].duration += stress_time_now() - t;
 		matrix_3d_metrics[matrix_3d_method].count += 1.0;
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+
+		if (verify) {
+			t = stress_time_now();
+			(void)func(n, a, b, s);
+			matrix_3d_metrics[matrix_3d_method].duration += stress_time_now() - t;
+			matrix_3d_metrics[matrix_3d_method].count += 1.0;
+			stress_bogo_inc(args);
+
+			if (shim_memcmp(r, s, matrix_3d_size)) {
+				pr_fail("%s: %s: data difference between identical matrix-3d computations\n",
+					args->name, current_method);
+			}
+		}
+		if (matrix_3d_method == 0) {
+			method_all_index++;
+			if (method_all_index >= SIZEOF_ARRAY(matrix_3d_methods))
+				method_all_index = 1;
+		}
+	} while (stress_continue(args));
 
 	/* Dump metrics except for 'all' method */
 	for (i = 1, j = 0; i < num_matrix_3d_methods; i++) {
@@ -977,20 +973,23 @@ static inline int stress_matrix_3d_exercise(
 	}
 
 	ret = EXIT_SUCCESS;
-	(void)munmap((void *)r, matrix_3d_size);
+	if (verify)
+		(void)munmap((void *)s, matrix_3d_mmap_size);
+tidy_r:
+	(void)munmap((void *)r, matrix_3d_mmap_size);
 tidy_b:
-	(void)munmap((void *)b, matrix_3d_size);
+	(void)munmap((void *)b, matrix_3d_mmap_size);
 tidy_a:
-	(void)munmap((void *)a, matrix_3d_size);
+	(void)munmap((void *)a, matrix_3d_mmap_size);
 tidy_ret:
 	return ret;
 }
 
 /*
- *  stress_matrix()
+ *  stress_matrix_3d()
  *	stress CPU by doing floating point math ops
  */
-static int stress_matrix(const stress_args_t *args)
+static int stress_matrix_3d(const stress_args_t *args)
 {
 	size_t matrix_3d_method = 0; 	/* All method */
 	size_t matrix_3d_size = DEFAULT_MATRIX3D_SIZE;
@@ -1033,16 +1032,18 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
 };
 
 stressor_info_t stress_matrix_3d_info = {
-	.stressor = stress_matrix,
+	.stressor = stress_matrix_3d,
 	.set_default = stress_matrix_3d_set_default,
 	.class = CLASS_CPU | CLASS_CPU_CACHE | CLASS_MEMORY,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
 #else
 stressor_info_t stress_matrix_3d_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_CPU | CLASS_CPU_CACHE | CLASS_MEMORY,
+	.verify = VERIFY_OPTIONAL,
 	.help = help,
 	.unimplemented_reason = "compiler does not support variable length array function arguments"
 };
