@@ -262,37 +262,37 @@ static inline HOT OPTIMIZE3 void stress_memory_contend(const stress_pthread_args
 #endif
 	for (i = 0; i < 1024; i++) {
 		vdata0[0] = (uint64_t)i;
-		shim_mb();
+		stress_asm_mb();
 		vdata1[0] = (uint64_t)i;
-		shim_mb();
+		stress_asm_mb();
 		vdata0[1] = (uint64_t)i;
-		shim_mb();
+		stress_asm_mb();
 		vdata1[1] = (uint64_t)i;
-		shim_mb();
+		stress_asm_mb();
 		vdata0[2] = (uint64_t)i;
-		shim_mb();
+		stress_asm_mb();
 		vdata1[2] = (uint64_t)i;
-		shim_mb();
+		stress_asm_mb();
 		vdata0[3] = (uint64_t)i;
-		shim_mb();
+		stress_asm_mb();
 		vdata1[3] = (uint64_t)i;
-		shim_mb();
+		stress_asm_mb();
 		vdata0[4] = (uint64_t)i;
-		shim_mb();
+		stress_asm_mb();
 		vdata1[4] = (uint64_t)i;
-		shim_mb();
+		stress_asm_mb();
 		vdata0[5] = (uint64_t)i;
-		shim_mb();
+		stress_asm_mb();
 		vdata1[5] = (uint64_t)i;
-		shim_mb();
+		stress_asm_mb();
 		vdata0[6] = (uint64_t)i;
-		shim_mb();
+		stress_asm_mb();
 		vdata1[6] = (uint64_t)i;
-		shim_mb();
+		stress_asm_mb();
 		vdata0[7] = (uint64_t)i;
-		shim_mb();
+		stress_asm_mb();
 		vdata1[7] = (uint64_t)i;
-		shim_mb();
+		stress_asm_mb();
 		read64(data0);
 		read64(data1);
 	}
