@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
 #include "core-pthread.h"
 
 #if defined(HAVE_LINUX_MEMBARRIER_H)
@@ -73,6 +74,7 @@ static int stress_membarrier_exercise(const stress_args_t *args, membarrier_info
 {
 	int ret;
 	unsigned int i, mask;
+	double t;
 
 	ret = shim_membarrier(MEMBARRIER_CMD_QUERY, 0, 0);
 	if (ret < 0) {
@@ -81,28 +83,30 @@ static int stress_membarrier_exercise(const stress_args_t *args, membarrier_info
 		return -1;
 	}
 	mask = (unsigned int)ret;
+
+	t = stress_time_now();
 	for (i = 1; i; i <<= 1) {
 		if (i & mask) {
-			double t;
 
-			t = stress_time_now();
 			VOID_RET(int, shim_membarrier((int)i, 0, 0));
-			info->duration += stress_time_now() - t;
 			info->count += 1.0;
 
+#if defined(MEMBARRIER_CMD_FLAG_CPU)
+			/* Exercise MEMBARRIER_CMD_FLAG_CPU flag */
+			VOID_RET(int, shim_membarrier((int)i, MEMBARRIER_CMD_FLAG_CPU, 0));
+			info->count += 1.0;
+#endif
+		}
+	}
+	info->duration += stress_time_now() - t;
+
+	for (i = 1; i; i <<= 1) {
+		if (i & mask) {
 			/* Exercise illegal flags */
 			VOID_RET(int, shim_membarrier((int)i, ~0, 0));
 
 			/* Exercise illegal cpu_id */
 			VOID_RET(int, shim_membarrier((int)i, 0, INT_MAX));
-
-#if defined(MEMBARRIER_CMD_FLAG_CPU)
-			/* Exercise MEMBARRIER_CMD_FLAG_CPU flag */
-			t = stress_time_now();
-			VOID_RET(int, shim_membarrier((int)i, MEMBARRIER_CMD_FLAG_CPU, 0));
-			info->duration += stress_time_now() - t;
-			info->count += 1.0;
-#endif
 		}
 	}
 
@@ -129,7 +133,7 @@ static void *stress_membarrier_thread(void *arg)
 	 */
 	(void)sigprocmask(SIG_BLOCK, &set, NULL);
 
-	while (keep_running && keep_stressing_flag()) {
+	while (keep_running && stress_continue_flag()) {
 		if (stress_membarrier_exercise(args, info) < 0)
 			break;
 	}
@@ -172,7 +176,7 @@ static int stress_membarrier(const stress_args_t *args)
 	(void)sigfillset(&set);
 	for (i = 0; i < MAX_MEMBARRIER_THREADS + 1; i++) {
 		info[i].pthread_ret = -1;
-		(void)memset(&info[i].pthread, 0, sizeof(info[i].pthread));
+		(void)shim_memset(&info[i].pthread, 0, sizeof(info[i].pthread));
 		info[i].duration = 0.0;
 		info[i].count = 0.0;
 	}
@@ -192,8 +196,8 @@ static int stress_membarrier(const stress_args_t *args)
 			pr_fail("%s: membarrier failed: errno=%d: (%s)\n",
 				args->name, errno, strerror(errno));
 		}
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	keep_running = false;
 
