@@ -18,6 +18,9 @@
  *
  */
 #include "stress-ng.h"
+#include "core-cpu.h"
+#include "core-madvise.h"
+#include "core-out-of-memory.h"
 #include "core-target-clones.h"
 
 #define MIN_MEMFD_BYTES		(2 * MB)
@@ -31,10 +34,16 @@ static const stress_help_t help[] = {
 	{ NULL,	"memfd N",	 "start N workers allocating memory with memfd_create" },
 	{ NULL,	"memfd-bytes N", "allocate N bytes for each stress iteration" },
 	{ NULL,	"memfd-fds N",	 "number of memory fds to open per stressors" },
+	{ NULL,	"memfd-mlock",	 "attempt to mlock pages into memory" },
 	{ NULL,	"memfd-ops N",	 "stop after N memfd bogo operations" },
 	{ NULL,	NULL,		 NULL }
 };
 
+static int stress_set_memfd_mlock(const char *opt)
+{
+	return stress_set_setting_true("memfd-mlock", opt);
+}
+
 /*
  *  stress_set_memfd_bytes
  *	set max size of each memfd size
@@ -66,6 +75,7 @@ static int stress_set_memfd_fds(const char *opt)
 static const stress_opt_set_func_t opt_set_funcs[] = {
 	{ OPT_memfd_bytes,	stress_set_memfd_bytes },
 	{ OPT_memfd_fds,	stress_set_memfd_fds },
+	{ OPT_memfd_mlock,	stress_set_memfd_mlock },
 	{ 0,			NULL }
 };
 
@@ -87,6 +97,12 @@ static const unsigned int flags[] = {
     defined(MFD_HUGE_1GB)
 	MFD_HUGETLB | MFD_HUGE_1GB,
 #endif
+#if defined(MFD_NOEXEC_SEAL)
+	MFD_NOEXEC_SEAL,
+#endif
+#if defined(MFD_EXEC)
+	MFD_EXEC,
+#endif
 };
 
 /*
@@ -102,7 +118,7 @@ static inline const uint64_t *uint64_ptr_offset(const uint64_t *ptr, const size_
  *  stress_memfd_fill_pages()
  *	fill pages with random uin64_t values
  */
-static inline void TARGET_CLONES stress_memfd_fill_pages(void *ptr, const size_t size)
+static void TARGET_CLONES stress_memfd_fill_pages_generic(void *ptr, const size_t size)
 {
 	register uint64_t *u64ptr = (uint64_t *)ptr;
 	register const uint64_t *u64end = uint64_ptr_offset(ptr, size);
@@ -145,6 +161,55 @@ static inline void TARGET_CLONES stress_memfd_fill_pages(void *ptr, const size_t
 	}
 }
 
+#if defined(HAVE_NT_STORE64)
+/*
+ *  stress_memfd_fill_pages_nt_store()
+ *	fill pages with random uin64_t values using nt_store
+ */
+static void OPTIMIZE3 stress_memfd_fill_pages_nt_store(void *ptr, const size_t size)
+{
+	register uint64_t *u64ptr = (uint64_t *)ptr;
+	register const uint64_t *u64end = uint64_ptr_offset(ptr, size);
+	register uint64_t v = stress_mwc64();
+
+	while (u64ptr < u64end) {
+		stress_nt_store64(u64ptr++, v);
+		stress_nt_store64(u64ptr++, v);
+		stress_nt_store64(u64ptr++, v);
+		stress_nt_store64(u64ptr++, v);
+		stress_nt_store64(u64ptr++, v);
+		stress_nt_store64(u64ptr++, v);
+		stress_nt_store64(u64ptr++, v);
+		stress_nt_store64(u64ptr++, v);
+		stress_nt_store64(u64ptr++, v);
+		stress_nt_store64(u64ptr++, v);
+		stress_nt_store64(u64ptr++, v);
+		stress_nt_store64(u64ptr++, v);
+		stress_nt_store64(u64ptr++, v);
+		stress_nt_store64(u64ptr++, v);
+		stress_nt_store64(u64ptr++, v);
+		stress_nt_store64(u64ptr++, v);
+		stress_nt_store64(u64ptr++, v);
+		stress_nt_store64(u64ptr++, v);
+		stress_nt_store64(u64ptr++, v);
+		stress_nt_store64(u64ptr++, v);
+		stress_nt_store64(u64ptr++, v);
+		stress_nt_store64(u64ptr++, v);
+		stress_nt_store64(u64ptr++, v);
+		stress_nt_store64(u64ptr++, v);
+		stress_nt_store64(u64ptr++, v);
+		stress_nt_store64(u64ptr++, v);
+		stress_nt_store64(u64ptr++, v);
+		stress_nt_store64(u64ptr++, v);
+		stress_nt_store64(u64ptr++, v);
+		stress_nt_store64(u64ptr++, v);
+		stress_nt_store64(u64ptr++, v);
+		stress_nt_store64(u64ptr++, v);
+		v++;
+	}
+}
+#endif
+
 #if defined(HAVE_MADVISE) &&	\
     defined(MADV_PAGEOUT)
 /*
@@ -181,6 +246,13 @@ static int stress_memfd_child(const stress_args_t *args, void *context)
 	uint32_t memfd_fds = DEFAULT_MEMFD_FDS;
 	int mmap_flags = MAP_FILE | MAP_SHARED;
 	double duration = 0.0, count = 0.0, rate;
+	bool memfd_mlock = false;
+#if defined(HAVE_NT_STORE64)
+	void (*stress_memfd_fill_pages)(void *ptr, const size_t size) =
+		stress_cpu_x86_has_sse2() ? stress_memfd_fill_pages_nt_store : stress_memfd_fill_pages_generic;
+#else
+	void (*stress_memfd_fill_pages)(void *ptr, const size_t size) = stress_memfd_fill_pages_generic;
+#endif
 
 #if defined(MAP_POPULATE)
 	mmap_flags |= MAP_POPULATE;
@@ -188,6 +260,8 @@ static int stress_memfd_child(const stress_args_t *args, void *context)
 
 	(void)context;
 
+	(void)stress_get_setting("memfd-mlock", &memfd_mlock);
+
 	if (!stress_get_setting("memfd-bytes", &memfd_bytes)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
 			memfd_bytes = MAX_32;
@@ -255,11 +329,11 @@ static int stress_memfd_child(const stress_args_t *args, void *context)
 				default:
 					pr_fail("%s: memfd_create failed: errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
-					keep_stressing_set_flag(false);
+					stress_continue_set_flag(false);
 					goto memfd_unmap;
 				}
 			}
-			if (!keep_stressing_flag())
+			if (!stress_continue_flag())
 				goto memfd_unmap;
 		}
 
@@ -274,7 +348,7 @@ static int stress_memfd_child(const stress_args_t *args, void *context)
 			if (fds[i] < 0)
 				continue;
 
-			if (!keep_stressing_flag())
+			if (!stress_continue_flag())
 				break;
 
 			/* Allocate space */
@@ -294,6 +368,10 @@ static int stress_memfd_child(const stress_args_t *args, void *context)
 			 * to force page it in
 			 */
 			maps[i] = mmap(NULL, size, PROT_WRITE, mmap_flags, fds[i], 0);
+			if (maps[i] == MAP_FAILED)
+				continue;
+			if (memfd_mlock)
+				(void)shim_mlock(maps[i], size);
 			stress_memfd_fill_pages(maps[i], size);
 			(void)stress_madvise_random(maps[i], size);
 
@@ -313,13 +391,15 @@ static int stress_memfd_child(const stress_args_t *args, void *context)
 			 */
 			VOID_RET(ssize_t, shim_fallocate(fds[i], 0, (off_t)size, 0));
 
-			if (!keep_stressing_flag())
+			if (!stress_continue_flag())
 				goto memfd_unmap;
 		}
 
 		for (i = 0; i < memfd_fds; i++) {
 			if (fds[i] < 0)
 				continue;
+			if (maps[i] == MAP_FAILED)
+				continue;
 #if defined(SEEK_SET)
 			if (lseek(fds[i], (off_t)size >> 1, SEEK_SET) < 0) {
 				if ((errno != ENXIO) && (errno != EINVAL))
@@ -355,7 +435,7 @@ static int stress_memfd_child(const stress_args_t *args, void *context)
 						args->name, errno, strerror(errno));
 			}
 #endif
-			if (!keep_stressing_flag())
+			if (!stress_continue_flag())
 				goto memfd_unmap;
 		}
 
@@ -370,7 +450,7 @@ memfd_unmap:
 		 *  Check for zap_pte bug, see Linux commit
 		 *  5abfd71d936a8aefd9f9ccd299dea7a164a5d455
 		 */
-		for (i = 0; keep_stressing_flag() && (i < memfd_fds); i++) {
+		for (i = 0; stress_continue_flag() && (i < memfd_fds); i++) {
 			uint64_t *buf, *ptr;
 			const uint64_t *end_ptr;
 			uint64_t val = stress_mwc64();
@@ -382,6 +462,8 @@ memfd_unmap:
 					MAP_PRIVATE, fds[i], 0);
 			if (buf == MAP_FAILED)
 				continue;
+			if (memfd_mlock)
+				(void)shim_mlock(buf, test_size);
 			end_ptr = uint64_ptr_offset(buf, test_size);
 			for (ptr = buf; ptr < end_ptr; ptr++)
 				*ptr = val;
@@ -460,8 +542,8 @@ buf_unmap:
 				(void)close(fd);
 			}
 		}
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
@@ -495,6 +577,7 @@ stressor_info_t stress_memfd_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_OS | CLASS_MEMORY,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without memfd_create() system call"
 };
