@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
 #include "core-capabilities.h"
 
 #if defined(__linux__)
@@ -58,7 +59,7 @@ static int stress_memhotplug_supported(const char *name)
 	return 0;
 }
 
-static bool stress_memhotplug_removable(char *name)
+static bool stress_memhotplug_removable(const char *name)
 {
 	char path[PATH_MAX];
 	char buf[64];
@@ -66,7 +67,7 @@ static bool stress_memhotplug_removable(char *name)
 
 	(void)snprintf(path, sizeof(path), "%s/%s/removable",
 		sys_memory_path, name);
-	if (system_read(path, buf, sizeof(buf)) < 0)
+	if (stress_system_read(path, buf, sizeof(buf)) < 0)
 		return false;
 	if (sscanf(buf, "%1d", &val) != 1)
 		return false;
@@ -79,7 +80,7 @@ static void stress_memhotplug_set_timer(const unsigned int secs)
 {
 	struct itimerval timer;
 
-	(void)memset(&timer, 0, sizeof(timer));
+	(void)shim_memset(&timer, 0, sizeof(timer));
 	timer.it_value.tv_sec = secs;
 	timer.it_value.tv_usec = 0;
 	timer.it_interval.tv_sec = secs;
@@ -229,17 +230,17 @@ static int stress_memhotplug(const stress_args_t *args)
 
 	do {
 		bool ok = false;
-		for (i = 0; keep_stressing(args) && (i < max); i++) {
+		for (i = 0; stress_continue(args) && (i < max); i++) {
 			stress_memhotplug_mem_toggle(&mem_info[i], &metrics);
 			if (!mem_info[i].timeout)
 				ok = true;
-			inc_counter(args);
+			stress_bogo_inc(args);
 		}
 		if (!ok) {
 			for (i = 0; i < max; i++)
 				stress_memhotplug_mem_online(&mem_info[i]);
 		}
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
