@@ -23,6 +23,8 @@
 #include "core-builtin.h"
 #include "core-cpu-cache.h"
 #include "core-nt-store.h"
+#include "core-numa.h"
+#include "core-out-of-memory.h"
 #include "core-pthread.h"
 #include "core-target-clones.h"
 
@@ -69,8 +71,8 @@ typedef struct {
 typedef void (*stress_memthrash_func_t)(const stress_memthrash_context_t *context, size_t mem_size);
 
 typedef struct stress_memthrash_method_info {
-	const char		*name;	/* human readable form of stressor */
-	stress_memthrash_func_t	func;	/* the method function */
+	const char		*name;		/* human readable form of stressor */
+	const stress_memthrash_func_t	func;	/* the method function */
 } stress_memthrash_method_info_t;
 
 /* Per-pthread information */
@@ -271,7 +273,7 @@ static void OPTIMIZE3 TARGET_CLONES stress_memthrash_swap64(
 		r5 = ptr[5];
 		r6 = ptr[6];
 		r7 = ptr[7];
-		shim_mb();
+		stress_asm_mb();
 
 		ptr[0] = r4;
 		ptr[1] = r5;
@@ -281,7 +283,7 @@ static void OPTIMIZE3 TARGET_CLONES stress_memthrash_swap64(
 		ptr[5] = r1;
 		ptr[6] = r2;
 		ptr[7] = r3;
-		shim_mb();
+		stress_asm_mb();
 		ptr += 8;
 
 		r0 = ptr[0];
@@ -292,7 +294,7 @@ static void OPTIMIZE3 TARGET_CLONES stress_memthrash_swap64(
 		r5 = ptr[5];
 		r6 = ptr[6];
 		r7 = ptr[7];
-		shim_mb();
+		stress_asm_mb();
 
 		ptr[0] = r4;
 		ptr[1] = r5;
@@ -302,7 +304,7 @@ static void OPTIMIZE3 TARGET_CLONES stress_memthrash_swap64(
 		ptr[5] = r1;
 		ptr[6] = r2;
 		ptr[7] = r3;
-		shim_mb();
+		stress_asm_mb();
 		ptr += 8;
 	}
 }
@@ -337,7 +339,7 @@ static void OPTIMIZE3 TARGET_CLONES stress_memthrash_copy128(
 		ptr[5] = r5;
 		ptr[6] = r6;
 		ptr[7] = r7;
-		shim_mb();
+		stress_asm_mb();
 
 		ptr += 8;
 	}
