@@ -150,7 +150,7 @@ static int stress_mergesort(const stress_args_t *args)
 				}
 			}
 		}
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 
 		/* Reverse sort */
@@ -175,7 +175,7 @@ static int stress_mergesort(const stress_args_t *args)
 				}
 			}
 		}
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 
 		/* And re-order */
@@ -205,11 +205,11 @@ static int stress_mergesort(const stress_args_t *args)
 				}
 			}
 		}
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	do_jmp = false;
 	(void)stress_sigrestore(args->name, SIGALRM, &old_action);
@@ -236,7 +236,8 @@ stressor_info_t stress_mergesort_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_CPU_CACHE | CLASS_CPU | CLASS_MEMORY,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_OPTIONAL,
 	.help = help,
-	.unimplemented_reason = "built without BSD library"
+	.unimplemented_reason = "built without the BSD library"
 };
 #endif
