@@ -76,19 +76,19 @@ static int stress_mincore_file(const stress_args_t *args)
 static void stress_mincore_expect(
 	const stress_args_t *args,
 	const int ret,		/* return value */
-	const int ret_expected,	/* exepected return value */
+	const int ret_expected,	/* expected return value */
 	const int err,		/* returned errno */
 	const int err_expected,	/* expected errno */
 	char *msg)		/* test message */
 {
-	if (ret == ret_expected) {
-		if (ret_expected == 0)
+	if (LIKELY(ret == ret_expected)) {
+		if (LIKELY(ret_expected == 0))
 			return;	/* Success! */
 		pr_fail("%s: unexpected success exercising %s\n",
 			args->name, msg);
 	}
 	/* Silently ignore ENOSYS for now */
-	if (err == ENOSYS)
+	if (UNLIKELY(err == ENOSYS))
 		return;
 	if (err != err_expected)
 		pr_fail("%s: expected errno %d, got %d instead while exercising %s\n",
@@ -137,14 +137,14 @@ static int stress_mincore(const stress_args_t *args)
 	do {
 		int i;
 
-		for (i = 0; (i < 100) && keep_stressing_flag(); i++) {
+		for (i = 0; (i < 100) && stress_continue_flag(); i++) {
 			int ret, redo = 0;
 			static unsigned char vec[1];
 			double t;
 
 redo: 			errno = 0;
 			ret = shim_mincore((void *)addr, page_size, vec);
-			if (ret < 0) {
+			if (UNLIKELY(ret < 0)) {
 				switch (errno) {
 				case ENOMEM:
 					/* Page not mapped */
@@ -185,7 +185,7 @@ redo: 			errno = 0;
 					}
 				}
 			}
-			if (fdmapped != MAP_FAILED) {
+			if (UNLIKELY(fdmapped != MAP_FAILED)) {
 				/* Force page to be resident */
 				*fdmapped = stress_mwc8();
 #if defined(MS_ASYNC)
@@ -202,7 +202,7 @@ redo: 			errno = 0;
 					}
 				}
 			}
-			if (unmapped != MAP_FAILED) {
+			if (UNLIKELY(unmapped != MAP_FAILED)) {
 				/* mincore on unmapped page should fail */
 				ret = shim_mincore((void *)unmapped, page_size, vec);
 				if (ret == 0) {
@@ -221,7 +221,7 @@ redo: 			errno = 0;
 				addr += page_size;
 			}
 
-			if (mapped != MAP_FAILED) {
+			if (UNLIKELY(mapped != MAP_FAILED)) {
 				/*
 				 *  Exercise with zero length, ignore return
 				 */
@@ -274,8 +274,8 @@ redo: 			errno = 0;
 			stress_mincore_expect(args, ret, 0, errno, EINVAL,
 				"NULL and zero arguments");
 		}
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	rate = (count > 0.0) ? duration / count : 0.0;
 	stress_metrics_set(args, 0, "nanosecs per mincore call", rate * STRESS_DBL_NANOSECOND);
@@ -307,6 +307,7 @@ stressor_info_t stress_mincore_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_OS | CLASS_MEMORY,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without mincore() system call support"
 };
