@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
 
 #if defined(HAVE_LIBGEN_H)
 #include <libgen.h>
@@ -84,7 +85,7 @@ static int stress_mknod_find_dev(mode_t mode, dev_t *dev)
 	struct dirent *d;
 	int rc = -1;
 
-	(void)memset(dev, 0, sizeof(*dev));
+	(void)shim_memset(dev, 0, sizeof(*dev));
 
 	dir = opendir("/dev");
 	if (!dir)
@@ -100,7 +101,7 @@ static int stress_mknod_find_dev(mode_t mode, dev_t *dev)
 
 		/* A match, cope it */
 		if ((statbuf.st_mode & S_IFMT) == mode) {
-			(void)memcpy(dev, &statbuf.st_dev, sizeof(*dev));
+			(void)shim_memcpy(dev, &statbuf.st_dev, sizeof(*dev));
 			rc = 0;
 			break;
 		}
@@ -247,7 +248,7 @@ static int stress_mknod(const stress_args_t *args)
 		if (blk_dev_ret == 0)
 			stress_mknod_test_dev(args, dir_fd, bad_fd, S_IFBLK, "S_IFBLK", blk_dev);
 
-		for (i = 0; keep_stressing(args) && (i < n); i++) {
+		for (i = 0; stress_continue(args) && (i < n); i++) {
 			char path[PATH_MAX];
 			const uint64_t gray_code = (i >> 1) ^ i;
 			size_t j = stress_mwc32modn(num_nodes);
@@ -259,14 +260,14 @@ static int stress_mknod(const stress_args_t *args)
 					continue;	/* Try again */
 				break;
 			}
-			inc_counter(args);
+			stress_bogo_inc(args);
 		}
 
 		stress_mknod_tidy(args, i);
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 		(void)sync();
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
@@ -288,6 +289,7 @@ stressor_info_t stress_mknod_info = {
 stressor_info_t stress_mknod_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "only supported on Linux"
 };
