@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-out-of-memory.h"
 
 static const stress_help_t help[] = {
 	{ NULL,	"mlock N",	"start N workers exercising mlock/munlock" },
@@ -30,6 +31,30 @@ static const stress_help_t help[] = {
 
 #define MLOCK_MAX	(256*1024)
 
+#if defined(__linux__)
+static uint64_t stress_mlock_pages(const size_t page_size)
+{
+	FILE *fp;
+	char buf[4096];
+	uint64_t mlocked = 0;
+	uint64_t kb_per_page = (uint64_t)page_size / 1024U;
+
+	fp = fopen("/proc/self/status", "r");
+	if (!fp)
+		return 0;
+	while ((fgets(buf, sizeof(buf), fp) != NULL)) {
+		if (strncmp(buf, "VmLck:", 6) == 0) {
+			if (sscanf(buf + 6, "%" SCNu64, &mlocked) == 1)
+				break;
+			mlocked = 0;
+			break;
+		}
+	}
+	(void)fclose(fp);
+	return mlocked / kb_per_page;
+}
+#endif
+
 #if defined(HAVE_MLOCK2)
 
 #ifndef MLOCK_ONFAULT
@@ -50,37 +75,65 @@ static int do_mlock(
 {
 	static bool use_mlock2 = true;
 	int ret;
-	double t;
+	static int metrics_count = 0;
 
-	if (use_mlock2) {
-		const uint32_t rnd = stress_mwc32() >> 5;
+	if (LIKELY(metrics_count++ < 1000)) {
+		/* faster non-metrics timed mlock operations */
+		if (use_mlock2) {
+			const uint8_t rnd = stress_mwc8() >> 5;
 
-		/* Randomly use mlock2 or mlock */
-		if (rnd & 1) {
-			const int flags = (rnd & 2) ?
-				0 : MLOCK_ONFAULT;
+			/* Randomly use mlock2 or mlock */
+			if (rnd & 1) {
+				const int flags = (rnd & 2) ?
+					0 : MLOCK_ONFAULT;
 
-			t = stress_time_now();
-			ret = shim_mlock2(addr, len, flags);
-			if (ret == 0) {
-				(*duration) += stress_time_now() - t;
-				(*count) += 1.0;
-				return 0;
+				ret = shim_mlock2(addr, len, flags);
+				if (ret == 0)
+					return 0;
+				if (errno != ENOSYS)
+					return ret;
+
+				/* mlock2 not supported... */
+				use_mlock2 = false;
 			}
-			if (errno != ENOSYS)
-				return ret;
-
-			/* mlock2 not supported... */
-			use_mlock2 = false;
 		}
-	}
 
-	/* Just do mlock */
-	t = stress_time_now();
-	ret = shim_mlock((const void *)addr, len);
-	if (ret == 0) {
-		(*duration) += stress_time_now() - t;
-		(*count) += 1.0;
+		/* Just do mlock */
+		ret = shim_mlock((const void *)addr, len);
+	} else {
+		double t;
+
+		/* slower metrics timed mlock operations */
+		if (use_mlock2) {
+			const uint8_t rnd = stress_mwc8() >> 5;
+
+			/* Randomly use mlock2 or mlock */
+			if (rnd & 1) {
+				const int flags = (rnd & 2) ?
+					0 : MLOCK_ONFAULT;
+
+				t = stress_time_now();
+				ret = shim_mlock2(addr, len, flags);
+				if (ret == 0) {
+					(*duration) += stress_time_now() - t;
+					(*count) += 1.0;
+					return 0;
+				}
+				if (errno != ENOSYS)
+					return ret;
+
+				/* mlock2 not supported... */
+				use_mlock2 = false;
+			}
+		}
+
+		/* Just do mlock */
+		t = stress_time_now();
+		ret = shim_mlock((const void *)addr, len);
+		if (ret == 0) {
+			(*duration) += stress_time_now() - t;
+			(*count) += 1.0;
+		}
 	}
 	return ret;
 }
@@ -92,13 +145,21 @@ static inline int do_mlock(
 	double *count)
 {
 	int ret;
-	double t;
+	static int metrics_count = 0;
 
-	t = stress_time_now();
-	ret = shim_mlock((const void *)addr, len);
-	if (ret == 0) {
-		(*duration) += stress_time_now() - t;
-		(*count) += 1.0;
+	if (LIKELY(metrics_count++ < 1000)) {
+		/* faster non-metrics timed mlock operations */
+		ret = shim_mlock((const void *)addr, len);
+	} else {
+		double t;
+
+		/* slower metrics timed mlock operations */
+		t = stress_time_now();
+		ret = shim_mlock((const void *)addr, len);
+		if (ret == 0) {
+			(*duration) += stress_time_now() - t;
+			(*count) += 1.0;
+		}
 	}
 	return ret;
 }
@@ -140,9 +201,10 @@ static size_t stress_mlock_max_lockable(void)
  *	perform various invalid or unusual calls to
  *	exercise kernel a little more.
  */
-static void stress_mlock_misc(const stress_args_t *args, const size_t page_size)
+static void stress_mlock_misc(const stress_args_t *args, const size_t page_size, const bool oom_avoid)
 {
 	(void)args;
+	(void)oom_avoid;
 
 	/*
 	 *  mlock/munlock with invalid or unusual arguments
@@ -162,44 +224,64 @@ static void stress_mlock_misc(const stress_args_t *args, const size_t page_size)
 #if defined(HAVE_MLOCKALL)
 	{
 		int flag = 0;
+
 #if defined(MCL_CURRENT)
-		if (!keep_stressing(args))
+		/* Low memory avoidance, re-start */
+		if (oom_avoid && stress_low_memory(page_size * 3))
+			return;
+		if (!stress_continue(args))
 			return;
 		(void)shim_mlockall(MCL_CURRENT);
 		flag |= MCL_CURRENT;
 #endif
+#if defined(HAVE_MUNLOCKALL)
+		(void)shim_munlockall();
+#endif
+
 #if defined(MCL_FUTURE)
-		if (!keep_stressing(args))
+		/* Low memory avoidance, re-start */
+		if (oom_avoid && stress_low_memory(page_size * 3))
+			return;
+		if (!stress_continue(args))
 			return;
 		(void)shim_mlockall(MCL_FUTURE);
 		flag |= MCL_FUTURE;
 #endif
 #if defined(MCL_ONFAULT) &&	\
     defined(MCL_CURRENT)
-		if (!keep_stressing(args))
+		/* Low memory avoidance, re-start */
+		if (oom_avoid && stress_low_memory(page_size * 3))
+			return;
+		if (!stress_continue(args))
 			return;
 		if (shim_mlockall(MCL_ONFAULT | MCL_CURRENT) == 0)
 			flag |= (MCL_ONFAULT | MCL_CURRENT);
 #endif
 #if defined(MCL_ONFAULT) &&	\
     defined(MCL_FUTURE)
-		if (!keep_stressing(args))
+		/* Low memory avoidance, re-start */
+		if (oom_avoid && stress_low_memory(page_size * 3))
+			return;
+		if (!stress_continue(args))
 			return;
 		if (shim_mlockall(MCL_ONFAULT | MCL_FUTURE) == 0)
 			flag |= (MCL_ONFAULT | MCL_FUTURE);
 #endif
 #if defined(MCL_ONFAULT)
-		if (!keep_stressing(args))
+		/* Low memory avoidance, re-start */
+		if (oom_avoid && stress_low_memory(page_size * 3))
+			return;
+		if (!stress_continue(args))
 			return;
 		/* Exercising Invalid mlockall syscall and ignoring failure */
 		(void)shim_mlockall(MCL_ONFAULT);
 #endif
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			return;
 		/* Exercise Invalid mlockall syscall with invalid flag */
 		(void)shim_mlockall(~0);
 		if (flag) { /* cppcheck-suppress knownConditionTrueFalse */
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				return;
 			(void)shim_mlockall(flag);
 		}
@@ -213,11 +295,17 @@ static int stress_mlock_child(const stress_args_t *args, void *context)
 	uint8_t **mappings;
 	const size_t page_size = args->page_size;
 	const size_t max = stress_mlock_max_lockable();
-	const size_t mappings_len = max * sizeof(*mappings);
+	size_t mappings_len = max * sizeof(*mappings);
 	size_t shmall, freemem, totalmem, freeswap, totalswap;
 	double mlock_duration = 0.0, mlock_count = 0.0;
 	double munlock_duration = 0.0, munlock_count = 0.0;
 	double rate;
+	int rc = EXIT_SUCCESS;
+#if defined(__linux__)
+	uint64_t max_mlocked_pages = 0;
+#endif
+	const size_t mappings_per_page = page_size / sizeof(*mappings);
+	const bool oom_avoid = !!(g_opt_flags & OPT_FLAGS_OOM_AVOID);
 
 	stress_get_memlimits(&shmall, &freemem, &totalmem, &freeswap, &totalswap);
 
@@ -228,13 +316,36 @@ static int stress_mlock_child(const stress_args_t *args, void *context)
 	 *  keep stressing before attempting a calloc that can
 	 *  for a OOM and a respawn if this function
 	 */
-	if (!keep_stressing(args))
+	if (!stress_continue(args))
 		return EXIT_SUCCESS;
 
-	mappings = (uint8_t **)mmap(NULL, mappings_len, PROT_READ | PROT_WRITE,
-		MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
+	/*
+	 *  The physical total mem is the upper bound of the number
+	 *  pages mappable, so allocate page mappings pointers that
+	 *  wont't exceed this upper limit.
+	 */
+	if (mappings_len > totalmem / mappings_per_page)
+		mappings_len = totalmem / mappings_per_page;
+	mappings_len &= ~(page_size - 1);
+	if (mappings_len < page_size)
+		mappings_len = page_size;
+
+	/*
+	 *  Under memory pressure we may need to scale back our
+	 *  mappings array to the point where it can fit into memory.
+	 */
+	mappings = MAP_FAILED;
+	while (stress_continue(args) && (mappings_len >= page_size)) {
+		mappings = (uint8_t **)mmap(NULL, mappings_len, PROT_READ | PROT_WRITE,
+			MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
+		if (mappings != MAP_FAILED)
+			break;
+		mappings_len = mappings_len >> 1;
+		/* mmap failed, yield a bit before retry */
+		shim_sched_yield();
+	}
 	if (mappings == MAP_FAILED) {
-		pr_fail("%s: cannot mmap mmapings table: %d (%s)\n",
+		pr_inf_skip("%s: cannot mmap mappings table: errno=%d (%s), skipping stressor\n",
 			args->name, errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	}
@@ -243,13 +354,15 @@ static int stress_mlock_child(const stress_args_t *args, void *context)
 
 	do {
 		int ret;
-
+#if defined(__linux__)
+		uint64_t mlocked_pages = 0, prev_mlocked_pages;
+#endif
 		for (n = 0; n < max; n++) {
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				break;
 
 			/* Low memory avoidance, re-start */
-			if ((g_opt_flags & OPT_FLAGS_OOM_AVOID) && stress_low_memory(page_size * 3))
+			if (oom_avoid && stress_low_memory(page_size * 3))
 				break;
 
 			mappings[n] = (uint8_t *)mmap(NULL, page_size * 3,
@@ -259,7 +372,7 @@ static int stress_mlock_child(const stress_args_t *args, void *context)
 				break;
 
 #if defined(HAVE_MLOCK2)
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				break;
 			/* Invalid mlock2 syscall with invalid flags and ignoring failure*/
 			(void)shim_mlock2((void *)(mappings[n] + page_size), page_size, ~0);
@@ -268,23 +381,24 @@ static int stress_mlock_child(const stress_args_t *args, void *context)
 			/*
 			 *  Attempt a bogus mlock, ignore failure
 			 */
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				break;
 			(void)do_mlock((void *)(mappings[n] + page_size), 0, &mlock_duration, &mlock_count);
 
 			/*
 			 *  Attempt a correct mlock
 			 */
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				break;
 			ret = do_mlock((void *)(mappings[n] + page_size), page_size, &mlock_duration, &mlock_count);
 			if (ret < 0) {
-				if (errno == EAGAIN)
+				if ((errno == EAGAIN) || (errno == EPERM))
 					continue;
-				if ((errno == ENOMEM) || (errno == EPERM))
+				if (errno == ENOMEM)
 					break;
 				pr_fail("%s: mlock failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
+				rc = EXIT_FAILURE;
 				break;
 			} else {
 				/*
@@ -295,19 +409,30 @@ static int stress_mlock_child(const stress_args_t *args, void *context)
 				 */
 				mappings[n] = (uint8_t *)
 					((intptr_t)mappings[n] | 1);
-				inc_counter(args);
+#if defined(__linux__)
+				prev_mlocked_pages = mlocked_pages;
+				mlocked_pages = stress_mlock_pages(page_size);
+				if ((mlocked_pages > 0) && (mlocked_pages < prev_mlocked_pages)) {
+					pr_dbg("%s: mlocked pages shrunk, before mlock: %" PRIu64 " pages mlocked, after: %" PRIu64 " pages mlocked\n",
+						args->name, prev_mlocked_pages, mlocked_pages);
+				}
+				if (mlocked_pages > max_mlocked_pages)
+					max_mlocked_pages = mlocked_pages;
+#endif
+				stress_bogo_inc(args);
 			}
 
 			if ((n & 1023) == 0)
-				stress_mlock_misc(args, page_size);
+				stress_mlock_misc(args, page_size, oom_avoid);
 		}
 
 		for (i = 0; i < n; i++) {
 			intptr_t addr = (intptr_t)mappings[i];
 			intptr_t mlocked = addr & 1;
 
-			if (keep_stressing(args)) {
-				addr ^= mlocked;
+			addr &= ~(intptr_t)1;
+
+			if (stress_continue(args)) {
 				if (mlocked) {
 					double t;
 
@@ -321,15 +446,19 @@ static int stress_mlock_child(const stress_args_t *args, void *context)
 				/*
 				 *  Attempt a bogus munlock, ignore failure
 				 */
-				(void)shim_munlock((void *)((uint8_t *)addr + page_size), 0);
+				VOID_RET(int, shim_munlock((void *)((uint8_t *)addr + page_size), 0));
 			}
-			(void)munmap((void *)addr, page_size * 3);
+			(void)stress_munmap_retry_enomem((void *)addr, page_size * 3);
+			mappings[i] = MAP_FAILED;
 		}
 
 		for (n = 0; n < max; n++) {
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				break;
 
+			/* Low memory avoidance, re-start */
+			if (oom_avoid && stress_low_memory(page_size * 3))
+				break;
 			mappings[n] = (uint8_t *)mmap(NULL, page_size,
 				PROT_READ | PROT_WRITE,
 				MAP_SHARED | MAP_ANONYMOUS, -1, 0);
@@ -340,19 +469,31 @@ static int stress_mlock_child(const stress_args_t *args, void *context)
 		(void)shim_munlockall();
 #endif
 		for (i = 0; i < n; i++)
-			(void)munmap((void *)mappings[i], page_size);
-	} while (keep_stressing(args));
+			(void)stress_munmap_retry_enomem((void *)mappings[i], page_size);
+
+	} while (stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
+#if defined(__linux__)
+	if (max_mlocked_pages > 0) {
+		uint64_t kb_per_page = (uint64_t)page_size / 1024U;
+
+		pr_dbg("%s: maximum of %" PRIu64 " pages (%" PRIu64 " MB)  mlocked\n",
+			args->name, max_mlocked_pages, (max_mlocked_pages / (kb_per_page * 1024)));
+	}
+#endif
+
 	rate = (mlock_count > 0.0) ? mlock_duration / mlock_count : 0.0;
 	stress_metrics_set(args, 0, "nanosecs per mlock call", rate * STRESS_DBL_NANOSECOND);
-	rate = (munlock_count > 0.0) ? munlock_duration / munlock_count : 0.0;
-	stress_metrics_set(args, 1, "nanosecs per munlock call", rate * STRESS_DBL_NANOSECOND);
+	if (munlock_count > 0.0) {
+		rate =  munlock_duration / munlock_count;
+		stress_metrics_set(args, 1, "nanosecs per munlock call", rate * STRESS_DBL_NANOSECOND);
+	}
 
 	(void)munmap((void *)mappings, mappings_len);
 
-	return EXIT_SUCCESS;
+	return rc;
 }
 
 /*
@@ -374,6 +515,7 @@ stressor_info_t stress_mlock_info = {
 stressor_info_t stress_mlock_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_VM | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without mlock() support or _POSIX_MEMLOCK_RANGE defined"
 };
