@@ -18,6 +18,10 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
+#include "core-killpid.h"
+#include "core-mincore.h"
+#include "core-out-of-memory.h"
 
 #define UNSET_MLOCK_PROCS		(0)
 #define DEFAULT_MLOCK_PROCS		(1024)
@@ -49,6 +53,49 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
 };
 
 #if defined(HAVE_MLOCK)
+
+static int stress_mlock_interruptible(
+	const stress_args_t *args,
+	void *addr,
+	size_t len)
+{
+	const size_t chunk_size = args->page_size << 4;
+	uintptr_t ptr = (uintptr_t)addr;
+
+	while ((len > 0) && (stress_continue(args))) {
+		size_t sz = (len > chunk_size) ? chunk_size : len;
+		int ret;
+
+		ret = shim_mlock((void *)ptr, sz);
+		if (ret < 0)
+			return ret;
+		ptr += sz;
+		len -= sz;
+	}
+	return 0;
+}
+
+static int stress_munlock_interruptible(
+	const stress_args_t *args,
+	void *addr,
+	size_t len)
+{
+	const size_t chunk_size = args->page_size << 4;
+	uintptr_t ptr = (uintptr_t)addr;
+
+	while ((len > 0) && (stress_continue(args))) {
+		size_t sz = (len > chunk_size) ? chunk_size : len;
+		int ret;
+
+		ret = shim_munlock((void *)ptr, sz);
+		if (ret < 0)
+			return ret;
+		ptr += sz;
+		len -= sz;
+	}
+	return 0;
+}
+
 /*
  *  stress_mlockmany()
  *	stress by forking and exiting
@@ -64,7 +111,7 @@ static int stress_mlockmany(const stress_args_t *args)
 
 	(void)stress_get_setting("mlockmany-procs", &mlockmany_procs);
 
-	stress_set_oom_adjustment(args->name, true);
+	stress_set_oom_adjustment(args, true);
 
 	/* Explicitly drop capabilities, makes it more OOM-able */
 	VOID_RET(int, stress_drop_capabilities(args->name));
@@ -94,18 +141,20 @@ static int stress_mlockmany(const stress_args_t *args)
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
-		unsigned int i, n;
+		unsigned int n;
 		size_t shmall, freemem, totalmem, freeswap, totalswap, last_freeswap, last_totalswap;
 
-		(void)memset(pids, 0, sizeof(*pids) * mlockmany_procs);
-
+		(void)shim_memset(pids, 0, sizeof(*pids) * mlockmany_procs);
 		stress_get_memlimits(&shmall, &freemem, &totalmem, &last_freeswap, &last_totalswap);
 
-		for (n = 0; n < mlockmany_procs; n++) {
+		for (n = 0; stress_continue(args) && (n < mlockmany_procs); n++) {
 			pid_t pid;
 
-			if (!keep_stressing(args))
+			/* In case we've missed SIGALRM */
+			if (stress_time_now() > args->time_end) {
+				stress_continue_set_flag(false);
 				break;
+			}
 
 			stress_get_memlimits(&shmall, &freemem, &totalmem, &freeswap, &totalswap);
 
@@ -122,8 +171,12 @@ static int stress_mlockmany(const stress_args_t *args)
 				void *ptr = MAP_FAILED;
 				size_t mmap_size = mlock_size;
 
+				/* In case we've missed SIGALRM */
+				if (stress_time_now() > args->time_end)
+					_exit(0);
+
 				stress_parent_died_alarm();
-				stress_set_oom_adjustment(args->name, true);
+				stress_set_oom_adjustment(args, true);
 				(void)sched_settings_apply(true);
 
 				shim_mlockall(0);
@@ -133,7 +186,7 @@ static int stress_mlockmany(const stress_args_t *args)
 					_exit(0);
 
 				while (mmap_size > args->page_size) {
-					if (!keep_stressing(args))
+					if (!stress_continue(args))
 						_exit(0);
 					ptr = mmap(NULL, mmap_size, PROT_READ | PROT_WRITE,
 						MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
@@ -144,61 +197,57 @@ static int stress_mlockmany(const stress_args_t *args)
 				if (ptr == MAP_FAILED)
 					_exit(0);
 
-
 				(void)stress_mincore_touch_pages(ptr, mmap_size);
 
 				mlock_size = mmap_size;
 				while (mlock_size > args->page_size) {
-					if (!keep_stressing(args))
+					if (!stress_continue(args))
 						_exit(0);
-					ret = shim_mlock(ptr, mlock_size);
+					ret = stress_mlock_interruptible(args, ptr, mlock_size);
 					if (ret == 0)
 						break;
 					mlock_size >>= 1;
 				}
 
-				for (;;) {
-					if (!keep_stressing(args))
-						goto unlock;
-					(void)shim_munlock(ptr, mlock_size);
-					if (!keep_stressing(args))
+				while (stress_continue(args)) {
+					(void)stress_munlock_interruptible(args, ptr, mlock_size);
+					if (!stress_continue(args))
 						goto unmap;
-					(void)shim_mlock(ptr, mlock_size);
-					if (!keep_stressing(args))
+					(void)stress_mlock_interruptible(args, ptr, mlock_size);
+					if (!stress_continue(args))
 						goto unlock;
 					/* Try invalid sizes */
 					(void)shim_mlock(ptr, 0);
 					(void)shim_munlock(ptr, 0);
 
-					(void)shim_mlock(ptr, mlock_size << 1);
-					(void)shim_munlock(ptr, mlock_size << 1);
+					(void)stress_mlock_interruptible(args, ptr, mlock_size << 1);
+					if (!stress_continue(args))
+						goto unlock;
+					(void)stress_munlock_interruptible(args, ptr, mlock_size << 1);
+					if (!stress_continue(args))
+						goto unlock;
 
-					(void)shim_mlock(ptr, ~(size_t)0);
 					(void)shim_munlock(ptr, ~(size_t)0);
-					if (!keep_stressing(args))
+					if (!stress_continue(args))
 						goto unlock;
 					(void)shim_usleep_interruptible(10000);
 				}
 unlock:
-				(void)shim_munlock(ptr, mlock_size);
+				(void)stress_munlock_interruptible(args, ptr, mlock_size);
 unmap:
 				(void)munmap(ptr, mmap_size);
 				_exit(0);
 			}
 			pids[n] = pid;
-			if (!keep_stressing_flag())
+			if (pid > 1) {
+				stress_bogo_inc(args);
+			} else if (pid < 0)
+				break;
+			if (!stress_continue(args))
 				break;
 		}
-		for (i = 0; i < n; i++) {
-			if (pids[i] > 0) {
-				int status;
-				/* Parent, wait for child */
-				(void)stress_killpid(pids[i]);
-				(void)shim_waitpid(pids[i], &status, 0);
-				inc_counter(args);
-			}
-		}
-	} while (keep_stressing(args));
+		stress_kill_and_wait_many(args, pids, n, SIGALRM, false);
+	} while (stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
