@@ -19,6 +19,11 @@
  */
 #include "stress-ng.h"
 #include "core-arch.h"
+#include "core-builtin.h"
+#include "core-madvise.h"
+#include "core-mincore.h"
+#include "core-mmap.h"
+#include "core-out-of-memory.h"
 
 #if defined(HAVE_SYS_PRCTL_H)
 #include <sys/prctl.h>
@@ -37,6 +42,7 @@ static const stress_help_t help[] = {
 	{ NULL,	"mmap-async",	 "using asynchronous msyncs for file based mmap" },
 	{ NULL,	"mmap-bytes N",	 "mmap and munmap N bytes for each stress iteration" },
 	{ NULL,	"mmap-file",	 "mmap onto a file using synchronous msyncs" },
+	{ NULL,	"mmap-mlock",	 "attempt to mlock mmap'd pages" },
 	{ NULL,	"mmap-mprotect", "enable mmap mprotect stressing" },
 	{ NULL, "mmap-odirect",	 "enable O_DIRECT on file" },
 	{ NULL,	"mmap-ops N",	 "stop after N mmap bogo operations" },
@@ -54,6 +60,7 @@ typedef struct {
 	bool mmap_mprotect;
 	bool mmap_file;
 	bool mmap_async;
+	bool mmap_mlock;
 	mmap_func_t mmap;
 	size_t mmap_prot_count;
 	int *mmap_prot_perms;
@@ -261,26 +268,16 @@ static int stress_set_mmap_odirect(const char *opt)
 	return stress_set_setting_true("mmap-odirect", opt);
 }
 
+static int stress_set_mmap_mlock(const char *opt)
+{
+	return stress_set_setting_true("mmap-mlock", opt);
+}
+
 static int stress_set_mmap_mmap2(const char *opt)
 {
 	return stress_set_setting_true("mmap-mmap2", opt);
 }
 
-/*
- *  attempt to set vma name using prctl SET_VMA_ANON_NAME
- */
-static void stress_mmap_set_vma_name(void *buf)
-{
-#if defined(HAVE_SYS_PRCTL_H) &&	\
-    defined(PR_SET_VMA) &&		\
-    defined(PR_SET_VMA_ANON_NAME)
-	/* set vma anon name, even if it's not anonymous */
-	VOID_RET(int, prctl(PR_SET_VMA, PR_SET_VMA_ANON_NAME, buf, "stress-mmap"));
-#else
-	(void)buf;
-#endif
-}
-
 /*
  *  stress_mmap_mprotect()
  *	cycle through page settings on a region of mmap'd memory
@@ -355,7 +352,7 @@ static void stress_mmap_invalid(
 
 	ptr = mmap(addr, length, prot, flags, fd, offset);
 	if (ptr != MAP_FAILED)
-		(void)munmap(ptr, length);
+		(void)stress_munmap_retry_enomem(ptr, length);
 
 #if defined(__NR_mmap) &&	\
     defined(HAVE_SYSCALL)
@@ -365,12 +362,40 @@ static void stress_mmap_invalid(
 	 */
 	ptr = (void *)(uintptr_t)syscall(__NR_mmap, addr, length, prot, flags, fd, offset + 1);
 	if (ptr != MAP_FAILED)
-		(void)munmap(ptr, length);
+		(void)stress_munmap_retry_enomem(ptr, length);
 #endif
 	/* Do the above via libc */
 	ptr = mmap(addr, length, prot, flags, fd, offset + 1);
 	if (ptr != MAP_FAILED)
-		(void)munmap(ptr, length);
+		(void)stress_munmap_retry_enomem(ptr, length);
+}
+
+/*
+ *  stress_mmap_index_shuffle()
+ *	single pass shuffle to mix up page mapping orders
+ */
+static void OPTIMIZE3 stress_mmap_index_shuffle(size_t *index, const size_t n)
+{
+	size_t i;
+
+	if (LIKELY(n <= 0xffffffff)) {
+		/* small index < 4GB of items we can use 32bit mod */
+		for (i = 0; i < n; i++) {
+			register size_t tmp, j = (size_t)stress_mwc32modn_maybe_pwr2(n);
+
+			tmp = index[i];
+			index[i] = index[j];
+			index[j] = tmp;
+		}
+	} else {
+		for (i = 0; i < n; i++) {
+			register size_t tmp, j = (size_t)stress_mwc64modn_maybe_pwr2(n);
+
+			tmp = index[i];
+			index[i] = index[j];
+			index[j] = tmp;
+		}
+	}
 }
 
 static int stress_mmap_child(const stress_args_t *args, void *ctxt)
@@ -378,32 +403,54 @@ static int stress_mmap_child(const stress_args_t *args, void *ctxt)
 	stress_mmap_context_t *context = (stress_mmap_context_t *)ctxt;
 	const size_t page_size = args->page_size;
 	const size_t sz = context->sz;
-	const size_t pages4k = sz / page_size;
+	const size_t pages = sz / page_size;
 	const bool mmap_file = context->mmap_file;
 	const int fd = context->fd;
 	NOCLOBBER int no_mem_retries = 0;
 	const int bad_fd = stress_get_bad_fd();
 	const int ms_flags = context->mmap_async ? MS_ASYNC : MS_SYNC;
 	uint8_t *mapped, **mappings;
+	size_t *index;
 	void *hint;
 	int ret;
 	NOCLOBBER int mask = ~0;
+	static const char mmap_name[] = "stress-mmap";
 
 	VOID_RET(int, stress_sighandler(args->name, SIGBUS, stress_mmap_sighandler, NULL));
 
-	mapped = calloc(pages4k, sizeof(*mapped));
-	if (!mapped) {
+	mapped = (uint8_t *)mmap(NULL, pages * sizeof(*mapped),
+				PROT_READ | PROT_WRITE,
+				MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
+	if (mapped == MAP_FAILED) {
 		pr_dbg("%s: cannot allocate mapped buffer: %d (%s)\n",
 			args->name, errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	}
-	mappings = calloc(pages4k, sizeof(*mappings));
-	if (!mappings) {
+	if (context->mmap_mlock)
+		(void)shim_mlock(mapped, pages * sizeof(*mapped));
+	mappings = (uint8_t **)mmap(NULL, pages * sizeof(*mappings),
+				PROT_READ | PROT_WRITE,
+				MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
+	if (mappings == MAP_FAILED) {
 		pr_dbg("%s: cannot allocate mappings buffer: %d (%s)\n",
 			args->name, errno, strerror(errno));
-		free(mapped);
+		(void)munmap((void *)mapped, pages * sizeof(*mapped));
 		return EXIT_NO_RESOURCE;
 	}
+	if (context->mmap_mlock)
+		(void)shim_mlock(mappings, pages * sizeof(*mappings));
+	index = (size_t *)mmap(NULL, pages * sizeof(*index),
+				PROT_READ | PROT_WRITE,
+				MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
+	if (index == MAP_FAILED) {
+		pr_dbg("%s: cannot allocate index buffer: %d (%s)\n",
+			args->name, errno, strerror(errno));
+		(void)munmap((void *)mappings, pages * sizeof(*mappings));
+		(void)munmap((void *)mapped, pages * sizeof(*mapped));
+		return EXIT_NO_RESOURCE;
+	}
+	if (context->mmap_mlock)
+		(void)shim_mlock(index, pages * sizeof(*index));
 
 	do {
 		size_t n;
@@ -419,7 +466,7 @@ retry:
 			break;
 		}
 
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 
 		if ((g_opt_flags & OPT_FLAGS_OOM_AVOID) && stress_low_memory(sz))
@@ -488,15 +535,17 @@ retry:
 			continue;	/* Try again */
 		}
 
+		if (context->mmap_mlock)
+			(void)shim_mlock(buf, sz);
 		no_mem_retries = 0;
 		if (mmap_file) {
-			(void)memset(buf, 0xff, sz);
+			(void)shim_memset(buf, 0xff, sz);
 			(void)shim_msync((void *)buf, sz, ms_flags);
 		}
 		(void)stress_madvise_random(buf, sz);
 		(void)stress_mincore_touch_pages(buf, context->mmap_bytes);
 		stress_mmap_mprotect(args->name, buf, sz, page_size, context->mmap_mprotect);
-		for (n = 0; n < pages4k; n++) {
+		for (n = 0; n < pages; n++) {
 			mapped[n] = PAGE_MAPPED;
 			mappings[n] = buf + (n * page_size);
 		}
@@ -516,7 +565,7 @@ retry:
 		if ((fd >= 0) && (mmap_file)) {
 			off_t offset = 0;
 
-			for (n = 0; n < pages4k; n++, offset += page_size) {
+			for (n = 0; n < pages; n++, offset += page_size) {
 				if (lseek(fd, offset, SEEK_SET) < 0)
 					continue;
 
@@ -528,87 +577,86 @@ retry:
 		/*
 		 *  Step #1, unmap all pages in random order
 		 */
+		for (n = 0; n < pages; n++)
+			index[n] = n;
+		stress_mmap_index_shuffle(index, n);
+
 		(void)stress_mincore_touch_pages(buf, context->mmap_bytes);
-		for (n = pages4k; n; ) {
-			uint64_t j, i = stress_mwc64modn(pages4k);
-			for (j = 0; j < n; j++) {
-				uint64_t page = (i + j) % pages4k;
-				if (mapped[page] == PAGE_MAPPED) {
-					mapped[page] = 0;
+		for (n = 0; n < pages; n++) {
+			register size_t page = index[n];
+
+			if (mapped[page] == PAGE_MAPPED) {
+				mapped[page] = 0;
 #if defined(HAVE_MQUERY) &&	\
     defined(MAP_FIXED)
-					{
-						/* Exercise OpenBSD mquery */
-						VOID_RET(void *, mquery(mappings[page], page_size,
-								PROT_READ, MAP_FIXED, -1, 0));
-					}
-#endif
-					(void)stress_madvise_random(mappings[page], page_size);
-					stress_mmap_mprotect(args->name, mappings[page],
-						page_size, page_size, context->mmap_mprotect);
-					(void)munmap((void *)mappings[page], page_size);
-					n--;
-					break;
+				{
+					/* Exercise OpenBSD mquery */
+					VOID_RET(void *, mquery(mappings[page], page_size,
+							PROT_READ, MAP_FIXED, -1, 0));
 				}
-				if (!keep_stressing_flag())
-					goto cleanup;
+#endif
+				(void)stress_madvise_random(mappings[page], page_size);
+				stress_mmap_mprotect(args->name, mappings[page],
+					page_size, page_size, context->mmap_mprotect);
+				(void)stress_munmap_retry_enomem((void *)mappings[page], page_size);
 			}
+			if (!stress_continue_flag())
+				goto cleanup;
 		}
-		(void)munmap((void *)buf, sz);
+		(void)stress_munmap_retry_enomem((void *)buf, sz);
 #if defined(MAP_FIXED)
+
 		/*
 		 *  Step #2, map them back in random order
 		 */
-		for (n = pages4k; n; ) {
-			uint64_t j, i = stress_mwc64modn(pages4k);
+		stress_mmap_index_shuffle(index, n);
 
-			for (j = 0; j < n; j++) {
-				uint64_t page = (i + j) % pages4k;
+		for (n = 0; n < pages; n++) {
+			register size_t page = index[n];
 
-				if (!mapped[page]) {
-					off_t offset = mmap_file ? (off_t)(page * page_size) : 0;
-					int fixed_flags = MAP_FIXED;
+			if (!mapped[page]) {
+				off_t offset = mmap_file ? (off_t)(page * page_size) : 0;
+				int fixed_flags = MAP_FIXED;
 
-					/*
-					 * Attempt to map them back into the original address, this
-					 * may fail (it's not the most portable operation), so keep
-					 * track of failed mappings too
-					 */
+				/*
+				 * Attempt to map them back into the original address, this
+				 * may fail (it's not the most portable operation), so keep
+				 * track of failed mappings too
+				 */
 #if defined(MAP_FIXED_NOREPLACE)
-					if (stress_mwc1())
-						fixed_flags = MAP_FIXED_NOREPLACE;
+				if (stress_mwc1())
+					fixed_flags = MAP_FIXED_NOREPLACE;
 #endif
-					mappings[page] = (uint8_t *)context->mmap((void *)mappings[page],
-						page_size, PROT_READ | PROT_WRITE, fixed_flags | context->flags, fd, offset);
+				mappings[page] = (uint8_t *)context->mmap((void *)mappings[page],
+					page_size, PROT_READ | PROT_WRITE, fixed_flags | context->flags, fd, offset);
 
-					if (mappings[page] == MAP_FAILED) {
-						mapped[page] = PAGE_MAPPED_FAIL;
-						mappings[page] = NULL;
-					} else {
-						(void)stress_mincore_touch_pages(mappings[page], page_size);
-						(void)stress_madvise_random(mappings[page], page_size);
-						stress_mmap_mprotect(args->name, mappings[page],
-							page_size, page_size, context->mmap_mprotect);
-						mapped[page] = PAGE_MAPPED;
-						/* Ensure we can write to the mapped page */
-						stress_mmap_set(mappings[page], page_size, page_size);
-						if (stress_mmap_check(mappings[page], page_size, page_size) < 0)
-							pr_fail("%s: mmap'd region of %zu bytes does "
-								"not contain expected data\n", args->name, page_size);
-						if (mmap_file) {
-							(void)memset(mappings[page], (int)n, page_size);
-							(void)shim_msync((void *)mappings[page], page_size, ms_flags);
+				if (mappings[page] == MAP_FAILED) {
+					mapped[page] = PAGE_MAPPED_FAIL;
+					mappings[page] = NULL;
+				} else {
+					if (context->mmap_mlock)
+						(void)shim_mlock(mappings[page], page_size);
+					(void)stress_mincore_touch_pages(mappings[page], page_size);
+					(void)stress_madvise_random(mappings[page], page_size);
+					stress_mmap_mprotect(args->name, mappings[page],
+						page_size, page_size, context->mmap_mprotect);
+					mapped[page] = PAGE_MAPPED;
+					/* Ensure we can write to the mapped page */
+					stress_mmap_set(mappings[page], page_size, page_size);
+					if (stress_mmap_check(mappings[page], page_size, page_size) < 0)
+						pr_fail("%s: mmap'd region of %zu bytes does "
+							"not contain expected data\n", args->name, page_size);
+					if (mmap_file) {
+						(void)shim_memset(mappings[page], (int)n, page_size);
+						(void)shim_msync((void *)mappings[page], page_size, ms_flags);
 #if defined(FALLOC_FL_KEEP_SIZE) &&	\
     defined(FALLOC_FL_PUNCH_HOLE)
-							(void)shim_fallocate(fd, FALLOC_FL_PUNCH_HOLE | FALLOC_FL_KEEP_SIZE,
-								offset, (off_t)page_size);
+						(void)shim_fallocate(fd, FALLOC_FL_PUNCH_HOLE | FALLOC_FL_KEEP_SIZE,
+							offset, (off_t)page_size);
 #endif
-						}
 					}
-					n--;
-					break;
 				}
-				if (!keep_stressing_flag())
+				if (!stress_continue_flag())
 					goto cleanup;
 			}
 		}
@@ -617,12 +665,12 @@ cleanup:
 		/*
 		 *  Step #3, unmap them all
 		 */
-		for (n = 0; n < pages4k; n++) {
+		for (n = 0; n < pages; n++) {
 			if (mapped[n] & PAGE_MAPPED) {
 				(void)stress_madvise_random(mappings[n], page_size);
 				stress_mmap_mprotect(args->name, mappings[n],
 					page_size, page_size, context->mmap_mprotect);
-				(void)munmap((void *)mappings[n], page_size);
+				(void)stress_munmap_retry_enomem((void *)mappings[n], page_size);
 			}
 		}
 
@@ -631,10 +679,10 @@ cleanup:
 		 *  was successfully mapped earlier. This page should be now
 		 *  unmapped so unmap it again in various ways
 		 */
-		for (n = 0; n < pages4k; n++) {
+		for (n = 0; n < pages; n++) {
 			if (mapped[n] & PAGE_MAPPED) {
-				(void)munmap((void *)mappings[n], 0);
-				(void)munmap((void *)mappings[n], page_size);
+				(void)stress_munmap_retry_enomem((void *)mappings[n], 0);
+				(void)stress_munmap_retry_enomem((void *)mappings[n], page_size);
 				break;
 			}
 		}
@@ -674,16 +722,19 @@ cleanup:
 			const int rnd_prot = context->mmap_prot_perms[rnd_sz];
 
 			buf = (uint8_t *)mmap(NULL, rnd_sz, rnd_prot, MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
-			if (buf != MAP_FAILED)
-				(void)munmap((void *)buf, rnd_sz);
+			if (buf != MAP_FAILED) {
+				if (context->mmap_mlock)
+					(void)shim_mlock(buf, rnd_sz);
+				(void)stress_munmap_retry_enomem((void *)buf, rnd_sz);
+			}
 		}
 
 		/*
 		 *  Step #8, work through all flag permutations
 		 */
 		if ((context->mmap_flag_perms) && (context->mmap_flag_count > 0)) {
-			static int index;
-			const int flag = context->mmap_flag_perms[index];
+			static size_t flag_perms_index;
+			const int flag = context->mmap_flag_perms[flag_perms_index];
 			int tmpfd;
 
 			if (flag & MAP_ANONYMOUS)
@@ -693,13 +744,16 @@ cleanup:
 
 			buf = (uint8_t *)mmap(NULL, page_size, PROT_READ, flag, tmpfd, 0);
 			if (buf != MAP_FAILED) {
-				stress_mmap_set_vma_name((void *)buf);
-				(void)munmap((void *)buf, page_size);
+				if (context->mmap_mlock)
+					(void)shim_mlock(buf, page_size);
+				stress_set_vma_anon_name((void *)buf, page_size, mmap_name);
+				(void)stress_munmap_retry_enomem((void *)buf, page_size);
 			}
 			if (tmpfd >= 0)
 				(void)close(tmpfd);
-			index++;
-			index %= context->mmap_flag_count;
+			flag_perms_index++;
+			if (flag_perms_index >= context->mmap_flag_count)
+				flag_perms_index = 0;
 		}
 #if defined(HAVE_MPROTECT)
 		/*
@@ -711,11 +765,13 @@ cleanup:
 		if (buf64 != MAP_FAILED) {
 			uint64_t val = stress_mwc64();
 
-			stress_mmap_set_vma_name((void *)buf64);
+			if (context->mmap_mlock)
+				(void)shim_mlock(buf64, page_size);
+			stress_set_vma_anon_name((void *)buf64, page_size, mmap_name);
 
 			*buf64 = val;
 			ret = mprotect((void *)buf64, page_size, PROT_READ);
-			if ((ret < 0) && (errno != ENOMEM)) {
+			if ((ret < 0) && (errno != ENOMEM) && (errno != EPERM)) {
 				pr_fail("%s: cannot set write-only page to read-only, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 			} else {
@@ -725,7 +781,7 @@ cleanup:
 						args->name, *buf64, val);
 				}
 			}
-			(void)munmap((void *)buf64, page_size);
+			(void)stress_munmap_retry_enomem((void *)buf64, page_size);
 		}
 #endif
 #if defined(HAVE_MPROTECT)
@@ -735,23 +791,26 @@ cleanup:
 		buf64 = (uint64_t *)mmap(NULL, page_size, PROT_READ,
 					MAP_SHARED | MAP_ANONYMOUS, -1, 0);
 		if (buf64 != MAP_FAILED) {
-			stress_mmap_set_vma_name((void *)buf64);
+			if (context->mmap_mlock)
+				(void)shim_mlock(buf64, page_size);
+			stress_set_vma_anon_name((void *)buf64, page_size, mmap_name);
 
 			ret = mprotect((void *)buf64, page_size, PROT_WRITE);
-			if ((ret < 0) && (errno != ENOMEM)) {
+			if ((ret < 0) && (errno != ENOMEM) && (errno != EPERM)) {
 				pr_fail("%s: cannot set read-only page to write-only, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 			}
-			(void)munmap((void *)buf64, page_size);
+			(void)stress_munmap_retry_enomem((void *)buf64, page_size);
 		}
 #endif
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	jmp_env_set = false;
 
-	free(mappings);
-	free(mapped);
+	(void)munmap((void *)index, pages * sizeof(*index));
+	(void)munmap((void *)mappings, pages * sizeof(*mappings));
+	(void)munmap((void *)mapped, pages * sizeof(*mapped));
 
 	return EXIT_SUCCESS;
 }
@@ -778,6 +837,7 @@ static int stress_mmap(const stress_args_t *args)
 	context.mmap_bytes = DEFAULT_MMAP_BYTES;
 	context.mmap_async = false;
 	context.mmap_file = false;
+	context.mmap_mlock = false;
 	context.mmap_mprotect = false;
 	context.flags = MAP_PRIVATE | MAP_ANONYMOUS;
 #if defined(MAP_POPULATE)
@@ -789,6 +849,7 @@ static int stress_mmap(const stress_args_t *args)
 	(void)stress_get_setting("mmap-mprotect", &context.mmap_mprotect);
 	(void)stress_get_setting("mmap-osync", &mmap_osync);
 	(void)stress_get_setting("mmap-odirect", &mmap_odirect);
+	(void)stress_get_setting("mmap-mlock", &context.mmap_mlock);
 	(void)stress_get_setting("mmap-mmap2", &mmap_mmap2);
 
 	for (all_flags = 0, i = 0; i < SIZEOF_ARRAY(mmap_prot); i++)
@@ -925,6 +986,7 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
 	{ OPT_mmap_mprotect,	stress_set_mmap_mprotect },
 	{ OPT_mmap_osync,	stress_set_mmap_osync },
 	{ OPT_mmap_odirect,	stress_set_mmap_odirect },
+	{ OPT_mmap_mlock,	stress_set_mmap_mlock },
 	{ OPT_mmap_mmap2,	stress_set_mmap_mmap2 },
 	{ 0,			NULL }
 };
