@@ -19,6 +19,8 @@
  */
 #include "stress-ng.h"
 #include "core-builtin.h"
+#include "core-madvise.h"
+#include "core-out-of-memory.h"
 #include "core-pragma.h"
 
 static const stress_help_t help[] = {
