@@ -18,14 +18,22 @@
  *
  */
 #include "stress-ng.h"
+#include "core-mmap.h"
+#include "core-out-of-memory.h"
 
 static const stress_help_t help[] = {
 	{ NULL,	"mmaphuge N",		"start N workers stressing mmap with huge mappings" },
+	{ NULL,	"mmaphuge-mlock",	"attempt to mlock pages into memory" },
 	{ NULL, "mmaphuge-mmaps N",	"select number of memory mappings per iteration" },
 	{ NULL,	"mmaphuge-ops N",	"stop after N mmaphuge bogo operations" },
 	{ NULL,	NULL,			NULL }
 };
 
+static int stress_set_mmaphuge_mlock(const char *opt)
+{
+	return stress_set_setting_true("mmaphuge-mlock", opt);
+}
+
 /*
  *  stress_set_mmaphuge_mmaps()
  *      set number of huge memory mappings to make per loop
@@ -41,7 +49,8 @@ static int stress_set_mmaphuge_mmaps(const char *opt)
 }
 
 static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_mmaphuge_mmaps,  stress_set_mmaphuge_mmaps },
+	{ OPT_mmaphuge_mlock,	stress_set_mmaphuge_mlock },
+	{ OPT_mmaphuge_mmaps,	stress_set_mmaphuge_mmaps },
 	{ 0,                    NULL }
 };
 
@@ -94,6 +103,10 @@ static int stress_mmaphuge_child(const stress_args_t *args, void *v_ctxt)
 	const size_t page_size = args->page_size;
 	stress_mmaphuge_buf_t *bufs = (stress_mmaphuge_buf_t *)ctxt->bufs;
 	size_t idx = 0;
+	int rc = EXIT_SUCCESS;
+	bool mmaphuge_mlock = false;
+
+	(void)stress_get_setting("mmaphuge-mlock", &mmaphuge_mlock);
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
@@ -103,7 +116,7 @@ static int stress_mmaphuge_child(const stress_args_t *args, void *v_ctxt)
 		for (i = 0; i < ctxt->mmaphuge_mmaps; i++)
 			bufs[i].buf = MAP_FAILED;
 
-		for (i = 0; keep_stressing(args) && (i < ctxt->mmaphuge_mmaps); i++) {
+		for (i = 0; stress_continue(args) && (i < ctxt->mmaphuge_mmaps); i++) {
 			size_t shmall, freemem, totalmem, freeswap, totalswap, last_freeswap, last_totalswap;
 			size_t j;
 
@@ -130,14 +143,31 @@ static int stress_mmaphuge_child(const stress_args_t *args, void *v_ctxt)
 					idx = 0;
 
 				if (buf != MAP_FAILED) {
-					register uint8_t val = stress_mwc8();
+					register uint64_t val = stress_mwc64();
 					register size_t k;
 
-					/* Touch every other 64 pages */
+					if (mmaphuge_mlock)
+						(void)shim_mlock(buf, sz);
+
+					/* Touch every other 64 pages.. */
 					for (k = 0; k < sz; k += page_size * 64) {
-						buf[k] = val;
+						register uint64_t *ptr64 = (uint64_t *)&buf[k];
+
+						*ptr64 = val + k;
 					}
-					inc_counter(args);
+					/* ..and sanity check */
+					for (k = 0; stress_continue(args) && (k < sz); k += page_size * 64) {
+						register uint64_t *ptr64 = (uint64_t *)&buf[k];
+
+						if (*ptr64 != val + k) {
+							pr_fail("%s: memory %p at offset 0x%zx check error, "
+								"got 0x%" PRIx64 ", expecting 0x%" PRIx64 "\n",
+								args->name, buf, k, *ptr64, val + k);
+							rc = EXIT_FAILURE;
+						}
+					}
+
+					stress_bogo_inc(args);
 					break;
 				}
 			}
@@ -148,7 +178,7 @@ static int stress_mmaphuge_child(const stress_args_t *args, void *v_ctxt)
 				break;
 		}
 
-		for (i = 0; keep_stressing(args) && (i < ctxt->mmaphuge_mmaps); i++) {
+		for (i = 0; stress_continue(args) && (i < ctxt->mmaphuge_mmaps); i++) {
 			if (bufs[i].buf != MAP_FAILED)
 				continue;
 			/* Try Transparent Huge Pages THP */
@@ -175,22 +205,22 @@ static int stress_mmaphuge_child(const stress_args_t *args, void *v_ctxt)
 				*buf = stress_mwc8();
 				*end_page = stress_mwc8();
 				/* Unmapping small page may fail on huge pages */
-				ret = munmap((void *)end_page, page_size);
+				ret = stress_munmap_retry_enomem((void *)end_page, page_size);
 				if (ret == 0)
-					ret = munmap((void *)buf, sz - page_size);
+					ret = stress_munmap_retry_enomem((void *)buf, sz - page_size);
 				if (ret != 0)
-					(void)munmap((void *)buf, sz);
+					(void)stress_munmap_retry_enomem((void *)buf, sz);
 			} else {
 				*buf = stress_mwc8();
-				(void)munmap((void *)buf, sz);
+				(void)stress_munmap_retry_enomem((void *)buf, sz);
 			}
 			bufs[i].buf = MAP_FAILED;
 		}
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
-	return EXIT_SUCCESS;
+	return rc;
 }
 
 /*
@@ -223,6 +253,7 @@ stressor_info_t stress_mmaphuge_info = {
 	.stressor = stress_mmaphuge,
 	.class = CLASS_VM | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 
@@ -232,6 +263,7 @@ stressor_info_t stress_mmaphuge_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_VM | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without mmap() MAP_HUGETLB support"
 };
