@@ -18,6 +18,10 @@
  *
  */
 #include "stress-ng.h"
+#include "core-madvise.h"
+#include "core-mincore.h"
+#include "core-mmap.h"
+#include "core-out-of-memory.h"
 
 #define DEFAULT_MREMAP_BYTES	(256 * MB)
 #define MIN_MREMAP_BYTES	(4 * KB)
@@ -26,7 +30,7 @@
 static const stress_help_t help[] = {
 	{ NULL,	"mremap N",	  "start N workers stressing mremap" },
 	{ NULL,	"mremap-bytes N", "mremap N bytes maximum for each stress iteration" },
-	{ NULL, "mremap-lock",	  "mlock remap pages, force pages to be unswappable" },
+	{ NULL, "mremap-mlock",	  "mlock remap pages, force pages to be unswappable" },
 	{ NULL,	"mremap-ops N",	  "stop after N mremap bogo operations" },
 	{ NULL,	NULL,		  NULL }
 };
@@ -75,7 +79,7 @@ static inline void *rand_mremap_addr(const size_t sz, int flags)
 	if (addr == MAP_FAILED)
 		return NULL;
 
-	(void)munmap(addr, sz);
+	(void)stress_munmap_retry_enomem(addr, sz);
 
 	/*
 	 * At this point, we know that we can remap to this addr
@@ -101,6 +105,7 @@ static int try_remap(
 {
 	uint8_t *newbuf;
 	int retry, flags = 0;
+	static int metrics_counter = 0;
 #if defined(MREMAP_MAYMOVE)
 	const int maymove = MREMAP_MAYMOVE;
 #else
@@ -115,31 +120,33 @@ static int try_remap(
 #endif
 
 	for (retry = 0; retry < 100; retry++) {
-		double t;
+		double t = 0.0;
+
 
 #if defined(MREMAP_FIXED)
 		void *addr = rand_mremap_addr(new_sz + args->page_size, flags);
 #endif
-		if (!keep_stressing_flag()) {
-			(void)munmap(*buf, old_sz);
+		if (!stress_continue_flag()) {
+			(void)stress_munmap_retry_enomem(*buf, old_sz);
 			*buf = 0;
 			return 0;
 		}
+		if (UNLIKELY(metrics_counter == 0))
+			t = stress_time_now();
 #if defined(MREMAP_FIXED)
 		if (addr) {
-			t = stress_time_now();
 			newbuf = mremap(*buf, old_sz, new_sz, flags, addr);
 		} else {
-			t = stress_time_now();
 			newbuf = mremap(*buf, old_sz, new_sz, flags & ~MREMAP_FIXED);
 		}
 #else
-		t = stress_time_now();
 		newbuf = mremap(*buf, old_sz, new_sz, flags);
 #endif
-		if (newbuf && newbuf != MAP_FAILED) {
-			(*duration) += stress_time_now() - t;
-			(*count) += 1.0;
+		if (newbuf && (newbuf != MAP_FAILED)) {
+			if (UNLIKELY(metrics_counter == 0)) {
+				(*duration) += stress_time_now() - t;
+				(*count) += 1.0;
+			}
 			*buf = newbuf;
 
 #if defined(MREMAP_DONTUNMAP)
@@ -148,15 +155,18 @@ static int try_remap(
 			 *  followed by an unmap of the old mapping for
 			 *  some more exercise
 			 */
-			t = stress_time_now();
+			if (UNLIKELY(metrics_counter == 0))
+				t = stress_time_now();
 			newbuf = mremap(*buf, new_sz, new_sz,
 					MREMAP_DONTUNMAP | MREMAP_MAYMOVE);
-			if (newbuf && newbuf != MAP_FAILED) {
-				(*duration) += stress_time_now() - t;
-				(*count) += 1.0;
+			if (newbuf && (newbuf != MAP_FAILED)) {
+				if (UNLIKELY(metrics_counter == 0)) {
+					(*duration) += stress_time_now() - t;
+					(*count) += 1.0;
+				}
 
 				if (*buf)
-					(void)munmap(*buf, new_sz);
+					(void)stress_munmap_retry_enomem(*buf, new_sz);
 				*buf = newbuf;
 			}
 #endif
@@ -167,6 +177,9 @@ static int try_remap(
 #else
 			(void)mremap_mlock;
 #endif
+			if (metrics_counter++ > 500)
+				metrics_counter = 0;
+
 			return 0;
 		}
 
@@ -230,7 +243,7 @@ static int stress_mremap_child(const stress_args_t *args, void *context)
 		uint8_t *buf = NULL, *ptr;
 		size_t old_sz;
 
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			goto deinit;
 
 		buf = mmap(NULL, new_sz, PROT_READ | PROT_WRITE, flags, -1, 0);
@@ -251,7 +264,7 @@ static int stress_mremap_child(const stress_args_t *args, void *context)
 				pr_fail("%s: mmap'd region of %zu "
 					"bytes does not contain expected data\n",
 					args->name, sz);
-				(void)munmap(buf, new_sz);
+				(void)stress_munmap_retry_enomem(buf, new_sz);
 				ret = EXIT_FAILURE;
 				goto deinit;
 			}
@@ -261,11 +274,11 @@ static int stress_mremap_child(const stress_args_t *args, void *context)
 		new_sz >>= 1;
 		while (new_sz > page_size) {
 			if (try_remap(args, &buf, old_sz, new_sz, mremap_mlock, &duration, &count) < 0) {
-				(void)munmap(buf, old_sz);
+				(void)stress_munmap_retry_enomem(buf, old_sz);
 				ret = EXIT_FAILURE;
 				goto deinit;
 			}
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				goto deinit;
 			(void)stress_madvise_random(buf, new_sz);
 			if (g_opt_flags & OPT_FLAGS_VERIFY) {
@@ -274,7 +287,7 @@ static int stress_mremap_child(const stress_args_t *args, void *context)
 						"of %zu bytes does "
 						"not contain expected data\n",
 						args->name, sz);
-					(void)munmap(buf, new_sz);
+					(void)stress_munmap_retry_enomem(buf, new_sz);
 					ret = EXIT_FAILURE;
 					goto deinit;
 				}
@@ -286,11 +299,11 @@ static int stress_mremap_child(const stress_args_t *args, void *context)
 		new_sz <<= 1;
 		while (new_sz < mremap_bytes) {
 			if (try_remap(args, &buf, old_sz, new_sz, mremap_mlock, &duration, &count) < 0) {
-				(void)munmap(buf, old_sz);
+				(void)stress_munmap_retry_enomem(buf, old_sz);
 				ret = EXIT_FAILURE;
 				goto deinit;
 			}
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				goto deinit;
 			(void)stress_madvise_random(buf, new_sz);
 			old_sz = new_sz;
@@ -299,21 +312,21 @@ static int stress_mremap_child(const stress_args_t *args, void *context)
 
 		/* Invalid remap flags */
 		ptr = mremap(buf, old_sz, old_sz, ~0);
-		if (ptr && ptr != MAP_FAILED)
+		if (ptr && (ptr != MAP_FAILED))
 			buf = ptr;
 		ptr = mremap(buf, old_sz, old_sz, MREMAP_FIXED | MREMAP_MAYMOVE, (void *)~(uintptr_t)0);
-		if (ptr && ptr != MAP_FAILED)
+		if (ptr && (ptr != MAP_FAILED))
 			buf = ptr;
 #if defined(MREMAP_MAYMOVE)
 		/* Invalid new size */
 		ptr = mremap(buf, old_sz, 0, MREMAP_MAYMOVE);
-		if (ptr && ptr != MAP_FAILED)
+		if (ptr && (ptr != MAP_FAILED))
 			buf = ptr;
 #endif
-		(void)munmap(buf, old_sz);
+		(void)stress_munmap_retry_enomem(buf, old_sz);
 
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 deinit:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
@@ -345,6 +358,7 @@ stressor_info_t stress_mremap_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_VM | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_OPTIONAL,
 	.help = help,
 	.unimplemented_reason = "built without mremap() system call support"
 };
