@@ -18,6 +18,8 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
+#include "core-out-of-memory.h"
 
 #define MIN_MSYNC_BYTES		(1 * MB)  /* MUST NOT BE page size or less! */
 #define MAX_MSYNC_BYTES		(MAX_FILE_LIMIT)
@@ -53,9 +55,10 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
 #if defined(HAVE_MSYNC)
 /*
  *  stress_page_check()
- *	check if mmap'd data is sane
+ *	check if mmap'd data is sane, sz is a page size
+ *	check in 64 bit byte chunks
  */
-static int stress_page_check(
+static int OPTIMIZE3 stress_page_check(
 	const uint8_t *buf,
 	const uint8_t val,
 	const size_t sz)
@@ -65,10 +68,20 @@ static int stress_page_check(
 	(void)val;
 	(void)sz;
 #else
-	size_t i;
+	uint16_t val16 = (uint16_t)val << 8 | val;
+	uint32_t val32 = (uint32_t)val16 << 16 | val16;
+	register uint64_t val64 = (uint64_t)val32 << 32 | val32;
+	register const uint64_t *buf64 = (const uint64_t *)buf;
+	register const uint64_t *buf64end = (const uint64_t *)(buf + sz);
 
-	for (i = 0; i < sz; i++) {
-		if (buf[i] != val)
+	while (buf64 < buf64end) {
+		if (UNLIKELY(*(buf64++) != val64))
+			return -1;
+		if (UNLIKELY(*(buf64++) != val64))
+			return -1;
+		if (UNLIKELY(*(buf64++) != val64))
+			return -1;
+		if (UNLIKELY(*(buf64++) != val64))
 			return -1;
 	}
 #endif
@@ -130,7 +143,7 @@ static int stress_msync(const stress_args_t *args)
 		sz = min_size;
 
 	/* Make sure this is killable by OOM killer */
-	stress_set_oom_adjustment(args->name, true);
+	stress_set_oom_adjustment(args, true);
 
 	rc = stress_temp_dir_mk_args(args);
 	if (rc < 0)
@@ -193,7 +206,7 @@ static int stress_msync(const stress_args_t *args)
 		offset = (off_t)stress_mwc64modn(sz - page_size) & ~((off_t)page_size - 1);
 		val = stress_mwc8();
 
-		(void)memset(buf + offset, val, page_size);
+		(void)shim_memset(buf + offset, val, page_size);
 		ret = shim_msync(buf + offset, page_size, MS_SYNC);
 		if (ret < 0) {
 			pr_fail("%s: msync MS_SYNC on "
@@ -229,7 +242,7 @@ do_invalidate:
 		offset = (off_t)stress_mwc64modn(sz - page_size) & ~((off_t)page_size - 1);
 		val = stress_mwc8();
 
-		(void)memset(buf + offset, val, page_size);
+		(void)shim_memset(buf + offset, val, page_size);
 
 		ret = lseek(fd, offset, SEEK_SET);
 		if (ret == (off_t)-1) {
@@ -280,8 +293,8 @@ do_invalidate:
 #endif
 
 do_next:
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
@@ -312,6 +325,7 @@ stressor_info_t stress_msync_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_VM | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without msync() system call support"
 };
