@@ -18,11 +18,12 @@
  *
  */
 #include "stress-ng.h"
+#include "core-out-of-memory.h"
 
 typedef struct {
 	const stress_args_t *args;	/* stress-ng arguments */
 	size_t page_shift;		/* log2(page_size) */
-	char *exe_path;			/* path of executable */
+	char *exec_path;		/* path of executable */
 	double duration;		/* mmap run time duration */
 	double count;			/* count of mmap calls */
 } munmap_context_t;
@@ -85,16 +86,24 @@ static void stress_munmap_range(
 	const size_t stride = stress_munmap_stride(n_pages + stress_mwc8());
 	size_t i, j;
 
-	for (i = 0, j = 0; keep_stressing(args) && (i < n_pages); i++) {
+	for (i = 0, j = 0; stress_continue(args) && (i < n_pages); i++) {
 		const size_t offset = j << page_shift;
 		void *addr = ((uint8_t *)start) + offset;
 		double t;
 
 		t = stress_time_now();
-		if (munmap(addr, page_size) == 0) {
+		if (stress_munmap_retry_enomem(addr, page_size) == 0) {
+			unsigned char vec[1];
+
 			ctxt->duration += stress_time_now() - t;
 			ctxt->count += 1.0;
-			inc_counter(args);
+			stress_bogo_inc(args);
+
+			if ((shim_mincore(addr, page_size, vec) == 0) &&
+			    (vec[0] != 0)) {
+				pr_fail("%s: unmapped page %p still resident in memory\n",
+					args->name, addr);
+			}
 		}
 		j += stride;
 		j %= n_pages;
@@ -141,7 +150,7 @@ static int stress_munmap_child(const stress_args_t *args, void *context)
 	/*
 	 *  Vainly attempt to reduce any potential core dump size
 	 */
-	while (keep_stressing(args) && fgets(buf, sizeof(buf), fp)) {
+	while (stress_continue(args) && fgets(buf, sizeof(buf), fp)) {
 		size_t size;
 
 		*path = '\0';
@@ -157,7 +166,7 @@ static int stress_munmap_child(const stress_args_t *args, void *context)
 	}
 	(void)rewind(fp);
 #endif
-	while (keep_stressing(args) && fgets(buf, sizeof(buf), fp)) {
+	while (stress_continue(args) && fgets(buf, sizeof(buf), fp)) {
 		*path = '\0';
 		n = sscanf(buf, "%p-%p %4s %p %x:%x %" PRIu64 " %s\n",
 			&start, &end, prot, &offset, &major, &minor,
@@ -182,7 +191,7 @@ static int stress_munmap_child(const stress_args_t *args, void *context)
 			continue;	/* don't unmap libc */
 		if (strstr(path, "/dev/zero"))
 			continue;	/* need this for zero'd page data */
-		if (!strcmp(path, ctxt->exe_path))
+		if (!strcmp(path, ctxt->exec_path))
 			continue;	/* don't unmap stress-ng */
 		if (prot[0] != 'r')
 			continue;	/* don't unmap non-readable pages */
@@ -192,8 +201,8 @@ static int stress_munmap_child(const stress_args_t *args, void *context)
 	}
 	(void)fclose(fp);
 
-	if (keep_stressing(args))
-		inc_counter(args);	/* bump per stressor */
+	if (stress_continue(args))
+		stress_bogo_inc(args);	/* bump per stressor */
 
 	return EXIT_SUCCESS;
 }
@@ -219,6 +228,7 @@ static int stress_munmap(const stress_args_t *args)
 {
 	munmap_context_t *ctxt;
 	double rate;
+	char exec_path[PATH_MAX];
 
 	ctxt = mmap(NULL, sizeof(*ctxt), PROT_READ | PROT_WRITE,
 		MAP_SHARED | MAP_ANONYMOUS, -1, 0);
@@ -231,17 +241,17 @@ static int stress_munmap(const stress_args_t *args)
 	ctxt->count = 0.0;
 	ctxt->args = args;
 	ctxt->page_shift = stress_munmap_log2(args->page_size);
-	ctxt->exe_path = stress_proc_self_exe();
-	if (!ctxt->exe_path) {
+	ctxt->exec_path = stress_get_proc_self_exe(exec_path, sizeof(exec_path));
+	if (!ctxt->exec_path) {
 		pr_inf_skip("%s: skipping stressor, cannot determine child executable path\n",
 			args->name);
 		(void)munmap((void *)ctxt, sizeof(*ctxt));
 		return EXIT_NO_RESOURCE;
 	}
-	stress_munmap_clean_path(ctxt->exe_path);
+	stress_munmap_clean_path(ctxt->exec_path);
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
-	while (keep_stressing(args)) {
+	while (stress_continue(args)) {
 		VOID_RET(int, stress_oomable_child(args, (void *)ctxt, stress_munmap_child, STRESS_OOMABLE_QUIET));
 	}
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
@@ -257,6 +267,7 @@ static int stress_munmap(const stress_args_t *args)
 stressor_info_t stress_munmap_info = {
 	.stressor = stress_munmap,
 	.class = CLASS_VM | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 
@@ -264,6 +275,7 @@ stressor_info_t stress_munmap_info = {
 stressor_info_t stress_munmap_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_VM | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
         .help = help,
 	.unimplemented_reason = "only supported on Linux"
 };
