@@ -34,7 +34,9 @@ typedef struct {
 #if defined(HAVE_CLOCK_GETTIME) &&	\
     defined(CLOCK_MONOTONIC)
 	double overrun_nsec;
-	double count;
+	double overrun_count;
+	double underrun_nsec;
+	double underrun_count;
 #endif
 } stress_ctxt_t;
 
@@ -83,14 +85,16 @@ static void *stress_pthread_func(void *c)
 	stress_ctxt_t *ctxt = (stress_ctxt_t *)c;
 	const stress_args_t *args = ctxt->args;
 
-	while (keep_stressing(args) &&
+	while (stress_continue(args) &&
 	       !thread_terminate &&
 	       (!ctxt->max_ops || (ctxt->counter < ctxt->max_ops))) {
-		unsigned long i;
+		register unsigned long i;
 
 		for (i = 1 << 18; i > 0; i >>=1) {
 			struct timespec tv;
-			long nsec = (long)stress_mwc32modn(i) + 8;
+			register const unsigned long mask = (i - 1);
+
+			long nsec = (long)(stress_mwc32() & mask) + 8;
 #if defined(HAVE_CLOCK_GETTIME) &&	\
     defined(CLOCK_MONOTONIC)
 			struct timespec t1;
@@ -101,15 +105,23 @@ static void *stress_pthread_func(void *c)
 			(void)clock_gettime(CLOCK_MONOTONIC, &t1);
 			if (LIKELY(nanosleep(&tv, NULL) == 0)) {
 				struct timespec t2;
-				long dt_nsec;
 
-				(void)clock_gettime(CLOCK_MONOTONIC, &t2);
+				if (clock_gettime(CLOCK_MONOTONIC, &t2) == 0) {
+					long dt_nsec;
 
-				dt_nsec = (t2.tv_sec - t1.tv_sec) * 1000000000;
-				dt_nsec += t2.tv_nsec - t1.tv_nsec;
-				dt_nsec -= nsec;
-				ctxt->overrun_nsec += (double)dt_nsec;
-				ctxt->count += 1.0;
+					dt_nsec = (t2.tv_sec - t1.tv_sec) * 1000000000;
+					dt_nsec += t2.tv_nsec - t1.tv_nsec;
+					dt_nsec -= nsec;
+
+					if (dt_nsec < 0) {
+						pr_inf("%lu vs %lu\n", nsec, dt_nsec);
+						ctxt->underrun_nsec += (double)labs(dt_nsec);
+						ctxt->underrun_count += 1.0;
+					} else {
+						ctxt->overrun_nsec += (double)dt_nsec;
+						ctxt->overrun_count += 1.0;
+					}
+				}
 			} else {
 				break;
 			}
@@ -139,7 +151,9 @@ static int stress_nanosleep(const stress_args_t *args)
 	int ret = EXIT_SUCCESS;
 #if defined(HAVE_CLOCK_GETTIME) &&	\
     defined(CLOCK_MONOTONIC)
-	double overhead_nsec, overrun_nsec, count;
+	double overhead_nsec;
+	double overrun_nsec, overrun_count;
+	double underrun_nsec, underrun_count;
 	const uint64_t benchmark_loops = 10000;
 #endif
 
@@ -166,7 +180,9 @@ static int stress_nanosleep(const stress_args_t *args)
 #if defined(HAVE_CLOCK_GETTIME) && 	\
     defined(CLOCK_MONOTONIC)
 		ctxts[n].overrun_nsec = 0.0;
-		ctxts[n].count = 0.0;
+		ctxts[n].overrun_count = 0.0;
+		ctxts[n].underrun_nsec = 0.0;
+		ctxts[n].underrun_count = 0.0;
 #endif
 		ret = pthread_create(&ctxts[n].pthread, NULL,
 			stress_pthread_func, &ctxts[n]);
@@ -183,22 +199,21 @@ static int stress_nanosleep(const stress_args_t *args)
 			goto tidy;
 		}
 		/* Timed out? abort! */
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			goto tidy;
 	}
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
-		set_counter(args, 0);
+		stress_bogo_set(args, 0);
 		(void)shim_usleep_interruptible(10000);
 		for (i = 0; i < n; i++)
-			add_counter(args, ctxts[i].counter);
-	}  while (!thread_terminate && keep_stressing(args));
+			stress_bogo_add(args, ctxts[i].counter);
+	}  while (!thread_terminate && stress_continue(args));
 
 	ret = EXIT_SUCCESS;
 
-
 tidy:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
@@ -224,15 +239,29 @@ tidy:
 	}
 	overhead_nsec /= (double)benchmark_loops;
 
-	count = 0.0;
+	overrun_count = 0.0;
 	overrun_nsec = 0.0;
+	underrun_count = 0.0;
+	underrun_nsec = 0.0;
 	for (i = 0; i < n; i++) {
 		overrun_nsec += ctxts[i].overrun_nsec;
-		count += (double)ctxts[i].count;
+		overrun_count += (double)ctxts[i].overrun_count;
+		underrun_nsec += ctxts[i].underrun_nsec;
+		underrun_count += (double)ctxts[i].underrun_count;
 	}
+
+	if (underrun_count > 0.0) {
+		pr_fail("%s: detected %.0f unexpected nanosleep underruns\n",
+			args->name, underrun_count);
+		ret = EXIT_FAILURE;
+	}
+
 	overrun_nsec -= overhead_nsec;
-	overrun_nsec = (count > 0.0) ? overrun_nsec / count : 0.0;
+	overrun_nsec = (overrun_count > 0.0) ? overrun_nsec / overrun_count : 0.0;
 	stress_metrics_set(args, 0, "nanosec sleep overrun", overrun_nsec);
+	underrun_nsec -= overhead_nsec;
+	underrun_nsec = (underrun_count > 0.0) ? underrun_nsec / underrun_count : 0.0;
+	stress_metrics_set(args, 1, "nanosec sleep underrun", underrun_nsec);
 #endif
 
 	if (limited) {
@@ -253,6 +282,7 @@ stressor_info_t stress_nanosleep_info = {
 	.stressor = stress_nanosleep,
 	.class = CLASS_INTERRUPT | CLASS_SCHEDULER | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
@@ -260,6 +290,7 @@ stressor_info_t stress_nanosleep_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_INTERRUPT | CLASS_SCHEDULER | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without pthread, librt or nanosleep() system call support"
 };
