@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
 
 #if defined(HAVE_LINUX_SOCKIOS_H)
 #include <linux/sockios.h>
@@ -35,7 +36,8 @@ static const stress_help_t help[] = {
 
 #if defined(__linux__) &&	\
     defined(SIOCGIFCONF) &&	\
-    defined(HAVE_IFCONF)
+    defined(HAVE_IFCONF) &&	\
+    defined(HAVE_IFREQ)
 
 /*
  *  As per man 7 netdevice advise, workaround glibc 2.1 missing
@@ -94,7 +96,7 @@ static int stress_netdev(const stress_args_t *args)
 		struct ifconf ifc;
 
 		/* Get list of transport layer addresses */
-		(void)memset(&ifc, 0, sizeof(ifc));
+		(void)shim_memset(&ifc, 0, sizeof(ifc));
 		rc = ioctl(fd, SIOCGIFCONF, &ifc);
 		if (rc < 0) {
 			pr_fail("%s: ioctl SIOCGIFCONF failed, errno=%d (%s)\n",
@@ -229,11 +231,27 @@ static int stress_netdev(const stress_args_t *args)
 #if defined(SIOCGIFLINK)
 			/* Get if link, not implemented */
 			STRESS_NETDEV_CHECK(args, ifr, fd, SIOCGIFLINK);
+#endif
+#if defined(SIOCGIFNAME)
+			/* Get name with illegal index */
+			ifr->ifr_ifindex = (int)stress_mwc32();
+			if (ioctl(fd, SIOCGIFNAME, ifr) < 0)
+				continue;
 #endif
 		}
+
+		/* Use invalid ifc_len */
+		if (n > 1) {
+			ifc.ifc_len = (int)(sizeof(struct ifreq) - 1);
+			VOID_RET(int, ioctl(fd, SIOCGIFCONF, &ifc));
+		}
+
+		ifc.ifc_len = -1;
+		VOID_RET(int, ioctl(fd, SIOCGIFCONF, &ifc));
+
 		free(ifc.ifc_buf);
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
@@ -252,6 +270,7 @@ stressor_info_t stress_netdev_info = {
 stressor_info_t stress_netdev_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_NETWORK,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without linux/sockios.h, net/if.h, struct ifconf or ioctl() SIOCGIFCONF command support"
 };
