@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
 #include "core-capabilities.h"
 
 #if defined(HAVE_LINUX_CN_PROC_H)
@@ -84,7 +85,7 @@ static int stress_netlink_task_supported(const char *name)
  *  stress_netlink_sendcmd()
  *	send a netlink cmd
  */
-static int stress_netlink_sendcmd(
+static int OPTIMIZE3 stress_netlink_sendcmd(
 	const stress_args_t *args,
 	const int sock,
 	const uint16_t nlmsg_type,
@@ -98,7 +99,7 @@ static int stress_netlink_sendcmd(
 	char *nlmsgbuf;
 	ssize_t nlmsgbuf_len;
 	struct sockaddr_nl addr;
-	stress_nlmsg_t nlmsg;
+	stress_nlmsg_t nlmsg ALIGN64;
 
 	nlmsg.n.nlmsg_len = NLMSG_LENGTH(GENL_HDRLEN);
 	nlmsg.n.nlmsg_type = nlmsg_type;
@@ -117,11 +118,11 @@ static int stress_netlink_sendcmd(
 	nlmsgbuf = (char *)&nlmsg;
 	nlmsgbuf_len = nlmsg.n.nlmsg_len;
 
-	(void)memset(&addr, 0, sizeof(addr));
+	(void)shim_memset(&addr, 0, sizeof(addr));
 	addr.nl_family = AF_NETLINK;
 
 	while (nlmsgbuf_len > 0) {
-		ssize_t len;
+		register ssize_t len;
 
 		len = sendto(sock, nlmsgbuf, (size_t)nlmsgbuf_len, 0,
 			(struct sockaddr *)&addr, sizeof(addr));
@@ -144,7 +145,7 @@ static int stress_netlink_sendcmd(
  * 	check that the pid matches and involuntary context
  *	switches are incrementing over time.
  */
-static void stress_parse_payload(
+static void OPTIMIZE3 stress_parse_payload(
 	const stress_args_t *args,
 	struct nlattr *na,
 	const pid_t pid,
@@ -162,7 +163,7 @@ static void stress_parse_payload(
 		switch (na->nla_type) {
 		case TASKSTATS_TYPE_PID:
 			task_pid = *(pid_t *)NLA_DATA(na);
-			if (task_pid != pid) {
+			if (UNLIKELY(task_pid != pid)) {
 				pr_fail("%s: TASKSTATS_TYPE_PID got pid %" PRIdMAX ", "
 					"expected %" PRIdMAX "\n",
 					args->name,
@@ -171,7 +172,7 @@ static void stress_parse_payload(
 			break;
 		case TASKSTATS_TYPE_STATS:
 			t = (struct taskstats *)NLA_DATA(na);
-			if ((uint64_t)t->nivcsw < *nivcsw) {
+			if (UNLIKELY((uint64_t)t->nivcsw < *nivcsw)) {
 				pr_fail("%s: TASKSTATS_TYPE_STATS got %"
 					PRIu64 " involuntary context switches, "
 					"expected at least %" PRIu64 "\n",
@@ -189,7 +190,7 @@ static void stress_parse_payload(
  *   stress_netlink_taskstats_monitor()
  *	monitor parent's activity using taskstats info
  */
-static int stress_netlink_taskstats_monitor(
+static int OPTIMIZE3 stress_netlink_taskstats_monitor(
 	const stress_args_t *args,
 	const int sock,
 	const pid_t pid,
@@ -197,7 +198,7 @@ static int stress_netlink_taskstats_monitor(
 	uint64_t *nivcsw)
 {
 	do {
-		stress_nlmsg_t msg;
+		stress_nlmsg_t msg ALIGN64;
 		ssize_t msg_len, len;
 		int ret;
 		pid_t pid_data = pid;
@@ -208,15 +209,15 @@ static int stress_netlink_taskstats_monitor(
 						TASKSTATS_CMD_ATTR_PID,
 						&pid_data,
 						(uint16_t)sizeof(pid_data));
-		if (ret < 0) {
+		if (UNLIKELY(ret < 0)) {
 			pr_fail("%s: sendto TASKSTATS_CMD_GET failed: %d (%s)\n",
 				args->name, errno, strerror(errno));
 			break;
 		}
 
-		(void)memset(&msg, 0, sizeof(msg));
+		(void)shim_memset(&msg, 0, sizeof(msg));
 		msg_len = recv(sock, &msg, sizeof(msg), 0);
-		if (msg_len < 0)
+		if (UNLIKELY(msg_len < 0))
 			continue;
 
 		if (!NLMSG_OK((&msg.n), (unsigned int)msg_len)) {
@@ -231,8 +232,8 @@ static int stress_netlink_taskstats_monitor(
 			if (na->nla_type == TASKSTATS_TYPE_AGGR_PID)
 				stress_parse_payload(args, na, pid, nivcsw);
 		}
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	return 0;
 }
@@ -247,7 +248,7 @@ static int stress_netlink_task(const stress_args_t *args)
 	ssize_t len;
 	struct sockaddr_nl addr;
 	struct nlattr *na;
-	stress_nlmsg_t nlmsg;
+	stress_nlmsg_t nlmsg ALIGN64;
 	const pid_t pid = getpid();
 	uint16_t id;
 	uint64_t nivcsw = 0ULL;	/* number of involuntary context switches */
@@ -264,7 +265,7 @@ static int stress_netlink_task(const stress_args_t *args)
 		return EXIT_FAILURE;
 	}
 
-	(void)memset(&addr, 0, sizeof(addr));
+	(void)shim_memset(&addr, 0, sizeof(addr));
 	addr.nl_family = AF_NETLINK;
 
 	if (bind(sock, (struct sockaddr *)&addr, sizeof(addr)) < 0) {
@@ -298,7 +299,7 @@ static int stress_netlink_task(const stress_args_t *args)
 	na = (struct nlattr *)(uintptr_t)GENL_MSG_DATA(&nlmsg);
 	na = (struct nlattr *)(uintptr_t)((char *) na + NLA_ALIGN(na->nla_len));
 	if (na->nla_type == CTRL_ATTR_FAMILY_ID) {
-		uint16_t *id_ptr = (uint16_t *)NLA_DATA(na);
+		const uint16_t *id_ptr = (uint16_t *)NLA_DATA(na);
 
 		id = *id_ptr;
 	} else {
@@ -312,7 +313,7 @@ static int stress_netlink_task(const stress_args_t *args)
 	do {
 		if (stress_netlink_taskstats_monitor(args, sock, pid, id, &nivcsw) < 0)
 			break;
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 #if 0
@@ -336,6 +337,7 @@ stressor_info_t stress_netlink_task_info = {
 stressor_info_t stress_netlink_task_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_SCHEDULER | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without linux/connector.h, linux/netlink.h, linux/cn_proc.h, linux/taskstats.h or linux/genetlink.h support"
 };
