@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
 #include "core-capabilities.h"
 
 #if defined(HAVE_LINUX_MEMPOLICY_H)
@@ -205,11 +206,11 @@ static long stress_numa_get_mem_nodes(
 	return n;
 }
 
-static void stress_set_numa_array(void *array, uint8_t val, size_t nmemb, size_t size)
+static inline void stress_set_numa_array(void *array, uint8_t val, size_t nmemb, size_t size)
 {
 	const size_t n = nmemb * size;
 
-	(void)memset(array, val, n);
+	(void)shim_memset(array, val, n);
 }
 
 /*
@@ -227,6 +228,7 @@ static int stress_numa(const stress_args_t *args)
 	int rc = EXIT_FAILURE;
 	const bool cap_sys_nice = stress_check_capability(SHIM_CAP_SYS_NICE);
 	int *status, *dest_nodes;
+	void **pages;
 	size_t mask_elements;
 	unsigned long *node_mask, *old_node_mask;
 
@@ -273,6 +275,13 @@ static int stress_numa(const stress_args_t *args)
 		rc = EXIT_NO_RESOURCE;
 		goto status_free;
 	}
+	pages = (void **)calloc(num_pages, sizeof(*pages));
+	if (!pages) {
+		pr_inf_skip("%s: cannot allocate pages array of %lu elements, skipping stressor\n",
+			args->name, num_pages);
+		rc = EXIT_NO_RESOURCE;
+		goto dest_nodes_free;
+	}
 
 	/*
 	 *  We need a buffer to migrate around NUMA nodes
@@ -283,7 +292,7 @@ static int stress_numa(const stress_args_t *args)
 		rc = stress_exit_status(errno);
 		pr_fail("%s: mmap'd region of %zu bytes failed\n",
 			args->name, (size_t)MMAP_SZ);
-		goto dest_nodes_free;
+		goto pages_free;
 	}
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
@@ -292,7 +301,6 @@ static int stress_numa(const stress_args_t *args)
 		int j, mode, ret;
 		long lret;
 		unsigned long i;
-		void *pages[num_pages];
 		uint8_t *ptr;
 		stress_node_t *n_tmp;
 		unsigned cpu, curr_node;
@@ -305,7 +313,7 @@ static int stress_numa(const stress_args_t *args)
 		 */
 		ret = shim_get_mempolicy(&mode, node_mask, max_nodes,
 					 buf, MPOL_F_ADDR);
-		if (ret < 0) {
+		if (UNLIKELY(ret < 0)) {
 			if (errno != ENOSYS) {
 				pr_fail("%s: get_mempolicy failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
@@ -329,11 +337,11 @@ static int stress_numa(const stress_args_t *args)
 
 		VOID_RET(int, shim_get_mempolicy(&mode, node_mask, max_nodes, buf, MPOL_F_MEMS_ALLOWED | MPOL_F_NODE));
 
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 
 		ret = shim_set_mempolicy(MPOL_PREFERRED, NULL, max_nodes);
-		if (ret < 0) {
+		if (UNLIKELY(ret < 0)) {
 			if (errno != ENOSYS) {
 				pr_fail("%s: set_mempolicy failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
@@ -341,8 +349,8 @@ static int stress_numa(const stress_args_t *args)
 			}
 		}
 
-		(void)memset(buf, 0xff, MMAP_SZ);
-		if (!keep_stressing_flag())
+		(void)shim_memset(buf, 0xff, MMAP_SZ);
+		if (!stress_continue_flag())
 			break;
 
 		/* Create a mix of _NONES options, invalid ones too */
@@ -434,18 +442,32 @@ static int stress_numa(const stress_args_t *args)
 		STRESS_SETBIT(node_mask, n->node_id);
 		lret = shim_mbind((void *)buf, MMAP_SZ, MPOL_BIND, node_mask,
 			max_nodes, MPOL_MF_STRICT);
-		if (lret < 0) {
+		if (UNLIKELY(lret < 0)) {
 			if ((errno != EIO) && (errno != ENOSYS)) {
 				pr_fail("%s: mbind failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 				goto err;
 			}
 		} else {
-			(void)memset(buf, 0xaa, MMAP_SZ);
+			(void)shim_set_mempolicy_home_node((unsigned long)buf,
+				(unsigned long)MMAP_SZ, n->node_id, 0);
+			(void)shim_memset(buf, 0xaa, MMAP_SZ);
 		}
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 
+		/*
+		 *  exericse set_mempolicy_home_node
+		 */
+		(void)shim_set_mempolicy_home_node((unsigned long)buf,
+				(unsigned long)MMAP_SZ, max_nodes - 1, 0);
+		(void)shim_set_mempolicy_home_node((unsigned long)buf,
+				(unsigned long)MMAP_SZ, 1, 0);
+		(void)shim_set_mempolicy_home_node((unsigned long)buf,
+				(unsigned long)0, n->node_id, 0);
+		(void)shim_set_mempolicy_home_node((unsigned long)buf,
+				(unsigned long)MMAP_SZ, n->node_id, 0);
+
 		/*
 		 *  mbind the buffer, now try MPOL_DEFAULT
 		 */
@@ -453,16 +475,18 @@ static int stress_numa(const stress_args_t *args)
 		STRESS_SETBIT(node_mask, n->node_id);
 		lret = shim_mbind((void *)buf, MMAP_SZ, MPOL_BIND, node_mask,
 			max_nodes, MPOL_DEFAULT);
-		if (lret < 0) {
+		if (UNLIKELY(lret < 0)) {
 			if ((errno != EIO) && (errno != ENOSYS)) {
 				pr_fail("%s: mbind failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 				goto err;
 			}
 		} else {
-			(void)memset(buf, 0x5c, MMAP_SZ);
+			(void)shim_set_mempolicy_home_node((unsigned long)buf,
+				(unsigned long)MMAP_SZ, n->node_id, 0);
+			(void)shim_memset(buf, 0x5c, MMAP_SZ);
 		}
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 
 		/* Exercise invalid start address */
@@ -528,7 +552,7 @@ static int stress_numa(const stress_args_t *args)
 		VOID_RET(long, shim_migrate_pages(args->pid, ~0UL, old_node_mask, node_mask));
 		VOID_RET(long, shim_migrate_pages(args->pid, 0, old_node_mask, node_mask));
 
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 
 		n_tmp = n;
@@ -543,15 +567,15 @@ static int stress_numa(const stress_args_t *args)
 			stress_set_numa_array(status, 0x00, num_pages, sizeof(*status));
 			lret = shim_move_pages(args->pid, num_pages, pages,
 				dest_nodes, status, MPOL_MF_MOVE);
-			if (lret < 0) {
+			if (UNLIKELY(lret < 0)) {
 				if (errno != ENOSYS) {
 					pr_fail("%s: move_pages failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
 					goto err;
 				}
 			}
-			(void)memset(buf, j, MMAP_SZ);
-			if (!keep_stressing_flag())
+			(void)shim_memset(buf, j, MMAP_SZ);
+			if (!stress_continue_flag())
 				break;
 		}
 
@@ -598,14 +622,16 @@ static int stress_numa(const stress_args_t *args)
 		pages[0] = buf;
 		VOID_RET(long, shim_move_pages(args->pid, 1, pages, NULL, status, MPOL_MF_MOVE));
 
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	rc = EXIT_SUCCESS;
 err:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	(void)munmap(buf, MMAP_SZ);
 
+pages_free:
+	free(pages);
 dest_nodes_free:
 	free(dest_nodes);
 status_free:
@@ -631,6 +657,7 @@ stressor_info_t stress_numa_info = {
 stressor_info_t stress_numa_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_CPU | CLASS_MEMORY | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without linux/mempolicy.h, get_mempolicy(), mbind(), migrate_pages(), move_pages() or set_mempolicy()"
 };
