@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-out-of-memory.h"
 
 static const stress_help_t help[] = {
 	{ NULL,	"oom-pipe N",	  "start N workers exercising large pipes" },
@@ -101,7 +102,7 @@ static int stress_oom_pipe_child(const stress_args_t *args, void *ctxt)
 	for (i = 0; i < max_pipes * 2; i++)
 		fds[i] = -1;
 
-	for (i = 0; keep_stressing(args) && (i < max_pipes); i++) {
+	for (i = 0; stress_continue(args) && (i < max_pipes); i++) {
 		int *pfd = fds + (2 * i);
 
 		if ((g_opt_flags & OPT_FLAGS_OOM_AVOID) && stress_low_memory(page_size))
@@ -134,7 +135,7 @@ static int stress_oom_pipe_child(const stress_args_t *args, void *ctxt)
 
 	do {
 		/* Set to maximum size */
-		for (i = 0, fd = fds; keep_stressing(args) && (i < max_pipes); i++, fd += 2) {
+		for (i = 0, fd = fds; stress_continue(args) && (i < max_pipes); i++, fd += 2) {
 			size_t max_size = context->max_pipe_size;
 
 			if ((fd[0] < 0) || (fd[1] < 0))
@@ -150,7 +151,7 @@ static int stress_oom_pipe_child(const stress_args_t *args, void *ctxt)
 				pipe_empty(fd[0], max_size, page_size, rd_buffer);
 		}
 		/* Set to invalid size */
-		for (i = 0, fd = fds; keep_stressing(args) && (i < max_pipes); i++, fd += 2) {
+		for (i = 0, fd = fds; stress_continue(args) && (i < max_pipes); i++, fd += 2) {
 			if ((fd[0] < 0) || (fd[1] < 0))
 				continue;
 			(void)fcntl(fd[0], F_SETPIPE_SZ, -1);
@@ -158,7 +159,7 @@ static int stress_oom_pipe_child(const stress_args_t *args, void *ctxt)
 		}
 
 		/* Set to minimum size */
-		for (i = 0, fd = fds; keep_stressing(args) && (i < max_pipes); i++, fd += 2) {
+		for (i = 0, fd = fds; stress_continue(args) && (i < max_pipes); i++, fd += 2) {
 			if ((fd[0] < 0) || (fd[1] < 0))
 				continue;
 			(void)fcntl(fd[0], F_SETPIPE_SZ, page_size);
@@ -167,8 +168,8 @@ static int stress_oom_pipe_child(const stress_args_t *args, void *ctxt)
 			if (!aggressive)
 				pipe_empty(fd[0], page_size, page_size, rd_buffer);
 		}
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	/* And close the pipes */
 clean:
@@ -237,6 +238,7 @@ stressor_info_t stress_oom_pipe_info = {
 stressor_info_t stress_oom_pipe_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_MEMORY | CLASS_OS | CLASS_PATHOLOGICAL,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without F_SETFL, F_SETPIPE_SZ or O_NONBLOCK fcntl() commands"
 };
