@@ -32,7 +32,7 @@ static const stress_help_t help[] = {
 #if defined(HAVE_PERSONALITY)
 
 /* Personalities are determined at build time */
-static const unsigned long personalities[] = {
+static const unsigned long personalities[] ALIGN64 = {
 #include "personality.h"
 };
 
@@ -69,19 +69,19 @@ static int stress_personality(const stress_args_t *args)
 			const unsigned long p = personalities[i];
 			int ret;
 
-			if (!keep_stressing_flag())
+			if (!stress_continue_flag())
 				break;
-			if (failed[i]) {
+			if (UNLIKELY(failed[i])) {
 				fails++;
 				continue;
 			}
 			ret = personality(p);
-			if (ret < 0) {
+			if (UNLIKELY(ret < 0)) {
 				failed[i] = true;
 				continue;
 			}
 			ret = personality(0xffffffffUL);
-			if (ret < 0) {
+			if (UNLIKELY(ret < 0)) {
 				pr_fail("%s: failed to get personality, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 			}
@@ -96,8 +96,8 @@ static int stress_personality(const stress_args_t *args)
 				"to be set\n", args->name, fails);
 			break;
 		}
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
@@ -116,6 +116,7 @@ stressor_info_t stress_personality_info = {
 stressor_info_t stress_personality_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without sys/personality.h or personality() system call"
 };
