@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
 
 static const stress_help_t help[] = {
 	{ NULL,	"pidfd N",	"start N workers exercising pidfd system call" },
@@ -86,7 +87,7 @@ static int stress_pidfd_supported(const char *name)
 	}
 
 	/* initialized info to be safe */
-	(void)memset(&info, 0, sizeof(info));
+	(void)shim_memset(&info, 0, sizeof(info));
 
 	/*
 	 * Exercise pidfd_send_signal with
@@ -106,7 +107,7 @@ static void stress_pidfd_reap(pid_t pid, int pidfd)
 	int status;
 
 	if (pid) {
-		(void)kill(pid, SIGKILL);
+		(void)shim_kill(pid, SIGKILL);
 		(void)shim_process_mrelease(pidfd, 0);
 		(void)shim_waitpid(pid, &status, 0);
 	}
@@ -124,15 +125,15 @@ static int stress_pidfd(const stress_args_t *args)
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
-	while (keep_stressing(args)) {
+	while (stress_continue(args)) {
 		pid_t pid;
 
 again:
 		pid = fork();
 		if (pid < 0) {
-			if (stress_redo_fork(errno))
+			if (stress_redo_fork(args, errno))
 				goto again;
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				goto finish;
 			pr_err("%s: fork failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
@@ -218,7 +219,7 @@ again:
 			}
 			stress_pidfd_reap(pid, pidfd);
 		}
-		inc_counter(args);
+		stress_bogo_inc(args);
 	}
 finish:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
@@ -245,6 +246,7 @@ stressor_info_t stress_pidfd_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_INTERRUPT | CLASS_OS,
 	.supported = stress_pidfd_supported,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without pidfd_send_signal() system call"
 };
