@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-affinity.h"
 
 #define MIN_POLL_FDS	(1)
 #define MAX_POLL_FDS	(8192)	/* Must be never larger than 65535 */
@@ -171,7 +172,7 @@ again:
 	parent_cpu = stress_get_cpu();
 	pid = fork();
 	if (pid < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
 		if (!stress_continue(args))
 			goto tidy;
