@@ -18,6 +18,7 @@
  */
 #include "stress-ng.h"
 #include "core-arch.h"
+#include "core-builtin.h"
 #include "core-cpu.h"
 #include "core-put.h"
 
@@ -476,7 +477,7 @@ static int stress_priv_instr(const stress_args_t *args)
 
 	do {
 		ret = sigsetjmp(jmp_env, 1);
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			goto finish;
 	} while (ret == 1);
 
@@ -484,13 +485,13 @@ static int stress_priv_instr(const stress_args_t *args)
 		if (idx >= SIZEOF_ARRAY(op_info))
 			idx = 0;
 
-		inc_counter(args);
+		stress_bogo_inc(args);
 		if (op_info[idx].op_func) {
 			t_start = stress_time_now();
 			op_info[idx].op_func();
 		}
 		idx++;
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 finish:
         stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
@@ -508,15 +509,15 @@ finish:
 		if (str) {
 			int unhandled = 0;
 
-			(void)memset(str, 0, len);
+			(void)shim_memset(str, 0, len);
 			for (i = 0; i < SIZEOF_ARRAY(op_info); i++) {
 				if (!op_info[i].trapped) {
 					unhandled++;
 					if (!*str)  {
-						shim_strlcpy(str, op_info[i].instr, len);
+						(void)shim_strlcpy(str, op_info[i].instr, len);
 					} else {
-						shim_strlcat(str, ", ", len);
-						shim_strlcat(str, op_info[i].instr, len);
+						(void)shim_strlcat(str, ", ", len);
+						(void)shim_strlcat(str, op_info[i].instr, len);
 					}
 				}
 			}
@@ -532,14 +533,18 @@ finish:
 	if (page != MAP_FAILED)
 		(void)munmap(page, args->page_size);
 #endif
-
+	if ((stress_bogo_get(args) > 1) && (count < 1.0)) {
+		pr_fail("%s: attempted to execute %" PRIu64 " privileged instructions, trapped none.\n",
+			args->name, stress_bogo_get(args));
+		return EXIT_FAILURE;
+	}
 	return EXIT_SUCCESS;
 }
 
 stressor_info_t stress_priv_instr_info = {
 	.stressor = stress_priv_instr,
 	.class = CLASS_CPU,
-	.verify = VERIFY_NONE,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 
@@ -548,6 +553,7 @@ stressor_info_t stress_priv_instr_info = {
 stressor_info_t stress_priv_instr_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_CPU,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "no privileged op-code test for this architecture"
 };
