@@ -155,7 +155,8 @@ static int stress_pty(const stress_args_t *args)
 #endif
 #if defined(HAVE_TCDRAIN)
 			{
-				if (UNLIKELY(tcdrain(ptys[i].follower) < 0)) {
+				if (UNLIKELY((tcdrain(ptys[i].follower) < 0) &&
+					     (errno != EINTR))) {
 					pr_fail("%s: tcdrain on follower pty failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
 				}
