@@ -18,7 +18,9 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
 #include "core-capabilities.h"
+#include "core-mounts.h"
 
 #if defined(HAVE_SYS_QUOTA_H)
 #include <sys/quota.h>
@@ -188,13 +190,13 @@ static int do_quotas(const stress_args_t *args, stress_dev_info_t *const dev)
 	char buffer[1024];
 	quotactl_status_t status;
 
-	(void)memset(&status, 0, sizeof(status));
+	(void)shim_memset(&status, 0, sizeof(status));
 
 #if defined(Q_GETQUOTA)
-	if (keep_stressing_flag()) {
+	if (stress_continue_flag()) {
 		struct dqblk dqblk;
 
-		(void)memset(&dqblk, 0, sizeof(dqblk));
+		(void)shim_memset(&dqblk, 0, sizeof(dqblk));
 		err = do_quotactl(args, "Q_GETQUOTA", &status,
 			QCMD(Q_GETQUOTA, USRQUOTA),
 			dev, 0, (caddr_t)&dqblk);
@@ -203,10 +205,10 @@ static int do_quotas(const stress_args_t *args, stress_dev_info_t *const dev)
 	}
 #endif
 #if defined(Q_GETNEXTQUOTA)
-	if (keep_stressing_flag()) {
+	if (stress_continue_flag()) {
 		struct shim_nextdqblk nextdqblk;
 
-		(void)memset(&nextdqblk, 0, sizeof(nextdqblk));
+		(void)shim_memset(&nextdqblk, 0, sizeof(nextdqblk));
 		err = do_quotactl(args, "Q_GETNEXTQUOTA", &status,
 			QCMD(Q_GETNEXTQUOTA, USRQUOTA),
 			dev, 0, (caddr_t)&nextdqblk);
@@ -215,10 +217,10 @@ static int do_quotas(const stress_args_t *args, stress_dev_info_t *const dev)
 	}
 #endif
 #if defined(Q_GETFMT)
-	if (keep_stressing_flag()) {
+	if (stress_continue_flag()) {
 		uint32_t format;
 
-		(void)memset(&format, 0, sizeof(format));
+		(void)shim_memset(&format, 0, sizeof(format));
 		err = do_quotactl(args, "Q_GETFMT", &status,
 			QCMD(Q_GETFMT, USRQUOTA),
 			dev, 0, (caddr_t)&format);
@@ -227,10 +229,10 @@ static int do_quotas(const stress_args_t *args, stress_dev_info_t *const dev)
 	}
 #endif
 #if defined(Q_GETINFO)
-	if (keep_stressing_flag()) {
+	if (stress_continue_flag()) {
 		struct dqinfo dqinfo;
 
-		(void)memset(&dqinfo, 0, sizeof(dqinfo));
+		(void)shim_memset(&dqinfo, 0, sizeof(dqinfo));
 		err = do_quotactl(args, "Q_GETINFO", &status,
 			QCMD(Q_GETINFO, USRQUOTA),
 			dev, 0, (caddr_t)&dqinfo);
@@ -240,10 +242,10 @@ static int do_quotas(const stress_args_t *args, stress_dev_info_t *const dev)
 #endif
 #if defined(Q_GETSTATS)
 	/* Obsolete in recent kernels */
-	if (keep_stressing_flag()) {
+	if (stress_continue_flag()) {
 		struct dqstats dqstats;
 
-		(void)memset(&dqstats, 0, sizeof(dqstats));
+		(void)shim_memset(&dqstats, 0, sizeof(dqstats));
 		err = do_quotactl(args, "Q_GETSTATS", &status,
 			QCMD(Q_GETSTATS, USRQUOTA),
 			dev, 0, (caddr_t)&dqstats);
@@ -252,7 +254,7 @@ static int do_quotas(const stress_args_t *args, stress_dev_info_t *const dev)
 	}
 #endif
 #if defined(Q_SYNC)
-	if (keep_stressing_flag()) {
+	if (stress_continue_flag()) {
 		err = do_quotactl(args, "Q_SYNC", &status,
 			QCMD(Q_SYNC, USRQUOTA),
 			dev, 0, 0);
@@ -268,7 +270,7 @@ static int do_quotas(const stress_args_t *args, stress_dev_info_t *const dev)
 	{
 		struct dqinfo dqinfo;
 
-		(void)memset(&dqinfo, 0, sizeof(dqinfo));
+		(void)shim_memset(&dqinfo, 0, sizeof(dqinfo));
 		VOID_RET(int, quotactl(QCMD(Q_GETQUOTA, USRQUOTA), "", 0, (caddr_t)&dqinfo));
 		VOID_RET(int, quotactl(QCMD(Q_GETQUOTA, USRQUOTA), dev->name, ~0, (caddr_t)&dqinfo));
 		VOID_RET(int, quotactl(QCMD(Q_GETQUOTA, -1), dev->name, ~0, (caddr_t)&dqinfo));
@@ -333,8 +335,8 @@ static int stress_quota(const stress_args_t *args)
 	struct dirent *d;
 	struct stat buf;
 
-	(void)memset(mnts, 0, sizeof(mnts));
-	(void)memset(devs, 0, sizeof(devs));
+	(void)shim_memset(mnts, 0, sizeof(mnts));
+	(void)shim_memset(devs, 0, sizeof(devs));
 
 	n_mounts = stress_mount_get(mnts, SIZEOF_ARRAY(mnts));
 
@@ -409,7 +411,7 @@ static int stress_quota(const stress_args_t *args)
 		do {
 			int failed = 0, skipped = 0;
 
-			for (i = 0; keep_stressing_flag() && (i < n_devs); i++) {
+			for (i = 0; stress_continue_flag() && (i < n_devs); i++) {
 				int ret;
 
 				/* This failed before, so don't re-test */
@@ -434,7 +436,7 @@ static int stress_quota(const stress_args_t *args)
 					break;
 				}
 			}
-			inc_counter(args);
+			stress_bogo_inc(args);
 
 			/*
 			 * Accounting not on for all the devices?
@@ -450,7 +452,7 @@ static int stress_quota(const stress_args_t *args)
 			/* All failed, then give up */
 			if (failed == n_devs)
 				goto tidy;
-		} while (keep_stressing(args));
+		} while (stress_continue(args));
 	}
 abort:
 	rc = EXIT_SUCCESS;
@@ -476,6 +478,7 @@ stressor_info_t stress_quota_info = {
 stressor_info_t stress_quota_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without sys/quota.h or only supported on Linux"
 };
