@@ -397,9 +397,9 @@ cleanup_fd:
 skip_fsopen:
 
 #endif
-		inc_counter(args);
-	} while (keep_mounting && keep_stressing(args) &&
-		 (!args->max_ops || (get_counter(args) < args->max_ops)));
+		stress_bogo_inc(args);
+	} while (keep_mounting && stress_continue(args) &&
+		 (!args->max_ops || (stress_bogo_get(args) < args->max_ops)));
 
 cleanup:
 	stress_ramfs_umount(args, realpathname);
@@ -420,14 +420,14 @@ static int stress_ramfs_mount(const stress_args_t *args)
 
 	do {
 again:
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 
 		pid = fork();
 		if (pid < 0) {
-			if (stress_redo_fork(errno))
+			if (stress_redo_fork(args, errno))
 				goto again;
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				goto finish;
 			pr_err("%s: fork failed: errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
@@ -440,8 +440,8 @@ again:
 				if (errno != EINTR) {
 					pr_dbg("%s: waitpid(): errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
-					(void)kill(pid, SIGTERM);
-					(void)kill(pid, SIGKILL);
+					(void)shim_kill(pid, SIGTERM);
+					(void)shim_kill(pid, SIGKILL);
 				}
 				(void)shim_waitpid(pid, &status, 0);
 			} else if (WIFSIGNALED(status)) {
@@ -463,7 +463,7 @@ again:
 		} else {
 			_exit(stress_ramfs_child(args));
 		}
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 finish:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
@@ -485,6 +485,7 @@ stressor_info_t stress_ramfs_info = {
 	.class = CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
 	.supported = stress_ramfs_supported,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without clone() or only supported on Linux"
 };
