@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-affinity.h"
 #include "core-builtin.h"
 #include "core-capabilities.h"
 #include "core-net.h"
@@ -115,7 +116,8 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
 };
 
 #if defined(HAVE_LINUX_UDP_H) &&	\
-    defined(HAVE_LINUX_IF_PACKET_H)
+    defined(HAVE_LINUX_IF_PACKET_H) &&	\
+    defined(HAVE_IFREQ)
 /*
  *  stress_rawpkt_sockopts()
  *	fetch some SOL_PACKET specific stats, ignore failures
@@ -523,7 +525,7 @@ again:
 	parent_cpu = stress_get_cpu();
 	pid = fork();
 	if (pid < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
 		if (!stress_continue(args)) {
 			rc = EXIT_SUCCESS;
