@@ -18,6 +18,8 @@
  *
  */
 #include "stress-ng.h"
+#include "core-affinity.h"
+#include "core-builtin.h"
 #include "core-capabilities.h"
 #include "core-net.h"
 
@@ -38,7 +40,6 @@
 #if !defined(SOL_UDP)
 #define SOL_UDP 	(17)
 #endif
-#define PACKET_SIZE	(2048)
 
 static const stress_help_t help[] = {
 	{ NULL, "rawudp N",	"start N workers exercising raw UDP socket I/O" },
@@ -89,13 +90,14 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
 
 #if defined(HAVE_LINUX_UDP_H)
 
+#define PACKET_SIZE	sizeof(struct iphdr) + sizeof(struct udphdr) + sizeof(pid_t)
+
 /*
  *  stress_rawudp_client()
  *	client sender
  */
-static void NORETURN stress_rawudp_client(
+static void NORETURN OPTIMIZE3 stress_rawudp_client(
 	const stress_args_t *args,
-	const pid_t ppid,
 	in_addr_t addr,
 	const int port)
 {
@@ -104,13 +106,14 @@ static void NORETURN stress_rawudp_client(
 	char buf[PACKET_SIZE];
 	struct iphdr *ip = (struct iphdr *)buf;
 	struct udphdr *udp = (struct udphdr *)(buf + sizeof(struct iphdr));
+	uint8_t *data = (uint8_t *)(buf + sizeof(struct iphdr) + sizeof(struct udphdr));
 	struct sockaddr_in s_in;
 	int one = 1;
 
 	stress_parent_died_alarm();
 	(void)sched_settings_apply(true);
 
-	(void)memset(buf, 0, sizeof(buf));
+	(void)shim_memset(buf, 0, sizeof(buf));
 
 	s_in.sin_family = AF_INET;
 	s_in.sin_port = (in_port_t)port;
@@ -119,7 +122,7 @@ static void NORETURN stress_rawudp_client(
 	ip->ihl      = 5;	/* Header length in 32 bit words */
 	ip->version  = 4;	/* IPv4 */
 	ip->tos      = stress_mwc8() & 0x1e;
-	ip->tot_len  = sizeof(struct iphdr) + sizeof(struct udphdr);
+	ip->tot_len  = PACKET_SIZE;
 	ip->ttl      = 16;  	/* Not too many hops! */
 	ip->protocol = SOL_UDP;	/* UDP protocol */
 	ip->saddr = (in_addr_t)addr;
@@ -133,13 +136,13 @@ static void NORETURN stress_rawudp_client(
 		int fd;
 		ssize_t n;
 
-		if ((fd = socket(PF_INET, SOCK_RAW, IPPROTO_UDP)) < 0) {
+		if (UNLIKELY((fd = socket(PF_INET, SOCK_RAW, IPPROTO_UDP)) < 0)) {
 			pr_fail("%s: socket failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			goto err;
 		}
 
-		if (setsockopt(fd,  IPPROTO_IP, IP_HDRINCL, &one, sizeof(one)) < 0) {
+		if (UNLIKELY(setsockopt(fd,  IPPROTO_IP, IP_HDRINCL, &one, sizeof(one)) < 0)) {
 			pr_fail("%s: setsocketopt failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			(void)close(fd);
@@ -148,21 +151,21 @@ static void NORETURN stress_rawudp_client(
 
 		ip->tos = stress_mwc8() & 0x1e;
 		ip->id = htons(id++);
-		ip->check = stress_ipv4_checksum((uint16_t *)buf, sizeof(struct iphdr) + sizeof(struct udphdr));
+		ip->check = stress_ipv4_checksum((uint16_t *)buf, PACKET_SIZE);
+
+		*(pid_t *)data = args->pid;
 
 		n = sendto(fd, buf, ip->tot_len, 0, (struct sockaddr *)&s_in, sizeof(s_in));
-		if (n < 0) {
+		if (UNLIKELY(n < 0)) {
 			pr_fail("%s: raw socket sendto failed on port %d, errno=%d (%s)\n",
 				args->name, port, errno, strerror(errno));
 		}
 		(void)close(fd);
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 	rc = EXIT_SUCCESS;
 
 err:
-	/* Inform parent we're all done */
-	(void)kill(ppid, SIGALRM);
 	_exit(rc);
 }
 
@@ -170,7 +173,7 @@ err:
  *  stress_rawudp_server()
  *	server reader
  */
-static int stress_rawudp_server(
+static int OPTIMIZE3 stress_rawudp_server(
 	const stress_args_t *args,
 	in_addr_t addr,
 	const int port)
@@ -182,7 +185,9 @@ static int stress_rawudp_server(
 	char buf[PACKET_SIZE];
 	const struct iphdr *ip = (struct iphdr *)buf;
 	const struct udphdr *udp = (struct udphdr *)(buf + sizeof(struct iphdr));
+	const uint8_t *data = (uint8_t *)(buf + sizeof(struct iphdr) + sizeof(struct udphdr));
 	double t_start, duration = 0.0, bytes = 0.0, rate;
+	char msg[64];
 
 	if (stress_sig_stop_stressing(args->name, SIGALRM) < 0) {
 		rc = EXIT_FAILURE;
@@ -212,19 +217,31 @@ static int stress_rawudp_server(
 		ssize_t n;
 
 		n = recv(fd, buf, sizeof(buf), 0);
-		if (n > 0) {
+		if (LIKELY(n > 0)) {
 			if ((ip->saddr == addr) &&
 			    (ip->protocol == SOL_UDP) &&
 			    (ntohs(udp->source) == port)) {
+				if (UNLIKELY(*(const pid_t *)data != args->pid)) {
+					pr_fail("%s: data check failure, "
+						"got 0x%" PRIxMAX ", "
+						 "expected 0x%" PRIxMAX "\n",
+						args->name,
+						(intmax_t)*(const pid_t *)data,
+						(intmax_t)args->pid);
+					rc = EXIT_FAILURE;
+				}
 				bytes += (double)n;
-				inc_counter(args);
+				stress_bogo_inc(args);
 			}
 		}
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 	duration = stress_time_now() - t_start;
 	rate = (duration > 0.0) ? bytes / duration : 0.0;
 	stress_metrics_set(args, 0, "MB recv'd per sec", rate / (double)MB);
+	rate = (duration > 0.0) ? (double)stress_bogo_get(args) / duration : 0.0;
+	(void)snprintf(msg, sizeof(msg), "packets (%zu bytes) received per sec", PACKET_SIZE);
+	stress_metrics_set(args, 1, msg, rate);
 
 die_close:
 	(void)close(fd);
@@ -236,7 +253,7 @@ static void stress_sock_sigpipe_handler(int signum)
 {
 	(void)signum;
 
-	keep_stressing_set_flag(false);
+	stress_continue_set_flag(false);
 }
 
 /*
@@ -247,10 +264,13 @@ static int stress_rawudp(const stress_args_t *args)
 {
 	pid_t pid;
 	int rawudp_port = DEFAULT_RAWUDP_PORT;
-	int rc = EXIT_FAILURE, reserved_port;
+	int rc = EXIT_FAILURE, reserved_port, parent_cpu;
 	in_addr_t addr = (in_addr_t)inet_addr("127.0.0.1");
 	char *rawudp_if = NULL;
 
+	if (stress_sigchld_set_handler(args) < 0)
+		return EXIT_NO_RESOURCE;
+
 	(void)stress_get_setting("rawudp-if", &rawudp_if);
 	(void)stress_get_setting("rawudp-port", &rawudp_port);
 
@@ -285,11 +305,12 @@ static int stress_rawudp(const stress_args_t *args)
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 again:
+	parent_cpu = stress_get_cpu();
 	pid = fork();
 	if (pid < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
-		if (!keep_stressing(args)) {
+		if (!stress_continue(args)) {
 			rc = EXIT_SUCCESS;
 			goto finish;
 		}
@@ -297,12 +318,13 @@ again:
 			args->name, errno, strerror(errno));
 		return rc;
 	} else if (pid == 0) {
-		stress_rawudp_client(args, args->pid, addr, rawudp_port);
+		(void)stress_change_cpu(args, parent_cpu);
+		stress_rawudp_client(args, addr, rawudp_port);
 	} else {
 		int status;
 
 		rc = stress_rawudp_server(args, addr, rawudp_port);
-		(void)kill(pid, SIGKILL);
+		(void)shim_kill(pid, SIGKILL);
 		(void)shim_waitpid(pid, &status, 0);
 	}
 finish:
@@ -317,6 +339,7 @@ stressor_info_t stress_rawudp_info = {
 	.class = CLASS_NETWORK | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
 	.supported = stress_rawudp_supported,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
@@ -325,6 +348,7 @@ stressor_info_t stress_rawudp_info = {
 	.class = CLASS_NETWORK | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
 	.supported = stress_rawudp_supported,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without linux/udp.h or only supported on Linux"
 };
