@@ -54,17 +54,33 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
 
 typedef uint64_t	buffer_t;
 
+static void OPTIMIZE3 stress_readahead_generate_offsets(
+	off_t *offsets,
+	const uint64_t rounded_readahead_bytes)
+{
+	register size_t i;
+
+	for (i = 0; i < MAX_OFFSETS; i++)
+		offsets[i] = (off_t)stress_mwc64modn(rounded_readahead_bytes - BUF_SIZE) & ~(BUF_SIZE - 1);
+}
+
+static void OPTIMIZE3 stress_readahead_modify_offsets(off_t *offsets)
+{
+	register int i;
+
+	for (i = 0; i < MAX_OFFSETS; i++)
+		offsets[i] = ((offsets[i] * 31) >> 5) & ~(BUF_SIZE - 1);
+}
+
 static int do_readahead(
 	const stress_args_t *args,
 	const int fd,
 	const char *fs_type,
-	off_t *offsets,
-	const uint64_t rounded_readahead_bytes)
+	off_t *offsets)
 {
-	int i;
+	register int i;
 
 	for (i = 0; i < MAX_OFFSETS; i++) {
-		offsets[i] = (off_t)stress_mwc64modn(rounded_readahead_bytes - BUF_SIZE) & ~(BUF_SIZE - 1);
 		if (readahead(fd, offsets[i], BUF_SIZE) < 0) {
 			pr_fail("%s: ftruncate failed, errno=%d (%s)%s\n",
 				args->name, errno, strerror(errno), fs_type);
@@ -74,7 +90,6 @@ static int do_readahead(
 	return 0;
 }
 
-
 /*
  *  stress_readahead
  *	stress file system cache via readahead calls
@@ -92,6 +107,9 @@ static int stress_readahead(const stress_args_t *args)
 	int fd, fd_wr;
 	struct stat statbuf;
 	const char *fs_type;
+	off_t offsets[MAX_OFFSETS] ALIGN64;
+	int generate_offsets = 0;
+	const bool verify = !!(g_opt_flags & OPT_FLAGS_VERIFY);
 
 	if (!stress_get_setting("readahead-bytes", &readahead_bytes)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
@@ -125,7 +143,7 @@ static int stress_readahead(const stress_args_t *args)
 			args->name, filename, errno, strerror(errno));
 		goto finish;
 	}
-	fs_type = stress_fs_type(filename);
+	fs_type = stress_get_fs_type(filename);
 
 	/* write-only open, ignore failure */
 	fd_wr = open(filename, O_WRONLY, S_IRUSR | S_IWUSR);
@@ -159,7 +177,7 @@ static int stress_readahead(const stress_args_t *args)
 		size_t j;
 		const off_t o = i / BUF_SIZE;
 seq_wr_retry:
-		if (!keep_stressing_flag()) {
+		if (!stress_continue_flag()) {
 			pr_inf("%s: test expired during test setup "
 				"(writing of data file)\n", args->name);
 			rc = EXIT_SUCCESS;
@@ -195,19 +213,20 @@ seq_wr_retry:
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
-	do {
-		off_t offsets[MAX_OFFSETS];
+	stress_readahead_generate_offsets(offsets, rounded_readahead_bytes);
 
-		if (do_readahead(args, fd, fs_type, offsets, rounded_readahead_bytes) < 0)
+	do {
+		if (UNLIKELY(do_readahead(args, fd, fs_type, offsets) < 0))
 			goto close_finish;
 
 		for (i = 0; i < MAX_OFFSETS; i++) {
 			ssize_t pret;
 rnd_rd_retry:
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				break;
+
 			pret = pread(fd, buf, BUF_SIZE, offsets[i]);
-			if (pret <= 0) {
+			if (UNLIKELY(pret <= 0)) {
 				if ((errno == EAGAIN) || (errno == EINTR))
 					goto rnd_rd_retry;
 				if (errno) {
@@ -217,26 +236,27 @@ rnd_rd_retry:
 				}
 				continue;
 			}
-			if (pret != BUF_SIZE)
+			if (UNLIKELY(pret != BUF_SIZE))
 				misreads++;
 
-			if (g_opt_flags & OPT_FLAGS_VERIFY) {
+			if (verify) {
 				size_t j;
 				const off_t o = offsets[i] / BUF_SIZE;
 
 				for (j = 0; j < (BUF_SIZE / sizeof(*buf)); j++) {
 					const buffer_t v = (buffer_t)o + j;
-					if (buf[j] != v)
+
+					if (UNLIKELY(buf[j] != v))
 						baddata++;
 				}
-				if (baddata) {
+				if (UNLIKELY(baddata)) {
 					pr_fail("%s: error in data between 0x%jx and 0x%jx\n",
 						args->name,
 						(intmax_t)offsets[i],
 						(intmax_t)offsets[i] + BUF_SIZE - 1);
 				}
 			}
-			inc_counter(args);
+			stress_bogo_inc(args);
 		}
 
 #if defined(HAVE_POSIX_FADVISE) &&	\
@@ -259,7 +279,15 @@ rnd_rd_retry:
 		for (i = 15; i < sizeof(size_t) * 8; i += 4) {
 			VOID_RET(ssize_t, readahead(fd, 0, 1ULL << i));
 		}
-	} while (keep_stressing(args));
+
+		if (LIKELY(generate_offsets++ < 16)) {
+			stress_readahead_modify_offsets(offsets);
+		} else {
+			stress_readahead_generate_offsets(offsets, rounded_readahead_bytes);
+			generate_offsets = 0;
+		}
+
+	} while (stress_continue(args));
 
 	rc = EXIT_SUCCESS;
 close_finish:
@@ -291,6 +319,7 @@ stressor_info_t stress_readahead_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_IO | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_OPTIONAL,
 	.help = help,
 	.unimplemented_reason = "only supported on Linux"
 };
