@@ -18,6 +18,9 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
+#include "core-killpid.h"
+#include "core-out-of-memory.h"
 #include "core-pragma.h"
 
 #define RMAP_CHILD_MAX		(16)
@@ -49,34 +52,53 @@ static void MLOCKED_TEXT NORETURN stress_rmap_handler(int signum)
 {
 	(void)signum;
 
-	(void)kill(getppid(), SIGALRM);
 	_exit(0);
 }
 
-static void OPTIMIZE3 stress_rmap_touch(uint32_t *addr, const size_t sz)
+static int OPTIMIZE3 stress_rmap_touch(
+	const stress_args_t *args,
+	const int child_index,
+	uint32_t *addr,
+	const size_t sz)
 {
-	static uint32_t val = ~0;
-	register const size_t inc = 64 >> 2; /* sizeof(*ptr) */
-	register uint32_t *ptr = addr;
-	register const uint32_t *end = (uint32_t *)((uintptr_t)ptr + sz);
-	register uint32_t v = val;
+	register uintptr_t *begin = ((uintptr_t *)addr) + child_index;
+	register const uintptr_t *end = (uintptr_t *)((uintptr_t)addr + sz);
+	register uintptr_t *ptr;
+	register const size_t inc = RMAP_CHILD_MAX * sizeof(uintptr_t);
+	register uintptr_t mix = stress_mwc64();
 
-	while (LIKELY(keep_stressing_flag() && (ptr < end))) {
-		*ptr = v;
-		ptr += inc;
-		/* Bump val, never fill memory with zero val */
-		v++;
-		v = v ? v : 1;
+	/* fill and put check value in that always has lowest bit set */
+PRAGMA_UNROLL_N(8)
+	for (ptr = begin; ptr < end; ptr += inc) {
+		register uintptr_t val = (uintptr_t)ptr ^ mix;
+
+		*ptr = val;
 	}
-	val = v;
+
+	/* read back and check */
+PRAGMA_UNROLL_N(8)
+	for (ptr = begin; ptr < end; ptr += inc) {
+		register uintptr_t chk = (uintptr_t)ptr ^ mix;
+
+		if (*ptr != chk) {
+			pr_fail("%s: address 0x%p check failure, "
+				"got 0x%" PRIxPTR ", "
+				"expected 0x%" PRIxPTR "\n",
+				args->name, ptr, chk, *ptr);
+			return -1;
+		}
+	}
+	return 0;
 }
 
 static void NORETURN stress_rmap_child(
 	const stress_args_t *args,
 	const size_t page_size,
+	const int child_index,
 	uint32_t *mappings[MAPPINGS_MAX])
 {
 	const size_t sz = MAPPING_PAGES * page_size;
+	int rc = EXIT_SUCCESS;
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
@@ -88,18 +110,24 @@ static void NORETURN stress_rmap_child(
 		switch (rnd8 & 3) {
 		case 0: for (i = 0; i < MAPPINGS_MAX; i++) {
 				if (mappings[i] != MAP_FAILED) {
-					if (!inc_counter_lock(args, counter_lock, false))
+					if (!stress_bogo_inc_lock(args, counter_lock, false))
 						break;
-					stress_rmap_touch(mappings[i], sz);
+					if (stress_rmap_touch(args, child_index, mappings[i], sz) < 0) {
+						rc = EXIT_FAILURE;
+						goto fail;
+					}
 					(void)shim_msync(mappings[i], sz, sync_flag);
 				}
 			}
 			break;
 		case 1: for (i = MAPPINGS_MAX - 1; i >= 0; i--) {
 				if (mappings[i] != MAP_FAILED) {
-					if (!inc_counter_lock(args, counter_lock, false))
+					if (!stress_bogo_inc_lock(args, counter_lock, false))
 						break;
-					stress_rmap_touch(mappings[i], sz);
+					if (stress_rmap_touch(args, child_index, mappings[i], sz) < 0) {
+						rc = EXIT_FAILURE;
+						goto fail;
+					}
 					(void)shim_msync(mappings[i], sz, sync_flag);
 				}
 			}
@@ -108,29 +136,34 @@ static void NORETURN stress_rmap_child(
 				const size_t j = stress_mwc32modn(MAPPINGS_MAX);
 
 				if (mappings[j] != MAP_FAILED) {
-					if (!inc_counter_lock(args, counter_lock, false))
+					if (!stress_bogo_inc_lock(args, counter_lock, false))
 						break;
-					stress_rmap_touch(mappings[j], sz);
+					if (stress_rmap_touch(args, child_index, mappings[j], sz) < 0) {
+						rc = EXIT_FAILURE;
+						goto fail;
+					}
 					(void)shim_msync(mappings[j], sz, sync_flag);
 				}
 			}
 			break;
 		case 3: for (i = 0; i < MAPPINGS_MAX - 1; i++) {
 				if (mappings[i] != MAP_FAILED) {
-					if (!inc_counter_lock(args, counter_lock, false))
+					if (!stress_bogo_inc_lock(args, counter_lock, false))
 						break;
-					stress_rmap_touch(mappings[i], sz);
+					if (stress_rmap_touch(args, child_index, mappings[i], sz) > 0) {
+						rc = EXIT_FAILURE;
+						goto fail;
+					}
 					(void)shim_msync(mappings[i], sz, sync_flag);
 				}
 			}
 			break;
 		}
-	} while (inc_counter_lock(args, counter_lock, true));
-
-	(void)kill(getppid(), SIGALRM);
+	} while (stress_bogo_inc_lock(args, counter_lock, true));
 
+fail:
 	stress_set_proc_state(args->name, STRESS_STATE_WAIT);
-	_exit(0);
+	_exit(rc);
 }
 
 /*
@@ -141,21 +174,23 @@ static int stress_rmap(const stress_args_t *args)
 {
 	const size_t page_size = args->page_size;
 	const size_t sz = ((MAPPINGS_MAX - 1) + MAPPING_PAGES) * page_size;
-	int fd = -1;
+	int fd = -1, rc;
 	size_t i;
-	ssize_t rc;
 	pid_t pids[RMAP_CHILD_MAX];
 	uint32_t *mappings[MAPPINGS_MAX];
 	uint32_t *paddings[MAPPINGS_MAX];
 	char filename[PATH_MAX];
 
+	if (stress_sigchld_set_handler(args) < 0)
+		return EXIT_NO_RESOURCE;
+
 	counter_lock = stress_lock_create();
 	if (!counter_lock) {
 		pr_inf_skip("%s: failed to create counter lock. skipping stressor\n", args->name);
 		return EXIT_NO_RESOURCE;
 	}
 
-	(void)memset(pids, 0, sizeof(pids));
+	(void)shim_memset(pids, 0, sizeof(pids));
 
 	for (i = 0; i < MAPPINGS_MAX; i++) {
 		mappings[i] = MAP_FAILED;
@@ -163,7 +198,7 @@ static int stress_rmap(const stress_args_t *args)
 	}
 
 	/* Make sure this is killable by OOM killer */
-	stress_set_oom_adjustment(args->name, true);
+	stress_set_oom_adjustment(args, true);
 
 	rc = stress_temp_dir_mk_args(args);
 	if (rc < 0)
@@ -197,7 +232,7 @@ static int stress_rmap(const stress_args_t *args)
 	for (i = 0; i < MAPPINGS_MAX; i++) {
 		const off_t offset = (off_t)(i * page_size);
 
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			goto cleanup;
 
 		mappings[i] =
@@ -213,7 +248,7 @@ static int stress_rmap(const stress_args_t *args)
 	 *  Spawn children workers
 	 */
 	for (i = 0; i < RMAP_CHILD_MAX; i++) {
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			goto cleanup;
 
 		pids[i] = fork();
@@ -230,9 +265,9 @@ static int stress_rmap(const stress_args_t *args)
 			(void)sched_settings_apply(true);
 
 			/* Make sure this is killable by OOM killer */
-			stress_set_oom_adjustment(args->name, true);
+			stress_set_oom_adjustment(args, true);
 
-			stress_rmap_child(args, page_size, mappings);
+			stress_rmap_child(args, page_size, i, mappings);
 		}
 	}
 
@@ -241,37 +276,14 @@ static int stress_rmap(const stress_args_t *args)
 	/*
 	 *  Wait for SIGINT or SIGALRM
 	 */
-	while (inc_counter_lock(args, counter_lock, false)) {
+	while (stress_bogo_inc_lock(args, counter_lock, false)) {
 		pause();
 	}
 
 cleanup:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
-	/*
-	 *  Kill and wait for children
-	 */
-	for (i = 0; i < RMAP_CHILD_MAX; i++) {
-		if (pids[i] <= 0)
-			continue;
-		(void)stress_killpid(pids[i]);
-	}
-	for (i = 0; i < RMAP_CHILD_MAX; i++) {
-		int status, ret;
-
-		if (pids[i] <= 0)
-			continue;
-
-		ret = shim_waitpid(pids[i], &status, 0);
-		if (ret < 0) {
-			if (errno != EINTR)
-				pr_dbg("%s: waitpid(): errno=%d (%s)\n",
-					args->name, errno, strerror(errno));
-			(void)kill(pids[i], SIGTERM);
-			(void)stress_killpid(pids[i]);
-			(void)shim_waitpid(pids[i], &status, 0);
-		}
-	}
+	rc = stress_kill_and_wait_many(args, pids, RMAP_CHILD_MAX, SIGALRM, true);
 
 	for (i = 0; i < MAPPINGS_MAX; i++) {
 		if (mappings[i] != MAP_FAILED)
@@ -284,11 +296,12 @@ cleanup:
 	(void)stress_temp_dir_rm_args(args);
 	(void)stress_lock_destroy(counter_lock);
 
-	return EXIT_SUCCESS;
+	return rc;
 }
 
 stressor_info_t stress_rmap_info = {
 	.stressor = stress_rmap,
 	.class = CLASS_OS | CLASS_MEMORY,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
