@@ -18,6 +18,8 @@
  *
  */
 #include "stress-ng.h"
+#include "core-affinity.h"
+#include "core-builtin.h"
 #include "core-net.h"
 
 #if defined(HAVE_SYS_UN_H)
@@ -380,7 +382,7 @@ static void stress_sctp_sockopts(const int fd)
  *  stress_sctp_client()
  *	client reader
  */
-static int stress_sctp_client(
+static int OPTIMIZE3 stress_sctp_client(
 	const stress_args_t *args,
 	const pid_t mypid,
 	const int sctp_port,
@@ -389,6 +391,7 @@ static int stress_sctp_client(
 	const char *sctp_if)
 {
 	struct sockaddr *addr;
+	int rc = EXIT_SUCCESS;
 
 	(void)sctp_sched;
 
@@ -404,27 +407,28 @@ static int stress_sctp_client(
 		struct sctp_event_subscribe events;
 #endif
 retry:
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			return EXIT_FAILURE;
-		if ((fd = socket(sctp_domain, SOCK_STREAM, IPPROTO_SCTP)) < 0) {
+		fd = socket(sctp_domain, SOCK_STREAM, IPPROTO_SCTP);
+		if (UNLIKELY(fd < 0)) {
 			if (errno == EPROTONOSUPPORT) {
 				if (args->instance == 0)
 					pr_inf_skip("%s: SCTP protocol not supported, skipping stressor\n",
 						args->name);
-				return EXIT_FAILURE;
+				return EXIT_NO_RESOURCE;
 			}
 			pr_fail("%s: socket failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			return EXIT_FAILURE;
 		}
 
-		if (stress_set_sockaddr_if(args->name, args->instance, mypid,
+		if (UNLIKELY(stress_set_sockaddr_if(args->name, args->instance, mypid,
 				sctp_domain, sctp_port, sctp_if,
-				&addr, &addr_len, NET_ADDR_LOOPBACK) < 0) {
+				&addr, &addr_len, NET_ADDR_LOOPBACK) < 0)) {
 			(void)close(fd);
 			return EXIT_FAILURE;
 		}
-		if (connect(fd, addr, addr_len) < 0) {
+		if (UNLIKELY(connect(fd, addr, addr_len) < 0)) {
 			const int save_errno = errno;
 
 			(void)close(fd);
@@ -439,10 +443,9 @@ retry:
 			goto retry;
 		}
 #if defined(HAVE_SCTP_EVENT_SUBSCRIBE)
-		(void)memset(&events, 0, sizeof(events));
+		(void)shim_memset(&events, 0, sizeof(events));
 		events.sctp_data_io_event = 1;
-		if (setsockopt(fd, SOL_SCTP, SCTP_EVENTS, &events,
-			sizeof(events)) < 0) {
+		if (UNLIKELY(setsockopt(fd, SOL_SCTP, SCTP_EVENTS, &events, sizeof(events)) < 0)) {
 			(void)close(fd);
 			pr_fail("%s: setsockopt failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
@@ -454,7 +457,7 @@ retry:
 		if (sctp_sched > -1) {
 			struct sctp_assoc_value val;
 
-			(void)memset(&val, 0, sizeof(val));
+			(void)shim_memset(&val, 0, sizeof(val));
 			val.assoc_value = (uint32_t)sctp_sched;
 			(void)setsockopt(fd, SOL_SCTP, SCTP_STREAM_SCHEDULER, &val, sizeof(val));
 		}
@@ -467,33 +470,46 @@ retry:
 
 			n = sctp_recvmsg(fd, buf, sizeof(buf),
 				NULL, 0, &sndrcvinfo, &flags);
-			if (n <= 0)
+			if (UNLIKELY(n <= 0))
 				break;
-		} while (keep_stressing(args));
+			if (n >= (ssize_t)sizeof(pid_t)) {
+				const pid_t *pidptr = (pid_t *)buf;
+				const pid_t pid = *pidptr;
+
+				if (UNLIKELY(pid != mypid)) {
+					pr_fail("%s: server received unexpected data "
+						"contents, got 0x%" PRIxMAX ", "
+						"expected 0x%" PRIxMAX "\n",
+						args->name, (intmax_t)pid,
+						(intmax_t)mypid);
+					rc = EXIT_FAILURE;
+					break;
+				}
+			}
+		} while (stress_continue_flag());
 		(void)shutdown(fd, SHUT_RDWR);
 		(void)close(fd);
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 #if defined(AF_UNIX) &&		\
     defined(HAVE_SOCKADDR_UN)
 	if (sctp_domain == AF_UNIX) {
-		struct sockaddr_un *addr_un = (struct sockaddr_un *)addr;
+		const struct sockaddr_un *addr_un = (struct sockaddr_un *)addr;
 
 		(void)shim_unlink(addr_un->sun_path);
 	}
 #else
 	UNEXPECTED
 #endif
-	return EXIT_SUCCESS;
+	return rc;
 }
 
 /*
  *  stress_sctp_server()
  *	server writer
  */
-static int stress_sctp_server(
+static int OPTIMIZE3 stress_sctp_server(
 	const stress_args_t *args,
-	const pid_t pid,
 	const pid_t mypid,
 	const int sctp_port,
 	const int sctp_domain,
@@ -501,11 +517,12 @@ static int stress_sctp_server(
 	const char *sctp_if)
 {
 	char ALIGN64 buf[SOCKET_BUF];
-	int fd, status;
+	int fd;
 	int so_reuseaddr = 1;
 	socklen_t addr_len = 0;
 	struct sockaddr *addr = NULL;
 	int rc = EXIT_SUCCESS;
+	int index = 0;
 
 	(void)sctp_sched;
 
@@ -513,12 +530,13 @@ static int stress_sctp_server(
 		rc = EXIT_FAILURE;
 		goto die;
 	}
-	if ((fd = socket(sctp_domain, SOCK_STREAM, IPPROTO_SCTP)) < 0) {
+	fd = socket(sctp_domain, SOCK_STREAM, IPPROTO_SCTP);
+	if (fd < 0) {
 		if (errno == EPROTONOSUPPORT) {
 			if (args->instance == 0)
 				pr_inf_skip("%s: SCTP protocol not supported, skipping stressor\n",
 					args->name);
-			rc = EXIT_NOT_IMPLEMENTED;
+			rc = EXIT_NO_RESOURCE;
 			goto die;
 		}
 		rc = stress_exit_status(errno);
@@ -572,57 +590,52 @@ static int stress_sctp_server(
 	if (sctp_sched > -1) {
 		struct sctp_assoc_value val;
 
-		(void)memset(&val, 0, sizeof(val));
+		(void)shim_memset(&val, 0, sizeof(val));
 		val.assoc_value = (uint32_t)sctp_sched;
 		(void)setsockopt(fd, SOL_SCTP, SCTP_STREAM_SCHEDULER, &val, sizeof(val));
 	}
 #endif
-
 	do {
 		int sfd;
 
-		if (!keep_stressing(args))
+		if (UNLIKELY(!stress_continue(args)))
 			break;
 
 		sfd = accept(fd, (struct sockaddr *)NULL, NULL);
-		if (sfd >= 0) {
+		if (LIKELY(sfd >= 0)) {
 			size_t i;
+			const int c = stress_ascii32[index++ & 0x1f];
+			pid_t *pidptr = (pid_t *)buf;
 
-			(void)memset(buf, 'A' + (get_counter(args) % 26), sizeof(buf));
-
+			(void)shim_memset(buf, c, sizeof(buf));
+			*pidptr = mypid;
 			for (i = 16; i < sizeof(buf); i += 16) {
 				ssize_t ret = sctp_sendmsg(sfd, buf, i,
 						NULL, 0, 0, 0,
 						LOCALTIME_STREAM, 0, 0);
-				if (ret < 0) {
+				if (UNLIKELY(ret < 0))
 					break;
-				} else {
-					inc_counter(args);
-				}
+
+				stress_bogo_inc(args);
 			}
 			stress_sctp_sockopts(sfd);
 			(void)close(sfd);
 		}
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 die_close:
 	(void)close(fd);
 die:
 #if defined(AF_UNIX) &&		\
     defined(HAVE_SOCKADDR_UN)
-	if (addr && sctp_domain == AF_UNIX) {
-		struct sockaddr_un *addr_un = (struct sockaddr_un *)addr;
+	if (addr && (sctp_domain == AF_UNIX)) {
+		const struct sockaddr_un *addr_un = (struct sockaddr_un *)addr;
 
 		(void)shim_unlink(addr_un->sun_path);
 	}
 #else
 	UNEXPECTED
 #endif
-	if (pid) {
-		(void)kill(pid, SIGKILL);
-		(void)shim_waitpid(pid, &status, 0);
-	}
-
 	return rc;
 }
 
@@ -643,9 +656,12 @@ static int stress_sctp(const stress_args_t *args)
 	int sctp_port = DEFAULT_SCTP_PORT;
 	int sctp_domain = AF_INET;
 	int sctp_sched = -1;	/* Undefined */
-	int ret, reserved_port;
+	int ret, reserved_port, parent_cpu;
 	char *sctp_if = NULL;
 
+	if (stress_sigchld_set_handler(args) < 0)
+		return EXIT_NO_RESOURCE;
+
 	(void)stress_get_setting("sctp-domain", &sctp_domain);
 	(void)stress_get_setting("sctp-if", &sctp_if);
 	(void)stress_get_setting("sctp-port", &sctp_port);
@@ -680,22 +696,31 @@ static int stress_sctp(const stress_args_t *args)
 	ret = EXIT_FAILURE;
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 again:
+	parent_cpu = stress_get_cpu();
 	pid = fork();
 	if (pid < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			goto finish;
 		pr_fail("%s: fork failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		return EXIT_FAILURE;
 	} else if (pid == 0) {
+		 (void)stress_change_cpu(args, parent_cpu);
 		ret = stress_sctp_client(args, mypid, sctp_port, sctp_domain, sctp_sched, sctp_if);
-		/* Inform parent we're all done */
-		(void)kill(getppid(), SIGALRM);
 		_exit(ret);
 	} else {
-		ret = stress_sctp_server(args, pid, mypid, sctp_port, sctp_domain, sctp_sched, sctp_if);
+		int status;
+
+		ret = stress_sctp_server(args, mypid, sctp_port, sctp_domain, sctp_sched, sctp_if);
+		(void)shim_kill(pid, SIGKILL);
+		(void)shim_waitpid(pid, &status, 0);
+		if (WIFEXITED(status)) {
+			if (WEXITSTATUS(status) != EXIT_SUCCESS) {
+				ret = WEXITSTATUS(status);
+			}
+		}
 	}
 
 finish:
@@ -712,6 +737,7 @@ stressor_info_t stress_sctp_info = {
 	.stressor = stress_sctp,
 	.class = CLASS_NETWORK,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
@@ -719,6 +745,7 @@ stressor_info_t stress_sctp_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_NETWORK,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without netinet/sctp.h or libsctp support"
 };
