@@ -18,6 +18,8 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
+#include "core-killpid.h"
 
 #if defined(HAVE_SEM_SYSV)
 #include <sys/sem.h>
@@ -92,7 +94,8 @@ static void stress_semaphore_sysv_init(void)
 		(void)semctl(sem_id, 0, IPC_RMID);
 
 	while (count < 100) {
-		g_shared->sem_sysv.key_id = (key_t)stress_mwc16();
+		/* use odd key so it won't clash with even core-resource sem keys */
+		g_shared->sem_sysv.key_id = (key_t)stress_mwc16() | 1;
 		g_shared->sem_sysv.sem_id =
 			semget(g_shared->sem_sysv.key_id, 3,
 				IPC_CREAT | S_IRUSR | S_IWUSR);
@@ -113,16 +116,6 @@ static void stress_semaphore_sysv_init(void)
 		/* Clean up */
 		(void)semctl(g_shared->sem_sysv.sem_id, 0, IPC_RMID);
 	}
-
-	if (g_opt_sequential) {
-		pr_inf_skip("semaphore init (System V) failed: errno=%d: "
-			"(%s), skipping semaphore stressor\n",
-			errno, strerror(errno));
-	} else {
-		pr_fail("semaphore init (System V) failed: errno=%d: "
-			"(%s)\n", errno, strerror(errno));
-		_exit(EXIT_FAILURE);
-	}
 }
 
 /*
@@ -166,7 +159,7 @@ static void stress_semaphore_sysv_get_procinfo(bool *get_procinfo)
  *  stress_semaphore_sysv_thrash()
  *	exercise the semaphore
  */
-static int stress_semaphore_sysv_thrash(const stress_args_t *args)
+static int OPTIMIZE3 stress_semaphore_sysv_thrash(const stress_args_t *args)
 {
 	const int sem_id = g_shared->sem_sysv.sem_id;
 	int rc = EXIT_SUCCESS;
@@ -187,8 +180,8 @@ static int stress_semaphore_sysv_thrash(const stress_args_t *args)
 
 #if defined(HAVE_SEMTIMEDOP)
 		if (got_semtimedop) {
-			struct timespec timeout;
-			struct sembuf sems[STRESS_MAX_SEMS * 3];
+			struct timespec timeout ALIGN64;
+			struct sembuf sems[STRESS_MAX_SEMS * 3] ALIGN64;
 
 			for (i = 0; i < STRESS_MAX_SEMS * 3; i += 3) {
 				sems[i].sem_num = 1;
@@ -210,7 +203,6 @@ static int stress_semaphore_sysv_thrash(const stress_args_t *args)
 #else
 		UNEXPECTED
 #endif
-
 		for (i = 0; i < 1000; i++) {
 			struct sembuf semwait, semsignal;
 
@@ -224,12 +216,12 @@ static int stress_semaphore_sysv_thrash(const stress_args_t *args)
 
 #if defined(HAVE_SEMTIMEDOP)
 			if (got_semtimedop) {
-				struct timespec timeout;
+				struct timespec timeout ALIGN64;
 
 				timeout.tv_sec = 1;
 				timeout.tv_nsec = 0;
 				ret = semtimedop(sem_id, &semwait, 1, &timeout);
-				if (ret < 0 && ((errno == ENOSYS) || (errno == EINVAL))) {
+				if ((ret < 0) && ((errno == ENOSYS) || (errno == EINVAL))) {
 					got_semtimedop = false;
 					ret = semop(sem_id, &semwait, 1);
 				}
@@ -239,7 +231,7 @@ static int stress_semaphore_sysv_thrash(const stress_args_t *args)
 #else
 			ret = semop(sem_id, &semwait, 1);
 #endif
-			if (ret < 0) {
+			if (UNLIKELY(ret < 0)) {
 				if (errno == EAGAIN)
 					goto timed_out;
 				if (errno != EINTR) {
@@ -249,7 +241,7 @@ static int stress_semaphore_sysv_thrash(const stress_args_t *args)
 				}
 				break;
 			}
-			if (semop(sem_id, &semsignal, 1) < 0) {
+			if (UNLIKELY(semop(sem_id, &semsignal, 1) < 0)) {
 				if (errno != EINTR) {
 					pr_fail("%s: semop signal failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
@@ -258,20 +250,22 @@ static int stress_semaphore_sysv_thrash(const stress_args_t *args)
 				break;
 			}
 timed_out:
-			if (!keep_stressing(args))
+			stress_bogo_inc(args);
+			if (UNLIKELY(!stress_continue(args)))
 				break;
-			inc_counter(args);
 		}
 #if defined(IPC_STAT)
 		{
 			struct semid_ds ds;
 			stress_semun_t s;
+#if defined(GETALL)
 			size_t nsems;
+#endif
 
-			(void)memset(&ds, 0, sizeof(ds));
+			(void)shim_memset(&ds, 0, sizeof(ds));
 
 			s.buf = &ds;
-			if (semctl(sem_id, 2, IPC_STAT, &s) < 0) {
+			if (UNLIKELY(semctl(sem_id, 2, IPC_STAT, &s) < 0)) {
 				pr_fail("%s: semctl IPC_STAT failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 				rc = EXIT_FAILURE;
@@ -283,7 +277,6 @@ timed_out:
 				UNEXPECTED
 #endif
 			}
-
 #if defined(GETALL)
 			/* Avoid zero array size allocation */
 			nsems = ds.sem_nsems;
@@ -345,7 +338,7 @@ timed_out:
 			stress_semun_t s;
 
 			s.__buf = &si;
-			if (semctl(sem_id, 0, IPC_INFO, &s) < 0) {
+			if (UNLIKELY(semctl(sem_id, 0, IPC_INFO, &s) < 0)) {
 				pr_fail("%s: semctl IPC_INFO failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 				rc = EXIT_FAILURE;
@@ -373,7 +366,7 @@ timed_out:
 			stress_semun_t s;
 
 			s.__buf = &si;
-			if (semctl(sem_id, 0, SEM_INFO, &s) < 0) {
+			if (UNLIKELY(semctl(sem_id, 0, SEM_INFO, &s) < 0)) {
 				pr_fail("%s: semctl SEM_INFO failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 				rc = EXIT_FAILURE;
@@ -388,6 +381,8 @@ timed_out:
 			VOID_RET(int, semctl(sem_id, ~0, IPC_INFO, &s));
 		}
 #endif
+		if (UNLIKELY(!stress_continue(args)))
+			break;
 #if defined(GETVAL)
 		VOID_RET(int, semctl(sem_id, 0, GETVAL, 0));
 
@@ -464,6 +459,8 @@ timed_out:
 			timeout.tv_sec = 0;
 			timeout.tv_nsec = 10000;
 			VOID_RET(int, semtimedop(sem_id, &semwait, (size_t)-1, &timeout));
+			if (UNLIKELY(!stress_continue(args)))
+				break;
 		}
 #else
 		UNEXPECTED
@@ -508,10 +505,7 @@ timed_out:
     defined(HAVE_SYSCALL)
 		(void)syscall(__NR_semctl, sem_id, 0, INT_MAX, 0);
 #endif
-	} while ((rc == EXIT_SUCCESS)  && keep_stressing(args));
-
-	if (rc == EXIT_FAILURE)
-		(void)kill(getppid(), SIGALRM);
+	} while ((rc == EXIT_SUCCESS) && stress_continue(args));
 
 	return rc;
 }
@@ -527,7 +521,7 @@ static pid_t semaphore_sysv_spawn(const stress_args_t *args)
 again:
 	pid = fork();
 	if (pid < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
 		return -1;
 	}
@@ -551,6 +545,14 @@ static int stress_sem_sysv(const stress_args_t *args)
 	uint64_t semaphore_sysv_procs = DEFAULT_SEM_SYSV_PROCS;
 	int rc = EXIT_SUCCESS;
 
+	if (stress_sigchld_set_handler(args) < 0)
+		return EXIT_NO_RESOURCE;
+
+	if (!g_shared->sem_sysv.init) {
+		pr_inf_skip("%s: skipping stressor, semaphore not initialised\n", args->name);
+		return EXIT_NO_RESOURCE;
+	}
+
 	if (!stress_get_setting("sem-sysv-procs", &semaphore_sysv_procs)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
 			semaphore_sysv_procs = MAX_SEM_SYSV_PROCS;
@@ -558,44 +560,25 @@ static int stress_sem_sysv(const stress_args_t *args)
 			semaphore_sysv_procs = MIN_SEM_SYSV_PROCS;
 	}
 
-	if (!g_shared->sem_sysv.init) {
-		pr_err("%s: aborting, semaphore not initialised\n", args->name);
-		return EXIT_FAILURE;
-	}
 
 	if (stress_sighandler(args->name, SIGCHLD, stress_sighandler_nop, NULL) < 0)
 		return EXIT_NO_RESOURCE;
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
-	(void)memset(pids, 0, sizeof(pids));
+	(void)shim_memset(pids, 0, sizeof(pids));
 	for (i = 0; i < semaphore_sysv_procs; i++) {
 		pids[i] = semaphore_sysv_spawn(args);
-		if (!keep_stressing_flag() || pids[i] < 0)
+		if (!stress_continue_flag() || pids[i] < 0)
 			goto reap;
 	}
 
 	/* Wait for termination */
-	while (keep_stressing(args))
+	while (stress_continue(args))
 		pause();
 reap:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
-
-	for (i = 0; i < semaphore_sysv_procs; i++) {
-		if (pids[i] > 0)
-			(void)stress_killpid(pids[i]);
-	}
-	for (i = 0; i < semaphore_sysv_procs; i++) {
-		if (pids[i] > 0) {
-			int status;
-
-			(void)shim_waitpid(pids[i], &status, 0);
-			if (WIFEXITED(status) &&
-			    (WEXITSTATUS(status) != EXIT_SUCCESS)) {
-				rc = EXIT_FAILURE;
-			}
-		}
-	}
+	stress_kill_and_wait_many(args, pids, semaphore_sysv_procs, SIGALRM, true);
 
 	return rc;
 }
@@ -614,6 +597,7 @@ stressor_info_t stress_sem_sysv_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_OS | CLASS_SCHEDULER,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without sys/sem.h"
 };
