@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
 #include "core-pthread.h"
 
 #if defined(HAVE_SEMAPHORE_H)
@@ -56,74 +57,116 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
     defined(HAVE_LIB_PTHREAD) && \
     defined(HAVE_SEM_POSIX)
 
+typedef struct {
+	pthread_t pthread;
+	int ret;
+	double trywait_count;
+	double timedwait_count;
+	double wait_count;
+} stress_sem_pthread_t;
+
 static sem_t sem;
-static pthread_t pthreads[MAX_SEM_POSIX_PROCS];
-static int p_ret[MAX_SEM_POSIX_PROCS];
+
+static stress_sem_pthread_t sem_pthreads[MAX_SEM_POSIX_PROCS] ALIGN64;
 
 /*
  *  semaphore_posix_thrash()
  *	exercise the semaphore
  */
-static void *semaphore_posix_thrash(void *arg)
+static void OPTIMIZE3 *semaphore_posix_thrash(void *arg)
 {
 	const stress_pthread_args_t *p_args = arg;
 	const stress_args_t *args = p_args->args;
+	stress_sem_pthread_t *pthread = (stress_sem_pthread_t *)p_args->data;
 	static void *nowt = NULL;
 
 	do {
-		int i;
+		int i, j = -1;
 
-		for (i = 0; keep_stressing_flag() && i < 1000; i++) {
+		for (i = 0; (i < 1000) && stress_continue_flag(); i++) {
 			int value;
+			struct timespec ts;
 
-			if (sem_getvalue(&sem, &value) < 0)
+			if (UNLIKELY(sem_getvalue(&sem, &value) < 0)) {
 				pr_fail("%s: sem_getvalue failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
+			}
+do_semwait:
+			j++;
+			if (j >= 3)
+				j = 0;
 
-			if (i & 1) {
+			switch (j) {
+			case 0:
+				/* Attempt a try wait */
 				if (sem_trywait(&sem) < 0) {
-					if (errno == 0 ||
-					    errno == EAGAIN)
-						continue;
-					if (errno != EINTR)
+					if (LIKELY(errno == EAGAIN))
+						goto do_semwait;
+					if (UNLIKELY(errno != EINTR))
 						pr_fail("%s: sem_trywait failed, errno=%d (%s)\n",
 							args->name, errno, strerror(errno));
-					break;
+					goto do_return;
 				}
-			} else {
-				struct timespec ts;
-
+				pthread->trywait_count += 1.0;
+				break;
+			case 1:
+				/* Attempt a very short timed wait */
 #if defined(CLOCK_REALTIME)
-				if (clock_gettime(CLOCK_REALTIME, &ts) < 0)
-					(void)memset(&ts, 0, sizeof(ts));
+				if (UNLIKELY(clock_gettime(CLOCK_REALTIME, &ts) < 0))
+					(void)shim_memset(&ts, 0, sizeof(ts));
 #else
-				(void)memset(&ts, 0, sizeof(ts));
+				(void)shim_memset(&ts, 0, sizeof(ts));
 #endif
+				ts.tv_nsec += 10000;
+				if (ts.tv_nsec >= 1000000000) {
+					ts.tv_nsec -= 1000000000;
+					ts.tv_sec++;
+				}
 
 				if (sem_timedwait(&sem, &ts) < 0) {
-					if (errno == 0 ||
-					    errno == EAGAIN ||
-					    errno == ETIMEDOUT)
-						continue;
-					if (errno != EINTR)
+					if (LIKELY(errno == EAGAIN ||
+						   errno == ETIMEDOUT ||
+						   errno == EINVAL))
+						goto do_semwait;
+					if (UNLIKELY(errno != EINTR))
 						pr_fail("%s: sem_timedwait failed, errno=%d (%s)\n",
 							args->name, errno, strerror(errno));
-					break;
+					goto do_return;
 				}
+				pthread->timedwait_count += 1.0;
+				break;
+			case 2:
+				if (UNLIKELY(sem_wait(&sem) < 0)) {
+					if (UNLIKELY(errno != EINTR))
+						pr_fail("%s: sem_wait failed, errno=%d (%s)\n",
+							args->name, errno, strerror(errno));
+					goto do_return;
+				}
+				pthread->wait_count += 1.0;
+				break;
+			default:
+				pr_inf("%s: bug detected, invalid state %d\n", args->name, j);
+				goto do_return;
 			}
-			inc_counter(args);
-			if (sem_post(&sem) < 0) {
+
+			/* Locked at this point, bump counter */
+			stress_bogo_inc(args);
+
+			if (UNLIKELY(sem_post(&sem) < 0)) {
 				pr_fail("%s: sem_post failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
-				break;
+				goto do_return;
 			}
-			if (stress_mwc1())
-				(void)shim_sched_yield();
-			else
-				(void)shim_usleep(0);
-		}
-	} while (keep_stressing(args));
 
+			/*
+			 *  force a scheduling yield to try and force alternative
+			 *  thread(s) to have a turn on wait/post combo
+			 */
+			shim_sched_yield();
+		}
+	} while (stress_continue(args));
+
+do_return:
 	return &nowt;
 }
 
@@ -137,6 +180,10 @@ static int stress_sem(const stress_args_t *args)
 	uint64_t i;
 	bool created = false;
 	stress_pthread_args_t p_args;
+	double wait_count = 0;
+	double trywait_count = 0;
+	double timedwait_count = 0;
+	double t, duration;
 
 	if (!stress_get_setting("sem-procs", &semaphore_posix_procs)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
@@ -152,22 +199,26 @@ static int stress_sem(const stress_args_t *args)
 		return EXIT_FAILURE;
 	}
 
-	(void)memset(pthreads, 0, sizeof(pthreads));
-	(void)memset(p_ret, 0, sizeof(p_ret));
+	(void)shim_memset(sem_pthreads, 0, sizeof(sem_pthreads));
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
+	t = stress_time_now();
 	for (i = 0; i < semaphore_posix_procs; i++) {
+		sem_pthreads[i].trywait_count = 0.0;
+		sem_pthreads[i].timedwait_count = 0.0;
+		sem_pthreads[i].wait_count = 0.0;
+
 		p_args.args = args;
-		p_args.data = NULL;
-		p_ret[i] = pthread_create(&pthreads[i], NULL,
+		p_args.data = &sem_pthreads[i];
+		sem_pthreads[i].ret = pthread_create(&sem_pthreads[i].pthread, NULL,
                                 semaphore_posix_thrash, (void *)&p_args);
-		if ((p_ret[i]) && (p_ret[i] != EAGAIN)) {
+		if ((sem_pthreads[i].ret) && (sem_pthreads[i].ret != EAGAIN)) {
 			pr_fail("%s: pthread create failed, errno=%d (%s)\n",
-				args->name, p_ret[i], strerror(p_ret[i]));
+				args->name, sem_pthreads[i].ret, strerror(sem_pthreads[i].ret));
 			break;
 		}
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 		created = true;
 	}
@@ -178,17 +229,28 @@ static int stress_sem(const stress_args_t *args)
 	}
 
 	/* Wait for termination */
-	while (keep_stressing(args))
+	while (stress_continue(args))
 		pause();
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
 	for (i = 0; i < semaphore_posix_procs; i++) {
-		if (p_ret[i])
+		if (sem_pthreads[i].ret)
 			continue;
 
-		VOID_RET(int, pthread_join(pthreads[i], NULL));
+		VOID_RET(int, pthread_join(sem_pthreads[i].pthread, NULL));
+
+		trywait_count += sem_pthreads[i].trywait_count;
+		timedwait_count += sem_pthreads[i].timedwait_count;
+		wait_count += sem_pthreads[i].wait_count;
 	}
+	duration = stress_time_now() - t;
+	if (duration > 0.0) {
+		stress_metrics_set(args, 0, "sem_trywait calls per sec", trywait_count / duration);
+		stress_metrics_set(args, 1, "sem_timedwait calls per sec", timedwait_count / duration);
+		stress_metrics_set(args, 2, "sem_wait calls per sec", wait_count / duration);
+	}
+
 	(void)sem_destroy(&sem);
 
 	return EXIT_SUCCESS;
@@ -206,6 +268,7 @@ stressor_info_t stress_sem_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_OS | CLASS_SCHEDULER,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without semaphore.h, pthread or POSIX semaphore support"
 };
