@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
 
 #define STRESS_SESSION_SUCCESS		(0x00)
 #define STRESS_SESSION_SETSID_FAILED	(0x10)
@@ -83,13 +84,13 @@ static int stress_session_set_and_get(const stress_args_t *args, const int fd)
 	pid_t sid, gsid;
 
 	sid = setsid();
-
 	if (sid == (pid_t)-1) {
 		stress_session_return_status(fd, errno, STRESS_SESSION_SETSID_FAILED);
 		pr_inf("%s: setsid failed: errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		return STRESS_SESSION_SETSID_FAILED;
 	}
+
 	gsid = getsid(getpid());
 	if (gsid == (pid_t)-1) {
 		stress_session_return_status(fd, errno, STRESS_SESSION_GETSID_FAILED);
@@ -97,6 +98,7 @@ static int stress_session_set_and_get(const stress_args_t *args, const int fd)
 			args->name, errno, strerror(errno));
 		return STRESS_SESSION_GETSID_FAILED;
 	}
+
 	if (gsid != sid) {
 		stress_session_return_status(fd, errno, STRESS_SESSION_WRONGSID_FAILED);
 		pr_inf("%s getsid failed, got session ID %d, expected %d\n",
@@ -139,7 +141,7 @@ static int stress_session_child(const stress_args_t *args, const int fd)
 	} else {
 		int status;
 
-		/* 25% of calls will be ophans */
+		/* 25% of calls will be orphans */
 		stress_mwc_reseed();
 		if (stress_mwc8() >= 64) {
 #if defined(HAVE_WAIT4)
@@ -172,6 +174,7 @@ static int stress_session_child(const stress_args_t *args, const int fd)
 static int stress_session(const stress_args_t *args)
 {
 	int fds[2];
+	int rc = EXIT_SUCCESS;
 
 	if (pipe(fds) < 0) {
 		pr_inf("%s: pipe failed: errno=%d (%s)\n",
@@ -181,7 +184,7 @@ static int stress_session(const stress_args_t *args)
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
-	while (keep_stressing(args)) {
+	while (stress_continue(args)) {
 		pid_t pid;
 
 		pid = fork();
@@ -204,35 +207,38 @@ static int stress_session(const stress_args_t *args)
 			ssize_t n;
 			session_error_t error;
 
-			(void)memset(&error, 0, sizeof(error));
+			(void)shim_memset(&error, 0, sizeof(error));
 			n = read(fds[0], &error, sizeof(error));
 
 			(void)shim_waitpid(pid, &status, 0);
 			if (WIFEXITED(status) &&
 			   (WEXITSTATUS(status) != STRESS_SESSION_SUCCESS)) {
 				if ((n < (ssize_t)sizeof(error)) ||
-				   ((n == (ssize_t)sizeof(error)) && error.err == 0)) {
+				   ((n == (ssize_t)sizeof(error)) && (error.err == 0))) {
 					pr_fail("%s: failure in child, %s\n", args->name,
 						stress_session_error(WEXITSTATUS(status)));
+					rc = EXIT_FAILURE;
 				} else {
 					pr_fail("%s: failure in child, %s: errno=%d (%s)\n",
 						args->name,
 						stress_session_error(error.status),
 						error.err, strerror(error.err));
+					rc = EXIT_FAILURE;
 				}
 			}
 		}
-		inc_counter(args);
+		stress_bogo_inc(args);
 	}
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	(void)close(fds[0]);
 	(void)close(fds[1]);
 
-	return EXIT_SUCCESS;
+	return rc;
 }
 
 stressor_info_t stress_session_info = {
 	.stressor = stress_session,
 	.class = CLASS_SCHEDULER | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = session_help
 };
