@@ -21,6 +21,9 @@
 #include "core-arch.h"
 #include "core-builtin.h"
 #include "core-capabilities.h"
+#include "core-madvise.h"
+#include "core-mincore.h"
+#include "core-out-of-memory.h"
 
 #if defined(HAVE_LINUX_MEMPOLICY_H)
 #include <linux/mempolicy.h>
@@ -451,7 +454,7 @@ static void exercise_shmget(const size_t sz, const char *name)
 		char buf[32];
 
 		/* Find size from shmmax proc value */
-		if (system_read("/proc/sys/kernel/shmmax", buf, sizeof(buf)) > 0) {
+		if (stress_system_read("/proc/sys/kernel/shmmax", buf, sizeof(buf)) > 0) {
 			size_t shmmax;
 
 			if (sscanf(buf, "%zu", &shmmax) == 1) {
