@@ -89,9 +89,9 @@ static int stress_sigabrt(const stress_args_t *args)
 again:
 		pid = fork();
 		if (pid < 0) {
-			if (stress_redo_fork(errno))
+			if (stress_redo_fork(args, errno))
 				goto again;
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				goto finish;
 			pr_fail("%s: fork failed: %d (%s)\n",
 				args->name, errno, strerror(errno));
@@ -114,7 +114,7 @@ again:
 
 				/* Raising SIGABRT without an handler will abort */
 				sigabrt_info->t_start = stress_time_now();
-				raise(SIGABRT);
+				shim_raise(SIGABRT);
 			}
 
 			_exit(EXIT_FAILURE);
@@ -143,14 +143,14 @@ rewait:
 								args->name);
 						}
 					}
-					inc_counter(args);
+					stress_bogo_inc(args);
 				} else if (WIFEXITED(status)) {
 					pr_fail("%s: child did not abort as expected\n",
 						args->name);
 				}
 			}
 		}
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 finish:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
