@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
 
 static const stress_help_t help[] = {
 	{ NULL,	"sigchld N",	 "start N workers that handle SIGCHLD" },
@@ -75,7 +76,7 @@ static int stress_sigchld(const stress_args_t *args)
 	cld_stopped = 0;
 	cld_continued = 0;
 
-	(void)memset(&sa, 0, sizeof(sa));
+	(void)shim_memset(&sa, 0, sizeof(sa));
 	sa.sa_sigaction = stress_sigchld_handler;
 #if defined(SA_SIGINFO)
 	sa.sa_flags = SA_SIGINFO;
@@ -94,9 +95,9 @@ static int stress_sigchld(const stress_args_t *args)
 again:
 		pid = fork();
 		if (pid < 0) {
-			if (stress_redo_fork(errno))
+			if (stress_redo_fork(args, errno))
 				goto again;
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				goto finish;
 			pr_err("%s: fork failed: %d (%s)\n",
 				args->name, errno, strerror(errno));
@@ -108,24 +109,37 @@ again:
 			/* Parent wait and reap for child */
 			int wstatus, ret;
 
-			ret = kill(pid, SIGSTOP);
+			ret = shim_kill(pid, SIGSTOP);
 			if (ret == 0) {
-				VOID_RET(int, kill(pid, SIGCONT));
+				VOID_RET(int, shim_kill(pid, SIGCONT));
 			}
-			VOID_RET(int, kill(pid, SIGKILL));
+			VOID_RET(int, shim_kill(pid, SIGKILL));
 			VOID_RET(int, waitpid(pid, &wstatus, 0));
 		}
-		set_counter(args, counter);
-	} while (keep_stressing(args));
+		stress_bogo_set(args, counter);
+	} while (stress_continue(args));
 
 finish:
+	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
+
 	pr_dbg("%s: exit: %" PRIu64 ", kill: %" PRIu64
 		", stop: %" PRIu64 ", continue: %" PRIu64 "\n",
 		args->name,
 		cld_exited, cld_killed,
 		cld_stopped, cld_continued);
 
-	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
+#if !defined(__OpenBSD__)
+	/*
+	 *  No si_code codes recognised and we handled SIGCHLD signals, then
+	 *  something is not conformant
+	 */
+	if ((cld_exited + cld_killed + cld_stopped + cld_continued == 0) &&
+	    (counter > 0)) {
+		pr_fail("%s: no SIGCHLD siginfo si_code detected in signal handler\n",
+			args->name);
+		return EXIT_FAILURE;
+	}
+#endif
 
 	return EXIT_SUCCESS;
 }
@@ -133,5 +147,6 @@ finish:
 stressor_info_t stress_sigchld_info = {
 	.stressor = stress_sigchld,
 	.class = CLASS_INTERRUPT | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
