@@ -18,6 +18,8 @@
  *
  */
 #include "stress-ng.h"
+#include "core-affinity.h"
+#include "core-builtin.h"
 
 #if defined(HAVE_SYS_SIGNALFD_H)
 #include <sys/signalfd.h>
@@ -40,7 +42,7 @@ static const stress_help_t help[] = {
     defined(__linux__) &&		\
     defined(HAVE_SYSCALL)
 #define HAVE_SIGNALFD4
-static int shim_signalfd4(
+static inline int shim_signalfd4(
 	int ufd,
 	sigset_t *user_mask,
 	size_t sizemask,
@@ -57,7 +59,7 @@ static int shim_signalfd4(
 static int stress_sigfd(const stress_args_t *args)
 {
 	pid_t pid, ppid = args->pid;
-	int sfd;
+	int sfd, parent_cpu;
 	const int bad_fd = stress_get_bad_fd();
 	sigset_t mask;
 
@@ -105,32 +107,33 @@ static int stress_sigfd(const stress_args_t *args)
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 again:
+	parent_cpu = stress_get_cpu();
 	pid = fork();
 	if (pid < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			goto finish;
 		pr_err("%s: fork failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		return EXIT_FAILURE;
 	} else if (pid == 0) {
 		int val = 0;
+		union sigval s ALIGN64;
 
+		(void)stress_change_cpu(args, parent_cpu);
 		stress_parent_died_alarm();
 		(void)sched_settings_apply(true);
 
-		while (keep_stressing_flag()) {
-			union sigval s;
+		(void)shim_memset(&s, 0, sizeof(s));
+
+		while (stress_continue_flag()) {
 			int ret;
 
-			(void)memset(&s, 0, sizeof(s));
 			s.sival_int = val++;
 			ret = sigqueue(ppid, SIGRTMIN, s);
-			if ((ret < 0) && (errno != EAGAIN)) {
+			if ((ret < 0) && (errno != EAGAIN))
 				break;
-			}
-
 		}
 		(void)close(sfd);
 		_exit(0);
@@ -138,13 +141,14 @@ again:
 		/* Parent */
 		int status;
 		const pid_t self = getpid();
+		const bool verify = !!(g_opt_flags & OPT_FLAGS_VERIFY);
 
 		do {
 			ssize_t ret;
-			struct signalfd_siginfo fdsi;
+			struct signalfd_siginfo fdsi ALIGN64;
 
 			ret = read(sfd, &fdsi, sizeof(fdsi));
-			if ((ret < 0) || (ret != sizeof(fdsi))) {
+			if (UNLIKELY((ret < 0) || (ret != sizeof(fdsi)))) {
 				if ((errno == EAGAIN) || (errno == EINTR))
 					continue;
 				if (errno) {
@@ -155,10 +159,10 @@ again:
 				}
 				continue;
 			}
-			if (ret == 0)
+			if (UNLIKELY(ret == 0))
 				break;
-			if (g_opt_flags & OPT_FLAGS_VERIFY) {
-				if (fdsi.ssi_signo != (uint32_t)SIGRTMIN) {
+			if (UNLIKELY(verify)) {
+				if (UNLIKELY(fdsi.ssi_signo != (uint32_t)SIGRTMIN)) {
 					pr_fail("%s: unexpected signal %d\n",
 						args->name, fdsi.ssi_signo);
 					break;
@@ -169,14 +173,14 @@ again:
 			 *  the signal fd to exercise the sigmask setting
 			 *  for this specific kind of fd info.
 			 */
-			if ((fdsi.ssi_int & 0xffff) == 0)
+			if (UNLIKELY((fdsi.ssi_int & 0xffff) == 0))
 				(void)stress_read_fdinfo(self, sfd);
 
-			inc_counter(args);
-		} while (keep_stressing(args));
+			stress_bogo_inc(args);
+		} while (stress_continue(args));
 
 		/* terminal child */
-		(void)kill(pid, SIGKILL);
+		(void)shim_kill(pid, SIGKILL);
 		(void)shim_waitpid(pid, &status, 0);
 	}
 
@@ -196,6 +200,7 @@ stressor_info_t stress_sigfd_info = {
 stressor_info_t stress_sigfd_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_INTERRUPT | CLASS_OS,
+	.verify = VERIFY_OPTIONAL,
 	.help = help,
 	.unimplemented_reason = "built without sys/signalfd.h, signalfd() or sigqueue() system calls"
 };
