@@ -19,6 +19,7 @@
  */
 #include "stress-ng.h"
 #include "core-arch.h"
+#include "core-builtin.h"
 #include "core-put.h"
 
 #if defined(HAVE_FENV_H)
@@ -132,7 +133,7 @@ static char *stress_sigfpe_errstr(const int err)
 static void NOINLINE OPTIMIZE0 stress_int_div_by_zero(void)
 {
 	uint8_t k = stress_mwc8();
-	uint64_t zero = stress_uint64_zero();
+	uint64_t zero = stress_get_uint64_zero();
 
 	stress_uint64_put(k / zero);
 }
@@ -140,7 +141,7 @@ static void NOINLINE OPTIMIZE0 stress_int_div_by_zero(void)
 static void NOINLINE OPTIMIZE0 stress_float_div_by_zero(void)
 {
 	float k = (float)stress_mwc8();
-	float zero = (float)stress_uint64_zero();
+	float zero = (float)stress_get_uint64_zero();
 
 	stress_float_put(k / zero);
 }
@@ -154,7 +155,9 @@ static int stress_sigfpe(const stress_args_t *args)
 	struct sigaction action;
 	static int i = 0;
 	int ret;
-	double time_end;
+#if defined(STRESS_CHECK_SIGINFO)
+	const bool verify = !!(g_opt_flags & OPT_FLAGS_VERIFY);
+#endif
 
 	typedef struct {
 		unsigned int exception;
@@ -164,7 +167,7 @@ static int stress_sigfpe(const stress_args_t *args)
 	/*
 	 *  FPE errors to raise
 	 */
-	static const stress_fpe_err_t fpe_errs[] = {
+	static const stress_fpe_err_t fpe_errs[] ALIGN64 = {
 #if defined(FPE_INTDIV)
 		{ SNG_INTDIV,	FPE_INTDIV },	/* can be trapped */
 #else
@@ -207,7 +210,7 @@ static int stress_sigfpe(const stress_args_t *args)
 #endif
 	};
 
-	(void)memset(&action, 0, sizeof action);
+	(void)shim_memset(&action, 0, sizeof action);
 
 #if defined(STRESS_CHECK_SIGINFO)
 	action.sa_sigaction = stress_fpehandler;
@@ -227,7 +230,6 @@ static int stress_sigfpe(const stress_args_t *args)
 	}
 
 	(void)alarm(0);
-	time_end = stress_time_now() + g_opt_timeout;
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	for (;;) {
@@ -248,9 +250,9 @@ static int stress_sigfpe(const stress_args_t *args)
 		 * We return here if we get SIGFPE, so
 		 * first check if we need to terminate
 		 */
-		if (UNLIKELY(!keep_stressing(args)))
+		if (UNLIKELY(!stress_continue(args)))
 			break;
-		if (UNLIKELY(stress_time_now() > time_end))
+		if (UNLIKELY(stress_time_now() > args->time_end))
 			break;
 
 		if (ret) {
@@ -261,7 +263,7 @@ static int stress_sigfpe(const stress_args_t *args)
 			(void)feclearexcept(FE_ALL_EXCEPT);
 
 #if defined(STRESS_CHECK_SIGINFO)
-			if ((g_opt_flags & OPT_FLAGS_VERIFY) &&
+			if (verify &&
 			    (siginfo.si_code >= 0) &&
 			    (siginfo.si_code != expected_err_code)) {
 				pr_fail("%s: got SIGFPE error %d (%s), expecting %d (%s)\n",
@@ -270,7 +272,7 @@ static int stress_sigfpe(const stress_args_t *args)
 					expected_err_code, stress_sigfpe_errstr(expected_err_code));
 			}
 #endif
-			inc_counter(args);
+			stress_bogo_inc(args);
 		} else {
 #if defined(STRESS_CHECK_SIGINFO)
 			siginfo.si_code = 0;
@@ -294,7 +296,8 @@ static int stress_sigfpe(const stress_args_t *args)
 			}
 		}
 		i++;
-		i %= SIZEOF_ARRAY(fpe_errs);
+		if (i >= (int)SIZEOF_ARRAY(fpe_errs))
+			i = 0;
 	}
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
@@ -313,6 +316,7 @@ stressor_info_t stress_sigfpe_info = {
 stressor_info_t stress_sigfpe_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_INTERRUPT | CLASS_OS,
+	.verify = VERIFY_OPTIONAL,
 	.help = help,
 	.unimplemented_reason = "built using uclibc or without fenv.h or float.h"
 };
