@@ -25,6 +25,7 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,		 NULL }
 };
 
+static bool jmp_env_ok;
 static sigjmp_buf jmp_env;
 
 static const int signals[] = {
@@ -98,8 +99,18 @@ typedef struct {
 } stress_signest_info_t;
 
 static volatile stress_signest_info_t signal_info;
+static uint64_t raised;
+static uint64_t handled;
 
-static ssize_t stress_signest_find(int signum)
+static void stress_signest_ignore(void)
+{
+	size_t i;
+
+	for (i = 0; i < SIZEOF_ARRAY(signals); i++)
+		VOID_RET(int, stress_sighandler("signest", signals[i], SIG_IGN, NULL));
+}
+
+static inline ssize_t stress_signest_find(int signum)
 {
 	size_t i;
 
@@ -116,8 +127,10 @@ static void MLOCKED_TEXT stress_signest_handler(int signum)
 	const intptr_t addr = (intptr_t)&i;
 	double run_time = stress_time_now() - signal_info.time_start;
 
+	handled++;
 	signal_info.depth++;
-	if (signal_info.depth > signal_info.max_depth)
+	/* After a while this becomes more unlikely than likely */
+	if (UNLIKELY(signal_info.depth > signal_info.max_depth))
 		signal_info.max_depth = signal_info.depth;
 
 	/* using alternative signal stack? */
@@ -132,29 +145,41 @@ static void MLOCKED_TEXT stress_signest_handler(int signum)
 			signal_info.stack_depth = delta;
 	}
 
-	if (run_time > (double)g_opt_timeout)
-		siglongjmp(jmp_env, 1);
+	if (UNLIKELY(run_time > (double)g_opt_timeout)) {
+		stress_signest_ignore();
+		if (jmp_env_ok)
+			siglongjmp(jmp_env, 1);
+	}
 
-	if (signal_info.stop)
-		siglongjmp(jmp_env, 1);
+	if (UNLIKELY(signal_info.stop)) {
+		stress_signest_ignore();
+		if (jmp_env_ok)
+			siglongjmp(jmp_env, 1);
+	}
 
-	if (!signal_info.args)
+	if (UNLIKELY(!signal_info.args))
 		goto done;
 
-	inc_counter(signal_info.args);
-	if (!keep_stressing(signal_info.args))
-		siglongjmp(jmp_env, 1);
+	stress_bogo_inc(signal_info.args);
+	if (UNLIKELY(!stress_continue(signal_info.args))) {
+		stress_signest_ignore();
+		if (jmp_env_ok)
+			siglongjmp(jmp_env, 1);
+	}
 
 	i = stress_signest_find(signum);
-	if ((i < 0) || (i == (ssize_t)SIZEOF_ARRAY(signals)))
+	if (UNLIKELY((i < 0) || (i == (ssize_t)SIZEOF_ARRAY(signals))))
 		goto done;
 
 	signal_info.signalled |= 1U << i;
 
 	for (; i < (ssize_t)SIZEOF_ARRAY(signals); i++) {
-		if (signal_info.stop || !keep_stressing(signal_info.args))
-			siglongjmp(jmp_env, 1);
-		raise(signals[i]);
+		if (signal_info.stop || !stress_continue(signal_info.args)) {
+			if (jmp_env_ok)
+				siglongjmp(jmp_env, 1);
+		}
+		(void)shim_raise(signals[i]);
+		raised++;
 	}
 
 done:
@@ -162,7 +187,6 @@ done:
 	return;
 }
 
-
 /*
  *  stress_signest
  *	stress by generating segmentation faults by
@@ -171,10 +195,16 @@ done:
 static int stress_signest(const stress_args_t *args)
 {
 	size_t i, sz;
-	int n, ret;
+	int n, ret, rc;
 	uint8_t *altstack;
 	char *buf, *ptr;
-	const size_t altstack_size = stress_min_sig_stack_size() * SIZEOF_ARRAY(signals);
+	const size_t altstack_size = stress_get_min_sig_stack_size() * SIZEOF_ARRAY(signals);
+	double rate;
+	NOCLOBBER double t, duration;
+
+	raised = 0;
+	handled = 0;
+	jmp_env_ok = false;
 
 	altstack = (uint8_t*)mmap(NULL, altstack_size, PROT_READ | PROT_WRITE,
 			MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
@@ -208,18 +238,24 @@ static int stress_signest(const stress_args_t *args)
 			return EXIT_NO_RESOURCE;
 	}
 
+	jmp_env_ok = true;
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
+	t = stress_time_now();
 	do {
-		raise(signals[0]);
-	} while (keep_stressing(args));
+		(void)shim_raise(signals[0]);
+		raised++;
+	} while (stress_continue(args));
 
 finish:
+	duration = stress_time_now() - t;
+	jmp_env_ok = false;
 	signal_info.stop = true;
+	stress_signest_ignore();
 
 	for (sz = 1, n = 0, i = 0; i < SIZEOF_ARRAY(signals); i++) {
 		if (signal_info.signalled & (1U << i)) {
-			const char *name = stress_signal_name(signals[i]);
+			const char *name = stress_get_signal_name(signals[i]);
 
 			n++;
 			sz += name ? (strlen(name) + 1) : 32;
@@ -231,7 +267,7 @@ finish:
 		if (buf) {
 			for (ptr = buf, i = 0; i < SIZEOF_ARRAY(signals); i++) {
 				if (signal_info.signalled & (1U << i)) {
-					const char *name = stress_signal_name(signals[i]);
+					const char *name = stress_get_signal_name(signals[i]);
 
 					if (name) {
 						if (strncmp(name, "SIG", 3) == 0)
@@ -258,14 +294,24 @@ finish:
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
+	rc = EXIT_SUCCESS;
+	if ((raised > 0) && (handled == 0)) {
+		pr_fail("%s: %" PRIu64 " signals raised and no signals handled\n",
+			args->name, raised);
+		rc = EXIT_FAILURE;
+	}
+	rate = handled > 0 ? duration / (double)handled : 0.0;
+	stress_metrics_set(args, 0, "nanosec to handle a signal", rate * 1000000000.0);
+
 	stress_sigaltstack_disable();
 	(void)munmap((void *)altstack, altstack_size);
 
-	return EXIT_SUCCESS;
+	return rc;
 }
 
 stressor_info_t stress_signest_info = {
 	.stressor = stress_signest,
 	.class = CLASS_INTERRUPT | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
