@@ -101,7 +101,7 @@ again:
 	pid = fork();
 #endif
 	if (UNLIKELY(pid < 0)) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
 
 		(void)close(pipefds[0]);
