@@ -18,6 +18,8 @@
  *
  */
 #include "stress-ng.h"
+#include "core-affinity.h"
+#include "core-builtin.h"
 #include "core-net.h"
 
 #if defined(HAVE_LINUX_SOCKIOS_H)
@@ -53,9 +55,13 @@ UNEXPECTED
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
-#define MIN_SOCKET_PORT		(1024)
+#define MIN_SOCKET_PORT		(1025)
 #define MAX_SOCKET_PORT		(65535)
-#define DEFAULT_SOCKET_PORT	(5000)
+#define DEFAULT_SOCKET_PORT	(1000)
+
+#define MIN_SOCKET_MSGS		(1)
+#define MAX_SOCKET_MSGS		(10000000)
+#define DEFAULT_SOCKET_MSGS	(5000)
 
 #define MMAP_BUF_SIZE		(65536)
 #define MMAP_IO_SIZE		(8192)	/* Must be less or equal to 8192 */
@@ -86,6 +92,7 @@ static const stress_help_t help[] = {
 	{ "S N", "sock N",		"start N workers exercising socket I/O" },
 	{ NULL,	"sock-domain D",	"specify socket domain, default is ipv4" },
 	{ NULL,	"sock-if I",		"use network interface I, e.g. lo, eth0, etc." },
+	{ NULL,	"sock-msgs N",		"number of messages to send per connection" },
 	{ NULL,	"sock-nodelay",		"disable Nagle algorithm, send data immediately" },
 	{ NULL,	"sock-ops N",		"stop after N socket bogo operations" },
 	{ NULL,	"sock-opts option", 	"socket options [send|sendmsg|sendmmsg]" },
@@ -145,6 +152,19 @@ static int stress_set_sock_opts(const char *opt)
 	return stress_set_sock_option("sock-opts", sock_opts, opt);
 }
 
+static const char *stress_recv_func_str(const int sock_opts)
+{
+	switch (sock_opts) {
+	case SOCKET_OPT_SEND:
+		return "recv";
+	case SOCKET_OPT_SENDMSG:
+		return "recvmsg";
+	case SOCKET_OPT_SENDMMSG:
+		return "recvmmsg";
+	}
+	return "unknown";
+}
+
 /*
  *  stress_set_sock_type()
  *	parse --sock-type
@@ -157,6 +177,9 @@ static int stress_set_sock_type(const char *opt)
 #endif
 #if defined(SOCK_SEQPACKET)
 		{ "seqpacket",	SOCK_SEQPACKET },
+#endif
+#if defined(SOCK_DGRAM)
+		{ "dgram",	SOCK_DGRAM },
 #endif
 		{ NULL,		0 }
 	};
@@ -164,6 +187,20 @@ static int stress_set_sock_type(const char *opt)
 	return stress_set_sock_option("sock-type", sock_types, opt);
 }
 
+/*
+ *  stress_set_sock_msgs()
+ *	set number of messages to send per connection
+ */
+static int stress_set_sock_msgs(const char *opt)
+{
+	size_t sock_msgs;
+
+	sock_msgs = (size_t)stress_get_uint64(opt);
+	stress_check_range("sock-msgs", (uint64_t)sock_msgs,
+                MIN_SOCKET_MSGS, MAX_SOCKET_MSGS);
+	return stress_set_setting("sock-msgs", TYPE_ID_SIZE_T, &sock_msgs);
+}
+
 /*
  *  stress_set_sock_port()
  *	set port to use
@@ -250,7 +287,7 @@ static char **stress_get_congestion_controls(const int sock_domain, size_t *n_ct
 	if (sock_domain != AF_INET)
 		return NULL;
 
-	buf_len = system_read(PROC_CONG_CTRLS, buf, sizeof(buf));
+	buf_len = stress_system_read(PROC_CONG_CTRLS, buf, sizeof(buf));
 	if (buf_len <= 0)
 		return NULL;
 
@@ -270,6 +307,11 @@ static char **stress_get_congestion_controls(const int sock_domain, size_t *n_ct
 		ctrls[n++] = ctrl;
 	}
 
+	if (n == 0) {
+		free(ctrls);
+		return NULL;
+	}
+
 	/* Shrink ctrls, hopefully should not fail, but check anyhow */
 	tmp = realloc(ctrls, n * sizeof(char *));
 	if (!tmp) {
@@ -323,7 +365,7 @@ static void stress_sock_ioctl(
 	if (!rt) {
 		struct ifconf ifc;
 
-		(void)memset(&ifc, 0, sizeof(ifc));
+		(void)shim_memset(&ifc, 0, sizeof(ifc));
 		VOID_RET(int, ioctl(fd, SIOCGIFCONF, &ifc));
 	}
 #endif
@@ -403,7 +445,7 @@ static void stress_sock_invalid_recv(const int fd, const int opt)
 	case SOCKET_OPT_RECVMSG:
 		vec[0].iov_base = buf;
 		vec[0].iov_len = sizeof(buf);
-		(void)memset(&msg, 0, sizeof(msg));
+		(void)shim_memset(&msg, 0, sizeof(msg));
 		msg.msg_iov = vec;
 		msg.msg_iovlen = 1;
 
@@ -415,7 +457,7 @@ static void stress_sock_invalid_recv(const int fd, const int opt)
 		break;
 #if defined(HAVE_RECVMMSG)
 	case SOCKET_OPT_RECVMMSG:
-		(void)memset(msgvec, 0, sizeof(msgvec));
+		(void)shim_memset(msgvec, 0, sizeof(msgvec));
 		vec[0].iov_base = buf;
 		vec[0].iov_len = sizeof(buf);
 		msgvec[0].msg_hdr.msg_iov = vec;
@@ -440,7 +482,7 @@ static void stress_sock_invalid_recv(const int fd, const int opt)
  *  stress_sock_client()
  *	client reader
  */
-static int stress_sock_client(
+static int OPTIMIZE3 stress_sock_client(
 	const stress_args_t *args,
 	char *buf,
 	const pid_t mypid,
@@ -457,6 +499,8 @@ static int stress_sock_client(
 	size_t n_ctrls;
 	char **ctrls;
 	int recvflag = 0, rc = EXIT_FAILURE;
+	uint64_t inq_bytes = 0, inq_samples = 0;
+	uint32_t count = 0;
 
 	stress_parent_died_alarm();
 	(void)sched_settings_apply(true);
@@ -472,10 +516,11 @@ static int stress_sock_client(
 	do {
 		int fd;
 		int retries = 0;
-		static int count = 0;
 		socklen_t addr_len = 0;
+		double metric;
+
 retry:
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			goto free_controls;
 
 		/* Exercise illegal socket family  */
@@ -719,7 +764,6 @@ retry:
 		do {
 			ssize_t n = 0;
 			size_t i, j;
-			char *recvfunc = "recv";
 			struct msghdr ALIGN64 msg;
 			struct iovec ALIGN64 vec[MMAP_IO_SIZE / 16];
 #if defined(HAVE_RECVMMSG)
@@ -741,26 +785,24 @@ retry:
 			 *  performance.
 			 */
 			if ((count & 0x3ff) == 0) {
-				size_t bytes = MMAP_IO_SIZE;
-
-				VOID_RET(int, ioctl(fd, FIONREAD, &bytes));
-				count = 0;
-
-				if (bytes > MMAP_IO_SIZE)
-					bytes = MMAP_IO_SIZE;
+				int val;
 
+				VOID_RET(int, ioctl(fd, FIONREAD, &val));
+#if defined(SIOCINQ)
+				if (LIKELY(ioctl(fd, SIOCINQ, &val) == 0)) {
+					inq_bytes += val;
+					inq_samples++;
+				}
+#endif
+#if defined(SIOCATMARK)
+				/* and exercise SIOCATMARK */
+				VOID_RET(int, ioctl(fd, SIOCATMARK, &val));
+#endif
 			}
 #endif
 			/*  Periodically exercise invalid recv calls */
 			if ((count & 0x7ff) == 0)
 				stress_sock_invalid_recv(fd, opt);
-#if defined(SIOCINQ)
-			{
-				int pending;
-
-				VOID_RET(int, ioctl(fd, SIOCINQ, &pending));
-			}
-#endif
 
 			/*
 			 *  Receive using equivalent receive method
@@ -768,24 +810,21 @@ retry:
 			 */
 			switch (opt) {
 			case SOCKET_OPT_RECV:
-				recvfunc = "recv";
 				n = recv(fd, buf, MMAP_IO_SIZE, recvflag);
 				break;
 			case SOCKET_OPT_RECVMSG:
-				recvfunc = "recvmsg";
 				for (j = 0, i = 16; i < MMAP_IO_SIZE; i += 16, j++) {
 					vec[j].iov_base = buf;
 					vec[j].iov_len = i;
 				}
-				(void)memset(&msg, 0, sizeof(msg));
+				(void)shim_memset(&msg, 0, sizeof(msg));
 				msg.msg_iov = vec;
 				msg.msg_iovlen = j;
 				n = recvmsg(fd, &msg, 0);
 				break;
 #if defined(HAVE_RECVMMSG)
 			case SOCKET_OPT_RECVMMSG:
-				recvfunc = "recvmmsg";
-				(void)memset(msgvec, 0, sizeof(msgvec));
+				(void)shim_memset(msgvec, 0, sizeof(msgvec));
 				for (j = 0, i = 16; i < MMAP_IO_SIZE; i += 16, j++) {
 					vec[j].iov_base = buf;
 					vec[j].iov_len = i;
@@ -795,24 +834,24 @@ retry:
 					msgvec[i].msg_hdr.msg_iovlen = j;
 				}
 				n = recvmmsg(fd, msgvec, MSGVEC_SIZE, 0, NULL);
-				if (n > 0) {
+				if (LIKELY(n > 0)) {
 					for (n = 0, i = 0; i < MSGVEC_SIZE; i++)
 						n += msgvec[i].msg_len;
 				}
 				break;
 #endif
 			}
-			if (n == 0)
-				break;
-			if (n < 0) {
+			if (UNLIKELY(n <= 0)) {
+				if (n == 0)
+					break;
 				if ((errno != EINTR) && (errno != ECONNRESET))
 					pr_fail("%s: %s failed, errno=%d (%s)\n",
-						recvfunc, args->name,
+						args->name, stress_recv_func_str(opt),
 						errno, strerror(errno));
 				break;
 			}
 			count++;
-		} while (keep_stressing(args));
+		} while (stress_continue(args));
 
 		stress_sock_ioctl(fd, sock_domain, rt);
 #if defined(AF_INET) && 	\
@@ -829,12 +868,14 @@ retry:
 
 		(void)shutdown(fd, SHUT_RDWR);
 		(void)close(fd);
-	} while (keep_stressing(args));
+		metric = (inq_samples > 0) ? (double)inq_bytes / (double)inq_samples : 0.0;
+		stress_metrics_set(args, 2, "byte average in queue length", metric);
+	} while (stress_continue(args));
 
 #if defined(AF_UNIX) &&		\
     defined(HAVE_SOCKADDR_UN)
 	if (sock_domain == AF_UNIX) {
-		struct sockaddr_un *addr_un = (struct sockaddr_un *)addr;
+		const struct sockaddr_un *addr_un = (struct sockaddr_un *)addr;
 
 		(void)shim_unlink(addr_un->sun_path);
 	}
@@ -858,7 +899,7 @@ static bool stress_send_error(const int err)
  *  stress_sock_server()
  *	server writer
  */
-static int stress_sock_server(
+static int OPTIMIZE3 stress_sock_server(
 	const stress_args_t *args,
 	char *buf,
 	const pid_t pid,
@@ -882,6 +923,14 @@ static int stress_sock_server(
 	void *ptr = MAP_FAILED;
 	const pid_t self = getpid();
 	int sendflag = 0;
+	double t, duration, metric;
+	uint64_t outq_bytes = 0, outq_samples = 0;
+	size_t sock_msgs = DEFAULT_SOCKET_MSGS;
+#if defined(SIOCOUTQ)
+	uint32_t count = 0;
+#endif
+
+	(void)stress_get_setting("sock-msgs", &sock_msgs);
 
 #if defined(MSG_ZEROCOPY)
 	if (sock_zerocopy)
@@ -913,7 +962,7 @@ static int stress_sock_server(
 
 	/* exercise invalid setsockopt fd */
 	(void)setsockopt(-1, SOL_SOCKET, SO_REUSEADDR, &so_reuseaddr, sizeof(so_reuseaddr));
-	
+
 	/* exercise invalid level */
 	(void)setsockopt(fd, -1, SO_REUSEADDR, &so_reuseaddr, sizeof(so_reuseaddr));
 
@@ -925,6 +974,18 @@ static int stress_sock_server(
 			&addr, &addr_len, NET_ADDR_ANY) < 0) {
 		goto die_close;
 	}
+
+#if defined(SIOCGIFADDR) &&	\
+    defined(HAVE_IFREQ)
+	{
+		struct ifreq ifaddr;
+
+		(void)shim_memset(&ifaddr, 0, sizeof(ifaddr));
+		(void)shim_strlcpy(ifaddr.ifr_name, sock_if ? sock_if : "lo", sizeof(ifaddr.ifr_name));
+		VOID_RET(int, ioctl(fd, SIOCGIFADDR, &ifaddr));
+	}
+#endif
+
 	if (bind(fd, addr, addr_len) < 0) {
 		rc = stress_exit_status(errno);
 		pr_fail("%s: bind failed on port %d, errno=%d (%s)\n",
@@ -944,10 +1005,11 @@ static int stress_sock_server(
 	 */
 	ptr = mmap(NULL, page_size, PROT_READ, MAP_PRIVATE, fd, 0);
 
+	t = stress_time_now();
 	do {
 		int sfd;
 
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			break;
 
 #if defined(HAVE_ACCEPT4)
@@ -961,7 +1023,7 @@ static int stress_sock_server(
 		sfd = accept(fd, (struct sockaddr *)NULL, NULL);
 #endif
 		if (sfd >= 0) {
-			size_t i, j;
+			size_t i, j, k;
 			struct sockaddr saddr;
 			socklen_t len;
 			int sndbuf, opt;
@@ -978,6 +1040,18 @@ static int stress_sock_server(
 				(void)close(sfd);
 				break;
 			}
+
+#if defined(SIOCUNIXFILE)
+			/* exercise SIOCUNIXFILE */
+			if (sock_domain == AF_UNIX) {
+				int unix_fd;
+
+				unix_fd = ioctl(sfd, SIOCUNIXFILE, 0);
+				if (unix_fd >= 0)
+					(void)close(unix_fd);
+			}
+#endif
+
 			/*
 			 *  Exercise illegal sockname lengths
 			 */
@@ -997,13 +1071,13 @@ static int stress_sock_server(
 				break;
 			}
 #if defined(SO_RESERVE_MEM)
-		{
-			const int mem = 4 * 1024 * 1024;
-			socklen_t optlen = sizeof(mem);
+			{
+				const int mem = 4 * 1024 * 1024;
+				socklen_t optlen = sizeof(mem);
 
-			(void)setsockopt(fd, SOL_SOCKET, SO_RESERVE_MEM,
-				&mem, optlen);
-		}
+				(void)setsockopt(fd, SOL_SOCKET, SO_RESERVE_MEM,
+					&mem, optlen);
+			}
 #endif
 #if defined(SOL_TCP) &&	\
     defined(TCP_QUICKACK)
@@ -1031,77 +1105,83 @@ static int stress_sock_server(
 				}
 			}
 #endif
-			(void)memset(buf, 'A' + (get_counter(args) % 26), MMAP_IO_SIZE);
+			(void)shim_memset(buf, stress_ascii64[stress_bogo_get(args) & 63], MMAP_IO_SIZE);
 
-			if (sock_opts == SOCKET_OPT_RANDOM)
-				opt = stress_mwc8modn(3);
-			else
-				opt = sock_opts;
+			opt = sock_opts;
 
-			switch (opt) {
-			case SOCKET_OPT_SEND:
-				for (i = 16; i < MMAP_IO_SIZE; i += 16) {
-					ssize_t ret = send(sfd, buf, i, sendflag);
-					if (ret < 0) {
-						if (stress_send_error(errno))
-							pr_fail("%s: send failed, errno=%d (%s)\n",
-								args->name, errno, strerror(errno));
-						break;
-					} else
-						msgs++;
-				}
-				break;
-			case SOCKET_OPT_SENDMSG:
-				for (j = 0, i = 16; i < MMAP_IO_SIZE; i += 16, j++) {
-					vec[j].iov_base = buf;
-					vec[j].iov_len = i;
-				}
-				(void)memset(&msg, 0, sizeof(msg));
-				msg.msg_iov = vec;
-				msg.msg_iovlen = j;
-				if (sendmsg(sfd, &msg, 0) < 0) {
+			for (k = 0; (k < sock_msgs) && stress_continue(args); k++) {
+				if (UNLIKELY(sock_opts == SOCKET_OPT_RANDOM))
+					opt = stress_mwc8modn(3);
+
+				switch (opt) {
+				case SOCKET_OPT_SEND:
+					for (i = 16; i < MMAP_IO_SIZE; i += 16) {
+						if (UNLIKELY(send(sfd, buf, i, sendflag) < 0)) {
+							if (stress_send_error(errno))
+								pr_fail("%s: send failed, errno=%d (%s)\n",
+									args->name, errno, strerror(errno));
+							break;
+						} else
+							msgs++;
+					}
+					break;
+				case SOCKET_OPT_SENDMSG:
+					for (j = 0, i = 16; i < MMAP_IO_SIZE; i += 16, j++) {
+						vec[j].iov_base = buf;
+						vec[j].iov_len = i;
+					}
+					(void)shim_memset(&msg, 0, sizeof(msg));
+					msg.msg_iov = vec;
+					msg.msg_iovlen = j;
+					if (UNLIKELY(sendmsg(sfd, &msg, 0) < 0)) {
 					if (stress_send_error(errno))
 						pr_fail("%s: sendmsg failed, errno=%d (%s)\n",
 							args->name, errno, strerror(errno));
-				} else
-					msgs += j;
-				break;
+					} else
+						msgs += j;
+					break;
 #if defined(HAVE_SENDMMSG)
-			case SOCKET_OPT_SENDMMSG:
-				(void)memset(msgvec, 0, sizeof(msgvec));
-				for (j = 0, i = 16; i < MMAP_IO_SIZE; i += 16, j++) {
-					vec[j].iov_base = buf;
-					vec[j].iov_len = i;
-				}
-				for (i = 0; i < MSGVEC_SIZE; i++) {
-					msgvec[i].msg_hdr.msg_iov = vec;
-					msgvec[i].msg_hdr.msg_iovlen = j;
-				}
-				if (sendmmsg(sfd, msgvec, MSGVEC_SIZE, 0) < 0) {
-					if (stress_send_error(errno))
-						pr_fail("%s: sendmmsg failed, errno=%d (%s)\n",
-							args->name, errno, strerror(errno));
-				} else
-					msgs += (MSGVEC_SIZE * j);
-				break;
+				case SOCKET_OPT_SENDMMSG:
+					(void)shim_memset(msgvec, 0, sizeof(msgvec));
+					for (j = 0, i = 16; i < MMAP_IO_SIZE; i += 16, j++) {
+						vec[j].iov_base = buf;
+						vec[j].iov_len = i;
+					}
+					for (i = 0; i < MSGVEC_SIZE; i++) {
+						msgvec[i].msg_hdr.msg_iov = vec;
+						msgvec[i].msg_hdr.msg_iovlen = j;
+					}
+					if (UNLIKELY(sendmmsg(sfd, msgvec, MSGVEC_SIZE, 0) < 0)) {
+						if (stress_send_error(errno))
+							pr_fail("%s: sendmmsg failed, errno=%d (%s)\n",
+								args->name, errno, strerror(errno));
+					} else
+						msgs += (MSGVEC_SIZE * j);
+					break;
 #endif
-			default:
-				/* Should never happen */
-				pr_err("%s: bad option %d\n", args->name, sock_opts);
-				(void)close(sfd);
-				goto die_close;
+				default:
+					/* Should never happen */
+					pr_err("%s: bad option %d\n", args->name, sock_opts);
+					(void)close(sfd);
+					goto die_close;
+				}
+				stress_bogo_inc(args);
 			}
-			if (getpeername(sfd, &saddr, &len) < 0) {
+			if (UNLIKELY(getpeername(sfd, &saddr, &len) < 0)) {
 				if (errno != ENOTCONN)
 					pr_fail("%s: getpeername failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
 			}
 #if defined(SIOCOUTQ)
-			{
-				int pending;
+			if ((count & 0x3ff) == 0) {
+				int outq_len;
 
-				VOID_RET(int, ioctl(sfd, SIOCOUTQ, &pending));
+				if (LIKELY(ioctl(sfd, SIOCOUTQ, &outq_len) == 0)) {
+					outq_bytes += outq_len;
+					outq_samples++;
+				}
 			}
+			count++;
 #endif
 			stress_sock_ioctl(fd, sock_domain, rt);
 			stress_read_fdinfo(self, sfd);
@@ -1117,8 +1197,13 @@ static int stress_sock_server(
 		if (sfd >= 0)
 			(void)close(sfd);
 #endif
-		inc_counter(args);
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
+
+	duration = stress_time_now() - t;
+	metric = (duration > 0.0) ? (double)msgs / duration : 0.0;
+	stress_metrics_set(args, 0, "messages sent per sec", metric);
+	metric = (outq_samples > 0) ? (double)outq_bytes / (double)outq_samples : 0.0;
+	stress_metrics_set(args, 1, "byte average out queue length", metric);
 
 die_close:
 	(void)close(fd);
@@ -1128,17 +1213,15 @@ die:
 #if defined(AF_UNIX) &&		\
     defined(HAVE_SOCKADDR_UN)
 	if (addr && (sock_domain == AF_UNIX)) {
-		struct sockaddr_un *addr_un = (struct sockaddr_un *)addr;
+		const struct sockaddr_un *addr_un = (struct sockaddr_un *)addr;
 
 		(void)shim_unlink(addr_un->sun_path);
 	}
 #endif
 	if (pid) {
-		(void)kill(pid, SIGKILL);
+		(void)shim_kill(pid, SIGKILL);
 		(void)shim_waitpid(pid, &status, 0);
 	}
-	pr_dbg("%s: %" PRIu64 " messages sent\n", args->name, msgs);
-
 	return rc;
 }
 
@@ -1146,7 +1229,7 @@ static void stress_sock_sigpipe_handler(int signum)
 {
 	(void)signum;
 
-	keep_stressing_set_flag(false);
+	stress_continue_set_flag(false);
 }
 
 /*
@@ -1184,17 +1267,16 @@ static int stress_sock(const stress_args_t *args)
 	int sock_domain = AF_INET;
 	int sock_type = SOCK_STREAM;
 	int sock_port = DEFAULT_SOCKET_PORT;
-#if defined(IPPROTO_TCP)
-	int sock_protocol = IPPROTO_TCP;
-#else
 	int sock_protocol = 0;
-#endif
 	int sock_zerocopy = false;
-	int rc = EXIT_SUCCESS, reserved_port;
+	int rc = EXIT_SUCCESS, reserved_port, parent_cpu;
 	const bool rt = stress_sock_kernel_rt();
 	char *mmap_buffer;
 	char *sock_if = NULL;
 
+	if (stress_sigchld_set_handler(args) < 0)
+		return EXIT_NO_RESOURCE;
+
 	(void)stress_get_setting("sock-if", &sock_if);
 	(void)stress_get_setting("sock-domain", &sock_domain);
 	(void)stress_get_setting("sock-type", &sock_type);
@@ -1239,11 +1321,12 @@ static int stress_sock(const stress_args_t *args)
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 again:
+	parent_cpu = stress_get_cpu();
 	pid = fork();
 	if (pid < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
-		if (!keep_stressing(args)) {
+		if (!stress_continue(args)) {
 			rc = EXIT_SUCCESS;
 			goto finish;
 		}
@@ -1251,13 +1334,12 @@ again:
 			args->name, errno, strerror(errno));
 		return EXIT_FAILURE;
 	} else if (pid == 0) {
+		(void)stress_change_cpu(args, parent_cpu);
+
 		rc = stress_sock_client(args, mmap_buffer, mypid, sock_opts,
 			sock_domain, sock_type, sock_protocol,
 			sock_port, sock_if, rt, sock_zerocopy);
 		(void)munmap((void *)mmap_buffer, MMAP_BUF_SIZE);
-
-		/* Inform parent we're all done */
-		(void)kill(getppid(), SIGALRM);
 		_exit(rc);
 	} else {
 		rc = stress_sock_server(args, mmap_buffer, pid, mypid, sock_opts,
@@ -1276,6 +1358,7 @@ finish:
 static const stress_opt_set_func_t opt_set_funcs[] = {
 	{ OPT_sock_domain,	stress_set_sock_domain },
 	{ OPT_sock_if,		stress_set_sock_if },
+	{ OPT_sock_msgs,	stress_set_sock_msgs },
 	{ OPT_sock_opts,	stress_set_sock_opts },
 	{ OPT_sock_type,	stress_set_sock_type },
 	{ OPT_sock_port,	stress_set_sock_port },
