@@ -18,7 +18,9 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
 #include "core-net.h"
+#include "core-out-of-memory.h"
 
 #if defined(HAVE_SYS_UN_H)
 #include <sys/un.h>
@@ -66,7 +68,7 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
 static inline ssize_t stress_socket_fd_sendmsg(const int fd, const int fd_send)
 {
 	struct iovec iov;
-	struct msghdr msg;
+	struct msghdr msg ALIGN64;
 	struct cmsghdr *cmsg;
 	int *ptr;
 
@@ -76,11 +78,11 @@ static inline ssize_t stress_socket_fd_sendmsg(const int fd, const int fd_send)
 	iov.iov_base = msg_data;
 	iov.iov_len = 1;
 
-	(void)memset(&msg, 0, sizeof(msg));
+	(void)shim_memset(&msg, 0, sizeof(msg));
 	msg.msg_iov = &iov;
 	msg.msg_iovlen = 1;
 
-	(void)memset(ctrl, 0, sizeof(ctrl));
+	(void)shim_memset(ctrl, 0, sizeof(ctrl));
 	msg.msg_control = ctrl;
 	msg.msg_controllen = sizeof(ctrl);
 
@@ -101,7 +103,7 @@ static inline ssize_t stress_socket_fd_sendmsg(const int fd, const int fd_send)
 static inline int stress_socket_fd_recv(const int fd)
 {
 	struct iovec iov;
-	struct msghdr msg;
+	struct msghdr ALIGN64 msg;
 	struct cmsghdr *cmsg;
 	char msg_data[1] = { 0 };
 	char ctrl[CMSG_SPACE(sizeof(int))];
@@ -109,19 +111,19 @@ static inline int stress_socket_fd_recv(const int fd)
 	iov.iov_base = msg_data;
 	iov.iov_len = 1;
 
-	(void)memset(&msg, 0, sizeof(msg));
+	(void)shim_memset(&msg, 0, sizeof(msg));
 	msg.msg_iov = &iov;
 	msg.msg_iovlen = 1;
 
-	(void)memset(ctrl, 0, sizeof(ctrl));
+	(void)shim_memset(ctrl, 0, sizeof(ctrl));
 	msg.msg_control = ctrl;
 	msg.msg_controllen = sizeof(ctrl);
 
-	if (recvmsg(fd, &msg, 0) <= 0)
+	if (UNLIKELY(recvmsg(fd, &msg, 0) <= 0))
 		return -1;
-	if (msg_data[0] != MSG_ID)
+	if (UNLIKELY(msg_data[0] != MSG_ID))
 		return -1;
-	if ((msg.msg_flags & MSG_CTRUNC) == MSG_CTRUNC)
+	if (UNLIKELY((msg.msg_flags & MSG_CTRUNC) == MSG_CTRUNC))
 		return -1;
 
 	cmsg = CMSG_FIRSTHDR(&msg);
@@ -140,7 +142,7 @@ static inline int stress_socket_fd_recv(const int fd)
  *  stress_socket_client()
  *	client reader
  */
-static int stress_socket_client(
+static int OPTIMIZE3 stress_socket_client(
 	const stress_args_t *args,
 	const pid_t mypid,
 	const ssize_t max_fd,
@@ -159,46 +161,45 @@ static int stress_socket_client(
 		socklen_t addr_len = 0;
 		int so_reuseaddr = 1;
 
-		(void)memset(fds, 0, fds_size);
+		(void)shim_memset(fds, 0, fds_size);
 retry:
-		if (!keep_stressing_flag())
-			return EXIT_FAILURE;
+		if (!stress_continue_flag())
+			return EXIT_NO_RESOURCE;
 
-		if ((fd = socket(AF_UNIX, SOCK_STREAM, 0)) < 0) {
+		if (UNLIKELY((fd = socket(AF_UNIX, SOCK_STREAM, 0)) < 0)) {
 			pr_fail("%s: socket failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			return EXIT_FAILURE;
 		}
-		if (setsockopt(fd, SOL_SOCKET, SO_REUSEADDR,
-				&so_reuseaddr, sizeof(so_reuseaddr)) < 0) {
+		if (UNLIKELY(setsockopt(fd, SOL_SOCKET, SO_REUSEADDR,
+				&so_reuseaddr, sizeof(so_reuseaddr)) < 0)) {
 			(void)close(fd);
 			pr_fail("%s: setsockopt SO_REUSEADDR failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			return EXIT_FAILURE;
 		}
 
-		if (stress_set_sockaddr(args->name, args->instance, mypid,
+		if (UNLIKELY(stress_set_sockaddr(args->name, args->instance, mypid,
 				AF_UNIX, socket_fd_port,
-				&addr, &addr_len, NET_ADDR_ANY) < 0) {
+				&addr, &addr_len, NET_ADDR_ANY) < 0)) {
 			return EXIT_FAILURE;
 		}
-		if (connect(fd, addr, addr_len) < 0) {
+		if (UNLIKELY(connect(fd, addr, addr_len) < 0)) {
 			(void)close(fd);
-			(void)shim_usleep(10000);
-			retries++;
-			if (retries > 100) {
+			if (retries++ > 100) {
 				/* Give up.. */
 				pr_fail("%s: connect failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
-				return EXIT_FAILURE;
+				return EXIT_NO_RESOURCE;
 			}
+			(void)shim_usleep(10000);
 			goto retry;
 		}
 
-		if (!keep_stressing_flag())
+		if (UNLIKELY(!stress_continue_flag()))
 			return EXIT_SUCCESS;
 
-		for (n = 0; keep_stressing(args) && (n < max_fd); n++) {
+		for (n = 0; stress_continue(args) && (n < max_fd); n++) {
 			int rc, nbytes;
 
 			fds[n] = stress_socket_fd_recv(fd);
@@ -217,11 +218,11 @@ retry:
 		stress_close_fds(fds, n);
 		(void)shutdown(fd, SHUT_RDWR);
 		(void)close(fd);
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 #if defined(HAVE_SOCKADDR_UN)
 	if (addr) {
-		struct sockaddr_un *addr_un = (struct sockaddr_un *)addr;
+		const struct sockaddr_un *addr_un = (struct sockaddr_un *)addr;
 
 		(void)shim_unlink(addr_un->sun_path);
 	}
@@ -236,14 +237,13 @@ retry:
  *  stress_socket_server()
  *	server writer
  */
-static int stress_socket_server(
+static int OPTIMIZE3 stress_socket_server(
 	const stress_args_t *args,
-	const pid_t pid,
 	const pid_t ppid,
 	const ssize_t max_fd,
 	const int socket_fd_port)
 {
-	int fd, status;
+	int fd;
 	int so_reuseaddr = 1;
 	socklen_t addr_len = 0;
 	struct sockaddr *addr = NULL;
@@ -297,14 +297,14 @@ static int stress_socket_server(
 	do {
 		int sfd;
 
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			break;
 
 		sfd = accept(fd, (struct sockaddr *)NULL, NULL);
 		if (sfd >= 0) {
 			ssize_t i;
 
-			for (i = 0; keep_stressing(args) && (i < max_fd); i++) {
+			for (i = 0; stress_continue(args) && (i < max_fd); i++) {
 				int new_fd;
 
 				new_fd = open("/dev/zero", O_RDWR);
@@ -330,28 +330,23 @@ static int stress_socket_server(
 					(void)close(new_fd);
 					VOID_RET(ssize_t, stress_socket_fd_sendmsg(sfd, bad_fd));
 					msgs++;
-					inc_counter(args);
+					stress_bogo_inc(args);
 				}
 			}
 			(void)close(sfd);
 		}
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 die_close:
 	(void)close(fd);
 die:
 #if defined(HAVE_SOCKADDR_UN)
 	if (addr) {
-		struct sockaddr_un *addr_un = (struct sockaddr_un *)addr;
+		const struct sockaddr_un *addr_un = (struct sockaddr_un *)addr;
 
 		(void)shim_unlink(addr_un->sun_path);
 	}
 #endif
-
-	if (pid) {
-		(void)kill(pid, SIGALRM);
-		(void)shim_waitpid(pid, &status, 0);
-	}
 	pr_dbg("%s: %" PRIu64 " messages sent\n", args->name, msgs);
 
 	return rc;
@@ -370,6 +365,9 @@ static int stress_sockfd(const stress_args_t *args)
 	int *fds;
 	size_t fds_size;
 
+	if (stress_sigchld_set_handler(args) < 0)
+		return EXIT_NO_RESOURCE;
+
 	(void)stress_get_setting("sockfd-port", &socket_fd_port);
 
 	socket_fd_port += args->instance;
@@ -408,9 +406,9 @@ static int stress_sockfd(const stress_args_t *args)
 again:
 	pid = fork();
 	if (pid < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
-		if (!keep_stressing(args)) {
+		if (!stress_continue(args)) {
 			ret = EXIT_SUCCESS;
 			goto finish;
 		}
@@ -419,15 +417,15 @@ again:
 		free(fds);
 		return EXIT_FAILURE;
 	} else if (pid == 0) {
-		stress_set_oom_adjustment(args->name, false);
+		stress_set_oom_adjustment(args, false);
 		ret = stress_socket_client(args, mypid, max_fd, socket_fd_port, fds, fds_size);
-
-		/* Inform parent we're all done */
-		(void)kill(getppid(), SIGALRM);
-
 		_exit(ret);
 	} else {
-		ret = stress_socket_server(args, pid, mypid, max_fd, socket_fd_port);
+		int status;
+
+		ret = stress_socket_server(args, mypid, max_fd, socket_fd_port);
+		(void)shim_kill(pid, SIGALRM);
+		(void)shim_waitpid(pid, &status, 0);
 	}
 
 finish:
@@ -450,6 +448,7 @@ stressor_info_t stress_sockfd_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_NETWORK | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "only supported on Linux"
 };
