@@ -18,6 +18,8 @@
  *
  */
 #include "stress-ng.h"
+#include "core-affinity.h"
+#include "core-builtin.h"
 #include "core-net.h"
 
 #if defined(HAVE_NETINET_TCP_H)
@@ -68,9 +70,9 @@ static int stress_set_sockmany_if(const char *name)
  *  stress_sockmany_cleanup()
  *	close sockets
  */
-static void stress_sockmany_cleanup(int fds[], const int n)
+static void OPTIMIZE3 stress_sockmany_cleanup(int fds[], const int n)
 {
-	int i;
+	register int i;
 
 	for (i = 0; i < n; i++) {
 		if (fds[i] >= 0) {
@@ -85,7 +87,7 @@ static void stress_sockmany_cleanup(int fds[], const int n)
  *  stress_sockmany_client()
  *	client reader
  */
-static int stress_sockmany_client(
+static int OPTIMIZE3 stress_sockmany_client(
 	const stress_args_t *args,
 	const int sockmany_port,
 	const pid_t mypid,
@@ -109,11 +111,11 @@ static int stress_sockmany_client(
 			int retries = 0;
 			socklen_t addr_len = 0;
 retry:
-			if (!keep_stressing_flag()) {
+			if (UNLIKELY(!stress_continue_flag())) {
 				stress_sockmany_cleanup(fds, i);
 				break;
 			}
-			if ((fds[i] = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
+			if (UNLIKELY((fds[i] = socket(AF_INET, SOCK_STREAM, 0)) < 0)) {
 				/* Out of resources? */
 				if ((errno == EMFILE) ||
 				    (errno == ENFILE) ||
@@ -128,12 +130,12 @@ retry:
 				return EXIT_FAILURE;
 			}
 
-			if (stress_set_sockaddr_if(args->name, args->instance, mypid,
+			if (UNLIKELY(stress_set_sockaddr_if(args->name, args->instance, mypid,
 					AF_INET, sockmany_port, sockmany_if,
-					&addr, &addr_len, NET_ADDR_ANY) < 0) {
+					&addr, &addr_len, NET_ADDR_ANY) < 0)) {
 				return EXIT_FAILURE;
 			}
-			if (connect(fds[i], addr, addr_len) < 0) {
+			if (UNLIKELY(connect(fds[i], addr, addr_len) < 0)) {
 				int save_errno = errno;
 
 				(void)close(fds[i]);
@@ -156,7 +158,7 @@ retry:
 			}
 
 			n = recv(fds[i], buf, sizeof(buf), 0);
-			if (n < 0) {
+			if (UNLIKELY(n < 0)) {
 				if ((errno != EINTR) && (errno != ECONNRESET))
 					pr_fail("%s: recv failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
@@ -166,7 +168,7 @@ retry:
 				sock_fds->max_fd = i;
 		}
 		stress_sockmany_cleanup(fds, i);
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 	return EXIT_SUCCESS;
 }
@@ -175,15 +177,14 @@ retry:
  *  stress_sockmany_server()
  *	server writer
  */
-static int stress_sockmany_server(
+static int OPTIMIZE3 stress_sockmany_server(
 	const stress_args_t *args,
 	const int sockmany_port,
-	const pid_t pid,
 	const pid_t mypid,
 	const char *sockmany_if)
 {
 	char ALIGN64 buf[SOCKET_MANY_BUF];
-	int fd, status;
+	int fd;
 	int so_reuseaddr = 1;
 	socklen_t addr_len = 0;
 	struct sockaddr *addr = NULL;
@@ -230,24 +231,25 @@ static int stress_sockmany_server(
 	do {
 		int sfd;
 
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			break;
 
 		sfd = accept(fd, (struct sockaddr *)NULL, NULL);
-		if (sfd >= 0) {
+		if (LIKELY(sfd >= 0)) {
 			ssize_t sret;
 			struct sockaddr saddr;
 			socklen_t len;
 			int sndbuf;
+
 			len = sizeof(saddr);
-			if (getsockname(fd, &saddr, &len) < 0) {
+			if (UNLIKELY(getsockname(fd, &saddr, &len) < 0)) {
 				pr_fail("%s: getsockname failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 				(void)close(sfd);
 				break;
 			}
 			len = sizeof(sndbuf);
-			if (getsockopt(fd, SOL_SOCKET, SO_SNDBUF, &sndbuf, &len) < 0) {
+			if (UNLIKELY(getsockopt(fd, SOL_SOCKET, SO_SNDBUF, &sndbuf, &len)) < 0) {
 				pr_fail("%s: getsockopt failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 				(void)close(sfd);
@@ -267,9 +269,9 @@ static int stress_sockmany_server(
 #else
 			UNEXPECTED
 #endif
-			(void)memset(buf, 'A' + (msgs % 26), sizeof(buf));
+			(void)shim_memset(buf, stress_ascii64[msgs & 63], sizeof(buf));
 			sret = send(sfd, buf, sizeof(buf), 0);
-			if (sret < 0) {
+			if (UNLIKELY(sret < 0)) {
 				if ((errno != EINTR) && (errno != EPIPE))
 					pr_fail("%s: send failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
@@ -280,17 +282,12 @@ static int stress_sockmany_server(
 			}
 			(void)close(sfd);
 		}
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 die_close:
 	(void)close(fd);
 die:
-	if (pid) {
-		(void)kill(pid, SIGKILL);
-		(void)shim_waitpid(pid, &status, 0);
-	}
-
 	return rc;
 }
 
@@ -298,7 +295,7 @@ static void stress_sockmany_sigpipe_handler(int signum)
 {
 	(void)signum;
 
-	keep_stressing_set_flag(false);
+	stress_continue_set_flag(false);
 }
 
 /*
@@ -310,9 +307,12 @@ static int stress_sockmany(const stress_args_t *args)
 	pid_t pid, ppid = getppid();
 	stress_sock_fds_t *sock_fds;
 	int sockmany_port = DEFAULT_SOCKET_MANY_PORT;
-	int rc = EXIT_SUCCESS, reserved_port;
+	int rc = EXIT_SUCCESS, reserved_port, parent_cpu;
 	char *sockmany_if = NULL;
 
+	if (stress_sigchld_set_handler(args) < 0)
+		return EXIT_NO_RESOURCE;
+
 	(void)stress_get_setting("sockmany-if", &sockmany_if);
 	(void)stress_get_setting("sockmany-port", &sockmany_port);
 
@@ -353,11 +353,12 @@ static int stress_sockmany(const stress_args_t *args)
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 again:
+	parent_cpu = stress_get_cpu();
 	pid = fork();
 	if (pid < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
-		if (!keep_stressing(args)) {
+		if (!stress_continue(args)) {
 			rc = EXIT_SUCCESS;
 			goto finish;
 		}
@@ -365,14 +366,16 @@ again:
 			args->name, errno, strerror(errno));
 		rc = EXIT_FAILURE;
 	} else if (pid == 0) {
+		(void)stress_change_cpu(args, parent_cpu);
+
 		rc = stress_sockmany_client(args, sockmany_port, ppid, sock_fds, sockmany_if);
-
-		/* Inform parent we're all done */
-		(void)kill(getppid(), SIGALRM);
-
 		_exit(rc);
 	} else {
-		rc = stress_sockmany_server(args, sockmany_port, pid, ppid, sockmany_if);
+		int status;
+
+		rc = stress_sockmany_server(args, sockmany_port, ppid, sockmany_if);
+		(void)shim_kill(pid, SIGKILL);
+		(void)shim_waitpid(pid, &status, 0);
 	}
 	pr_dbg("%s: %d sockets opened at one time\n", args->name, sock_fds->max_fd);
 
