@@ -18,6 +18,9 @@
  *
  */
 #include "stress-ng.h"
+#include "core-affinity.h"
+#include "core-builtin.h"
+#include "core-out-of-memory.h"
 
 #define MAX_SOCKET_PAIRS	(32768)
 #define SOCKET_PAIR_BUF         (4096)	/* Socket pair I/O buffer size */
@@ -49,7 +52,7 @@ static inline void socket_pair_memset(
  *  socket_pair_memchk()
  *	check data contains incrementing chars from val upwards
  */
-static inline int socket_pair_memchk(
+static inline int OPTIMIZE3 socket_pair_memchk(
 	uint8_t *buf,
 	const size_t sz)
 {
@@ -82,7 +85,7 @@ static void socket_pair_try_leak(void)
 {
 	int fds[2];
 
-	if (socketpair(AF_UNIX, SOCK_STREAM, 0, fds) < 0)
+	if (UNLIKELY(socketpair(AF_UNIX, SOCK_STREAM, 0, fds) < 0))
 		return;
 
 	(void)send(fds[0], "0", 1, MSG_OOB);
@@ -102,9 +105,11 @@ static int stress_sockpair_oomable(const stress_args_t *args, void *context)
 	pid_t pid;
 	static int socket_pair_fds[MAX_SOCKET_PAIRS][2];
 	int socket_pair_fds_bad[2];
-	int i, max, ret;
+	int i, max, ret, parent_cpu;
 	double t, duration, rate, bytes = 0.0;
-
+	uint64_t low_memory_count = 0;
+	const size_t low_mem_size = args->page_size * 32 * args->num_instances;
+	const bool oom_avoid = !!(g_opt_flags & OPT_FLAGS_OOM_AVOID);
 	(void)context;
 
 	/* exercise invalid socketpair domain */
@@ -132,12 +137,12 @@ static int stress_sockpair_oomable(const stress_args_t *args, void *context)
 		(void)close(socket_pair_fds_bad[1]);
 	}
 
-	(void)memset(socket_pair_fds, 0, sizeof(socket_pair_fds));
+	(void)shim_memset(socket_pair_fds, 0, sizeof(socket_pair_fds));
 	errno = 0;
 
 	t = stress_time_now();
 	for (max = 0; max < MAX_SOCKET_PAIRS; max++) {
-		if (!keep_stressing(args)) {
+		if (!stress_continue(args)) {
 			socket_pair_close(socket_pair_fds, max, 0);
 			socket_pair_close(socket_pair_fds, max, 1);
 			return EXIT_SUCCESS;
@@ -188,34 +193,38 @@ static int stress_sockpair_oomable(const stress_args_t *args, void *context)
 	}
 
 again:
+	parent_cpu = stress_get_cpu();
 	pid = fork();
 	if (pid < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
 
 		socket_pair_close(socket_pair_fds, max, 0);
 		socket_pair_close(socket_pair_fds, max, 1);
 
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			goto finish;
 		pr_err("%s: fork failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		return EXIT_FAILURE;
 	} else if (pid == 0) {
-		stress_set_oom_adjustment(args->name, true);
+		const bool verify = !!(g_opt_flags & OPT_FLAGS_VERIFY);
+
+		(void)stress_change_cpu(args, parent_cpu);
+		stress_set_oom_adjustment(args, true);
 		stress_parent_died_alarm();
 		(void)sched_settings_apply(true);
 
 		socket_pair_close(socket_pair_fds, max, 1);
-		while (keep_stressing(args)) {
-			uint8_t buf[SOCKET_PAIR_BUF];
+		while (stress_continue(args)) {
+			uint8_t buf[SOCKET_PAIR_BUF] ALIGN64;
 			ssize_t n;
 
-			for (i = 0; keep_stressing(args) && (i < max); i++) {
+			for (i = 0; stress_continue(args) && (i < max); i++) {
 				errno = 0;
 
 				n = read(socket_pair_fds[i][0], buf, sizeof(buf));
-				if (n <= 0) {
+				if (UNLIKELY(n <= 0)) {
 					if ((errno == EAGAIN) || (errno == EINTR))
 						continue;
 					else if (errno == ENFILE) /* Too many files! */
@@ -233,11 +242,12 @@ again:
 					}
 					continue;
 				}
-				if ((g_opt_flags & OPT_FLAGS_VERIFY) &&
-				    socket_pair_memchk(buf, (size_t)n)) {
+				if (UNLIKELY(verify && socket_pair_memchk(buf, (size_t)n))) {
 					pr_fail("%s: socket_pair read error detected, "
 						"failed to read expected data\n", args->name);
 				}
+				if (UNLIKELY(oom_avoid && stress_low_memory(low_mem_size)))
+					continue;
 				socket_pair_try_leak();
 			}
 		}
@@ -245,10 +255,10 @@ abort:
 		socket_pair_close(socket_pair_fds, max, 0);
 		_exit(EXIT_SUCCESS);
 	} else {
-		uint8_t buf[SOCKET_PAIR_BUF];
+		uint8_t buf[SOCKET_PAIR_BUF] ALIGN64;
 		int val = 0, status;
 
-		stress_set_oom_adjustment(args->name, true);
+		stress_set_oom_adjustment(args, true);
 		stress_parent_died_alarm();
 		(void)sched_settings_apply(true);
 
@@ -256,9 +266,19 @@ abort:
 		socket_pair_close(socket_pair_fds, max, 0);
 
 		do {
-			for (i = 0; keep_stressing(args) && (i < max); i++) {
+			for (i = 0; stress_continue(args) && (i < max); i++) {
 				ssize_t wret;
 
+				/* Low memory avoidance, re-start */
+				if (UNLIKELY(oom_avoid)) {
+					while (stress_low_memory(low_mem_size)) {
+						low_memory_count++;
+						if (!stress_continue_flag())
+							goto tidy;
+						shim_usleep(100000);
+					}
+				}
+
 				socket_pair_memset(buf, (uint8_t)val++, sizeof(buf));
 				t = stress_time_now();
 				wret = write(socket_pair_fds[i][1], buf, sizeof(buf));
@@ -277,13 +297,20 @@ abort:
 					}
 					continue;
 				}
-				inc_counter(args);
+				shim_sched_yield();
+				stress_bogo_inc(args);
 			}
-		} while (keep_stressing(args));
+		} while (stress_continue(args));
 
+tidy:
 		rate = (duration > 0.0) ? (double)bytes / duration : 0.0;
 		stress_metrics_set(args, 1, "MB written per sec", rate / (double)MB);
 
+		if (low_memory_count > 0) {
+			pr_dbg("%s: %.2f%% of writes backed off due to low memory\n",
+				args->name, 100.0 * (double)low_memory_count / (double)stress_bogo_get(args));
+		}
+
 		for (i = 0; i < max; i++) {
 			if (shutdown(socket_pair_fds[i][1], SHUT_RDWR) < 0)
 				if (errno != ENOTCONN) {
@@ -291,7 +318,7 @@ abort:
 						args->name, errno, strerror(errno));
 				}
 		}
-		(void)kill(pid, SIGKILL);
+		(void)shim_kill(pid, SIGKILL);
 		(void)shim_waitpid(pid, &status, 0);
 		socket_pair_close(socket_pair_fds, max, 1);
 	}
