@@ -18,6 +18,9 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
+#include "core-killpid.h"
+#include "core-out-of-memory.h"
 #include "core-put.h"
 
 #if defined(HAVE_UCONTEXT_H)
@@ -37,41 +40,73 @@ static const stress_help_t help[] = {
 
 #define MMAPSTACK_SIZE		(256 * KB)
 
+/*
+ *  stress_stack_check sanity check list
+ */
+typedef struct stress_stack_check {
+	struct stress_stack_check *prev;	/* Previous item on stack list */
+	struct stress_stack_check *self_addr;	/* Address of this struct to check */
+	uint32_t waste[2];			/* Waste data on stack */
+} stress_stack_check_t;
+
 static ucontext_t c_main, c_test;
 static uint8_t *stack_mmap;		/* mmap'd stack */
 static uintptr_t page_mask;
 static size_t page_size;
+static int check_status;		/* sanity check status */
 
 /*
  *  push values onto file backed mmap'd stack and
  *  force msync on the map'd region if page boundary
  *  has changed
  */
-static void stress_stackmmap_push_msync(void)
+static void stress_stackmmap_push_msync(stress_stack_check_t *prev_check)
 {
 	void *addr = (void *)(((uintptr_t)&addr) & page_mask);
 	static void *laddr;
-	uint32_t waste[2];
+	stress_stack_check_t check;
+	register stress_stack_check_t *ptr;
+	register int i;
+	static char name[] = "stackmmap";
+
+	check.prev = prev_check;
+	check.self_addr = &check;
 
 	/*
 	 * Ensure something is written to the stack that
 	 * won't get optimized away
 	 */
-	waste[0] = stress_mwc32();
-	stress_uint32_put(waste[0]);
-	waste[1] = stress_mwc32();
-	stress_uint32_put(waste[1]);
-	stress_uint64_put((uint64_t)(intptr_t)&waste);
+	check.waste[0] = stress_mwc32();
+	stress_uint32_put(check.waste[0]);
+	check.waste[1] = ~check.waste[0];
+	stress_uint32_put(check.waste[1]);
+	stress_uint64_put((uint64_t)(intptr_t)&check);
 
 	if (addr != laddr) {
-		(void)shim_msync(addr, page_size,
-			(stress_mwc8() & 1) ? MS_ASYNC : MS_SYNC);
+		(void)shim_msync(addr, page_size, stress_mwc1() ? MS_ASYNC : MS_SYNC);
 		laddr = addr;
 	}
-	if (keep_stressing_flag())
-		stress_stackmmap_push_msync();
 
-	stress_uint64_put((uint64_t)waste[1]);
+	for (i = 0, ptr = &check; (i < 256) && ptr; ptr = ptr->prev, i++) {
+		if (ptr->self_addr != ptr) {
+			pr_inf("%s: sanity check address mismatch, got 0x%p, "
+				"expecting 0x%p\n", name, ptr, ptr->self_addr);
+			check_status = EXIT_FAILURE;
+			return;
+		}
+		if (ptr->waste[0] != ~(ptr->waste[1])) {
+			pr_inf("%s: sanity check data mismatch, got 0x%" PRIx32
+				", expecting 0x%" PRIx32 "\n", name,
+				ptr->waste[0], ptr->waste[1]);
+			check_status = EXIT_FAILURE;
+			return;
+		}
+	}
+
+	if (stress_continue_flag())
+		stress_stackmmap_push_msync(&check);
+
+	stress_uint32_put(check.waste[1]);
 }
 
 /*
@@ -79,7 +114,7 @@ static void stress_stackmmap_push_msync(void)
  */
 static void stress_stackmmap_push_start(void)
 {
-	stress_stackmmap_push_msync();
+	stress_stackmmap_push_msync(NULL);
 }
 
 /*
@@ -149,7 +184,7 @@ static int stress_stackmmap(const stress_args_t *args)
 		pr_dbg("%s: madvise failed: errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 	}
-	(void)memset(stack_mmap, 0, MMAPSTACK_SIZE);
+	(void)shim_memset(stack_mmap, 0, MMAPSTACK_SIZE);
 
 #if defined(HAVE_MPROTECT)
 	/*
@@ -161,7 +196,7 @@ static int stress_stackmmap(const stress_args_t *args)
 	UNEXPECTED
 #endif
 
-	(void)memset(&c_test, 0, sizeof(c_test));
+	(void)shim_memset(&c_test, 0, sizeof(c_test));
 	if (getcontext(&c_test) < 0) {
 		pr_fail("%s: getcontext failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
@@ -180,14 +215,16 @@ static int stress_stackmmap(const stress_args_t *args)
 	 */
 	do {
 		pid_t pid;
+
+		(void)stress_mwc32();
 again:
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 		pid = fork();
 		if (pid < 0) {
-			if (stress_redo_fork(errno))
+			if (stress_redo_fork(args, errno))
 				goto again;
-			if (!keep_stressing(args))
+			if (!stress_continue(args))
 				goto finish;
 			pr_err("%s: fork failed: errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
@@ -200,9 +237,14 @@ again:
 				if (errno != EINTR)
 					pr_dbg("%s: waitpid(): errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
-				(void)kill(pid, SIGTERM);
-				(void)stress_killpid(pid);
-				(void)shim_waitpid(pid, &status, 0);
+				stress_kill_and_wait(args, pid, SIGTERM, false);
+			} else {
+				if (WIFEXITED(status)) {
+					if (WEXITSTATUS(status) != EXIT_SUCCESS) {
+						rc = status;
+						goto tidy_mmap;
+					}
+				}
 			}
 		} else {
 			/* Child */
@@ -211,7 +253,7 @@ again:
 			(void)sched_settings_apply(true);
 
 			/* Make sure this is killable by OOM killer */
-			stress_set_oom_adjustment(args->name, true);
+			stress_set_oom_adjustment(args, true);
 
 			/*
 			 *  We need to handle SEGV signals when we
@@ -219,7 +261,7 @@ again:
 			 *  an alternative signal handling stack
 			 *  is required because we ran out of stack
 			 */
-			(void)memset(&new_action, 0, sizeof new_action);
+			(void)shim_memset(&new_action, 0, sizeof new_action);
 			new_action.sa_handler = stress_sig_handler_exit;
 			(void)sigemptyset(&new_action.sa_mask);
 			new_action.sa_flags = SA_ONSTACK;
@@ -234,13 +276,13 @@ again:
 			if (stress_sigaltstack(stack_sig, STRESS_SIGSTKSZ) < 0)
 				_exit(EXIT_FAILURE);
 
+			check_status = EXIT_SUCCESS;
 			(void)makecontext(&c_test, stress_stackmmap_push_start, 0);
 			(void)swapcontext(&c_main, &c_test);
-
-			_exit(0);
+			_exit(check_status);
 		}
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 finish:
 	rc = EXIT_SUCCESS;
@@ -259,12 +301,14 @@ tidy_dir:
 stressor_info_t stress_stackmmap_info = {
 	.stressor = stress_stackmmap,
 	.class = CLASS_VM | CLASS_MEMORY,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
 stressor_info_t stress_stackmmap_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_VM | CLASS_MEMORY,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without ucontext.h or swapcontext()"
 };
