@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-affinity.h"
 #include "core-builtin.h"
 
 #if defined(HAVE_MQUEUE_H)
@@ -40,10 +41,13 @@ static const stress_help_t help[] = {
 	{ NULL, NULL, 			NULL }
 };
 
+typedef int (*switch_func_t)(const stress_args_t *args,
+			     const uint64_t switch_freq,
+			     const uint64_t switch_delay,
+			     const uint64_t threshold);
 typedef struct {
 	const char *name;
-	int (*func)(const stress_args_t *args, const uint64_t switch_freq,
-		    const uint64_t switch_delay, const uint64_t threshold);
+	const switch_func_t switch_func;
 } stress_switch_method_t;
 
 #define THRESH_FREQ	(100)		/* Delay adjustment rate in HZ */
@@ -67,7 +71,7 @@ static int stress_set_switch_freq(const char *opt)
  */
 static void stress_switch_rate(
 	const stress_args_t *args,
-	char *method,
+	const char *method,
 	const double t_start,
 	const double t_end,
 	uint64_t counter)
@@ -195,7 +199,7 @@ again:
 	parent_cpu = stress_get_cpu();
 	pid = fork();
 	if (pid < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
 		(void)close(pipefds[0]);
 		(void)close(pipefds[1]);
@@ -315,7 +319,7 @@ again:
 	parent_cpu = stress_get_cpu();
 	pid = fork();
 	if (pid < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
 		if (!stress_continue(args))
 			goto finish;
@@ -433,7 +437,7 @@ again:
 	parent_cpu = stress_get_cpu();
 	pid = fork();
 	if (pid < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
 		if (!stress_continue(args))
 			goto finish;
@@ -535,7 +539,7 @@ static int stress_switch(const stress_args_t *args)
 	switch_delay = (switch_freq == 0) ? 0 : STRESS_NANOSECOND / switch_freq;
 	threshold = switch_freq / THRESH_FREQ;
 
-	return stress_switch_methods[switch_method].func(args, switch_freq, switch_delay, threshold);
+	return stress_switch_methods[switch_method].switch_func(args, switch_freq, switch_delay, threshold);
 }
 
 static void stress_switch_set_default(void)
