@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-out-of-memory.h"
 
 #if defined(HAVE_SYS_STATFS_H)
 #include <sys/statfs.h>
@@ -1032,7 +1033,7 @@ static inline int stress_do_syscall(
 	pid_t pid;
 	int rc = 0;
 
-	if (!keep_stressing(args))
+	if (!stress_continue(args))
 		return 0;
 	pid = fork();
 	if (pid < 0) {
@@ -1089,13 +1090,13 @@ static inline int stress_do_syscall(
 			if (errno != EINTR)
 				pr_dbg("%s: waitpid(): errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
-			(void)kill(pid, SIGKILL);
+			(void)shim_kill(pid, SIGKILL);
 			(void)shim_waitpid(pid, &status, 0);
 
 		}
 		rc = WEXITSTATUS(status);
 
-		inc_counter(args);
+		stress_bogo_inc(args);
 	}
 	return rc;
 }
@@ -1118,7 +1119,7 @@ static int stress_sysbadaddr_child(const stress_args_t *args, void *context)
 				}
 			}
 		}
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 	return EXIT_SUCCESS;
 }
