@@ -18,6 +18,8 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
+#include "core-mounts.h"
 
 #if defined(HAVE_LINUX_FS_H)
 #include <linux/fs.h>
@@ -56,6 +58,7 @@ static const stress_help_t help[] = {
 static int stress_sysinfo(const stress_args_t *args)
 {
 	int n_mounts;
+	const int verify = !!(g_opt_flags & OPT_FLAGS_VERIFY);
 	char *mnts[128];
 #if defined(HAVE_SYS_SYSINFO_H) &&	\
     defined(HAVE_SYSINFO) &&		\
@@ -63,7 +66,7 @@ static int stress_sysinfo(const stress_args_t *args)
 	const int bad_fd = stress_get_bad_fd();
 #endif
 
-	(void)memset(mnts, 0, sizeof(mnts));
+	(void)shim_memset(mnts, 0, sizeof(mnts));
 
 	n_mounts = stress_mount_get(mnts, SIZEOF_ARRAY(mnts));
 	if (n_mounts < 0) {
@@ -89,32 +92,32 @@ static int stress_sysinfo(const stress_args_t *args)
 
 			ret = sysinfo(&sysinfo_buf);
 			if ((ret < 0) &&
-			    (g_opt_flags & OPT_FLAGS_VERIFY) &&
+			    (verify) &&
 			    (errno != EPERM))
 			 	pr_fail("%s: sysinfo failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 
-			if (!keep_stressing_flag())
+			if (!stress_continue_flag())
 				break;
 
 			/* Linux statfs variant */
 			for (i = 0; i < n_mounts; i++) {
 				int fd;
 
-				if (!keep_stressing_flag())
+				if (UNLIKELY(!stress_continue_flag()))
 					break;
 
-				if (!mnts[i])
+				if (UNLIKELY(!mnts[i]))
 					continue;
 
 				ret = statfs(mnts[i], &statfs_buf);
 				/* Mount may have been removed, so purge it */
-				if ((ret < 0) && (errno == ENOENT)) {
+				if (UNLIKELY((ret < 0) && (errno == ENOENT))) {
 					free(mnts[i]);
 					mnts[i] = NULL;
 					continue;
 				}
-				if ((ret < 0) && (g_opt_flags & OPT_FLAGS_VERIFY)) {
+				if (UNLIKELY((ret < 0) && (verify))) {
 					if ((errno != ENOSYS) &&
 					    (errno != EOVERFLOW) &&
 					    (errno != EACCES) &&
@@ -133,7 +136,7 @@ static int stress_sysinfo(const stress_args_t *args)
 				VOID_RET(int, statfs("", &statfs_buf));
 
 				fd = open(mnts[i], O_RDONLY | O_DIRECTORY);
-				if (fd < 0)
+				if (UNLIKELY(fd < 0))
 					continue;
 #if defined(FS_IOC_GETFSLABEL) &&	\
     defined(FSLABEL_MAX)
@@ -148,9 +151,9 @@ static int stress_sysinfo(const stress_args_t *args)
 
 				ret = fstatfs(fd, &statfs_buf);
 				(void)close(fd);
-				if ((ret < 0) && (errno == ENOENT))
+				if (UNLIKELY((ret < 0) && (errno == ENOENT)))
 					continue;
-				if ((ret < 0) && (g_opt_flags & OPT_FLAGS_VERIFY)) {
+				if (UNLIKELY((ret < 0) && (verify))) {
 					if ((errno != ENOSYS) &&
 					    (errno != EOVERFLOW) &&
 					    (errno != EACCES) &&
@@ -174,7 +177,7 @@ static int stress_sysinfo(const stress_args_t *args)
 			struct stat sbuf;
 			struct shim_ustat ubuf;
 
-			if (!keep_stressing_flag())
+			if (!stress_continue_flag())
 				break;
 
 			for (i = 0; i < n_mounts; i++) {
@@ -182,11 +185,11 @@ static int stress_sysinfo(const stress_args_t *args)
 					continue;
 
 				ret = stat(mnts[i], &sbuf);
-				if (ret < 0)
+				if (UNLIKELY(ret < 0))
 					continue;
 
 				ret = shim_ustat(sbuf.st_dev, &ubuf);
-				if ((ret < 0) && (g_opt_flags & OPT_FLAGS_VERIFY)) {
+				if (UNLIKELY((ret < 0) && (verify))) {
 					if ((errno != EINVAL) &&
 					    (errno != EFAULT) &&
 					    (errno != ENOSYS) &&
@@ -208,7 +211,7 @@ static int stress_sysinfo(const stress_args_t *args)
 #endif
 		}
 
-		if (!keep_stressing_flag())
+		if (UNLIKELY(!stress_continue_flag()))
 			break;
 
 #if defined(HAVE_SYS_STATVFS_H)
@@ -220,14 +223,14 @@ static int stress_sysinfo(const stress_args_t *args)
 			for (i = 0; i < n_mounts; i++) {
 				int ret;
 
-				if (!keep_stressing_flag())
+				if (UNLIKELY(!stress_continue_flag()))
 					break;
 
-				if (!mnts[i])
+				if (UNLIKELY(!mnts[i]))
 					continue;
 
 				ret = statvfs(mnts[i], &statvfs_buf);
-				if ((ret < 0) && (g_opt_flags & OPT_FLAGS_VERIFY)) {
+				if (UNLIKELY((ret < 0) && (verify))) {
 					if ((errno != ENOSYS) &&
 					    (errno != EOVERFLOW) &&
 					    (errno != EACCES) &&
@@ -246,15 +249,15 @@ static int stress_sysinfo(const stress_args_t *args)
 		}
 #endif
 
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 		clk = times(&tms_buf);
-		if ((clk == (clock_t)-1) && (g_opt_flags & OPT_FLAGS_VERIFY)) {
+		if ((clk == (clock_t)-1) && (verify)) {
 			 pr_fail("%s: times failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 		}
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
