@@ -18,6 +18,8 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
+#include "core-out-of-memory.h"
 
 static const stress_help_t help[] = {
 	{ NULL,	"sysinval N",		"start N workers that pass invalid args to syscalls" },
@@ -107,14 +109,10 @@ static const stress_help_t help[] = {
 /*
  *  rotate right for hashing
  */
-#define ROR(val)						\
+#define RORn(val, n)						\
 do {								\
-	unsigned long tmp = val;				\
-	const size_t bits = (sizeof(unsigned long) * 8) - 1;	\
-	const unsigned long bit0 = (tmp & 1) << bits;		\
-	tmp >>= 1;                              		\
-	tmp |= bit0;                            		\
-	val = tmp;                              		\
+	val = (sizeof(unsigned long) == sizeof(uint32_t)) ?	\
+		shim_ror32n(val, n) : shim_ror64n(val, n);	\
 } while (0)
 
 #define SHR_UL(v, shift) ((unsigned long)(((unsigned long long)v) << shift))
@@ -170,14 +168,13 @@ typedef struct {
 /*
  *  hash table - in the parent context this records system
  *  calls that crash the child. in the child context this
- *  contains the same crasher data that the parent has plus
+ *  contains the same crash data that the parent has plus
  *  a cache of the system calls that return 0 and we don't
  *  want to retest
  */
 static stress_syscall_hash_table_t *hash_table;
 
 static sigjmp_buf jmpbuf;
-static double time_end;
 
 static const int sigs[] = {
 #if defined(SIGILL)
@@ -735,7 +732,7 @@ static const stress_syscall_arg_t stress_syscall_args[] = {
 	{ SYS(getpid), 0, { 0, 0, 0, 0, 0, 0 } },
 #endif
 #if DEFSYS(getpmsg)
-	/* Unimplimented */
+	/* Unimplemented */
 	/* { SYS(getpmsg), 0, { 0, 0, 0, 0, 0, 0 } }, */
 #endif
 #if DEFSYS(getppid)
@@ -959,7 +956,7 @@ static const stress_syscall_arg_t stress_syscall_args[] = {
 	{ SYS(llseek), 5, { ARG_FD, ARG_UINT, ARG_UINT, ARG_PTR, ARG_UINT, 0 } },
 #endif
 #if DEFSYS(lock)
-	/* Unimplimented, deprecated */
+	/* Unimplemented, deprecated */
 #endif
 #if DEFSYS(lookup_dcookie)
 	{ SYS(lookup_dcookie), 3, { ARG_UINT, ARG_PTR, ARG_LEN, 0, 0, 0 } },
@@ -993,7 +990,7 @@ static const stress_syscall_arg_t stress_syscall_args[] = {
 	{ SYS(madvise), 3, { ARG_PTR, ARG_LEN, ARG_INT, 0, 0, 0 } },
 #endif
 #if DEFSYS(madvise1)
-	/* Unimplimented, deprecated */
+	/* Unimplemented, deprecated */
 #endif
 #if DEFSYS(mbind)
 	{ SYS(mbind), 6, { ARG_PTR, ARG_UINT, ARG_INT, ARG_PTR, ARG_UINT, ARG_UINT } },
@@ -1071,7 +1068,7 @@ static const stress_syscall_arg_t stress_syscall_args[] = {
 	{ SYS(mprotect), 3, { ARG_PTR, ARG_LEN, ARG_UINT, 0, 0, 0 } },
 #endif
 #if DEFSYS(mpx)
-	/* Unimplimented, deprecated */
+	/* Unimplemented, deprecated */
 #endif
 #if DEFSYS(mq_close)
 	{ SYS(mq_close), 1, { ARG_INT, 0, 0, 0, 0, 0 } },
@@ -1460,6 +1457,9 @@ static const stress_syscall_arg_t stress_syscall_args[] = {
 #if DEFSYS(riscv_flush_icache)
 	{ SYS(riscv_flush_icache), 3, { ARG_PTR, ARG_PTR, ARG_INT, 0, 0, 0 } },
 #endif
+#if DEFSYS(riscv_hwprobe)
+	{ SYS(riscv_hwprobe), 5, { ARG_PTR, ARG_UINT, ARG_UINT, ARG_PTR, ARG_INT, 0 } },
+#endif
 #if DEFSYS(rmdir)
 	/* { SYS(rmdir), 1, { ARG_EMPTY_FILENAME, 0, 0, 0, 0, 0 } }, */
 #endif
@@ -2300,23 +2300,17 @@ static unsigned long stress_syscall_hash(
 {
 	unsigned long hash = syscall_num;
 
-	ROR(hash);
-	ROR(hash);
+	RORn(hash, 2);
 	hash ^= (args[0]);
-	ROR(hash);
-	ROR(hash);
+	RORn(hash, 2);
 	hash ^= (args[1]);
-	ROR(hash);
-	ROR(hash);
+	RORn(hash, 2);
 	hash ^= (args[2]);
-	ROR(hash);
-	ROR(hash);
+	RORn(hash, 2);
 	hash ^= (args[3]);
-	ROR(hash);
-	ROR(hash);
+	RORn(hash, 2);
 	hash ^= (args[4]);
-	ROR(hash);
-	ROR(hash);
+	RORn(hash, 2);
 	hash ^= (args[5]);
 
 	return hash % SYSCALL_HASH_TABLE_SIZE;
@@ -2341,7 +2335,7 @@ static void hash_table_add(
 	h->hash = hash;
 	h->syscall = syscall_num;
 	h->type = type;
-	(void)memcpy(h->args, args, sizeof(h->args));
+	(void)shim_memcpy(h->args, args, sizeof(h->args));
 	h->next = hash_table->table[hash];
 	hash_table->table[hash] = h;
 	hash_table->index++;
@@ -2380,7 +2374,7 @@ static void syscall_permute(
 	unsigned long rnd_values[4];
 	size_t num_values = 0;
 
-	if (stress_time_now() > time_end)
+	if (stress_time_now() > args->time_end)
 		_exit(EXIT_SUCCESS);
 
 	if (arg_num >= stress_syscall_arg->num_args) {
@@ -2391,7 +2385,7 @@ static void syscall_permute(
 		struct itimerval it;
 
 		while (h) {
-			if (!memcmp(h->args, current_context->args, sizeof(h->args))) {
+			if (!shim_memcmp(h->args, current_context->args, sizeof(h->args))) {
 				switch (h->type) {
 				case SYSCALL_CRASH:
 					current_context->skip_crashed++;
@@ -2512,7 +2506,7 @@ timed_out:
 	}
 
 	if (arg_bitmask & ARG_PTR_WR)
-		(void)memset(page_ptr_wr, 0, args->page_size);
+		(void)shim_memset(page_ptr_wr, 0, args->page_size);
 	/*
 	 *  This should not fail!
 	 */
@@ -2544,7 +2538,7 @@ static inline int stress_do_syscall(const stress_args_t *args)
 
 	(void)stress_mwc32();
 
-	if (!keep_stressing_flag())
+	if (!stress_continue_flag())
 		return 0;
 
 	if (stress_drop_capabilities(args->name) < 0)
@@ -2578,7 +2572,7 @@ static inline int stress_do_syscall(const stress_args_t *args)
 		(void)sched_settings_apply(true);
 		stress_mwc_reseed();
 
-		while (keep_stressing_flag()) {
+		while (stress_continue_flag()) {
 			const size_t sz = SIZEOF_ARRAY(reorder);
 
 			for (i = 0; i < SIZEOF_ARRAY(reorder); i++)
@@ -2603,10 +2597,10 @@ static inline int stress_do_syscall(const stress_args_t *args)
 				}
 			}
 
-			for (i = 0; keep_stressing(args) && (i < SYSCALL_ARGS_SIZE); i++) {
+			for (i = 0; stress_continue(args) && (i < SYSCALL_ARGS_SIZE); i++) {
 				const size_t j = reorder[i];
 
-				(void)memset(current_context->args, 0, sizeof(current_context->args));
+				(void)shim_memset(current_context->args, 0, sizeof(current_context->args));
 				current_context->syscall = stress_syscall_args[j].syscall;
 				current_context->idx = j;
 				current_context->name = stress_syscall_args[j].name;
@@ -2632,7 +2626,7 @@ static inline int stress_do_syscall(const stress_args_t *args)
 			 *  kill and reap of child to make sure
 			 *  it is really dead and buried
 			 */
-			(void)kill(pid, SIGKILL);
+			(void)shim_kill(pid, SIGKILL);
 			VOID_RET(int, waitpid(pid, &status, 0));
 		}
 		if (current_context->type == SYSCALL_CRASH) {
@@ -2658,7 +2652,7 @@ static int stress_sysinval_child(const stress_args_t *args, void *context)
 	do {
 		(void)stress_mwc32();
 		stress_do_syscall(args);
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 	return EXIT_SUCCESS;
 }
@@ -2682,8 +2676,6 @@ static int stress_sysinval(const stress_args_t *args)
 	bool syscall_exercised[SYSCALL_ARGS_SIZE];
 	bool syscall_unique[SYSCALL_ARGS_SIZE];
 
-	time_end = stress_time_now() + (double)g_opt_timeout;
-
 	/*
 	 *  Run-time sanity check of zero syscalls, maybe __NR or SYS_ is not
 	 *  defined.
@@ -2787,14 +2779,14 @@ static int stress_sysinval(const stress_args_t *args)
 	futex_ptrs[0] = (unsigned long)(small_ptr + page_size -1);
 	futex_ptrs[1] = (unsigned long)page_ptr;
 
-	memset(current_context->crash_count, 0, sizeof(current_context->crash_count));
+	(void)shim_memset(current_context->crash_count, 0, sizeof(current_context->crash_count));
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	rc = stress_oomable_child(args, NULL, stress_sysinval_child, STRESS_OOMABLE_DROP_CAP);
 
-	(void)memset(syscall_exercised, 0, sizeof(syscall_exercised));
-	(void)memset(syscall_unique, 0, sizeof(syscall_unique));
+	(void)shim_memset(syscall_exercised, 0, sizeof(syscall_exercised));
+	(void)shim_memset(syscall_unique, 0, sizeof(syscall_unique));
 	syscalls_exercised = 0;
 	syscalls_unique = 0;
 	syscalls_crashed = 0;
@@ -2838,21 +2830,21 @@ static int stress_sysinval(const stress_args_t *args)
 		args->name, current_context->skip_errno_zero, current_context->skip_timed_out);
 	pr_unlock();
 
-	set_counter(args, current_context->counter);
+	stress_bogo_set(args, current_context->counter);
 
 tidy:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
-	if (stress_syscall_exercised && stress_syscall_exercised != MAP_FAILED)
+	if (stress_syscall_exercised && (stress_syscall_exercised != MAP_FAILED))
 		(void)munmap((void *)stress_syscall_exercised, stress_syscall_exercised_sz);
-	if (hash_table && hash_table != MAP_FAILED)
+	if (hash_table && (hash_table != MAP_FAILED))
 		(void)munmap((void *)hash_table, sizeof(*hash_table));
-	if (page_ptr_wr && page_ptr_wr != MAP_FAILED)
+	if (page_ptr_wr && (page_ptr_wr != MAP_FAILED))
 		(void)munmap((void *)page_ptr_wr, page_ptr_wr_size);
-	if (page_ptr && page_ptr != MAP_FAILED)
+	if (page_ptr && (page_ptr != MAP_FAILED))
 		(void)munmap((void *)page_ptr, page_size);
-	if (small_ptr && small_ptr != MAP_FAILED)
+	if (small_ptr && (small_ptr != MAP_FAILED))
 		(void)munmap((void *)small_ptr, small_ptr_size);
-	if (current_context && current_context != MAP_FAILED)
+	if (current_context && (current_context != MAP_FAILED))
 		(void)munmap((void *)current_context, current_context_size);
 	if (sockfds[0] >= 0)
 		(void)close((int)sockfds[0]);
