@@ -18,6 +18,11 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
+#include "core-cpu-cache.h"
+#include "core-killpid.h"
+#include "core-out-of-memory.h"
+#include "core-pragma.h"
 
 static const stress_help_t help[] = {
 	{ NULL,	"tlb-shootdown N",	"start N workers that force TLB shootdowns" },
@@ -38,15 +43,31 @@ static const stress_help_t help[] = {
  *  stress_tlb_shootdown_read_mem()
  *	read from every cache line in mem
  */
-static inline void stress_tlb_shootdown_read_mem(const uint8_t *mem, const size_t size, const size_t page_size)
+static inline void OPTIMIZE3 stress_tlb_shootdown_read_mem(const uint8_t *mem, const size_t size, const size_t page_size)
 {
 	const volatile uint8_t *vmem;
 
 	for (vmem = mem; vmem < mem + size; vmem += page_size) {
 		register size_t m;
 
-		for (m = 0; m < page_size; m += STRESS_CACHE_LINE_SIZE)
+		for (m = 0; m < page_size; ) {
 			(void)vmem[m];
+			m += STRESS_CACHE_LINE_SIZE;
+			(void)vmem[m];
+			m += STRESS_CACHE_LINE_SIZE;
+			(void)vmem[m];
+			m += STRESS_CACHE_LINE_SIZE;
+			(void)vmem[m];
+			m += STRESS_CACHE_LINE_SIZE;
+			(void)vmem[m];
+			m += STRESS_CACHE_LINE_SIZE;
+			(void)vmem[m];
+			m += STRESS_CACHE_LINE_SIZE;
+			(void)vmem[m];
+			m += STRESS_CACHE_LINE_SIZE;
+			(void)vmem[m];
+			m += STRESS_CACHE_LINE_SIZE;
+		}
 	}
 }
 
@@ -54,7 +75,7 @@ static inline void stress_tlb_shootdown_read_mem(const uint8_t *mem, const size_
  *  stress_tlb_shootdown_read_mem()
  *	write to every cache line in mem
  */
-static inline void stress_tlb_shootdown_write_mem(uint8_t *mem, const size_t size, const size_t page_size)
+static inline void OPTIMIZE3 stress_tlb_shootdown_write_mem(uint8_t *mem, const size_t size, const size_t page_size)
 {
 	volatile uint8_t *vmem;
 	uint8_t rnd8 = stress_mwc8();
@@ -62,9 +83,26 @@ static inline void stress_tlb_shootdown_write_mem(uint8_t *mem, const size_t siz
 	for (vmem = mem; vmem < mem + size; vmem += page_size) {
 		register size_t m;
 
-		for (m = 0; m < page_size; m += STRESS_CACHE_LINE_SIZE)
+		for (m = 0; m < page_size; ) {
 			vmem[m] = m + rnd8;
+			m += STRESS_CACHE_LINE_SIZE;
+			vmem[m] = m + rnd8;
+			m += STRESS_CACHE_LINE_SIZE;
+			vmem[m] = m + rnd8;
+			m += STRESS_CACHE_LINE_SIZE;
+			vmem[m] = m + rnd8;
+			m += STRESS_CACHE_LINE_SIZE;
+			vmem[m] = m + rnd8;
+			m += STRESS_CACHE_LINE_SIZE;
+			vmem[m] = m + rnd8;
+			m += STRESS_CACHE_LINE_SIZE;
+			vmem[m] = m + rnd8;
+			m += STRESS_CACHE_LINE_SIZE;
+			vmem[m] = m + rnd8;
+			m += STRESS_CACHE_LINE_SIZE;
+		}
 	}
+	(void)shim_cacheflush((char *)mem, (int)size, SHIM_DCACHE);
 }
 
 /*
@@ -162,7 +200,7 @@ static int stress_tlb_shootdown(const stress_args_t *args)
 		rc = EXIT_NO_RESOURCE;
 		goto err_munmap_memfd;
 	}
-	(void)memset(mem, 0xff, mmap_size);
+	(void)shim_memset(mem, 0xff, mmap_size);
 
 	if (sched_getaffinity(0, sizeof(proc_mask_initial), &proc_mask_initial) < 0) {
 		pr_fail("%s: sched_getaffinity could not get CPU affinity, errno=%d (%s)\n",
@@ -205,17 +243,21 @@ static int stress_tlb_shootdown(const stress_args_t *args)
 			continue;
 		} else if (pids[i] == 0) {
 			cpu_set_t mask;
+			double t_start, t_next;
 
 			stress_parent_died_alarm();
 			(void)sched_settings_apply(true);
 
 			/* Make sure this is killable by OOM killer */
-			stress_set_oom_adjustment(args->name, true);
+			stress_set_oom_adjustment(args, true);
 
 			CPU_ZERO(&mask);
 			CPU_SET(cpu % max_cpus, &mask);
 			(void)sched_setaffinity(args->pid, sizeof(mask), &mask);
 
+			t_start = stress_time_now();
+			t_next = t_start + 1.0;
+
 			do {
 				size_t l;
 				size_t k = stress_mwc32() & mem_mask;
@@ -230,19 +272,33 @@ static int stress_tlb_shootdown(const stress_args_t *args)
 
 				vmem = mem;
 				(void)mprotect(mem, mmap_size, PROT_READ);
+PRAGMA_UNROLL_N(8)
 				for (l = 0; l < cache_lines; l++) {
 					(void)vmem[k];
 					k = (k + stride) & mem_mask;
 				}
 				(void)mprotect(mem, mmap_size, PROT_WRITE);
+PRAGMA_UNROLL_N(8)
 				for (l = 0; l < cache_lines; l++) {
 					vmem[k] = (uint8_t)(k + rnd8);
 					k = (k + stride) & mem_mask;
 				}
-				inc_counter(args);
-			} while (keep_stressing(args));
+				stress_bogo_inc(args);
 
-			(void)kill(pid, SIGALRM);
+				/*
+				 *  periodically change cpu affinity
+				 */
+				if (stress_time_now() >= t_next) {
+					cpu++;
+					cpu %= max_cpus;
+					CPU_ZERO(&mask);
+					CPU_SET(cpu, &mask);
+					(void)sched_setaffinity(args->pid, sizeof(mask), &mask);
+					t_next += 1.0;
+				}
+			} while (stress_continue(args));
+
+			(void)shim_kill(pid, SIGALRM);
 			_exit(0);
 		}
 	}
@@ -263,38 +319,24 @@ static int stress_tlb_shootdown(const stress_args_t *args)
 		(void)madvise(mem, mmap_size, MADV_DONTNEED);
 		stress_tlb_shootdown_write_mem(mem, mmap_size, page_size);
 
-		inc_counter(args);
-	} while(keep_stressing(args));
+#if defined(__linux__)
+		{
+			static const char flush_ceiling[] = "/sys/kernel/debug/x86/tlb_single_page_flush_ceiling";
+			char buf[64];
+			ssize_t rd_ret;
 
-	for (i = 0; i < tlb_procs; i++) {
-		int status;
-
-		if (pids[i] == -1)
-			continue;
-
-		ret = waitpid(pids[i], &status, 0);
-		if ((ret < 0) && (errno == EINTR)) {
-			int j;
-
-			/*
-			 * We got interrupted, so assume
-			 * it was the alarm (timedout) or
-			 * SIGINT so force terminate
-			 */
-			for (j = i; j < tlb_procs; j++) {
-				if (pids[j] != -1)
-					(void)kill(pids[j], SIGKILL);
-			}
-
-			/* re-wait on the failed wait */
-			(void)waitpid(pids[i], &status, 0);
-
-			/* and continue waitpid on the pids */
+			rd_ret = stress_system_read(flush_ceiling, buf, sizeof(buf));
+			if (rd_ret > 0)
+				VOID_RET(ssize_t, stress_system_write(flush_ceiling, buf, rd_ret));
 		}
-	}
+#endif
+
+		stress_bogo_inc(args);
+	} while(stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
+	stress_kill_and_wait_many(args, pids, tlb_procs, SIGALRM, true);
 err_munmap_mem:
 	(void)munmap(mem, mmap_size);
 err_munmap_memfd:
