@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
 #include "core-pragma.h"
 #include "core-target-clones.h"
 
@@ -308,7 +309,7 @@ PRAGMA_UNROLL_N(4)
 	for (node = SPLAY_MIN(stress_splay_tree, &splay_root); node; node = next) {
 		next = SPLAY_NEXT(stress_splay_tree, &splay_root, node);
 		SPLAY_REMOVE(stress_splay_tree, &splay_root, node);
-		(void)memset(&node->u.splay, 0, sizeof(node->u.splay));
+		(void)shim_memset(&node->u.splay, 0, sizeof(node->u.splay));
 	}
 	metrics->remove += stress_time_now() - t;
 	metrics->count += (double)n;
@@ -329,10 +330,10 @@ static void OPTIMIZE3 TARGET_CLONES binary_insert(
 
 static struct tree_node * OPTIMIZE3 TARGET_CLONES binary_find(
 	struct tree_node *head,
-	struct tree_node *node)
+	const struct tree_node *node)
 {
 	while (head) {
-		if (node->value == head->value)
+		if (UNLIKELY(node->value == head->value))
 			return head;
 		head = (node->value <= head->value) ?
 				head->u.binary.left :
@@ -534,10 +535,10 @@ static void OPTIMIZE3 avl_insert(
 
 static struct tree_node OPTIMIZE3 TARGET_CLONES *avl_find(
 	struct tree_node *head,
-	struct tree_node *node)
+	const struct tree_node *node)
 {
 	while (LIKELY(head != NULL)) {
-		if (node->value == head->value)
+		if (UNLIKELY(node->value == head->value))
 			return head;
 		head = (node->value <= head->value) ?
 				head->u.avl.left :
@@ -983,8 +984,8 @@ static int stress_tree(const stress_args_t *args)
 		func(args, n, nodes, metrics);
 		stress_tree_shuffle(nodes, n);
 
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	do_jmp = false;
 	(void)stress_sigrestore(args->name, SIGALRM, &old_action);
