@@ -18,6 +18,8 @@
  *
  */
 #include "stress-ng.h"
+#include "core-affinity.h"
+#include "core-builtin.h"
 #include "core-capabilities.h"
 #include "core-net.h"
 
@@ -51,6 +53,7 @@ static const stress_help_t help[] = {
 };
 
 #if defined(HAVE_LINUX_IF_TUN_H) &&	\
+    defined(HAVE_IFREQ) &&		\
     defined(IFF_TUN) &&			\
     defined(TUNSETIFF) && 		\
     defined(TUNSETOWNER) && 		\
@@ -107,7 +110,7 @@ static int stress_tun(const stress_args_t *args)
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
-		int i, fd, sfd, ret, status;
+		int i, fd, sfd, ret, status, parent_cpu;
 		pid_t pid;
 		struct ifreq ifr;
 		struct sockaddr_in *tun_addr;
@@ -125,7 +128,7 @@ static int stress_tun(const stress_args_t *args)
 			return EXIT_FAILURE;
 		}
 
-		(void)memset(&ifr, 0, sizeof(ifr));
+		(void)shim_memset(&ifr, 0, sizeof(ifr));
 		ifr.ifr_flags = tun_tap ? IFF_TAP : IFF_TUN;
 
 		ret = ioctl(fd, TUNSETIFF, (void *)&ifr);
@@ -177,6 +180,7 @@ static int stress_tun(const stress_args_t *args)
 		if (ret < 0)
 			goto clean_up;
 
+		parent_cpu = stress_get_cpu();
 		pid = fork();
 		if (pid < 0) {
 			goto clean_up;
@@ -188,6 +192,7 @@ static int stress_tun(const stress_args_t *args)
 			ssize_t n;
 			char buffer[4];
 
+			(void)stress_change_cpu(args, parent_cpu);
 			stress_parent_died_alarm();
 			(void)sched_settings_apply(true);
 
@@ -204,7 +209,7 @@ static int stress_tun(const stress_args_t *args)
 					rc = EXIT_SUCCESS;
 					break;
 				default:
-					pr_dbg("%s: child socket failed, errno = %d (%s)\n",
+					pr_dbg("%s: child socket failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
 					rc = EXIT_FAILURE;
 					break;
@@ -212,7 +217,7 @@ static int stress_tun(const stress_args_t *args)
 				goto child_cleanup_fd;
 			}
 
-			(void)memset(&addr, 0, sizeof(addr));
+			(void)shim_memset(&addr, 0, sizeof(addr));
 			addr.sin_family = AF_INET;
 			addr.sin_port = (in_port_t)port;
 			len = sizeof(addr);
@@ -229,7 +234,7 @@ static int stress_tun(const stress_args_t *args)
 					rc = EXIT_SUCCESS;
 					break;
 				default:
-					pr_dbg("%s: child bind failed, errno = %d (%s)\n",
+					pr_dbg("%s: child bind failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
 					rc = EXIT_FAILURE;
 					break;
@@ -351,18 +356,18 @@ child_cleanup_fd:
 
 			sfd = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
 			if (sfd < 0) {
-				pr_fail("%s: parent socket failed, errno = %d (%s)\n",
+				pr_fail("%s: parent socket failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 				goto child_reap;
 			}
 
-			(void)memset(&addr, 0, sizeof(addr));
+			(void)shim_memset(&addr, 0, sizeof(addr));
 			addr.sin_family = AF_INET;
 			addr.sin_port = (in_port_t)port;
 			len = sizeof(addr);
 			inet_pton(AF_INET, ip_addr, &addr.sin_addr.s_addr);
 
-			for (i = 0; keep_stressing(args) && (i < PACKETS_TO_SEND); i++) {
+			for (i = 0; stress_continue(args) && (i < PACKETS_TO_SEND); i++) {
 				n = sendto(sfd, buffer, sizeof(buffer), 0,
 					(struct sockaddr *)&addr, len);
 				if (n < 0)
@@ -372,7 +377,7 @@ child_cleanup_fd:
 			(void)close(sfd);
 		}
 child_reap:
-		(void)kill(pid, SIGKILL);
+		(void)shim_kill(pid, SIGKILL);
 		(void)shim_waitpid(pid, &status, 0);
 		if (WEXITSTATUS(status) == EXIT_FAILURE)
 			pr_fail("%s: child reading process failed\n", args->name);
@@ -380,8 +385,8 @@ child_reap:
 clean_up:
 		(void)close(fd);
 		stress_net_release_ports(port, port);
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
@@ -406,6 +411,7 @@ stressor_info_t stress_tun_info = {
 stressor_info_t stress_tun_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_NETWORK | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without linux/if_tun.h and various undefined TUN related macros"
 };
