@@ -18,6 +18,7 @@
  */
 #include "stress-ng.h"
 #include "core-capabilities.h"
+#include "core-killpid.h"
 
 #if defined(HAVE_SYS_MOUNT_H)
 #include <sys/mount.h>
@@ -105,7 +106,7 @@ static void stress_umount_umount(const stress_args_t *args, const char *path, co
  *  stress_umount_read_proc_mounts()
  *	exercise reading of proc mounts
  */
-static void stress_umount_read_proc_mounts(const stress_args_t *args, const pid_t parent, const char *path)
+static void stress_umount_read_proc_mounts(const stress_args_t *args, const char *path)
 {
 	(void)path;
 
@@ -123,9 +124,8 @@ static void stress_umount_read_proc_mounts(const stress_args_t *args, const pid_
 		(void)close(fd);
 
 		shim_nanosleep_uint64(stress_mwc64modn(1000000));
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
-	(void)kill(parent, SIGALRM);
 	_exit(0);
 }
 
@@ -133,7 +133,7 @@ static void stress_umount_read_proc_mounts(const stress_args_t *args, const pid_
  *  stress_umount_umounter()
  *	racy unmount, hammer time!
  */
-static void stress_umount_umounter(const stress_args_t *args, const pid_t parent, const char *path)
+static void stress_umount_umounter(const stress_args_t *args, const char *path)
 {
 	stress_parent_died_alarm();
 	(void)sched_settings_apply(true);
@@ -141,19 +141,17 @@ static void stress_umount_umounter(const stress_args_t *args, const pid_t parent
 	do {
 		stress_umount_umount(args, path, 10000);
 		shim_nanosleep_uint64(stress_mwc64modn(10000));
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
-	(void)kill(parent, SIGALRM);
 	_exit(0);
 }
 
-
 /*
  *  stress_umount_mounter()
  *	aggressively perform ramfs mounts, this can force out of memory
  *	conditions
  */
-static void stress_umount_mounter(const stress_args_t *args, const pid_t parent, const char *path)
+static void stress_umount_mounter(const stress_args_t *args, const char *path)
 {
 	const uint64_t ramfs_size = 64 * KB;
 	int i = 0;
@@ -177,12 +175,11 @@ static void stress_umount_mounter(const stress_args_t *args, const pid_t parent,
 			/* Just in case, force umount */
 			goto cleanup;
 		} else {
-			inc_counter(args);
+			stress_bogo_inc(args);
 		}
 		stress_umount_umount(args, path, 1000000);
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
-	(void)kill(parent, SIGALRM);
 cleanup:
 	stress_umount_umount(args, path, 100000000);
 	_exit(0);
@@ -194,18 +191,17 @@ cleanup:
  */
 static pid_t stress_umount_spawn(
 	const stress_args_t *args,
-	const pid_t parent,
 	const char *path,
-	void (*func)(const stress_args_t *args, const pid_t parent, const char *path))
+	void (*func)(const stress_args_t *args, const char *path))
 {
 	pid_t pid;
 
 again:
 	pid = fork();
 	if (pid < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			return 0;
 		pr_inf("%s: fork failed: %d (%s), skipping stressor\n",
 			args->name, errno, strerror(errno));
@@ -216,7 +212,7 @@ again:
 		(void)sched_settings_apply(true);
 
 		stress_set_proc_state(args->name, STRESS_STATE_RUN);
-		func(args, parent, path);
+		func(args, path);
 		stress_set_proc_state(args->name, STRESS_STATE_WAIT);
 
 		_exit(EXIT_SUCCESS);
@@ -231,11 +227,12 @@ again:
 static int stress_umount(const stress_args_t *args)
 {
 	pid_t pids[3] = { -1, -1, -1 };
-	const pid_t mypid = getpid();
-	size_t i;
 	int ret = EXIT_NO_RESOURCE;
 	char pathname[PATH_MAX], realpathname[PATH_MAX];
 
+	if (stress_sigchld_set_handler(args) < 0)
+		return EXIT_NO_RESOURCE;
+
 	stress_temp_dir(pathname, sizeof(pathname), args->name, args->pid, args->instance);
 	if (mkdir(pathname, S_IRGRP | S_IWGRP) < 0) {
 		pr_fail("%s: cannot mkdir %s, errno=%d (%s)\n",
@@ -249,13 +246,13 @@ static int stress_umount(const stress_args_t *args)
 		return EXIT_FAILURE;
 	}
 
-	pids[0] = stress_umount_spawn(args, mypid, realpathname, stress_umount_mounter);
+	pids[0] = stress_umount_spawn(args, realpathname, stress_umount_mounter);
 	if (pids[0] < 1)
 		goto reap;
-	pids[1] = stress_umount_spawn(args, mypid, realpathname, stress_umount_umounter);
+	pids[1] = stress_umount_spawn(args, realpathname, stress_umount_umounter);
 	if (pids[1] < 1)
 		goto reap;
-	pids[2] = stress_umount_spawn(args, mypid, realpathname, stress_umount_read_proc_mounts);
+	pids[2] = stress_umount_spawn(args, realpathname, stress_umount_read_proc_mounts);
 	if (pids[2] < 1)
 		goto reap;
 
@@ -264,21 +261,12 @@ static int stress_umount(const stress_args_t *args)
 	/* Wait for SIGALARMs */
 	do {
 		pause();
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 	ret = EXIT_SUCCESS;
 reap:
-	for (i = 0; i < SIZEOF_ARRAY(pids); i++) {
-		if (pids[i] > 1) {
-			int status;
-
-			(void)kill(pids[i], SIGALRM);
-			(void)waitpid(pids[i], &status, 0);
-		}
-	}
-
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
-
+	stress_kill_and_wait_many(args, pids, SIZEOF_ARRAY(pids), SIGALRM, true);
 	(void)stress_temp_dir_rm_args(args);
 
 	return ret;
@@ -296,6 +284,7 @@ stressor_info_t stress_umount_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_OS,
 	.supported = stress_umount_supported,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "only supported on Linux"
 };
