@@ -47,7 +47,7 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
 
 #if defined(HAVE_UTIME) &&	\
     defined(HAVE_UTIMBUF)
-static int shim_utime(const char *filename, const struct utimbuf *times)
+static inline int shim_utime(const char *filename, const struct utimbuf *times)
 {
 #if defined(__NR_utime) &&	\
     defined(HAVE_SYSCALL)
@@ -62,7 +62,7 @@ static int shim_utime(const char *filename, const struct utimbuf *times)
  *  stress_utime()
  *	stress system by setting file utime
  */
-static int stress_utime(const stress_args_t *args)
+static int OPTIMIZE3 stress_utime(const stress_args_t *args)
 {
 #if defined(O_DIRECTORY) &&	\
     defined(O_PATH) &&		\
@@ -80,6 +80,7 @@ static int stress_utime(const stress_args_t *args)
 	const bool verify = !!(g_opt_flags & OPT_FLAGS_VERIFY);
 #endif
 	double duration = 0.0, count = 0.0, rate;
+	int metrics_count = 0;
 
 	(void)stress_get_setting("utime-fsync", &utime_fsync);
 
@@ -122,25 +123,43 @@ static int stress_utime(const stress_args_t *args)
 
 		(void)gettimeofday(&timevals[0], NULL);
 		timevals[1] = timevals[0];
-		t = stress_time_now();
-		if (utimes(filename, timevals) < 0) {
-			pr_dbg("%s: utimes failed: errno=%d (%s)%s\n",
-				args->name, errno, strerror(errno),
-				stress_fs_type(filename));
-			break;
+		if (LIKELY(metrics_count > 0)) {
+			if (UNLIKELY(utimes(filename, timevals) < 0)) {
+				pr_dbg("%s: utimes failed: errno=%d (%s)%s\n",
+					args->name, errno, strerror(errno),
+					stress_get_fs_type(filename));
+				break;
+			}
+		} else {
+			t = stress_time_now();
+			if (UNLIKELY(utimes(filename, timevals) < 0)) {
+				pr_dbg("%s: utimes failed: errno=%d (%s)%s\n",
+					args->name, errno, strerror(errno),
+					stress_get_fs_type(filename));
+				break;
+			}
+			duration += stress_time_now() - t;
+			count += 1.0;
 		}
-		duration += stress_time_now() - t;
-		count += 1.0;
 
-		t = stress_time_now();
-		if (utimes(filename, NULL) < 0) {
-			pr_dbg("%s: utimes failed: errno=%d (%s)%s\n",
-				args->name, errno, strerror(errno),
-				stress_fs_type(filename));
-			break;
+		if (LIKELY(metrics_count > 0)) {
+			if (UNLIKELY(utimes(filename, NULL) < 0)) {
+				pr_dbg("%s: utimes failed: errno=%d (%s)%s\n",
+					args->name, errno, strerror(errno),
+					stress_get_fs_type(filename));
+				break;
+			}
+		} else {
+			t = stress_time_now();
+			if (UNLIKELY(utimes(filename, NULL) < 0)) {
+				pr_dbg("%s: utimes failed: errno=%d (%s)%s\n",
+					args->name, errno, strerror(errno),
+					stress_get_fs_type(filename));
+				break;
+			}
+			duration += stress_time_now() - t;
+			count += 1.0;
 		}
-		duration += stress_time_now() - t;
-		count += 1.0;
 
 		/* Exercise with invalid filename, ENOENT */
 		VOID_RET(int, utimes("", timevals));
@@ -149,15 +168,24 @@ static int stress_utime(const stress_args_t *args)
 		VOID_RET(int, utimes(hugename, timevals));
 
 #if defined(HAVE_FUTIMENS)
-		t = stress_time_now();
-		if (futimens(fd, NULL) < 0) {
-			pr_dbg("%s: futimens failed: errno=%d (%s)%s\n",
-				args->name, errno, strerror(errno),
-				stress_fs_type(filename));
-			break;
+		if (LIKELY(metrics_count > 0)) {
+			if (UNLIKELY(futimens(fd, NULL) < 0)) {
+				pr_dbg("%s: futimens failed: errno=%d (%s)%s\n",
+					args->name, errno, strerror(errno),
+					stress_get_fs_type(filename));
+				break;
+			}
+		} else {
+			t = stress_time_now();
+			if (UNLIKELY(futimens(fd, NULL) < 0)) {
+				pr_dbg("%s: futimens failed: errno=%d (%s)%s\n",
+					args->name, errno, strerror(errno),
+					stress_get_fs_type(filename));
+				break;
+			}
+			duration += stress_time_now() - t;
+			count += 1.0;
 		}
-		duration += stress_time_now() - t;
-		count += 1.0;
 
 		/* Exercise with invalid fd */
 		ts[0].tv_sec = UTIME_NOW;
@@ -171,15 +199,24 @@ static int stress_utime(const stress_args_t *args)
 		ts[0].tv_nsec = UTIME_NOW;
 		ts[1].tv_sec = UTIME_NOW;
 		ts[1].tv_nsec = UTIME_NOW;
-		t = stress_time_now();
-		if (futimens(fd, ts) < 0) {
-			pr_dbg("%s: futimens failed: errno=%d (%s)%s\n",
-				args->name, errno, strerror(errno),
-				stress_fs_type(filename));
-			break;
+		if (LIKELY(metrics_count > 0)) {
+			if (UNLIKELY(futimens(fd, ts) < 0)) {
+				pr_dbg("%s: futimens failed: errno=%d (%s)%s\n",
+					args->name, errno, strerror(errno),
+					stress_get_fs_type(filename));
+				break;
+			}
+		} else {
+			t = stress_time_now();
+			if (UNLIKELY(futimens(fd, ts) < 0)) {
+				pr_dbg("%s: futimens failed: errno=%d (%s)%s\n",
+					args->name, errno, strerror(errno),
+					stress_get_fs_type(filename));
+				break;
+			}
+			duration += stress_time_now() - t;
+			count += 1.0;
 		}
-		duration += stress_time_now() - t;
-		count += 1.0;
 #else
 		UNEXPECTED
 #endif
@@ -187,15 +224,24 @@ static int stress_utime(const stress_args_t *args)
 #if defined(UTIME_OMIT)
 		ts[0].tv_sec = UTIME_OMIT;
 		ts[0].tv_nsec = UTIME_OMIT;
-		t = stress_time_now();
-		if (futimens(fd, ts) < 0) {
-			pr_dbg("%s: futimens failed: errno=%d (%s)%s\n",
-				args->name, errno, strerror(errno),
-				stress_fs_type(filename));
-			break;
+		if (LIKELY(metrics_count > 0)) {
+			if (UNLIKELY(futimens(fd, ts) < 0)) {
+				pr_dbg("%s: futimens failed: errno=%d (%s)%s\n",
+					args->name, errno, strerror(errno),
+					stress_get_fs_type(filename));
+				break;
+			}
+		} else {
+			t = stress_time_now();
+			if (UNLIKELY(futimens(fd, ts) < 0)) {
+				pr_dbg("%s: futimens failed: errno=%d (%s)%s\n",
+					args->name, errno, strerror(errno),
+					stress_get_fs_type(filename));
+				break;
+			}
+			duration += stress_time_now() - t;
+			count += 1.0;
 		}
-		duration += stress_time_now() - t;
-		count += 1.0;
 #else
 		UNEXPECTED
 #endif
@@ -210,10 +256,14 @@ static int stress_utime(const stress_args_t *args)
 		ts[1].tv_sec = UTIME_NOW;
 		ts[1].tv_nsec = UTIME_NOW;
 
-		t = stress_time_now();
-		VOID_RET(int, utimensat(AT_FDCWD, filename, ts, 0));
-		duration += stress_time_now() - t;
-		count += 1.0;
+		if (LIKELY(metrics_count > 0)) {
+			VOID_RET(int, utimensat(AT_FDCWD, filename, ts, 0));
+		} else {
+			t = stress_time_now();
+			VOID_RET(int, utimensat(AT_FDCWD, filename, ts, 0));
+			duration += stress_time_now() - t;
+			count += 1.0;
+		}
 
 		/* Exercise invalid filename, ENOENT */
 		VOID_RET(int, utimensat(AT_FDCWD, "", ts, 0));
@@ -282,15 +332,24 @@ STRESS_PRAGMA_POP
 			utbuf.actime = (time_t)tv.tv_sec;
 			utbuf.modtime = utbuf.actime;
 
-			t = stress_time_now();
-			if (shim_utime(filename, &utbuf) < 0) {
-				pr_fail("%s: utime failed: errno=%d (%s)%s\n",
-					args->name, errno, strerror(errno),
-					stress_fs_type(filename));
-				break;
+			if (LIKELY(metrics_count > 0)) {
+				if (UNLIKELY(shim_utime(filename, &utbuf) < 0)) {
+					pr_fail("%s: utime failed: errno=%d (%s)%s\n",
+						args->name, errno, strerror(errno),
+						stress_get_fs_type(filename));
+					break;
+				}
+			} else {
+				t = stress_time_now();
+				if (UNLIKELY(shim_utime(filename, &utbuf) < 0)) {
+					pr_fail("%s: utime failed: errno=%d (%s)%s\n",
+						args->name, errno, strerror(errno),
+						stress_get_fs_type(filename));
+					break;
+				}
+				duration += stress_time_now() - t;
+				count += 1.0;
 			}
-			duration += stress_time_now() - t;
-			count += 1.0;
 
 			if (verify) {
 				struct stat statbuf;
@@ -306,15 +365,24 @@ STRESS_PRAGMA_POP
 					}
 				}
 			}
-			t = stress_time_now();
-			if (shim_utime(filename, NULL) < 0) {
-				pr_fail("%s: utime failed: errno=%d (%s)%s\n",
-					args->name, errno, strerror(errno),
-					stress_fs_type(filename));
-				break;
+			if (LIKELY(metrics_count > 0)) {
+				if (UNLIKELY(shim_utime(filename, NULL) < 0)) {
+					pr_fail("%s: utime failed: errno=%d (%s)%s\n",
+						args->name, errno, strerror(errno),
+						stress_get_fs_type(filename));
+					break;
+				}
+			} else {
+				t = stress_time_now();
+				if (UNLIKELY(shim_utime(filename, NULL) < 0)) {
+					pr_fail("%s: utime failed: errno=%d (%s)%s\n",
+						args->name, errno, strerror(errno),
+						stress_get_fs_type(filename));
+					break;
+				}
+				duration += stress_time_now() - t;
+				count += 1.0;
 			}
-			duration += stress_time_now() - t;
-			count += 1.0;
 
 			/* Exercise invalid timename, ENOENT */
 			VOID_RET(int, shim_utime("", &utbuf));
@@ -322,15 +390,17 @@ STRESS_PRAGMA_POP
 			/* Exercise huge filename, ENAMETOOLONG */
 			VOID_RET(int, shim_utime(hugename, &utbuf));
 		}
-#else
-		UNEXPECTED
-#endif
 		/* forces metadata writeback */
 		if (utime_fsync) {
 			VOID_RET(int, shim_fsync(fd));
 		}
-		inc_counter(args);
-	} while (keep_stressing(args));
+#else
+		UNEXPECTED
+#endif
+		if (metrics_count++ > 1000)
+			metrics_count = 0;
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
