@@ -63,8 +63,8 @@ typedef int (*stress_vdso_func_t)(void *);
  *  Symbol name to wrapper function lookup
  */
 typedef struct stress_wrap_func {
-	stress_vdso_func_t func;/* Wrapper function */
-	char *name;		/* Function name */
+	const stress_vdso_func_t func;	/* Wrapper function */
+	char *name;			/* Function name */
 } stress_wrap_func_t;
 
 /*
@@ -174,7 +174,7 @@ static const stress_wrap_func_t wrap_funcs[] = {
  *  func_find()
  *	find wrapper function by symbol name
  */
-static stress_vdso_func_t func_find(char *name)
+static stress_vdso_func_t func_find(const char *name)
 {
 	size_t i;
 
