@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
 
 #define MIN_VM_SPLICE_BYTES	(4 * KB)
 #define MAX_VM_SPLICE_BYTES	(64 * MB)
@@ -54,12 +55,15 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
  */
 static int stress_vm_splice(const stress_args_t *args)
 {
-	int fd, fds[2];
+	int fd, fds[2], rc = EXIT_SUCCESS;
 	uint8_t *buf;
 	const size_t page_size = args->page_size;
 	size_t sz, vm_splice_bytes = DEFAULT_VM_SPLICE_BYTES;
 	char *data;
 	double duration = 0.0, bytes = 0.0, vm_splices = 0.0, rate;
+	int metrics_counter = 0;
+	uint64_t checkval = stress_mwc64();
+	uint64_t prime;
 
 	if (!stress_get_setting("vm-splice-bytes", &vm_splice_bytes)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
@@ -110,52 +114,77 @@ static int stress_vm_splice(const stress_args_t *args)
 	}
 
 	stress_rndbuf(data, page_size);
+	prime = stress_get_prime64(vm_splice_bytes);
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
+	(void)shim_memset(buf, 0, sz);
 	do {
 		ssize_t ret, n_bytes;
-		struct iovec iov;
+		struct iovec iov ALIGN64;
 		double t;
 
 		/*
 		 *  vmsplice from memory to pipe
 		 */
-		(void)memset(buf, 0, sz);
 		iov.iov_base = buf;
 		iov.iov_len = sz;
-		t = stress_time_now();
-		n_bytes = vmsplice(fds[1], &iov, 1, 0);
-		if (UNLIKELY(n_bytes < 0))
-			break;
-		duration += stress_time_now() - t;
+
+		if (LIKELY(metrics_counter != 0)) {
+			n_bytes = vmsplice(fds[1], &iov, 1, 0);
+			if (UNLIKELY(n_bytes < 0))
+				break;
+		} else {
+			t = stress_time_now();
+			n_bytes = vmsplice(fds[1], &iov, 1, 0);
+			if (UNLIKELY(n_bytes < 0))
+				break;
+			duration += stress_time_now() - t;
+		}
 		bytes += (double)n_bytes;
 		vm_splices += 1.0;
 
 		ret = splice(fds[0], NULL, fd, NULL,
 			vm_splice_bytes, SPLICE_F_MOVE);
-		if (ret < 0)
+		if (UNLIKELY(ret < 0))
 			break;
 
 		/*
 		 *  vmsplice from pipe to memory
 		 */
+		checkval += prime;
+		*(uint64_t *)data = checkval;
 		n_bytes = write(fds[1], data, page_size);
-		if (n_bytes > 0) {
+		if (LIKELY(n_bytes > 0)) {
 			iov.iov_base = buf;
 			iov.iov_len = (size_t)n_bytes;
 
-			t = stress_time_now();
-			n_bytes = vmsplice(fds[0], &iov, 1, 0);
-			if (n_bytes < 0)
-				break;
-			duration += stress_time_now() - t;
+			if (LIKELY(metrics_counter != 0)) {
+				n_bytes = vmsplice(fds[0], &iov, 1, 0);
+				if (UNLIKELY(n_bytes < 0))
+					break;
+			} else {
+				t = stress_time_now();
+				n_bytes = vmsplice(fds[0], &iov, 1, 0);
+				if (UNLIKELY(n_bytes < 0))
+					break;
+				duration += stress_time_now() - t;
+			}
+			/* Sanity check the data */
+			if (LIKELY(n_bytes > (ssize_t)sizeof(checkval)) &&
+			    UNLIKELY(checkval != *(uint64_t *)buf)) {
+				pr_fail("%s: data check pattern failed, got %" PRIx64 ", expected %" PRIx64 "\n",
+					args->name, *(uint64_t *)buf, checkval);
+				rc = EXIT_FAILURE;
+			}
 			bytes += (double)n_bytes;
 			vm_splices += 1.0;
 		}
+		if (metrics_counter++ > 1000)
+			metrics_counter = 0;
 
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	rate = (duration > 0.0) ? bytes / duration : 0.0;
 	stress_metrics_set(args, 0, "MB per sec vm-splice rate", rate / (double)MB);
@@ -170,13 +199,14 @@ static int stress_vm_splice(const stress_args_t *args)
 	(void)close(fds[0]);
 	(void)close(fds[1]);
 
-	return EXIT_SUCCESS;
+	return rc;
 }
 
 stressor_info_t stress_vm_splice_info = {
 	.stressor = stress_vm_splice,
 	.class = CLASS_VM | CLASS_PIPE_IO | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
@@ -184,6 +214,7 @@ stressor_info_t stress_vm_splice_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_VM | CLASS_PIPE_IO | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without vmsplice() or undefined SPLICE_F_MOVE"
 };
