@@ -71,9 +71,15 @@ static const stress_wcs_method_info_t wcs_methods[];
 static void stress_wcs_fill(wchar_t *wcstr, const size_t len)
 {
 	register size_t i;
+	static const wchar_t letters[32] ALIGN64 = {
+		L'a', L'b', L'c', L'd', L'e', L'f', L'g', L'h',
+		L'i', L'j', L'k', L'l', L'm', L'n', L'o', L'p',
+		L'q', L'r', L's', L't', L'u', L'v', L'w', L'x',
+		L'y', L'z', L'~', L'+', L'!', L'#', L'*', L'+',
+	};
 
-	for (i = 0; i < (len-1); i++) {
-		*wcstr++ = (stress_mwc8modn(26)) + L'a';
+	for (i = 0; i < (len - 1); i++) {
+		*wcstr++ = letters[stress_mwc8() & 31];
 	}
 	*wcstr = L'\0';
 }
@@ -125,7 +131,7 @@ static size_t stress_wcscasecmp(const stress_args_t *args, stress_wcs_args_t *in
 	const size_t len1 = info->len1;
 	register size_t i;
 
-	for (i = 1; keep_stressing_flag() && (i < len1); i++) {
+	for (i = 1; stress_continue_flag() && (i < len1); i++) {
 		WCSCHK(info, 0 == test_wcscasecmp(str1, str1));
 		WCSCHK(info, 0 == test_wcscasecmp(str2, str2));
 
@@ -139,7 +145,7 @@ static size_t stress_wcscasecmp(const stress_args_t *args, stress_wcs_args_t *in
 		WCSCHK(info, 0 != test_wcscasecmp(str1 + i, str2));
 		WCSCHK(info, 0 != test_wcscasecmp(str2, str1 + i));
 	}
-	add_counter(args, 9);
+	stress_bogo_add(args, 9);
 	return i * 9;
 }
 #endif
@@ -160,7 +166,7 @@ static size_t stress_wcsncasecmp(const stress_args_t *args, stress_wcs_args_t *i
 	const size_t len2 = info->len2;
 	register size_t i;
 
-	for (i = 1; keep_stressing_flag() && (i < len1); i++) {
+	for (i = 1; stress_continue_flag() && (i < len1); i++) {
 		WCSCHK(info, 0 == test_wcsncasecmp(str1, str1, len1));
 		WCSCHK(info, 0 == test_wcsncasecmp(str2, str2, len2));
 
@@ -174,14 +180,14 @@ static size_t stress_wcsncasecmp(const stress_args_t *args, stress_wcs_args_t *i
 		WCSCHK(info, 0 != test_wcsncasecmp(str1 + i, str2, len1));
 		WCSCHK(info, 0 != test_wcsncasecmp(str2, str1 + i, len2));
 	}
-	add_counter(args, 9);
+	stress_bogo_add(args, 9);
 	return i * 9;
 }
 #endif
 
-#if defined(HAVE_WCSLCPY) &&	\
-    defined(HAVE_WCSLEN) &&	\
-    !defined(__PCC__) &&	\
+#if defined(HAVE_WCSLCPY) &&		\
+    defined(HAVE_WCSLEN) &&		\
+    !defined(HAVE_COMPILER_PCC) &&	\
     !defined(BUILD_STATIC)
 /*
  *  stress_wcslcpy()
@@ -201,11 +207,11 @@ static size_t stress_wcslcpy(const stress_args_t *args, stress_wcs_args_t *info)
 	const size_t strlen2 = wcslen(str2);
 	register size_t i;
 
-	for (i = 0; keep_stressing_flag() && (i < len1 - 1); i++) {
+	for (i = 0; stress_continue_flag() && (i < len1 - 1); i++) {
 		WCSCHK(info, strlen1 == test_wcslcpy(strdst, str1, strdstlen));
 		WCSCHK(info, strlen2 == test_wcslcpy(strdst, str2, strdstlen));
 	}
-	add_counter(args, 2);
+	stress_bogo_add(args, 2);
 	return i * 2;
 }
 #elif defined(HAVE_WCSCPY)
@@ -224,18 +230,18 @@ static size_t stress_wcscpy(const stress_args_t *args, stress_wcs_args_t *info)
 	const size_t len1 = info->len1;
 	register size_t i;
 
-	for (i = 0; keep_stressing_flag() && (i < len1 - 1); i++) {
+	for (i = 0; stress_continue_flag() && (i < len1 - 1); i++) {
 		WCSCHK(info, strdst == test_wcscpy(strdst, str1));
 		WCSCHK(info, strdst == test_wcscpy(strdst, str2));
 	}
-	add_counter(args, 2);
+	stress_bogo_add(args, 2);
 	return i * 2;
 }
 #endif
 
-#if defined(HAVE_WCSLCAT) &&	\
-    defined(HAVE_WCSLEN) &&	\
-    !defined(__PCC__) &&	\
+#if defined(HAVE_WCSLCAT) &&		\
+    defined(HAVE_WCSLEN) &&		\
+    !defined(HAVE_COMPILER_PCC) &&	\
     !defined(BUILD_STATIC)
 /*
  *  stress_wcslcat()
@@ -256,7 +262,7 @@ static size_t stress_wcslcat(const stress_args_t *args, stress_wcs_args_t *info)
 	const size_t strdstlen = info->strdstlen;
 	register size_t i;
 
-	for (i = 0; keep_stressing_flag() && (i < len1 - 1); i++) {
+	for (i = 0; stress_continue_flag() && (i < len1 - 1); i++) {
 		*strdst = L'\0';
 		WCSCHK(info, str1_len == test_wcslcat(strdst, str1, strdstlen));
 		*strdst = L'\0';
@@ -268,7 +274,7 @@ static size_t stress_wcslcat(const stress_args_t *args, stress_wcs_args_t *info)
 		WCSCHK(info, str2_len == test_wcslcat(strdst, str2, strdstlen));
 		WCSCHK(info, str_len  == test_wcslcat(strdst, str1, strdstlen));
 	}
-	add_counter(args, 6);
+	stress_bogo_add(args, 6);
 	return i * 6;
 }
 #elif defined(HAVE_WCSCAT)
@@ -287,7 +293,7 @@ static size_t stress_wcscat(const stress_args_t *args, stress_wcs_args_t *info)
 	const size_t len1 = info->len1;
 	register size_t i;
 
-	for (i = 0; keep_stressing_flag() && (i < len1 - 1); i++) {
+	for (i = 0; stress_continue_flag() && (i < len1 - 1); i++) {
 		*strdst = L'\0';
 		WCSCHK(info, strdst == test_wcscat(strdst, str1));
 		*strdst = L'\0';
@@ -299,7 +305,7 @@ static size_t stress_wcscat(const stress_args_t *args, stress_wcs_args_t *info)
 		WCSCHK(info, strdst == test_wcscat(strdst, str2));
 		WCSCHK(info, strdst == test_wcscat(strdst, str1));
 	}
-	add_counter(args, 6);
+	stress_bogo_add(args, 6);
 	return i * 6;
 }
 #endif
@@ -321,7 +327,7 @@ static size_t stress_wcsncat(const stress_args_t *args, stress_wcs_args_t *info)
 	const size_t len2 = info->len2;
 	register size_t i;
 
-	for (i = 0; keep_stressing_flag() && (i < len1 - 1); i++) {
+	for (i = 0; stress_continue_flag() && (i < len1 - 1); i++) {
 		*strdst = '\0';
 		WCSCHK(info, strdst == test_wcsncat(strdst, str1, len1));
 		*strdst = '\0';
@@ -333,7 +339,7 @@ static size_t stress_wcsncat(const stress_args_t *args, stress_wcs_args_t *info)
 		WCSCHK(info, strdst == test_wcsncat(strdst, str2, i));
 		WCSCHK(info, strdst == test_wcsncat(strdst, str1, i));
 	}
-	add_counter(args, 6);
+	stress_bogo_add(args, 6);
 	return i * 6;
 }
 #endif
@@ -353,14 +359,14 @@ static size_t stress_wcschr(const stress_args_t *args, stress_wcs_args_t *info)
 	const size_t len1 = info->len1;
 	register size_t i;
 
-	for (i = 0; keep_stressing_flag() && (i < len1 - 1); i++) {
+	for (i = 0; stress_continue_flag() && (i < len1 - 1); i++) {
 		WCSCHK(info, NULL == test_wcschr(str1, '_'));
 		WCSCHK(info, NULL != test_wcschr(str1, str1[0]));
 
 		WCSCHK(info, NULL == test_wcschr(str2, '_'));
 		WCSCHK(info, NULL != test_wcschr(str2, str2[0]));
 	}
-	add_counter(args, 4);
+	stress_bogo_add(args, 4);
 	return i * 4;
 }
 #endif
@@ -380,14 +386,14 @@ static size_t stress_wcsrchr(const stress_args_t *args, stress_wcs_args_t *info)
 	const size_t len1 = info->len1;
 	register size_t i;
 
-	for (i = 0; keep_stressing_flag() && (i < len1 - 1); i++) {
+	for (i = 0; stress_continue_flag() && (i < len1 - 1); i++) {
 		WCSCHK(info, NULL == test_wcsrchr(str1, '_'));
 		WCSCHK(info, NULL != test_wcsrchr(str1, str1[0]));
 
 		WCSCHK(info, NULL == test_wcsrchr(str2, '_'));
 		WCSCHK(info, NULL != test_wcsrchr(str2, str2[0]));
 	}
-	add_counter(args, 4);
+	stress_bogo_add(args, 4);
 	return i * 4;
 }
 #endif
@@ -408,7 +414,7 @@ static size_t stress_wcscmp(const stress_args_t *args, stress_wcs_args_t *info)
 	const size_t len1 = info->len1;
 	register size_t i;
 
-	for (i = 1; keep_stressing_flag() && (i < len1); i++) {
+	for (i = 1; stress_continue_flag() && (i < len1); i++) {
 		WCSCHK(info, 0 == test_wcscmp(str1, str1));
 		WCSCHK(info, 0 == test_wcscmp(str2, str2));
 
@@ -422,7 +428,7 @@ static size_t stress_wcscmp(const stress_args_t *args, stress_wcs_args_t *info)
 		WCSCHK(info, 0 != test_wcscmp(str1 + i, str2));
 		WCSCHK(info, 0 != test_wcscmp(str2, str1 + i));
 	}
-	add_counter(args, 9);
+	stress_bogo_add(args, 9);
 	return i * 9;
 }
 #endif
@@ -443,7 +449,7 @@ static size_t stress_wcsncmp(const stress_args_t *args, stress_wcs_args_t *info)
 	const size_t len2 = info->len2;
 	register size_t i;
 
-	for (i = 1; keep_stressing_flag() && (i < len1); i++) {
+	for (i = 1; stress_continue_flag() && (i < len1); i++) {
 		WCSCHK(info, 0 == test_wcsncmp(str1, str1, len1));
 		WCSCHK(info, 0 == test_wcsncmp(str2, str2, len2));
 
@@ -457,7 +463,7 @@ static size_t stress_wcsncmp(const stress_args_t *args, stress_wcs_args_t *info)
 		WCSCHK(info, 0 != test_wcsncmp(str1 + i, str2, len2));
 		WCSCHK(info, 0 != test_wcsncmp(str2, str1 + i, len2));
 	}
-	add_counter(args, 9);
+	stress_bogo_add(args, 9);
 	return i * 9;
 }
 #endif
@@ -478,16 +484,16 @@ static size_t stress_wcslen(const stress_args_t *args, stress_wcs_args_t *info)
 	const size_t len2 = info->len2;
 	register size_t i;
 
-	for (i = 0; keep_stressing_flag() && (i < len1 - 1); i++) {
+	for (i = 0; stress_continue_flag() && (i < len1 - 1); i++) {
 		WCSCHK(info, len1 - 1 == test_wcslen(str1));
 		WCSCHK(info, len1 - 1 - i == test_wcslen(str1 + i));
 	}
 
-	for (i = 0; keep_stressing_flag() && (i < len2 - 1); i++) {
+	for (i = 0; stress_continue_flag() && (i < len2 - 1); i++) {
 		WCSCHK(info, len2 - 1 == test_wcslen(str2));
 		WCSCHK(info, len2 - 1 - i == test_wcslen(str2 + i));
 	}
-	add_counter(args, 4);
+	stress_bogo_add(args, 4);
 	return i * 4;
 }
 #endif
@@ -507,7 +513,7 @@ static size_t stress_wcscoll(const stress_args_t *args, stress_wcs_args_t *info)
 	const size_t len1 = info->len1;
 	register size_t i;
 
-	for (i = 1; keep_stressing_flag() && (i < len1); i++) {
+	for (i = 1; stress_continue_flag() && (i < len1); i++) {
 		WCSCHK(info, 0 == test_wcscoll(str1, str1));
 		WCSCHK(info, 0 == test_wcscoll(str2, str2));
 
@@ -521,7 +527,7 @@ static size_t stress_wcscoll(const stress_args_t *args, stress_wcs_args_t *info)
 		WCSCHK(info, 0 != test_wcscoll(str1 + i, str2));
 		WCSCHK(info, 0 != test_wcscoll(str2, str1 + i));
 	}
-	add_counter(args, 9);
+	stress_bogo_add(args, 9);
 	return i * 9;
 }
 #endif
@@ -543,7 +549,7 @@ static size_t stress_wcsxfrm(const stress_args_t *args, stress_wcs_args_t *info)
 	const size_t strdstlen = info->strdstlen;
 	register size_t i;
 
-	for (i = 0; keep_stressing_flag() && (i < len1 - 1); i++) {
+	for (i = 0; stress_continue_flag() && (i < len1 - 1); i++) {
 		*strdst = '\0';
 		WCSCHK(info, 0 != test_wcsxfrm(strdst, str1, strdstlen));
 		*strdst = '\0';
@@ -555,7 +561,7 @@ static size_t stress_wcsxfrm(const stress_args_t *args, stress_wcs_args_t *info)
 		WCSCHK(info, 0 != test_wcsxfrm(strdst, str2, strdstlen));
 		WCSCHK(info, 0 != test_wcsxfrm(strdst, str1, strdstlen));
 	}
-	add_counter(args, 6);
+	stress_bogo_add(args, 6);
 	return i * 6;
 }
 #endif
@@ -570,9 +576,9 @@ static const stress_wcs_method_info_t wcs_methods[] = {
 #if defined(HAVE_WCSCASECMP)
 	{ "wcscasecmp",		stress_wcscasecmp,	(void *)wcscasecmp },
 #endif
-#if defined(HAVE_WCSLCAT) &&	\
-    defined(HAVE_WCSLEN) &&	\
-    !defined(__PCC__) &&	\
+#if defined(HAVE_WCSLCAT) &&		\
+    defined(HAVE_WCSLEN) &&		\
+    !defined(HAVE_COMPILER_PCC) &&	\
     !defined(BUILD_STATIC)
 	{ "wcslcat",		stress_wcslcat,		(void *)wcslcat },
 #elif defined(HAVE_WCSCAT)
@@ -585,9 +591,9 @@ static const stress_wcs_method_info_t wcs_methods[] = {
     !defined(STRESS_ARCH_M68K)
 	{ "wcscmp",		stress_wcscmp,		(void *)wcscmp },
 #endif
-#if defined(HAVE_WCSLCPY) &&	\
-    defined(HAVE_WCSLEN) &&	\
-    !defined(__PCC__) &&	\
+#if defined(HAVE_WCSLCPY) &&		\
+    defined(HAVE_WCSLEN) &&		\
+    !defined(HAVE_COMPILER_PCC) &&	\
     !defined(BUILD_STATIC)
 	{ "wcslcpy",		stress_wcslcpy,		(void *)wcslcpy },
 #elif defined(HAVE_WCSCPY)
@@ -628,6 +634,9 @@ static size_t stress_wcs_all(const stress_args_t *args, stress_wcs_args_t *info)
 	stress_wcs_args_t info_all = *info;
 	double t;
 
+	if (UNLIKELY(SIZEOF_ARRAY(wcs_methods) < 2))
+		return 0;
+
 	info_all.libc_func = wcs_methods[i].libc_func;
 
 	t = stress_time_now();
@@ -677,9 +686,10 @@ static int stress_wcs(const stress_args_t *args)
 	wchar_t ALIGN64 str1[STR1LEN], ALIGN64 str2[STR2LEN];
 	wchar_t strdst[STRDSTLEN];
 	stress_wcs_args_t info;
+	int metrics_count = 0;
 
 	/* No wcs* functions available on this system? */
-	if (SIZEOF_ARRAY(wcs_methods) <= 2)
+	if (SIZEOF_ARRAY(wcs_methods) < 2)
 		return stress_unimplemented(args);
 
 	(void)stress_get_setting("wcs-method", &wcs_method);
@@ -692,6 +702,7 @@ static int stress_wcs(const stress_args_t *args)
 	info.strdst = strdst;
 	info.strdstlen = STRDSTLEN;
 	info.failed = false;
+	info.name = args->name;
 
 	stress_wcs_fill(info.str1, info.len1);
 
@@ -705,12 +716,18 @@ static int stress_wcs(const stress_args_t *args)
 	do {
 		register wchar_t *tmpptr;
 		register size_t tmplen;
-		double t;
 
 		stress_wcs_fill(info.str2, info.len2);
-		t = stress_time_now();
-		metrics[wcs_method].count += (double)wcs_method_info->func(args, &info);
-		metrics[wcs_method].duration += (stress_time_now() - t);
+		if (UNLIKELY(metrics_count++ > 1000)) {
+			double t;
+
+			metrics_count = 0;
+			t = stress_time_now();
+			metrics[wcs_method].count += (double)wcs_method_info->func(args, &info);
+			metrics[wcs_method].duration += (stress_time_now() - t);
+		} else {
+			(void)wcs_method_info->func(args, &info);
+		}
 
 		tmpptr = info.str1;
 		info.str1 = info.str2;
@@ -720,8 +737,8 @@ static int stress_wcs(const stress_args_t *args)
 		info.len1 = info.len2;
 		info.len2 = tmplen;
 
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
